//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2008
//
// Add node to polygon/polyline.
//

# include "StdAfx.h"
# include "AddPoint.h"
# include "UndoRedo.h"
# include "Label.h"
# include "MapView.h"

class CAddPointAction : public CEditAction {
  public:
	CAddPointAction (rgn_t * _pRgn, size_t _cElement, size_t _cPointBefore, const point_t & _pointNew) :
		m_pRgn (_pRgn),
		m_cElement (_cElement),
		m_cPointBefore (_cPointBefore),
        m_pointNew (_pointNew),
		m_bWasModified ((_pRgn->dwFlags & rgn_t::fModified) != 0)
	{
		assert (0 <= m_cPointBefore);
	}

	virtual void GetName (string_t & _str) const;
	virtual cu_t CanUndo () const {return cuYes;}
	virtual bool Do ();
	virtual void Undo ();
	virtual void Redo ();

  private:
	rgn_t * const m_pRgn;
	size_t  const m_cElement;
	size_t  const m_cPointBefore;
    point_t const m_pointNew;
	bool    const m_bWasModified;
};

void CAddPointAction::GetName (string_t & _str) const {
	_str.append ("insert node [");
	GetHintText (* m_pRgn, _str);
	_str.append ("]");
}

bool CAddPointAction::Do () {
	CCritSec::CLock lock (g_cs);

	assert (m_pRgn);
	assert (m_cElement < m_pRgn->elements.size ());
	rgn_element_t & element = m_pRgn->elements [m_cElement];

	points_t & points = element.points;
	const size_t cPrevSize = points.size ();
	points.insert (points.begin () + m_cPointBefore, m_pointNew);

	if (element.pExtra != NULL) {
		point_extras_t & PointExtras = element.pExtra->PointExtras;
		if (! PointExtras.empty ()) {
			PointExtras.resize (cPrevSize);
			PointExtras.insert (PointExtras.begin () + m_cPointBefore, point_extra_t ());
			assert (PointExtras.size () == points.size ());
		}
	}

	m_pRgn->RefreshAttributes (g_map);
	m_pRgn->dwFlags |= rgn_t::fModified;

	return true;
}

void CAddPointAction::Undo () {
	CCritSec::CLock lock (g_cs);

	assert (m_pRgn);
	assert (m_cElement < m_pRgn->elements.size ());
	rgn_element_t & element = m_pRgn->elements [m_cElement];

	points_t & points = element.points;
	const size_t cPrevSize = points.size ();
	points.erase (points.begin () + m_cPointBefore);

	if (element.pExtra != NULL) {
		point_extras_t & PointExtras = element.pExtra->PointExtras;
		if (! PointExtras.empty ()) {
			PointExtras.resize (cPrevSize);
			PointExtras.erase (PointExtras.begin () + m_cPointBefore);
			assert (PointExtras.size () == points.size ());
		}
	}

	m_pRgn->RefreshAttributes (g_map);
	if (! m_bWasModified)
		m_pRgn->dwFlags &= ~rgn_t::fModified;
}

void CAddPointAction::Redo () {
	CAddPointAction::Do ();
}

void AddPoint (rgn_t * _pRgn, size_t _cElement, size_t _cPointAfter, const point_t & _pointNew) {
	assert (_pRgn);
	assert (_pRgn->IsPolyline () || _pRgn->IsPolygon ());
	assert (_cPointAfter == -1 || _cPointAfter < _pRgn->elements [_cElement].points.size ());

	PerformEditAction (new CAddPointAction (_pRgn, _cElement, _cPointAfter + 1, _pointNew));

	Refresh ();
}
