//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Backup tool for actions not suppoting Undo() method.
//

# include "StdAfx.h"
# include "Resource.h"
# include "BackupAction.h"
# include "Status.h"
# include "Globals.h"

CInvokeOnBackupMap * CInvokeOnBackupMap::s_pFirst;

CInvokeOnBackupMap::CInvokeOnBackupMap (pvfv_t _pf) :
	m_pf (_pf)
{
	// Insert to the head of the list.
	m_pNext = CInvokeOnBackupMap::s_pFirst;
	CInvokeOnBackupMap::s_pFirst = this;
}

static
void _ResetAnyRgnLists () {
	for (const CInvokeOnBackupMap * pBM = CInvokeOnBackupMap::s_pFirst; pBM != NULL; pBM = pBM->m_pNext)
		pBM->m_pf ();
}

///////////////////////////////////////////////////////////////////////

CBackupAction::~CBackupAction () {
	delete m_pAction;
}

void CBackupAction::GetName (string_t & _str) const {
	m_pAction->GetName (_str);
}

bool CBackupAction::Do () {
	CCritSec::CLock lock (g_cs);

	m_bWasModified = g_bModified;
	m_Selection = g_Selection;

	SetStatus (GetStr (IDS_STATUS_BACKING_UP));
	m_map.Copy (g_map, true);
	SetStatusReady ();

	g_map.swap (m_map);

	bool bRet = m_pAction->Do ();
	if (! bRet) {
		g_map.swap (m_map);
		g_Selection.swap (m_Selection);
	}

	if (bRet)
		_ResetAnyRgnLists ();

	return bRet;
}

void CBackupAction::Undo () {
	CCritSec::CLock lock (g_cs);

	g_map.swap (m_map);
	g_Selection.swap (m_Selection);
	m_pAction->Undo ();

	g_map.bDirtyRGNodes = true;
	g_bModified = m_bWasModified;

	_ResetAnyRgnLists ();
}

void CBackupAction::Redo () {
	CCritSec::CLock lock (g_cs);

	g_map.swap (m_map);
	g_Selection.swap (m_Selection);
	m_pAction->Redo ();

	g_map.bDirtyRGNodes = true;
	m_bWasModified = g_bModified;

	_ResetAnyRgnLists ();
}
