//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Bookmark class.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Bookmark.h"
# include "BookmarkTypes.h"
# include "TrackVideo.h"
# include "CreateObject.h"
# include "SetBookmark.h"
# include "DeleteObject.h"
# include "Map.h"
# include "MapView.h"
# include "Load.h"
# include "Window.h"
# include "Dialog.h"
# include "ListView.h"
# include "ComboBox.h"
# include "EditText.h"
# include "DrawHelpers.h"
# include "ParseHelpers.h"
# include "Settings.h"
# include "fast_floor.h"
# include "Globals.h"

static string_t g_strLastBookmarkType;
static string_t g_strLastBookmarkSubType;

rgn_t & CreateBookmark (const point_t & _pt) {
	points_t points (1);
	points [0] = _pt;
	rgn_t & rgn = CreateObject (points, rkPoint, 0);
	rgn.pExtra->pBookmark = new bookmark_t;

	rgn.pExtra->pBookmark->strType    = g_strLastBookmarkType.empty () ? "comment" : g_strLastBookmarkType;
	rgn.pExtra->pBookmark->strSubType = g_strLastBookmarkType.empty () ? "info"    : g_strLastBookmarkSubType;
	rgn.pExtra->pBookmark->FixPos (rgn);

//	SYSTEMTIME st; 
//	::GetSystemTime (& st);
//	::SystemTimeToVariantTime (& st, & rgn.pExtra->pBookmark->dt);

	g_bHaveBookmarks = true;

	return rgn;
}

/////////////////////////////////////////////////////////////////////

enum {
	colKey   = 0,
	colValue = 1
};

class CBookmarkProperties : public CDialog {
  public:
	CBookmarkProperties (rgn_t & _rgn, const bookmark_t & _bm, bool _bInitModified, bool & _bDeleteApplied) :
		CDialog (IDD_BOOKMARK),
		m_Types (BookmarkTypes ()),
		m_pRgn (& _rgn),
		m_bookmark (_bm),
		m_cBookmarkOrdinal (0),
		m_cBookmarksHere (1),
		m_bUpdateFields (false),
		m_bHasHeadingIsConst (false),
		m_bApplied (false),
		m_bInitModified (_bInitModified),
		m_bDeleteApplied (_bDeleteApplied)
	{
		bookmark_t::GetPos (* m_pRgn, m_ptPosHere);

		_FindNeighbours ();
	}

	// Override.
	virtual bool OnInit (HWND _hwndFocus);
	virtual void OnMove (int _x, int _y);
	virtual	void OnPaint (const PAINTSTRUCT & _ps) const;
	virtual void OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl);
	virtual bool OnNotify (int _idCtrl, NMHDR * _pInfo);
	virtual INT_PTR OnMessage (UINT _uMsg, WPARAM _wParam, LPARAM _lParam);

  private:
	static CBookmarkProperties * s_pThis;

	rgn_t      * m_pRgn;
	bookmark_t m_bookmark;
	point_t m_ptPosHere;
	size_t m_cBookmarkOrdinal;
	size_t m_cBookmarksHere;
	typedef std::vector<rgn_t *> vecBookmarks_t;
	vecBookmarks_t m_vecBookmarks;
	void _FindNeighbours ();

	void _Apply ();
	bool m_bUpdateFields;
	void _UpdateFields ();
	void _UpdateSubType (bool _bReset);
	void _UpdateAttributes ();
	void _UpdateOrdinal ();

	const bm_types_t & m_Types;
	int _GetSelType () const;

	HWND m_hAttributes;
	bool _GetAttribute (size_t _cAttributeIdx, bm_attribute_t & _attribute);
	size_t m_cCurrentAttributeIdx;
	BmAttributeKind m_CurrentAttributeKind;

	HWND m_hDate;

	HWND m_hAttributeEdit;
	bool m_bEditAttribute;
	static WNDPROC s_wpAttributeEdit;
	static LRESULT APIENTRY _AttributeEditProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam);
	void _OnEditAttributeStart (const string_t & _strValue, int _x, int _y, int _w, int _h);
	void _OnEditAttributeComplete (bool _bDone);

	HWND m_hHeading;
	void _UpdateHeading ();
	void _UpdateHeadingV ();
	void _EnablePos ();
	void _SetHasHeadingByDefault ();
	void _EnableHasHeading ();
	bool m_bHasHeadingIsConst;

	bool m_bApplied;
	bool m_bModified;
	const bool m_bInitModified;
	void _SetModified (bool _bModified = true);

	bool & m_bDeleteApplied;
};

CBookmarkProperties * CBookmarkProperties::s_pThis;
WNDPROC CBookmarkProperties::s_wpAttributeEdit = NULL;

void CBookmarkProperties::_FindNeighbours () {
	m_vecBookmarks.clear ();
	m_cBookmarkOrdinal = 0;

	CCritSec::CLock lock (g_cs);

	rgns_t & rgns = g_map.layers [rkPoint].rgns;
	for (rgns_t::iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
		rgn_t & rgn = * iRgn;
		if (rgn.IsImmutable () || ! rgn.IsBookmark ())
			continue;

		bool bNeighbour = false;

		point_t ptPos;
		bookmark_t::GetPos (rgn, ptPos);
		if (ptPos == m_ptPosHere)
			bNeighbour = true;
		else if (m_pRgn != NULL) {
			const bookmark_t & bm = * rgn.pExtra->pBookmark;
			const bookmark_t & bm0 = * m_pRgn->pExtra->pBookmark;
			if (bm.dwFilePos == bm0.dwFilePos && ! bm.strFileName.empty () && bm.strFileName == bm0.strFileName)
				bNeighbour = true;
		}

		if (bNeighbour) {
			if (& rgn == m_pRgn)
				m_cBookmarkOrdinal = m_vecBookmarks.size ();

			m_vecBookmarks.push_back (& rgn);
		}
	}

	m_cBookmarksHere = m_vecBookmarks.size ();

	if (m_pRgn == NULL)
		m_cBookmarkOrdinal = 0;
}

void CBookmarkProperties::_SetModified (bool _bModified) {
	m_bModified = _bModified;
	EnableDlgItem (IDC_APPLY, m_bModified);
}

static const UINT c_uiRadioBtns [] = {IDC_LEFT, IDC_RIGHT};

bool CBookmarkProperties::OnInit (HWND _hwndFocus) {
	s_pThis = this;

	// Show only used types.
	CheckDlgButton (IDC_SHOW_USED_ONLY, GetSetting ("Bookmark_ShowUsedTypes", false));

	_SetModified (m_bInitModified);
	EnableDlgItem (IDC_DELETE, ! m_bInitModified);

	// Attributes.
	m_hAttributes = GetDlgItem (IDC_ATTRIBUTES);
	ListView_SetExtendedListViewStyleEx (m_hAttributes, LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES, LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);
	AddColumn (m_hAttributes, colKey,   "");
	AddColumn (m_hAttributes, colValue, "");

	m_cCurrentAttributeIdx = -1;
	m_CurrentAttributeKind = bakString;

	// Editbox to edit attributes' values in-place.
	m_hAttributeEdit = ::CreateWindow (
		"EDIT", NULL,
		ES_LEFT | ES_AUTOHSCROLL | WS_CHILDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | ES_WANTRETURN,
		0, 0, 1, 1,
		m_hWnd, NULL, g_hInst, NULL
	);
	if (m_hAttributeEdit) {
		// Subclass the edit box to hook ENTER and ESC.
		CBookmarkProperties::s_wpAttributeEdit = reinterpret_cast<WNDPROC> (::GetWindowLong (m_hAttributeEdit, GWL_WNDPROC));
		::SetWindowLong (m_hAttributeEdit, GWL_WNDPROC, reinterpret_cast<long> (_AttributeEditProc));
	}
	m_bEditAttribute = false;

	_UpdateFields ();

	// Restore position.
	if (IsSettingDefined ("Bookmark_X")) {
		const DWORD dwX = GetSetting ("Bookmark_X", (DWORD) 0);
		const DWORD dwY = GetSetting ("Bookmark_Y", (DWORD) 0);
		::SetWindowPos (m_hWnd, NULL, dwX, dwY, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
		EnsureVisibleOnScreen (m_hWnd);
	}

	// Heading.
	_EnableHasHeading ();
	if (m_bInitModified)
		_SetHasHeadingByDefault ();
	CheckDlgButton (IDC_HAS_HEADING, m_bookmark.bHasHeading);
	_UpdateHeadingV ();
	m_hHeading = GetDlgItem (IDC_HEADING);
	::ShowWindow (m_hHeading, SW_HIDE);
	_UpdateHeading ();

	return true;
}

void CBookmarkProperties::_UpdateHeadingV () {
	m_bUpdateFields = true;
	SetDlgItemInt (IDC_HEADING_V, fast_floor (m_bookmark.fHeading + .5f));
	m_bUpdateFields = false;
}

void CBookmarkProperties::_UpdateHeading () {
	::InvalidateRect (m_hWnd, NULL, TRUE);
	::UpdateWindow (m_hWnd);
}

void CBookmarkProperties::_UpdateFields () {
	m_bUpdateFields = true;

	// Type.
	const HWND hType = GetDlgItem (IDC_TYPE);
	size_t cSelType = -1;
	const size_t cTypes = m_Types.size ();
	for (size_t c = 0; c < cTypes; ++ c) {
		const bm_type_t & t = m_Types [c];
		AddComboBox (hType, c, c, t.strName.c_str ());
		if (t.strId == m_bookmark.strType)
			cSelType = c;
	}
	if (cSelType == -1) {
		AddComboBox (hType, cTypes, cTypes, m_bookmark.strType.c_str ());
		cSelType = cTypes;
	}
	SelectComboBox (hType, cSelType);

	// Sub-type.
	_UpdateSubType (false);

	// Distance along road.
	SetDlgItemInt  (IDC_DISTANCE,         fast_floor (m_bookmark.fDistanceAlongRoad + .5f));
	// Distance to road.
	SetDlgItemUInt (IDC_DISTANCE_TO_ROAD, fast_floor (m_bookmark.fDistanceToRoad    + .5f));
	// Left/Right.
	CheckRadioButton (m_bookmark.bLeftToRoad ? IDC_LEFT : IDC_RIGHT, c_uiRadioBtns, ARRAY_SIZE (c_uiRadioBtns));
	_EnablePos ();

	// Date.
	m_hDate = GetDlgItem (IDC_DATE);
	SYSTEMTIME st;
	::VariantTimeToSystemTime (m_bookmark.dt, & st);
	DateTime_SetSystemtime (m_hDate, m_bookmark.dt != 0 ? GDT_VALID : GDT_NONE, & st);

	// Processed.
	CheckDlgButton (IDC_PROCESSED, m_bookmark.bProcessed);

	// File.
	SetDlgItemText (IDC_FILE, ! m_bookmark.strFileName.empty () ? m_bookmark.strFileName.c_str () : "-");
	EnableDlgItem (IDC_SHOW_VT, ! m_bookmark.strFileName.empty ());

	// Ordinal.
	SetDlgItemTextF (IDC_ORDINAL, "%d/%d", m_cBookmarkOrdinal + 1, m_cBookmarksHere);
	EnableDlgItem (IDC_PREV, m_cBookmarkOrdinal > 0);
	EnableDlgItem (IDC_NEXT, m_cBookmarkOrdinal + 1 < m_cBookmarksHere);

	m_bUpdateFields = false;
}

int CBookmarkProperties::_GetSelType () const {
	int nType = 0;
	if (! GetComboBoxItemData (m_hWnd, IDC_TYPE, nType))
		return -1;
	if (nType < 0 || nType >= (int) m_Types.size ())
		return -1;

	return nType;
}

void CBookmarkProperties::_UpdateSubType (bool _bReset) {
	const HWND hSubType = GetDlgItem (IDC_SUBTYPE);
	EmptyComboBox (hSubType);

	size_t cSelSubType = -1;
	const int nType = _GetSelType ();
	if (nType == -1) {
		AddComboBox (hSubType, 0, 0, m_bookmark.strSubType.c_str ());
		cSelSubType = 0;
	} else {
		assert (nType < (int) m_Types.size ());
		const bm_type_t & Type = m_Types [nType];
		const bm_subtypes_t & vecSubTypes = Type.vecSubTypes;
		const size_t cSubTypes = vecSubTypes.size ();

		// Prepare used sub-types.
		std::set<string_t> setSubTypes;
		const bool bShowUsedTypes = GetSetting ("Bookmark_ShowUsedTypes", false);
		if (bShowUsedTypes) {
			const layer_t & layer = g_map.layers [rkPoint];
			const rgns_t & rgns = layer.rgns;
			rgns_t::const_iterator iRgn;
			for (iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
				const rgn_t & rgn = * iRgn;
				if (rgn.IsImmutable () || ! rgn.IsBookmark ())
					continue;
				
				const bookmark_t & bm = * rgn.pExtra->pBookmark;
				if (! (bm.strType == Type.strId))
					continue;

				setSubTypes.insert (bm.strSubType);
			}
		}

		// Fill out the list.
		size_t cItem = 0;
		for (size_t c = 0; c < cSubTypes; ++ c) {
			const bm_subtype_t & st = vecSubTypes [c];
			const bool bShow = bShowUsedTypes == false || setSubTypes.find (st.strId) != setSubTypes.end ();
			if (! bShow)
				continue;

			AddComboBox (hSubType, cItem, c, st.strName.c_str ());
			if (st.strId == m_bookmark.strSubType)
				cSelSubType = cItem;
			++ cItem;
		}
		if (cSelSubType == -1) {
			if (_bReset) {
				cSelSubType = 0;
			} else {
				AddComboBox (hSubType, cItem, cSubTypes, m_bookmark.strSubType.c_str ());
				cSelSubType = cItem;
			}
		}
	}
	SelectComboBox (hSubType, cSelSubType);

	_UpdateAttributes ();
}

void CBookmarkProperties::_UpdateAttributes () {
	ListView_DeleteAllItems (m_hAttributes);

	bool bTypeIsFound = false;

	const int nType = _GetSelType ();
	if (nType != -1) {
		const bm_subtypes_t & vecSubTypes = m_Types [nType].vecSubTypes;
		const size_t cSubTypes = vecSubTypes.size ();

		int nSubType = 0;
		if (GetComboBoxItemData (m_hWnd, IDC_SUBTYPE, nSubType)) {
			if (0 <= nSubType && nSubType < (int) vecSubTypes.size ()) {
				bTypeIsFound = true;

				const bm_attributes_t & vecAttributes = vecSubTypes [nSubType].vecAttributes;
				const size_t cAttributes = vecAttributes.size ();

				for (size_t c = 0; c < cAttributes; ++ c) {
					const bm_attribute_t & att = vecAttributes [c];
					InsertItem     (m_hAttributes, c, c, "%s%s", att.strName.c_str (), att.bObligatory ? "*" : "");
					SetSubItemText (m_hAttributes, c, colValue, m_bookmark.attributes [att.strId].c_str ());
				}
			}
		}
	}

	if (! bTypeIsFound) {
		size_t cItem = 0;
		for (bookmark_t::k2v_t::const_iterator i = m_bookmark.attributes.begin (); i != m_bookmark.attributes.end (); ++ i) {
			const string_t & strKey   = i->first;
			const string_t & strValue = i->second;

			InsertItem     (m_hAttributes, cItem, -1, "%s", strKey.c_str ());
			SetSubItemText (m_hAttributes, cItem, colValue, strValue.c_str ());

			++ cItem;
		}
	}

	AutoSizeColumns (m_hAttributes);
}

void CBookmarkProperties::_UpdateOrdinal () {
	_Apply ();

	if (m_cBookmarkOrdinal >= m_vecBookmarks.size ())
		m_cBookmarkOrdinal = 0;
	m_pRgn = m_vecBookmarks.empty () ? NULL : m_vecBookmarks [m_cBookmarkOrdinal];
	if (m_pRgn == NULL)
		return;
	m_bookmark = m_pRgn->pExtra->pBookmark ? * m_pRgn->pExtra->pBookmark : bookmark_t ();

	_UpdateFields ();
}

void CBookmarkProperties::_Apply () {
	if (m_pRgn == NULL) {
		m_bApplied = true;
		return;
	}

	// Type, subtype.
	const int nType = _GetSelType ();
	if (nType != -1) {
		const bm_type_t & t = m_Types [nType];
		m_bookmark.strType = t.strId;
		if (m_bInitModified)
			g_strLastBookmarkType = t.strId;

		int nSubType = 0;
		if (GetComboBoxItemData (m_hWnd, IDC_SUBTYPE, nSubType)) {
			if (nSubType >= 0 && nSubType < (int) t.vecSubTypes.size ()) {
				const bm_subtype_t & st = t.vecSubTypes [nSubType];
				m_bookmark.strSubType = st.strId;

				if (m_bInitModified)
					g_strLastBookmarkSubType = st.strId;
			}
		}
	}

	// Attributes.
	if (m_bookmark.RemoveDummyAttributes ())
		m_bModified = true;

	// Processed.
	m_bookmark.bProcessed         = IsDlgButtonChecked (IDC_PROCESSED);

	// Position.
	m_bookmark.fDistanceAlongRoad = GetDlgItemInt  (IDC_DISTANCE);
	m_bookmark.fDistanceToRoad    = GetDlgItemUInt (IDC_DISTANCE_TO_ROAD);
	m_bookmark.bLeftToRoad        = IsDlgButtonChecked (IDC_LEFT);
	m_bookmark.FixPos (* m_pRgn);

	if (m_bModified)
		SetBookmark (m_pRgn, m_bookmark);

	_SetModified (false);

	m_bApplied = true;
}

void CBookmarkProperties::OnMove (int _x, int _y) {
	RECT rect;
	::GetWindowRect (m_hWnd, & rect);
	if (rect.left < 0)
		rect.left = 0;
	if (rect.top < 0)
		rect.top = 0;

	SaveSetting ("Bookmark_X", (DWORD) rect.left);
	SaveSetting ("Bookmark_Y", (DWORD) rect.top);
}

void CBookmarkProperties::OnPaint (const PAINTSTRUCT & _ps) const {
	WINDOWPLACEMENT wp = {sizeof (WINDOWPLACEMENT)};
	::GetWindowPlacement (m_hHeading, & wp);

	const int x0 = (wp.rcNormalPosition.left + wp.rcNormalPosition.right)/2;
	const int y0 = (wp.rcNormalPosition.top + wp.rcNormalPosition.bottom)/2;
	const int r  = (wp.rcNormalPosition.right - wp.rcNormalPosition.left)/2;

	::SelectObject (_ps.hdc, ::GetStockObject (BLACK_PEN));
	::SelectObject (_ps.hdc, ::GetStockObject (m_bookmark.bHasHeading ? WHITE_BRUSH : GRAY_BRUSH));
	::Ellipse (_ps.hdc, x0 - r, y0 - r, x0 + r, y0 + r);

	if (m_bookmark.bHasHeading) {
		DrawLine (
			_ps.hdc, x0, y0,
			x0 + fast_floor (r*::cos (c_PI/2 - m_bookmark.fHeading*c_PI/180) + .5f),
			y0 - fast_floor (r*::sin (c_PI/2 - m_bookmark.fHeading*c_PI/180) + .5f)
		);
	}
}

static
const rgn_t * _FindAttachment (const string_t & _strFileName) {
	CCritSec::CLock lock (g_cs);

	const rgns_t & rgns = g_map.layers [rkPolyline].rgns;
	for (rgns_t::const_iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
		const rgn_t & rgn = * iRgn;
		if (rgn.IsDeleted () || ! rgn.IsTrack ())
			continue;
		if (rgn.pFileRef && rgn.pFileRef->strFileName == _strFileName)
			return & rgn;
	}

	return NULL;
}

void CBookmarkProperties::_EnablePos () {
	EnableDlgItem (IDC_HEADING_V,        m_bookmark.bHasHeading);
	EnableDlgItem (IDC_DISTANCE,         m_bookmark.bHasHeading);
	EnableDlgItem (IDC_DISTANCE_TO_ROAD, m_bookmark.bHasHeading);
	EnableDlgItem (IDC_LEFT,             m_bookmark.bHasHeading);
	EnableDlgItem (IDC_RIGHT,            m_bookmark.bHasHeading);
}

void CBookmarkProperties::_SetHasHeadingByDefault () {
	const int nType = _GetSelType ();
	if (nType == -1)
		return;
	const bm_type_t & t = m_Types [nType];

	if (m_bookmark.bHasHeading == false && t.bHasHeadingByDefault) {
		m_bookmark.bHasHeading = true;
		CheckDlgButton (IDC_HAS_HEADING, m_bookmark.bHasHeading);
		_EnablePos ();
		_UpdateHeading ();
	}
}

void CBookmarkProperties::_EnableHasHeading () {
	const int nType = _GetSelType ();
	if (nType == -1)
		return;
	const bm_type_t & t = m_Types [nType];

	m_bHasHeadingIsConst = t.bHasHeadingIsConst;
	EnableDlgItem (IDC_HAS_HEADING, ! m_bHasHeadingIsConst);

	if (m_bHasHeadingIsConst && m_bookmark.bHasHeading != t.bHasHeadingByDefault) {
		m_bookmark.bHasHeading = t.bHasHeadingByDefault;
		_SetModified ();

		CheckDlgButton (IDC_HAS_HEADING, m_bookmark.bHasHeading);
		_EnablePos ();
		_UpdateHeading ();
	}
}

void CBookmarkProperties::OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
	switch (_wCmd) {
		case IDC_SHOW_USED_ONLY: {
			const bool bShowUsedTypes = IsDlgButtonChecked (IDC_SHOW_USED_ONLY);
			SaveSetting ("Bookmark_ShowUsedTypes", bShowUsedTypes);

			_UpdateSubType (true);
			break;
		}

		case IDC_HAS_HEADING: {
			_SetModified ();

			m_bookmark.bHasHeading = IsDlgButtonChecked (IDC_HAS_HEADING);
			_EnablePos ();
			_UpdateHeading ();
			break;
		}

		case IDC_DISTANCE:
		case IDC_DISTANCE_TO_ROAD:
			if (_wNotifyCode == EN_CHANGE) {
				if (! m_bUpdateFields)
					_SetModified ();
			}
			break;

		case IDC_HEADING_V:
			if (_wNotifyCode == EN_CHANGE) {
				if (! m_bUpdateFields) {
					m_bookmark.fHeading = GetDlgItemInt (IDC_HEADING_V);
					if (0 > m_bookmark.fHeading || m_bookmark.fHeading >= 360)
						m_bookmark.fHeading = (m_bookmark.fHeading/360 - ::floor (m_bookmark.fHeading/360))*360;
					_UpdateHeading ();
					_SetModified ();
				}
			}
			break;

		case IDC_LEFT:
		case IDC_RIGHT:
			CheckRadioButton (_wCmd, c_uiRadioBtns, ARRAY_SIZE (c_uiRadioBtns));
			if (! m_bUpdateFields)
				_SetModified ();
			break;

		case IDC_TYPE:
			if (_wNotifyCode == CBN_SELCHANGE) {
				_UpdateSubType (true);
				_SetModified ();

				_SetHasHeadingByDefault ();
				_EnableHasHeading ();
			}
			break;

		case IDC_SUBTYPE:
			if (_wNotifyCode == CBN_SELCHANGE) {
				_UpdateAttributes ();
				_SetModified ();
			}
			break;

		case IDC_PREV:
			if (m_cBookmarkOrdinal > 0) {
				-- m_cBookmarkOrdinal;
				_UpdateOrdinal ();
			}
			break;
		case IDC_NEXT:
			if (m_cBookmarkOrdinal + 1 < m_cBookmarksHere) {
				++ m_cBookmarkOrdinal;
				_UpdateOrdinal ();
			}
			break;

		case IDC_PROCESSED:
			_SetModified ();
			break;

		case IDC_SHOW_VT: {
			if (m_bookmark.strFileName.empty ())
				break;

			const rgn_t * pRgnVT = _FindAttachment (m_bookmark.strFileName);
			if (! pRgnVT) {
				LoadMap (m_bookmark.strFileName.c_str (), 0, false, false, g_map);
				WaitLoadThread ();

				pRgnVT = _FindAttachment (m_bookmark.strFileName);
			}

			if (pRgnVT) {
				ShowTrackVideo (* pRgnVT, m_bookmark.dwFilePos);
				SendCommand (IDOK);
			}
			break;
		}

		case IDC_DELETE:
			DeleteObject (m_pRgn);
			Refresh ();

			m_bDeleteApplied = true;

			m_pRgn = NULL;
			_FindNeighbours ();
			if (m_cBookmarksHere == 0)
				EndDialog (0);
			else
				_UpdateOrdinal ();
			break;

		case IDC_APPLY:
		case IDOK: {
			if (m_bEditAttribute)
				_OnEditAttributeComplete (true);

			_Apply ();
			Refresh ();

			if (_wCmd == IDOK)
				EndDialog (1);
			break;
		}

		case IDCANCEL:
			EndDialog (m_bApplied ? 1 : 0);
			break;
	}
}

void CBookmarkProperties::_OnEditAttributeStart (const string_t & _strValue, int _x, int _y, int _w, int _h) {
	m_bEditAttribute = true;

	// Set font.
	const HFONT hFont = (HFONT) ::SendMessage (::GetParent (m_hAttributeEdit), WM_GETFONT, 0, 0);
	::SendMessage (m_hAttributeEdit, WM_SETFONT, reinterpret_cast<WPARAM> (hFont), FALSE);

	// Set text.
	::SetWindowText (m_hAttributeEdit, _strValue.c_str ());

	// Set position & size.
	::SetWindowPos (m_hAttributeEdit, HWND_TOP, _x, _y, _w, _h, SWP_SHOWWINDOW);

	// Set initial selection.
	::SetFocus (m_hAttributeEdit);
	::SendMessage (m_hAttributeEdit, EM_SETSEL, 0, _strValue.size ());
	::SendMessage (m_hAttributeEdit, EM_SCROLLCARET, 0, 0);
}

bool CBookmarkProperties::_GetAttribute (size_t _cAttributeIdx, bm_attribute_t & _attribute) {
	const int nType = _GetSelType ();
	if (nType == -1)
		return false;
	const bm_subtypes_t & vecSubTypes = m_Types [nType].vecSubTypes;
	const size_t cSubTypes = vecSubTypes.size ();

	int nSubType = 0;
	if (! GetComboBoxItemData (m_hWnd, IDC_SUBTYPE, nSubType))
		return false;
	if (nSubType < 0 || nSubType >= (int) vecSubTypes.size ())
		return false;
	const bm_attributes_t & vecAttributes = vecSubTypes [nSubType].vecAttributes;
	const size_t cAttributes = vecAttributes.size ();

	const size_t cAttribute = _cAttributeIdx;
	if (cAttribute >= cAttributes)
		return false;

	_attribute = vecAttributes [cAttribute];
	return true;
}

void CBookmarkProperties::_OnEditAttributeComplete (bool _bDone) {
//	assert (m_bEditAttribute);
	if (! m_bEditAttribute)
		return;

	// Return keyboard input from the edit box.
//	::SetFocus (m_hWnd);
	::ShowWindow (m_hAttributeEdit, SW_HIDE);

	CBookmarkProperties::s_pThis->m_bEditAttribute = false;

	if (! _bDone)
		return;

	if (m_cCurrentAttributeIdx == -1)
		return;

	const size_t cLen = ::GetWindowTextLength (m_hAttributeEdit) + 1;
	char * const strLabel = reinterpret_cast<char *> (::alloca (sizeof (char)*cLen));
	::GetWindowText (m_hAttributeEdit, strLabel, cLen);

	bm_attribute_t attribute;
	if (! _GetAttribute (m_cCurrentAttributeIdx, attribute))
		return;
	m_bookmark.attributes [attribute.strId] = strLabel;

	m_cCurrentAttributeIdx = -1;
	m_CurrentAttributeKind = bakString;

	_UpdateAttributes ();
	_SetModified ();
}

LRESULT APIENTRY CBookmarkProperties::_AttributeEditProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_GETDLGCODE:
			return DLGC_WANTALLKEYS | DLGC_WANTARROWS | DLGC_WANTCHARS | DLGC_HASSETSEL;

		case WM_KILLFOCUS:
			CBookmarkProperties::s_pThis->_OnEditAttributeComplete (true);
			break;

		case WM_CHAR:
			switch (_wParam) {
				case VK_RETURN: CBookmarkProperties::s_pThis->_OnEditAttributeComplete (true);  return 0; // NOTE: don't call ::CallWindowProc() to avoid beep.
				case VK_ESCAPE: CBookmarkProperties::s_pThis->_OnEditAttributeComplete (false); return 0;
			}
			break;
	}

	assert (CBookmarkProperties::s_wpAttributeEdit);
	return ::CallWindowProc (CBookmarkProperties::s_wpAttributeEdit, _hWnd, _uMsg, _wParam, _lParam);
}

bool CBookmarkProperties::OnNotify (int _idCtrl, NMHDR * _pInfo) {
	switch (_pInfo->code) {
		/*case DTN_USERSTRING:
			if (_idCtrl == IDC_DATE) {
				NMDATETIMESTRING * const pInfo2 = reinterpret_cast<NMDATETIMESTRING *> (_pInfo);
				::SystemTimeToVariantTime (& pInfo2->st, & m_bookmark.dt);
				_SetModified ();
			}
			break;*/
		case DTN_DATETIMECHANGE:
			if (_idCtrl == IDC_DATE) {
				NMDATETIMECHANGE * const pInfo2 = reinterpret_cast<NMDATETIMECHANGE *> (_pInfo);
				::SystemTimeToVariantTime (& pInfo2->st, & m_bookmark.dt);
				if (pInfo2->dwFlags != GDT_VALID)
					m_bookmark.dt = 0;

				_SetModified ();
			}
			break;

		case NM_CUSTOMDRAW: {
			NMLVCUSTOMDRAW * const pInfo2 = reinterpret_cast<NMLVCUSTOMDRAW *> (_pInfo);
			if (pInfo2->nmcd.dwDrawStage == CDDS_PREPAINT) {
				SetMsgResult (CDRF_NOTIFYITEMDRAW);
				return true;
			} else if (pInfo2->nmcd.dwDrawStage == CDDS_ITEMPREPAINT) {
				SetMsgResult (CDRF_NOTIFYSUBITEMDRAW);
				return true;
			} else if (pInfo2->nmcd.dwDrawStage == (CDDS_SUBITEM | CDDS_ITEMPREPAINT)) {
				const size_t cAttributeIdx = (size_t) pInfo2->nmcd.dwItemSpec;
				bm_attribute_t attribute;
				if (! _GetAttribute (cAttributeIdx, attribute))
					break;
				const string_t & strValue = m_bookmark.attributes [attribute.strId];

				bool bValid = ValidateBookmarkAttribute (attribute.Kind, strValue.c_str (), strValue.size ());
				if (pInfo2->iSubItem == colKey) {
					if (attribute.bObligatory && strValue.empty ())
						bValid = false;
				}

				pInfo2->clrText = ! bValid ? c_colRed : ::GetSysColor (COLOR_BTNTEXT);
				SetMsgResult (CDRF_NEWFONT);
				return true;
			}
			break;
		}

		case NM_CLICK:
		case LVN_ITEMACTIVATE: {
			const NMITEMACTIVATE * const pInfo2 = reinterpret_cast<const NMITEMACTIVATE *> (_pInfo);
			if (_idCtrl == IDC_ATTRIBUTES) {
				if (m_bEditAttribute)
					_OnEditAttributeComplete (true);

				if (pInfo2->iSubItem == 1) {
					m_cCurrentAttributeIdx = pInfo2->iItem;

					bm_attribute_t attribute;
					if (! _GetAttribute (m_cCurrentAttributeIdx, attribute))
						break;
					string_t & strValue = m_bookmark.attributes [attribute.strId];

					if (attribute.Kind == bakMultiString) {
						if (EditText (m_hWnd, strValue, attribute.strName.c_str (), true)) {
							_UpdateAttributes ();
							_SetModified ();
						}
						break;
					}

					m_CurrentAttributeKind = attribute.Kind;

					RECT rect;
					ListView_GetSubItemRect (m_hAttributes, pInfo2->iItem, pInfo2->iSubItem, LVIR_BOUNDS, & rect);
					::MapWindowPoints (m_hAttributes, m_hWnd, (POINT *) & rect, 2);

					_OnEditAttributeStart (strValue, rect.left + 1, rect.top, rect.right - rect.left - 1, rect.bottom - rect.top - 1);
				}
			}
			break;
		}
	}
	return false;
}

INT_PTR CBookmarkProperties::OnMessage (UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_LBUTTONUP: {
			const int x = GET_X_LPARAM (_lParam);
			const int y = GET_Y_LPARAM (_lParam);
			const POINT pt = {x, y};

			WINDOWPLACEMENT wp = {sizeof (WINDOWPLACEMENT)};
			::GetWindowPlacement (m_hHeading, & wp);

			if (::PtInRect (& wp.rcNormalPosition, pt)) {
				const int x0 = (wp.rcNormalPosition.left + wp.rcNormalPosition.right)/2;
				const int y0 = (wp.rcNormalPosition.top + wp.rcNormalPosition.bottom)/2;

				m_bookmark.fHeading = 90 - ::atan2 (double (y0 - y), double (x - x0))*180/c_PI;
				if (m_bookmark.fHeading < 0)
					m_bookmark.fHeading += 360;
				_SetModified ();

				_UpdateHeadingV ();
				_UpdateHeading ();
			}

			break;
		}

		case WM_MOUSEACTIVATE:
			if (LOWORD (_lParam) == HTVSCROLL) {
				if (m_bEditAttribute)
					_OnEditAttributeComplete (true);
			}
			break;

		case WM_CTLCOLOREDIT: {
			const HWND hCtrl = reinterpret_cast<HWND> (_lParam);
			const HDC hDC = reinterpret_cast<HDC> (_wParam);

			if (hCtrl != m_hAttributeEdit)
				break;
			if (CBookmarkProperties::s_pThis == NULL)
				break;

			const size_t cLen = ::GetWindowTextLength (m_hAttributeEdit) + 1;
			char * const strLabel = reinterpret_cast<char *> (::alloca (sizeof (char)*cLen));
			::GetWindowText (m_hAttributeEdit, strLabel, cLen);

			const bool bValid = ValidateBookmarkAttribute (m_CurrentAttributeKind, strLabel, cLen - 1);

			::SetTextColor (hDC, ! bValid ? c_colRed : ::GetSysColor (COLOR_BTNTEXT));
			return (LRESULT) ::GetSysColorBrush (COLOR_WINDOW);
		}

		case WM_MEASUREITEM: {
			MEASUREITEMSTRUCT * const pMI = reinterpret_cast<MEASUREITEMSTRUCT *> (_lParam);

			const int nType = _GetSelType ();
			if (nType != -1) {
				const bm_subtypes_t & vecSubTypes = m_Types [nType].vecSubTypes;
				const size_t cSubTypes = vecSubTypes.size ();
				if (pMI->itemData >= 0 && pMI->itemData < cSubTypes) {
					CCPtr<CIcon> pIcon = vecSubTypes [pMI->itemData].pIcon;
					if (pIcon) {
						if (pMI->itemHeight < pIcon->Height () + 2)
							pMI->itemHeight = pIcon->Height () + 2; 
					}
				}
			}

            break;
		}

		case WM_DRAWITEM: {
			DRAWITEMSTRUCT * const pDI = reinterpret_cast<DRAWITEMSTRUCT *> (_lParam);
			if (pDI->itemID == -1)
				break;

			// Get the icon.
			CCPtr<CIcon> pIcon;
			const int nType = _GetSelType ();
			if (nType != -1) {
				const bm_subtypes_t & vecSubTypes = m_Types [nType].vecSubTypes;
				const size_t cSubTypes = vecSubTypes.size ();
				if (pDI->itemData >= 0 && pDI->itemData < cSubTypes)
					pIcon = vecSubTypes [pDI->itemData].pIcon;
			}

			const int nIconWidth = pIcon ? pIcon->Width () : 0;

			// Calculate the vertical and horizontal position.
			TEXTMETRIC tm;
			::GetTextMetrics (pDI->hDC, & tm);
			const int y = (pDI->rcItem.bottom + pDI->rcItem.top - tm.tmHeight)/2;
			const int x = LOWORD (::GetDialogBaseUnits ())/4;

			const COLORREF colForeground_prev = ::SetTextColor (pDI->hDC, ::GetSysColor (pDI->itemState & ODS_SELECTED ? COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT));
			const COLORREF colBackground_prev = ::SetBkColor   (pDI->hDC, ::GetSysColor (pDI->itemState & ODS_SELECTED ? COLOR_HIGHLIGHT     : COLOR_WINDOW));

			// Get and display the text for the list item. 
			char strTemp [256];
			::SendMessage (pDI->hwndItem, CB_GETLBTEXT, pDI->itemID, (LPARAM) strTemp);
			::ExtTextOut (pDI->hDC, nIconWidth + 1 + 2*x, y, ETO_CLIPPED | ETO_OPAQUE, & pDI->rcItem, strTemp, ::strlen (strTemp), NULL); 

			// Restore the previous colors. 
			::SetTextColor (pDI->hDC, colForeground_prev); 
			::SetBkColor   (pDI->hDC, colBackground_prev);

			// Show the icon.
			if (pIcon)
				pIcon->Draw (pDI->hDC, x + 1, pDI->rcItem.top + 1, nIconWidth, pIcon->Height ());

			// Draw focus rectangle, if any.
			if (pDI->itemState & ODS_FOCUS) 
				::DrawFocusRect (pDI->hDC, & pDI->rcItem); 

			break;
		}
	}

	return CDialog::OnMessage (_uMsg, _wParam, _lParam);
}

bool ShowBookmarkProperties (HWND _hWnd, rgn_t & _rgn, bool _bInitModified, bool & _bDeleteApplied) {
	assert (_rgn.IsPoint () && _rgn.IsMutable () && _rgn.IsBookmark ());

	_bDeleteApplied = false;

	bookmark_t bmDef;
	CBookmarkProperties bp (_rgn, _rgn.pExtra && _rgn.pExtra->pBookmark ? * _rgn.pExtra->pBookmark : bmDef, _bInitModified, _bDeleteApplied);
	const int nRet = bp.ShowModal (_hWnd);
	return nRet != 0;
//	return static_cast<ShowBookmarkProperties_t> (nRet);
}

bool ShowBookmarkProperties (HWND _hWnd, rgn_t & _rgn) {
	bool bDeleteApplied = false;
	return ShowBookmarkProperties (_hWnd, _rgn, false, bDeleteApplied);
}

/////////////////////////////////////////////////////////

void bookmark_t::GetPos (const rgn_t & _rgn, point_t & _pt) {
	assert (_rgn.origins.size () == 1);
	assert (! _rgn.origins [0].bDeleted);
	_pt = _rgn.origins [0].point;
}

void bookmark_t::FixPos (const rgn_t & _rgn) {
//	assert (_rgn.pExtra == NULL || _rgn.pExtra->pBookmark == NULL || _rgn.pExtra->pBookmark == this);

	point_t ptPos;
	bookmark_t::GetPos (_rgn, ptPos);

	if (this->bHasHeading) {
		const float fCosY = ::cos (ptPos.y*c_PI/180);

		const float dx0 = this->fDistanceAlongRoad*::cos ((90 - this->fHeading)*c_PI/180)*360/40e6/fCosY;
		const float dy0 = this->fDistanceAlongRoad*::sin ((90 - this->fHeading)*c_PI/180)*360/40e6;

		const float fNormal = this->fHeading + (this->bLeftToRoad ? -90 : 90);
		const float dx1 = this->fDistanceToRoad*::cos ((90 - fNormal)*c_PI/180)*360/40e6/fCosY;
		const float dy1 = this->fDistanceToRoad*::sin ((90 - fNormal)*c_PI/180)*360/40e6;

		this->ptPosFixed.x = ptPos.x + dx0 + dx1;
		this->ptPosFixed.y = ptPos.y + dy0 + dy1;
	} else {
		this->ptPosFixed = ptPos;
	}
}

bool bookmark_t::RemoveDummyAttributes () {
	// Find the type.
	const bm_subtype_t * pST = NULL;
	const bm_types_t & Types = BookmarkTypes ();
	const size_t cTypes = Types.size ();
	for (size_t cType = 0; cType < cTypes; ++ cType) {
		const bm_type_t & type = Types [cType];
		if (! (type.strId == this->strType))
			continue;

		const size_t cSubTypes = type.vecSubTypes.size ();
		for (size_t cSubType = 0; cSubType < cSubTypes; ++ cSubType) {
			const bm_subtype_t & st = type.vecSubTypes [cSubType];
			if (! (st.strId == this->strSubType))
				continue;

			pST = & st;
		}
	}
	if (pST == NULL)
		return false;

	bool bModified = false;

	// Check all the attributes to be allowed for type/subtype.
	const size_t cAttributes = pST->vecAttributes.size ();
	for (bookmark_t::k2v_t::iterator i = this->attributes.begin (); i != this->attributes.end ();) {
		const string_t & strId    = i->first;
		const string_t & strValue = i->second;

		bool bRemove = false;
		if (strValue.empty ())
			bRemove = true;
		else {
			// Is this attribute defined for the type/subtype?
			bRemove = true;
			for (size_t c = 0; c < cAttributes; ++ c) {
				const bm_attribute_t & attr = pST->vecAttributes [c];
				if (attr.strId == strId) {
					bRemove = false;
					break;
				}
			}
		}

		if (! bRemove)
			++ i;
		else {
			const bookmark_t::k2v_t::iterator iToErase = i;
			++ i;
			this->attributes.erase (iToErase);

			bModified = true;
		}
	}

	return bModified;
}

bool bookmark_t::AreAllAttributesValid () const {
	const bm_attributes_t * const pAttributes = GetBookmarkTypeAttributes (* this);
	if (pAttributes == NULL)
		return false;

	const bm_attributes_t & attributes = * pAttributes;
	const size_t cAttributes = attributes.size ();
	for (size_t c = 0; c < cAttributes; ++ c) {
		const bm_attribute_t & attr = attributes [c];
		const bookmark_t::k2v_t::const_iterator i = this->attributes.find (attr.strId);
		if (i == this->attributes.end ()) {
			if (attr.bObligatory)
				return false;
		} else {
			const string_t & strValue = i->second;
			if (strValue.empty () && attr.bObligatory)
				return false;

			if (! ValidateBookmarkAttribute (attr.Kind, strValue.c_str (), strValue.size ()))
				return false;
		}
	}
	return true;
}

/////////////////////////////////////////////////////////

void bookmark_t::Save (const rgn_t & _rgn, string_t & _str) {
	assert (_rgn.IsBookmark ());
	const bookmark_t & bm = * _rgn.pExtra->pBookmark;

	point_t ptPos;
	bookmark_t::GetPos (_rgn, ptPos);

	char strHeading [10] = "";
	if (bm.bHasHeading)
		::sprintf (strHeading, "%d", fast_floor (bm.fHeading + .5f));

	// Standard fields.
	char strBuf [512];
	::sprintf (
		strBuf, "%s,%s,%d,%.6f,%.6f,%.6f,%s,%d,%d,%d,",
		bm.strType.c_str (),
		bm.strSubType.c_str (),
		bm.bProcessed ? 1 : 0,
		bm.dt,
		ptPos.x, ptPos.y,
		strHeading,
		fast_floor (bm.fDistanceAlongRoad + .5f),
		fast_floor (bm.fDistanceToRoad + .5f),
		bm.bLeftToRoad ? 1 : 0
	);
	_str.append (strBuf);

	// File ref.
	_str.append (1, '\"');
	_str.append (bm.strFileName);
	::sprintf (strBuf, "\",%d,", bm.dwFilePos);
	_str.append (strBuf);

	// Image pos.
	::sprintf (
		strBuf, "%d,%d,",
		bm.bImagePosDefined ? bm.cChannel : 0,
		bm.bImagePosDefined ? bm.cPoints : 0
	);
	_str.append (strBuf);

	if (bm.bImagePosDefined && bm.cPoints != 0) {
		for (size_t c = 0; c < bookmark_t::MAX_POINTS; ++ c) {
			::sprintf (strBuf, "%d,%d,", bm.points [c].x, bm.points [c].y);
			_str.append (strBuf);
		}
	}

	for (bookmark_t::k2v_t::const_iterator i = bm.attributes.begin (); i != bm.attributes.end (); ++ i) {
		const string_t & strKey   = i->first;
		const string_t & strValue = i->second;
		if (strValue.empty ())
			continue;

		_str.append (strKey);
		_str.append  ("=\"");
		const size_t cValueLen = strValue.size ();
		for (size_t c = 0; c < cValueLen; ++ c) {
			const char ch = strValue [c];
			switch (ch) {
				case '\\':
					_str.append (2, '\\');
					break;
				case '\n':
					_str.append (1, '\\');
					_str.append (1, 'n');
					break;
				case '\r':
					_str.append (1, '\\');
					_str.append (1, 'r');
					break;
				case '\t':
					_str.append (1, '\\');
					_str.append (1, 't');
					break;
				case '\"':
					_str.append (1, '\\');
					_str.append (1, '\"');
					break;
				default:
					_str.append (1, ch);
					break;
			}
		}
		_str.append ("\",");
	}
}

static
bool _ParseString (const char ** _pp, const char * _pEnd, char _chDelimiter, string_t & _str) {
	if (** _pp == '\"') {
		++ * _pp;
		const char * const pStart = * _pp;
		while (** _pp != '\"' && * _pp < _pEnd)
			++ * _pp;
		_str.assign (pStart, * _pp);
		if (* _pp < _pEnd)
			++ * _pp;
	} else {
		const char * const pStart = * _pp;
		while (** _pp != _chDelimiter && * _pp < _pEnd)
			++ * _pp;
		_str.assign (pStart, * _pp);
	}

	return true;
}

bool bookmark_t::Load (rgn_t & _rgn, const char * _pStart, const char * _pEnd, const char ** _ppEnd) {
	const char * & p = * _ppEnd;
	p = _pStart;
	OmitSpaces (& p, _pEnd);

	// Type.
	string_t strType;
	_ParseString (& p, _pEnd, ',', strType);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Sub-type.
	string_t strSubType;
	_ParseString (& p, _pEnd, ',', strSubType);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Processed.
	unsigned long ulProcessed = 0;
	if (! ParseDecU (p, _pEnd, & p, ulProcessed))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Date/time.
	DATE dt;
	if (! ParseReal (p, _pEnd, & p, dt))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// X.
	point_t pt;
	if (! ParseReal (p, _pEnd, & p, pt.x))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Y.
	if (! ParseReal (p, _pEnd, & p, pt.y))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Heading.
	float fHeading = 0;
	bool bHasHeading = false;
	if (* p != ',') {
		if (! ParseReal (p, _pEnd, & p, fHeading))
			return false;
		OmitSpaces (& p, _pEnd);
		if (* p != ',')
			return false;
		bHasHeading = true;
	}
	++ p;
	OmitSpaces (& p, _pEnd);

	// Distance along road.
	float fDistanceAlongRoad = 0;
	if (! ParseReal (p, _pEnd, & p, fDistanceAlongRoad))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Distance to road.
	float fDistanceToRoad = 0;
	if (! ParseReal (p, _pEnd, & p, fDistanceToRoad))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Left to road.
	unsigned long ulLeftToRoad = 0;
	if (! ParseDecU (p, _pEnd, & p, ulLeftToRoad))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// File name.
	string_t strFileName;
	_ParseString (& p, _pEnd, ',', strFileName);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// File pos.
	unsigned long ulProgressPrec = 0;
	if (! ParseDecU (p, _pEnd, & p, ulProgressPrec))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Channel.
	unsigned long ulChannel = 0;
	if (! ParseDecU (p, _pEnd, & p, ulChannel))
		return false;
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Position in image.
	POINT points [bookmark_t::MAX_POINTS];
	unsigned long ulPoints = 0;
	if (! ParseDecU (p, _pEnd, & p, ulPoints))
		return false;
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	if (ulPoints != 0) {
		for (size_t c = 0; c < bookmark_t::MAX_POINTS; ++ c) {
			unsigned long ulImageX = 0;
			if (! ParseDecU (p, _pEnd, & p, ulImageX))
				return false;
			if (* p != ',')
				return false;
			++ p;
			OmitSpaces (& p, _pEnd);

			unsigned long ulImageY = 0;
			if (! ParseDecU (p, _pEnd, & p, ulImageY))
				return false;
			if (* p != ',')
				return false;
			++ p;
			OmitSpaces (& p, _pEnd);

			points [c].x = ulImageX;
			points [c].y = ulImageY;
		}
	}

	// Parse attributes.
	bookmark_t::k2v_t attributes;
	while (p < _pEnd) {
		string_t strKey;
		_ParseString (& p, _pEnd, '=', strKey);
		if (* p != '=')
			break;
		++ p;
		OmitSpaces (& p, _pEnd);

		string_t strValue;
		if (* p != '\"')
			break;
		++ p;
		while (p < _pEnd) {
			const char ch = * p;
			++ p;

			if (ch == '\"')
				break;
			else if (ch == '\\' && p + 1 < _pEnd) {
				const char ch2 = * p;
				++ p;

				switch (ch2) {
					case '\\': strValue.append (1, '\\'); break;
					case 'n':  strValue.append (1, '\n'); break;
					case 'r':  strValue.append (1, '\r'); break;
					case 't':  strValue.append (1, '\t'); break;
					case '\"': strValue.append (1, '\"'); break;
					default:
						strValue.append (1, '\\'); break;
						strValue.append (1, ch2); break;
				}
			} else
				strValue.append (1, ch);
		}
		attributes [strKey] = strValue;

		if (* p != ',')
			break;
		++ p;
	}

	if (_rgn.origins.empty ()) {
		_rgn.origins.push_back (rgn_origin_t ());
		_rgn.origins [0].cLevel = 0;
	}
	_rgn.origins [0].point = pt;
//	_rgn.uchEndLevel = 10;

	if (! _rgn.IsBookmark ())
		_rgn.pExtra->pBookmark = new bookmark_t;
	bookmark_t & bm = * _rgn.pExtra->pBookmark;
	bm.strType   .swap (strType);
	bm.strSubType.swap (strSubType);
	bm.attributes.swap (attributes);
	bm.bProcessed         = ulProcessed != 0;
	bm.dt                 = dt;

	bm.bHasHeading        = bHasHeading;
	bm.fHeading           = fHeading;
	bm.fDistanceAlongRoad = fDistanceAlongRoad;
	bm.fDistanceToRoad    = fDistanceToRoad;
	bm.bLeftToRoad        = ulLeftToRoad != 0;
	bm.FixPos (_rgn);

	bm.strFileName        = strFileName;
	bm.dwFilePos          = ulProgressPrec;
	bm.bImagePosDefined   = ulPoints != 0;
	bm.cChannel           = ulPoints != 0 ? ulChannel : -1;
	bm.cPoints            = ulPoints;
	::memcpy (bm.points, points, sizeof (points));

	return true;
}

////////////////////////////////////////////////////////////////

# include "Position.h"
# include "DrawHelpers.h"

void bookmark_t::Draw (HDC _hDC, const rgn_t & _rgn, const rect_t & _rectVisible) const {
	assert (_rgn.pExtra && _rgn.pExtra->pBookmark == this);

	point_t ptPos;
	bookmark_t::GetPos (_rgn, ptPos);

	if (! _rectVisible.Contains (ptPos) && ! _rectVisible.Contains (this->ptPosFixed))
		return;

	const int x0 = XToScreen (ptPos.x);
	const int y0 = YToScreen (ptPos.y);

	const int x1 = XToScreen (this->ptPosFixed.x);
	const int y1 = YToScreen (this->ptPosFixed.y);

	if (x0 != x1 || y0 != y1) {
		::SelectObject (_hDC, g_hpenGrayThin);
		::SelectObject (_hDC, g_hbrWhite);
		DrawLine   (_hDC, x0, y0, x1, y1);
		DrawSquare (_hDC, x0, y0, 2);
	}

	// Draw icon.
	const CCPtr<CIcon> pIcon = GetBookmarkIcon (this->strType, this->strSubType);
	if (pIcon)
		pIcon->Draw (_hDC, x1 - pIcon->Width ()/2, y1 - pIcon->Height ()/2);

	// Draw label, if any.
	const bm_attributes_t * const pAttributes = GetBookmarkTypeAttributes (* this);
	if (pAttributes) {
		const size_t cAttributes = pAttributes->size ();
		for (size_t cAttribute = 0; cAttribute < cAttributes; ++ cAttribute) {
			const bm_attribute_t & attribute = (* pAttributes) [cAttribute];
			if (attribute.bUseAsLabel) {
				const k2v_t::const_iterator i = this->attributes.find (attribute.strId);
				if (i != this->attributes.end ()) {
					const string_t & strLabel = i->second;
					size_t cLen = strLabel.size ();
					if (cLen != 0) {
						// Trim all except the first line of label.
						for (size_t c = 0; c < cLen; ++ c)
							if (strLabel [c] == '\n' || strLabel [c] == '\r') {
								cLen = c;
								break;
							}

						::SetTextColor (_hDC, c_colBlack);
						::SetTextAlign (_hDC, TA_TOP | TA_CENTER);
						::SetBkMode    (_hDC, TRANSPARENT); 
						::SelectObject (_hDC, g_hFont);
						::TextOut      (_hDC, x1 + 0, y1 + 5, strLabel.c_str (), cLen);
					}
				}
				break;
			}
		}
	}

	// Draw heading, if any.
	if (this->bHasHeading) {
		::SelectObject (_hDC, g_hpenGrayThin);
		::SelectObject (_hDC, g_hbrYellow);
		DrawTriangleArrow (_hDC, x1, y1, this->fHeading - 90, 9);
	}
}

////////////////////////////////////////////////////////////////

void bookmark_t::GetMaptipText (string_t & _str) const {
	// Type.
	_str.append ("Bookmark\nType: ");

	string_t strType, strSubType;
	if (GetBookmarkDisplayType (* this, strType, strSubType))
		_str.append (strSubType);
	else {
		_str.append (this->strType);
		_str.append (1, ':');
		_str.append (this->strSubType);
	}
	_str.append ("\n");

	// Attributes.
	for (k2v_t::const_iterator i = attributes.begin (); i != attributes.end (); ++ i) {
		_str.append (i->first);
		_str.append ("=");
		_str.append (i->second);
		_str.append ("\n");
	}
	if (! _str.empty () && _str [_str.size () - 1] == '\n')
		_str [_str.size () - 1] = '\0';
}

////////////////////////////////////////////////////////////////

bool bookmark_t::IsDuplicate (const bookmark_t & _other) const {
	if (! (this->ptPosFixed  == _other.ptPosFixed))
		return false;

	if (! (this->bHasHeading == _other.bHasHeading))
		return false;
	if (this->bHasHeading && ! (this->fHeading == _other.fHeading))
		return false;

	if (! (this->strType     == _other.strType))
		return false;
	if (! (this->strSubType  == _other.strSubType))
		return false;

	if (! (this->attributes  == _other.attributes))
		return false;

	return true;
}