# ifndef __Bookmark_H_
# define __Bookmark_H_

# include "PtAlgo.h"

struct rgn_t;

struct bookmark_t {
	string_t strType;
	string_t strSubType;
	
	typedef std::map<string_t, string_t> k2v_t;
	k2v_t attributes;
	// Dummy attribues are empty ones, or whose keys are not defined for given type/subtype.
	bool RemoveDummyAttributes ();
	// Invalid attributes are missed obligatory ones, or whose value does not correspond its kind (e.g. string value for numeric attribute).
	bool AreAllAttributesValid () const;

	DATE dt;
	float fHeading;           // In degrees.

	// The 'true' position of bookmark based on 'basic' one, fHeading, fDistanceAlongRoad and fDistanceToRoad.
	point_t ptPosFixed;
	void FixPos (const rgn_t & _rgn);
	// The 'basic' position of bookmark.
	static void GetPos (const rgn_t & _rgn, point_t & _pt);

	float fDistanceAlongRoad; // In m, forward is positive.
	float fDistanceToRoad;    // In m.
	bool bLeftToRoad;

	bool bHasHeading;
	bool bProcessed;

	//
	// Videotracks only.
	//
	string_t strFileName;
	DWORD dwFilePos;

	enum {MAX_POINTS = 4};
	bool bImagePosDefined;
	size_t cChannel;
	size_t cPoints;
	POINT points [MAX_POINTS];
	void SetEllipse (int _x0, int _y0, int _rx, int _ry) {
		cPoints = 1;
		points [0].x = _x0;
		points [0].y = _y0;
		points [1].x = _rx;
		points [1].y = _ry;
	}
	void SetPolygon (POINT * _pPoints, size_t _cPoints) {
		assert (2 <= _cPoints && _cPoints <= MAX_POINTS);
		if (_cPoints <= MAX_POINTS) {
			cPoints = _cPoints;
			::memcpy (points, _pPoints, _cPoints*sizeof (POINT));
		}
	}

	bookmark_t () :
		dt (0),
		bHasHeading (false), fHeading (0),
		fDistanceAlongRoad (0), fDistanceToRoad (0), bLeftToRoad (false),
		bProcessed (false),
		dwFilePos (0), bImagePosDefined (false), cChannel (-1), cPoints (0)
	{
		::memset (points, 0, sizeof (points));
	}

	static void Save (const rgn_t & _rgn, string_t & _str);
	static bool Load (rgn_t & _rgn, const char * _pStart, const char * _pEnd, const char ** _ppEnd);

	void Draw (HDC _hdc, const rgn_t & _rgn, const rect_t & _rectVisible) const;

	void GetMaptipText (string_t & _str) const;

	bool IsDuplicate (const bookmark_t & _other) const;
};

struct rgn_t;

bool ShowBookmarkProperties (HWND _hWnd, rgn_t & _rgn, bool _bInitModified, bool & _bDeleteApplied);
bool ShowBookmarkProperties (HWND _hWnd, rgn_t & _rgn);

rgn_t & CreateBookmark (const point_t & _pt);

# endif // __Bookmark_H_
