# ifndef __BookmarkTypes_H_
# define __BookmarkTypes_H_

# include "Icon.h"

struct bookmark_t;

enum BmAttributeKind {
	bakString,
	bakMultiString,
	bakReal,
	bakInteger,
	bakBoolean,
	bakTime,
	bakDate
};

bool ValidateBookmarkAttribute (BmAttributeKind _CurrentAttributeKind, const char * _strValue, size_t _cLen);

struct bm_attribute_t {
	string_t strId;
	string_t strName;
	BmAttributeKind Kind;
	bool bObligatory;
	bool bUseAsLabel;

	bm_attribute_t () : Kind (bakString), bObligatory (false), bUseAsLabel (false) {}
};

typedef std::vector<bm_attribute_t> bm_attributes_t;

struct bm_subtype_t {
	string_t strId;
	string_t strName;
	CCPtr<CIcon> pIcon;

	bm_attributes_t vecAttributes;
};

typedef std::vector<bm_subtype_t> bm_subtypes_t;

struct bm_type_t {
	string_t strId;
	string_t strName;

	bm_subtypes_t vecSubTypes;

	bool bHasHeadingByDefault;
	bool bHasHeadingIsConst;

	bm_type_t () : bHasHeadingByDefault (false), bHasHeadingIsConst (false) {}
};

typedef std::vector<bm_type_t> bm_types_t;

bm_types_t & BookmarkTypes ();
CCPtr<CIcon> GetBookmarkIcon (const string_t & _strType, const string_t & _strSubType);
bool GetBookmarkDisplayType (const bookmark_t & _bm, string_t & _strType, string_t & _strSubType);
const bm_attributes_t * GetBookmarkTypeAttributes (const bookmark_t & _bm);
const bm_attribute_t * FindBookmarkTypeAttribute (const bm_attributes_t & _attributes, const char * _strId);

# endif // __BookmarkTypes_H_