//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2005-2008
//
// COM object model: the cursor object.
//

# include "StdAfx.h"
# include "COMCursor.h"
# include "Cursors.h"

CImagesCache g_ImagesCache;

long CImagesCache::PictureToCookie (IPicture * _pPicture) const {
	return reinterpret_cast<long> (_pPicture);
}

bool CImagesCache::CookieToPicture (long _lCookie, IPicture ** _ppPicture) const {
	IPicture * const pPicture = reinterpret_cast<IPicture *> (_lCookie);
	const setPictures_t::const_iterator i = m_setPictures.find (pPicture);
	if (i == m_setPictures.end ())
		return false;

	* _ppPicture = pPicture;
	if (pPicture)
		pPicture->AddRef ();
	return true;
}

long CImagesCache::RegisterPicture (IPicture * _pPicture) {
	if (_pPicture) {
		_pPicture->AddRef ();
		m_setPictures.insert (_pPicture);
	}
	return PictureToCookie (_pPicture);
}

bool CImagesCache::UnregisterPicture (long _lCookie) {
	IPicture * const pPicture = reinterpret_cast<IPicture *> (_lCookie);
	const setPictures_t::iterator i = m_setPictures.find (pPicture);
	if (i == m_setPictures.end ())
		return false;
	if (pPicture)
		pPicture->Release ();
	m_setPictures.erase (i);
	return true;
}

bool CImagesCache::UnregisterPicture (IPicture * _pPicture) {
	return UnregisterPicture (PictureToCookie (_pPicture));
}

void CImagesCache::Clear () {
	for (setPictures_t::iterator i = m_setPictures.begin (); i != m_setPictures.end (); ++ i) {
		IPicture * const pPicture = * i;
		if (pPicture)
			pPicture->Release ();
	}
	m_setPictures.clear ();
}

///////////////////////////////////////////////////////////////

STDMETHODIMP CComCursor::Delete () {
	DeleteCursor (m_cursor);
	return S_OK;
}

STDMETHODIMP CComCursor::get_Latitude  (double * _ret) {
	* _ret = m_cursor.pt.y;
	return S_OK;
}

STDMETHODIMP CComCursor::get_Longitude (double * _ret) {
	* _ret = m_cursor.pt.x;
	return S_OK;
}

STDMETHODIMP CComCursor::SetPosition (double _dfLatitude, double _dfLongitude) {
	m_cursor.pt.x = _dfLongitude;
	m_cursor.pt.y = _dfLatitude;
	return S_OK;
}

STDMETHODIMP CComCursor::get_Heading (double * _ret) {
	* _ret = m_cursor.fHeading;
	return S_OK;
}

STDMETHODIMP CComCursor::put_Heading (double _dfVal) {
	m_cursor.fHeading = _dfVal;
	return S_OK;
}

STDMETHODIMP CComCursor::get_HasHeading (VARIANT_BOOL * _ret) {
	* _ret = m_cursor.bHasHeading ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComCursor::put_HasHeading (VARIANT_BOOL _Val) {
	m_cursor.bHasHeading = _Val != VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComCursor::get_Color (long * _ret) {
	* _ret = m_cursor.col;
	return S_OK;
}

STDMETHODIMP CComCursor::put_Color (long _lVal) {
	m_cursor.col = _lVal;
	return S_OK;
}

STDMETHODIMP CComCursor::get_ContourColor (long * _ret) {
	* _ret = m_cursor.colContour;
	return S_OK;
}

STDMETHODIMP CComCursor::put_ContourColor (long _lVal) {
	m_cursor.colContour = _lVal;
	return S_OK;
}

STDMETHODIMP CComCursor::get_Radius (long * _ret) {
	* _ret = m_cursor.nR;
	return S_OK;
}

STDMETHODIMP CComCursor::put_Radius (long _lVal) {
	m_cursor.nR = _lVal;
	return S_OK;
}

STDMETHODIMP CComCursor::get_ArrowColor (long * _ret) {
	* _ret = m_cursor.colArrow;
	return S_OK;
}

STDMETHODIMP CComCursor::put_ArrowColor (long _lVal) {
	m_cursor.colArrow = _lVal;
	return S_OK;
}

STDMETHODIMP CComCursor::get_ArrowShape (ArrowShape * _ret) {
	* _ret = static_cast<ArrowShape> (m_cursor.nArrowShape);
	return S_OK;
}

STDMETHODIMP CComCursor::put_ArrowShape (ArrowShape _Val) {
	m_cursor.nArrowShape = _Val;
	return S_OK;
}

STDMETHODIMP CComCursor::get_Label (BSTR * _ret) {
	USES_CONVERSION;
	* _ret = ::SysAllocStringLen (A2OLE (m_cursor.strLabel.c_str ()), m_cursor.strLabel.size ());
	return S_OK;
}

STDMETHODIMP CComCursor::put_Label (BSTR _bstrVal) {
	USES_CONVERSION;
	m_cursor.strLabel = OLE2A (_bstrVal);
	return S_OK;
}

STDMETHODIMP CComCursor::get_LabelColor (long * _ret) {
	* _ret = m_cursor.colLabel;
	return S_OK;
}

STDMETHODIMP CComCursor::put_LabelColor (long _lVal) {
	m_cursor.colLabel = _lVal;
	return S_OK;
}

STDMETHODIMP CComCursor::get_LabelX0 (long * _ret) {
	* _ret = m_cursor.xLabel0;
	return S_OK;
}

STDMETHODIMP CComCursor::get_LabelY0 (long * _ret) {
	* _ret = m_cursor.yLabel0;
	return S_OK;
}

STDMETHODIMP CComCursor::get_LabelAlignment (long * _ret) {
	* _ret = m_cursor.uiAlignMask;
	return S_OK;
}

STDMETHODIMP CComCursor::SetLabelAlignment (long _x0, long _y0, long _lAlignMask) {
	m_cursor.xLabel0 = _x0;
	m_cursor.yLabel0 = _y0;
	m_cursor.uiAlignMask = _lAlignMask;
	return S_OK;
}

STDMETHODIMP CComCursor::get_Hidden (VARIANT_BOOL * _ret) {
	* _ret = m_cursor.bHidden ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComCursor::put_Hidden (VARIANT_BOOL _Val) {
	m_cursor.bHidden = _Val != VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComCursor::get_Selected (VARIANT_BOOL * _ret) {
	* _ret = m_cursor.bSelected ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComCursor::put_Selected (VARIANT_BOOL _Val) {
	m_cursor.bSelected = _Val != VARIANT_FALSE;
	return S_OK;
}


///////////////////////////////////////////////

# include "Wrappers.h"

STDMETHODIMP CComCursor::LoadImageFromFile (BSTR _bstrFileName, long _x0, long _y0) {
	CComPtr<IPicture> pPicture;
	const HRESULT hr = ::OleLoadPicturePath (_bstrFileName, NULL, 0, 0xFFFFFF, IID_IPicture, reinterpret_cast<void **> (& pPicture));
	if (FAILED (hr))
		return hr;

	g_ImagesCache.RegisterPicture (pPicture);

	m_cursor.pIcon  = pPicture;
	m_cursor.xIcon0 = _x0;
	m_cursor.yIcon0 = _y0;

	return S_OK;
}

STDMETHODIMP CComCursor::UnloadImage () {
	if (m_cursor.pIcon) {
		g_ImagesCache.UnregisterPicture (m_cursor.pIcon);
		m_cursor.pIcon.Release ();
	}
	return S_OK;
}

STDMETHODIMP CComCursor::get_Image (long * _ret) {
	if (_ret == NULL)
		return E_INVALIDARG;
	* _ret = g_ImagesCache.PictureToCookie (m_cursor.pIcon.p);
	return S_OK;
}

STDMETHODIMP CComCursor::put_Image (long _lImage) {
	CComPtr<IPicture> pPicture;
	if (! g_ImagesCache.CookieToPicture (_lImage, & pPicture))
		return E_INVALIDARG;

	m_cursor.pIcon = pPicture;
	m_cursor.SetCursorShift (CClientDC (NULL));

	return S_OK;
}

STDMETHODIMP CComCursor::get_Tag (VARIANT * _ret) {
	return ::VariantCopy (_ret, & m_cursor.vTag);
}

STDMETHODIMP CComCursor::put_Tag (const VARIANT * _pVal) {
	return m_cursor.vTag.Copy (_pVal);
}

STDMETHODIMP CComCursor::get_Id (long * _ret) {
	* _ret = m_cursor.lID;
	return S_OK;
}

STDMETHODIMP CComCursor::put_Id (long _lVal) {
	m_cursor.lID = _lVal;
	return S_OK;
}

STDMETHODIMP CComCursor::_GetPrivatePtr (long * _ret) {
	* _ret = reinterpret_cast<long> (& m_cursor);
	return S_OK;
}
