//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// COM object model: the Document object.
//

# include "StdAfx.h"
# include "COMDocument.h"
# include "Map.h"
# include "FileTypeInfo.h"
# include "ChangeMapHeader.h"
# include "Globals.h"

STDMETHODIMP CComDocument::get_Name (BSTR * _ret) {
	CCritSec::CLock lock (g_cs);

	CComBSTR bstrName (m_map.Header.strName.c_str ());
	* _ret = bstrName.Detach ();
	return S_OK;
}

STDMETHODIMP CComDocument::put_Name (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComDocument::get_ID (BSTR * _ret) {
	CCritSec::CLock lock (g_cs);

	CComBSTR bstrID (m_map.Header.strID.c_str ());
	* _ret = bstrID.Detach ();
	return S_OK;
}

STDMETHODIMP CComDocument::put_ID (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComDocument::get_Copyright (BSTR * _ret) {
	CCritSec::CLock lock (g_cs);

	CComBSTR bstrCopyright (m_map.Header.strCopyright.c_str ());
	* _ret = bstrCopyright.Detach ();
	return S_OK;
}

STDMETHODIMP CComDocument::put_Copyright (BSTR _bstrVal) {
	CComBSTR strCopyright (_bstrVal);

	if (strCopyright != CComBSTR (g_map.Header.strCopyright.c_str ())) {
		map_header_t Header = g_map.Header;

		USES_CONVERSION;
		g_map.Header.strCopyright = OLE2T (strCopyright);

		ChangeMapHeader (Header);
	}
	return S_OK;
}

STDMETHODIMP CComDocument::get_Objects (long * _ret) {
	CCritSec::CLock lock (g_cs);

	* _ret = g_map.GetRgnCount ();
	return S_OK;
}

# include "COMMapObject.h"

STDMETHODIMP CComDocument::get_FirstObject (IMapObject ** _ppRet) {
	CCritSec::CLock lock (g_cs);

	rgn_t * pRgn = m_map.pFirstRgn;
	while (pRgn && pRgn->IsDeleted ())
		pRgn = pRgn->pNextRgn;

	if (pRgn == NULL) {
		* _ppRet = NULL;
		return S_FALSE;
	}
	CComMapObject * const pMapObject = new CComMapObject (pRgn);
	return pMapObject->QueryInterface (__uuidof (IMapObject), reinterpret_cast<void **> (_ppRet));
}

STDMETHODIMP CComDocument::CreateObject (ObjectKind _Kind, IMapObject ** _ret) {
	* _ret = NULL;
	return E_NOTIMPL;
}

STDMETHODIMP CComDocument::get_ZoomLevels (long * _ret) {
	* _ret = g_map.Levels.size ();
	return S_OK;
}

# include "ChangeLevel.h"
# include "RemoveLevel.h"
# include "Types.h"

STDMETHODIMP CComDocument::GetZoomLevel (long _lLevel, long * _plBits, long * _plBitsRangeTo, long * _plMSZoom) {
	* _plBits        = 0;
	* _plBitsRangeTo = 0;
	* _plMSZoom      = 0;
	return E_NOTIMPL;
}

STDMETHODIMP CComDocument::SetZoomLevel (long _lLevel, long _lBits, long _lBitsRangeTo, long _lMSZoom) {
	CCritSec::CLock lock (g_cs);

	if (_lLevel < 0 || _lLevel >= (long) m_map.Levels.size ())
		return E_INVALIDARG;

	const long lMaxBits = m_map.pTypeSet ? m_map.pTypeSet->cMaxBits : 24;
	if (_lBits < 1 || _lBits > lMaxBits)
		return E_INVALIDARG;

	const map_level_t & level = m_map.Levels [_lLevel];
	if (& m_map != & g_map)
		return E_INVALIDARG;
	ChangeLevel (_lLevel, _lBits, _lBitsRangeTo != 0 ? _lBitsRangeTo : _lBits, _lMSZoom != 0 ? _lMSZoom : level.cZoom);

	return S_OK;
}

STDMETHODIMP CComDocument::InsertZoomLevel (long _lLevel, long _lBits, long _lBitsRangeTo, long _lMSZoom) {
	return E_NOTIMPL;
}

STDMETHODIMP CComDocument::DeleteZoomLevel (long _lLevel) {
	RemoveLevel (_lLevel);
	return S_OK;
}

STDMETHODIMP CComDocument::get_MinLatitude  (double * _ret) {
	* _ret = g_map.rectBound.y0;
	return S_OK;
}

STDMETHODIMP CComDocument::get_MaxLatitude  (double * _ret) {
	* _ret = g_map.rectBound.y1;
	return S_OK;
}

STDMETHODIMP CComDocument::get_MinLongitude (double * _ret) {
	* _ret = g_map.rectBound.x0;
	return S_OK;
}

STDMETHODIMP CComDocument::get_MaxLongitude (double * _ret) {
	* _ret = g_map.rectBound.x1;
	return S_OK;
}

STDMETHODIMP CComDocument::get_FormatID (BSTR * _ret) {
	if (g_pLoaderInfo == NULL) {
		* _ret = NULL;
		return S_OK;
	}
	
	CComBSTR bstrName (g_pLoaderInfo->m_strClass);
	* _ret = bstrName.Detach ();

	return S_OK;
}

STDMETHODIMP CComDocument::get_Invisible (VARIANT_BOOL * _ret) {
	* _ret = g_map.Header.bInvisible ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComDocument::put_Invisible (VARIANT_BOOL _Val) {
	const bool bIsInvisible = (_Val != VARIANT_FALSE);
	if (bIsInvisible != g_map.Header.bInvisible) {
		map_header_t Header = g_map.Header;
		Header.bInvisible = bIsInvisible;
		ChangeMapHeader (Header);
	}
	return S_OK;
}
