//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// COM object model: the Element object.
//

# include "StdAfx.h"
# include "COMElement.h"
# include "Map.h"
# include "Selection.h"

STDMETHODIMP CComElement::get_Points (VARIANT * _ret) {
	::VariantInit (_ret);

	const bool bPoint = m_pRgn->IsPoint ();
	const size_t cElements = bPoint ? m_pRgn->origins.size () : m_pRgn->elements.size ();
	if (m_cElement >= cElements)
		return E_UNEXPECTED;
	const rgn_element_t & element = m_pRgn->elements [m_cElement];

	// Create SAFEARRAY.
	const size_t cPoints = bPoint ? 1 : element.points.size ();
	SAFEARRAYBOUND sab = {cPoints*2, 0};
	SAFEARRAY * const pSA = ::SafeArrayCreate (VT_R8, 1, & sab);
	double * const pData = reinterpret_cast<double *> (pSA->pvData);

	// Fill the SAFEARRAY.
	if (bPoint) {
		const rgn_origin_t & origin = m_pRgn->origins [m_cElement];
		pData [0] = origin.point.y;
		pData [1] = origin.point.x;
	} else {
		for (size_t c = 0; c < cPoints; ++ c) {
			pData [c*2]     = element.points [c].y;
			pData [c*2 + 1] = element.points [c].x;
		}
	}

	// Return the SAFEARRAY.
	_ret->parray = pSA;
	_ret->vt     = VT_ARRAY | VT_R8;
	return S_OK;
}

STDMETHODIMP CComElement::put_Points (const VARIANT * _pPoints) {
	return E_NOTIMPL;
}

STDMETHODIMP CComElement::get_IsHole (VARIANT_BOOL * _ret) {
	if (! m_pRgn->IsPolygon ()) {
		* _ret = VARIANT_FALSE;
		return S_FALSE;
	}

	if (m_cElement >= m_pRgn->elements.size ())
		return E_UNEXPECTED;

	const rgn_element_t & element = m_pRgn->elements [m_cElement];
	* _ret = element.bHole ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComElement::put_IsHole (VARIANT_BOOL _lVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComElement::get_Level (long * _ret) {
	if (m_pRgn->IsPoint ()) {
		if (m_cElement >= m_pRgn->origins.size ())
			return E_UNEXPECTED;
		const rgn_origin_t & origin = m_pRgn->origins [m_cElement];
		* _ret = origin.cLevel;
	} else {
		if (m_cElement >= m_pRgn->elements.size ())
			return E_UNEXPECTED;
		const rgn_element_t & element = m_pRgn->elements [m_cElement];
		* _ret = element.cLevel;
	}
	return S_OK;
}

STDMETHODIMP CComElement::put_Level (long _lVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComElement::get_NextElement (IElement ** _ppRet) {
	const size_t cElements = m_pRgn->IsPoint () ? m_pRgn->origins.size () : m_pRgn->elements.size ();
	const size_t cNextElement = m_cElement + 1;
	if (cNextElement >= cElements) {
		* _ppRet = NULL;
		return S_FALSE;
	}

	CComElement * const pElement = new CComElement (m_pRgn, cNextElement);
	return pElement->QueryInterface (__uuidof (IElement), reinterpret_cast<void **> (_ppRet));

	return S_OK;
}

STDMETHODIMP CComElement::Delete () {
	return E_NOTIMPL;
}

STDMETHODIMP CComElement::get_IsDeleted (VARIANT_BOOL * _ret) {
	if (m_pRgn->IsPoint ()) {
		if (m_cElement > m_pRgn->origins.size ())
			return E_UNEXPECTED;
		const rgn_origin_t & origin = m_pRgn->origins [m_cElement];
		* _ret = origin.bDeleted ? VARIANT_TRUE : VARIANT_FALSE;
	} else {
		if (m_cElement > m_pRgn->elements.size ())
			return E_UNEXPECTED;
		const rgn_element_t & element = m_pRgn->elements [m_cElement];
		* _ret = element.bDeleted ? VARIANT_TRUE : VARIANT_FALSE;
	}
	return S_OK;
}

STDMETHODIMP CComElement::AddToSelection () {
	return E_NOTIMPL;
}

STDMETHODIMP CComElement::get_IsSelected (VARIANT_BOOL * _ret) {
	* _ret = VARIANT_FALSE;

	CCritSec::CLock lock (g_cs);

	const size_t cItems = g_Selection.size ();
	for (size_t c = 0; c < cItems; ++ c) {
		const find_info_t & fi = g_Selection [c];
		if (fi.pRenderer || fi.pLayer == NULL || & * fi.iRgn != m_pRgn)
			continue;

		if (fi.cElementOrOrigin == -1 || fi.cElementOrOrigin == m_cElement) {
			* _ret = VARIANT_TRUE;
			return S_OK;
		}
	}
	return S_OK;
}
