//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// COM object model: the MapObject object.
//

# include "StdAfx.h"
# include "COMMapObject.h"
# include "Map.h"
# include "Selection.h"

STDMETHODIMP CComMapObject::get_Label (BSTR * _ret) {
	CComBSTR bstr (m_pRgn->strLabel.c_str ());
	* _ret = bstr.Detach ();
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Label (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Kind (ObjectKind * _ret) {
	switch (m_pRgn->GetKind ()) {
		case rkPoint:    * _ret = okPoint; break;
		case rkPolyline: * _ret = okPolyline; break;
		case rkPolygon:  * _ret = okPolygon; break;
	}
	return S_OK;
}

STDMETHODIMP CComMapObject::get_Type (long * _ret) {
	* _ret = m_pRgn->GetType ();
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Type (long _lVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_EndLevel (long * _ret) {
	* _ret = m_pRgn->uchEndLevel;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_EndLevel (long _lVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_FileRef (BSTR * _ret) {
	if (m_pRgn->pFileRef) {
		CComBSTR bstr (m_pRgn->pFileRef->strFileName.c_str ());
		* _ret = bstr.Detach ();
		return S_OK;
	} else {
		* _ret = NULL;
		return S_FALSE;
	}
}

STDMETHODIMP CComMapObject::get_Elements (long * _ret) {
	* _ret = m_pRgn->IsPoint () ? m_pRgn->origins.size () : m_pRgn->elements.size ();
	return S_OK;
}

# include "COMElement.h"

STDMETHODIMP CComMapObject::GetElement (long _lIndex, IElement ** _ppRet) {
	const size_t cElements = m_pRgn->IsPoint () ? m_pRgn->origins.size () : m_pRgn->elements.size ();
	if (_lIndex < 0 || _lIndex >= (long) cElements)
		return E_INVALIDARG;

	CComElement * const pElement = new CComElement (m_pRgn, _lIndex);
	return pElement->QueryInterface (__uuidof (IElement), reinterpret_cast<void **> (_ppRet));
}

STDMETHODIMP CComMapObject::AddElement (IElement ** _ppRet) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_IsMapObject (VARIANT_BOOL * _ret) {* _ret = m_pRgn->IsMapObject  () ? VARIANT_TRUE : VARIANT_FALSE; return S_OK;}
STDMETHODIMP CComMapObject::get_IsAttached  (VARIANT_BOOL * _ret) {* _ret = m_pRgn->IsAttachment () ? VARIANT_TRUE : VARIANT_FALSE; return S_OK;}
STDMETHODIMP CComMapObject::get_IsBookmark  (VARIANT_BOOL * _ret) {* _ret = m_pRgn->IsBookmark   () ? VARIANT_TRUE : VARIANT_FALSE; return S_OK;}
STDMETHODIMP CComMapObject::get_IsNote      (VARIANT_BOOL * _ret) {* _ret = m_pRgn->IsNote       () ? VARIANT_TRUE : VARIANT_FALSE; return S_OK;}
STDMETHODIMP CComMapObject::get_IsWTR       (VARIANT_BOOL * _ret) {* _ret = m_pRgn->IsWTR        () ? VARIANT_TRUE : VARIANT_FALSE; return S_OK;}
STDMETHODIMP CComMapObject::get_IsTrack     (VARIANT_BOOL * _ret) {* _ret = m_pRgn->IsTrack      () ? VARIANT_TRUE : VARIANT_FALSE; return S_OK;}
STDMETHODIMP CComMapObject::get_IsWaypoint  (VARIANT_BOOL * _ret) {* _ret = m_pRgn->IsWaypoint   () ? VARIANT_TRUE : VARIANT_FALSE; return S_OK;}
STDMETHODIMP CComMapObject::get_IsRoute     (VARIANT_BOOL * _ret) {* _ret = m_pRgn->IsRoute      () ? VARIANT_TRUE : VARIANT_FALSE; return S_OK;}

STDMETHODIMP CComMapObject::get_Country     (BSTR * _ret) {
	* _ret = NULL;
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::put_Country     (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Region      (BSTR * _ret) {
	* _ret = NULL;
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::put_Region      (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_City        (BSTR * _ret) {
	if (m_pRgn->pExtra && m_pRgn->pExtra->pCity && ! m_pRgn->pExtra->pCity->IsDeleted ()) {
		CComBSTR bstr (m_pRgn->pExtra->pCity->strName.c_str ());
		* _ret = bstr.Detach ();
	} else
		* _ret = NULL;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_City        (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Street      (BSTR * _ret) {
	if (m_pRgn->pExtra) {
		CComBSTR bstr (m_pRgn->pExtra->strStreetDesc.c_str ());
		* _ret = bstr.Detach ();
	} else
		* _ret = NULL;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Street      (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Number      (BSTR * _ret) {
	if (m_pRgn->pExtra) {
		CComBSTR bstr (m_pRgn->pExtra->strNumber.c_str ());
		* _ret = bstr.Detach ();
	} else
		* _ret = NULL;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Number      (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Phone       (BSTR * _ret) {
	if (m_pRgn->pExtra) {
		CComBSTR bstr (m_pRgn->pExtra->strPhone.c_str ());
		* _ret = bstr.Detach ();
	} else
		* _ret = NULL;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Phone       (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Fax         (BSTR * _ret) {
	if (m_pRgn->pExtra) {
		CComBSTR bstr (m_pRgn->pExtra->strFax.c_str ());
		* _ret = bstr.Detach ();
	} else
		* _ret = NULL;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Fax         (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Email       (BSTR * _ret) {
	if (m_pRgn->pExtra) {
		CComBSTR bstr (m_pRgn->pExtra->strEmail.c_str ());
		* _ret = bstr.Detach ();
	} else
		* _ret = NULL;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Email       (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_WebPage     (BSTR * _ret) {
	if (m_pRgn->pExtra) {
		CComBSTR bstr (m_pRgn->pExtra->strWebPage.c_str ());
		* _ret = bstr.Detach ();
	} else
		* _ret = NULL;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_WebPage     (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Description (BSTR * _ret) {
	if (m_pRgn->pExtra) {
		CComBSTR bstr (m_pRgn->pExtra->strDescription.c_str ());
		* _ret = bstr.Detach ();
	} else
		* _ret = NULL;
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Description (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_Comment     (BSTR * _ret) {
	CComBSTR bstr (m_pRgn->strComment.c_str ());
	* _ret = bstr.Detach ();
	return S_OK;
}

STDMETHODIMP CComMapObject::put_Comment     (BSTR _bstrVal) {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_NextObject (IMapObject ** _ppRet) {
	rgn_t * pRgn = m_pRgn->pNextRgn;
	while (pRgn && pRgn->IsDeleted ())
		pRgn = pRgn->pNextRgn;

	if (pRgn == NULL) {
		* _ppRet = NULL;
		return S_FALSE;
	}
	CComMapObject * const pMapObject = new CComMapObject (pRgn);
	return pMapObject->QueryInterface (__uuidof (IMapObject), reinterpret_cast<void **> (_ppRet));
}

STDMETHODIMP CComMapObject::Delete () {
	return E_NOTIMPL;
};

STDMETHODIMP CComMapObject::get_IsDeleted (VARIANT_BOOL * _ret) {
	* _ret = m_pRgn->IsDeleted () ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComMapObject::AddToSelection () {
	return E_NOTIMPL;
}

STDMETHODIMP CComMapObject::get_IsSelected (VARIANT_BOOL * _ret) {
	* _ret = VARIANT_FALSE;

	CCritSec::CLock lock (g_cs);

	const size_t cItems = g_Selection.size ();
	for (size_t c = 0; c < cItems; ++ c) {
		const find_info_t & fi = g_Selection [c];
		if (fi.pRenderer || fi.pLayer == NULL)
			continue;

		if (& * fi.iRgn == m_pRgn && fi.cElementOrOrigin == -1) {
			* _ret = VARIANT_TRUE;
			return S_OK;
		}
	}

	return S_OK;
}

STDMETHODIMP CComMapObject::get_HasSelectedElements (VARIANT_BOOL * _ret) {
	* _ret = VARIANT_FALSE;

	CCritSec::CLock lock (g_cs);

	const size_t cItems = g_Selection.size ();
	for (size_t c = 0; c < cItems; ++ c) {
		const find_info_t & fi = g_Selection [c];
		if (fi.pRenderer || fi.pLayer == NULL)
			continue;

		if (& * fi.iRgn != m_pRgn && fi.cElementOrOrigin != -1) {
			* _ret = VARIANT_TRUE;
			return S_OK;
		}
	}

	return S_OK;
}

