//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2005-2009
//
// COM object model: the path object.
//

# include "StdAfx.h"
# include "COMPath.h"
# include "Paths.h"
# include "Map.h"
# include "Wrappers.h"

STDMETHODIMP CComPath::Delete () {
	DeletePath (m_path);
	return S_OK;
}

STDMETHODIMP CComPath::get_Color (long * _ret) {
	* _ret = m_path.col;
	return S_OK;
}

STDMETHODIMP CComPath::put_Color (long _lVal) {
	m_path.col = _lVal;
	return S_OK;
}

STDMETHODIMP CComPath::get_HighlightedColor (long * _ret) {
	* _ret = m_path.colHS;
	return S_OK;
}

STDMETHODIMP CComPath::put_HighlightedColor (long _lVal) {
	m_path.colHS = _lVal;
	return S_OK;
}

STDMETHODIMP CComPath::get_Width (long * _ret) {
	* _ret = m_path.cWidth;
	return S_OK;
}

STDMETHODIMP CComPath::put_Width (long _lVal) {
	m_path.cWidth = _lVal;
	return S_OK;
}

STDMETHODIMP CComPath::get_HighlightedSegmentFrom (long * _ret) {
	* _ret = m_path.cHSFrom;
	return S_OK;
}

STDMETHODIMP CComPath::get_HighlightedSegmentTo (long * _ret) {
	* _ret = m_path.cHSTo;
	return S_OK;
}

STDMETHODIMP CComPath::SetHighlightedSegment (long _lFrom, long _lTo) {
	m_path.cHSFrom = _lFrom;
	m_path.cHSTo   = _lTo;
	return S_OK;
}

STDMETHODIMP CComPath::get_Points (long * _ret) {
	* _ret = m_path.points.size ();
	return S_OK;
}

STDMETHODIMP CComPath::AppendPoint (double _Latitude, double _Longitude) {
	point_t pt (_Longitude, _Latitude);
	m_path.points.push_back (pt);
	m_path.rectBound.Extend (pt);
	return S_OK;
}

STDMETHODIMP CComPath::AppendPoints (const VARIANT * _pvPoints) {
	if ((_pvPoints->vt & VT_R8) == 0 || (_pvPoints->vt & VT_ARRAY) == 0)
		return E_INVALIDARG;

	SAFEARRAY * const pPoints = _pvPoints->vt & VT_BYREF ? * _pvPoints->pparray : _pvPoints->parray;
	if (pPoints->cDims != 1)
		return E_INVALIDARG;
	if (pPoints->cbElements != sizeof (double))
		return E_INVALIDARG;
	const size_t cElements = pPoints->rgsabound [0].cElements;
	if ((cElements & 0x1) != 0)
		return E_INVALIDARG;

	const size_t cSize0 = m_path.points.size ();
	m_path.points.resize (cSize0 + cElements/2);

	double * pArray = NULL;
	::SafeArrayAccessData (pPoints, reinterpret_cast<void **> (& pArray));
	for (size_t c = 0; c < cElements/2; ++ c) {
		m_path.points [cSize0 + c].x = pArray [c*2 + 1];
		m_path.points [cSize0 + c].y = pArray [c*2];
	}
	::SafeArrayUnaccessData (pPoints);

	m_path.rectBound.Extend (m_path.points);

	return S_OK;
}

STDMETHODIMP CComPath::DeleteAllPoints () {
	m_path.points.clear ();
	m_path.rectBound = rect_t ();
	return S_OK;
}

STDMETHODIMP CComPath::get_Hidden (VARIANT_BOOL * _ret) {
	* _ret = m_path.bHidden ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::put_Hidden (VARIANT_BOOL _bVal) {
	m_path.bHidden = _bVal != VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::get_Tag (VARIANT * _ret) {
	return ::VariantCopy (_ret, & m_path.vTag);
}

STDMETHODIMP CComPath::put_Tag (const VARIANT * _pVal) {
	return m_path.vTag.Copy (_pVal);
}

STDMETHODIMP CComPath::SetInfotipsForPoints (BSTR _bstrInfotips, BSTR _bstrDelimiter) {
	// NOTE: _bstrInfotips may contain inner '\0's.
	CComBSTR bstrInfotips (::SysStringLen (_bstrInfotips), _bstrInfotips);
	m_path.bstrInfotipsForPoints.Empty ();
	m_path.bstrInfotipsForPoints.Attach (bstrInfotips.Detach ());

	m_path.InfotipsForPoints.clear ();
	m_path.InfotipsForPoints.push_back (0);

	const wchar_t wc0 = _bstrDelimiter == NULL ? L'\0' : _bstrDelimiter [0];
	const size_t cDelimiterLen = ::SysStringLen (_bstrDelimiter);
	const size_t cLen = m_path.bstrInfotipsForPoints.Length ();
	for (size_t c = 0; c < cLen; ++ c) {
		if (m_path.bstrInfotipsForPoints [c] != wc0)
			continue;

		if (cDelimiterLen <= 1) {
			m_path.bstrInfotipsForPoints [c] = L'\0';
			m_path.InfotipsForPoints.push_back (c + 1);
			continue;
		}

		if (
			cDelimiterLen > 1 &&
			c + cDelimiterLen <= cLen &&
			::memcmp (m_path.bstrInfotipsForPoints.m_str + c + 1, _bstrDelimiter + 1, (cDelimiterLen - 1)*sizeof (OLECHAR)) == 0
		) {
			m_path.bstrInfotipsForPoints [c] = L'\0';
			c += cDelimiterLen - 1;
			m_path.InfotipsForPoints.push_back (c + 1);
		}
	}
	return S_OK;
}

STDMETHODIMP CComPath::AppendInfotipForPoints (BSTR _bstrInfotip) {
	if (m_path.bstrInfotipsForPoints.Length () == 0)
		m_path.bstrInfotipsForPoints = L" ";

	if (::SysStringLen (_bstrInfotip) == 0) {
		m_path.InfotipsForPoints.push_back (m_path.bstrInfotipsForPoints.Length ());
		return S_OK;
	}

	const size_t cLen0 = m_path.bstrInfotipsForPoints.Length () + 1;
	CComBSTR bstrInfotips (cLen0 + ::SysStringLen (_bstrInfotip));
	if (m_path.bstrInfotipsForPoints.Length () != 0)
		::memcpy (bstrInfotips.m_str, m_path.bstrInfotipsForPoints, cLen0*sizeof (OLECHAR));

	::memcpy (bstrInfotips.m_str + cLen0, _bstrInfotip, (::SysStringLen (_bstrInfotip) + 1)*sizeof (OLECHAR));
	m_path.InfotipsForPoints.push_back (cLen0);

	m_path.bstrInfotipsForPoints.Empty ();
	m_path.bstrInfotipsForPoints.Attach (bstrInfotips.Detach ());
	return S_OK;
}

STDMETHODIMP CComPath::put_Infotip (BSTR _bstrVal) {
	m_path.bstrInfotip = _bstrVal;
	return S_OK;
}

STDMETHODIMP CComPath::get_Infotip (BSTR * _ret) {
	* _ret = m_path.bstrInfotip.Copy ();
	return S_OK;
}

STDMETHODIMP CComPath::get_ShowDirection (VARIANT_BOOL * _ret) {
	* _ret = m_path.bShowDirection ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::put_ShowDirection (VARIANT_BOOL _bVal) {
	m_path.bShowDirection = _bVal != VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::get_ShowVertices (VARIANT_BOOL * _ret) {
	* _ret = m_path.bShowVertices ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::put_ShowVertices (VARIANT_BOOL _bVal) {
	m_path.bShowVertices = _bVal != VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::get_FillArea (VARIANT_BOOL * _ret) {
	* _ret = m_path.bFillArea ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::put_FillArea (VARIANT_BOOL _bVal) {
	m_path.bFillArea = _bVal != VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::get_VertexSize (long * _ret) {
	* _ret = m_path.cVertexSize;
	return S_OK;
}

STDMETHODIMP CComPath::put_VertexSize (long _lVal) {
	m_path.cVertexSize = _lVal;
	return S_OK;
}

STDMETHODIMP CComPath::get_AreaColor (long * _ret) {
	* _ret = m_path.colArea;
	return S_OK;
}

STDMETHODIMP CComPath::put_AreaColor (long _lVal) {
	m_path.colArea = _lVal;
	return S_OK;
}

STDMETHODIMP CComPath::get_TransparentArea (VARIANT_BOOL * _ret) {
	* _ret = m_path.bTransparentArea ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::put_TransparentArea (VARIANT_BOOL _bVal) {
	m_path.bTransparentArea = _bVal != VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComPath::get_Label (BSTR * _ret) {
	USES_CONVERSION;
	* _ret = ::SysAllocStringLen (A2OLE (m_path.strLabel.c_str ()), m_path.strLabel.size ());
	return S_OK;
}

STDMETHODIMP CComPath::put_Label (BSTR _bstrVal) {
	USES_CONVERSION;
	m_path.strLabel = OLE2A (_bstrVal);
	return S_OK;
}

STDMETHODIMP CComPath::get_LabelColor (long * _ret) {
	* _ret = m_path.colLabel;
	return S_OK;
}

STDMETHODIMP CComPath::put_LabelColor (long _lVal) {
	m_path.colLabel = _lVal;
	return S_OK;
}
