//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// COM object model: the root object.
//

# include "StdAfx.h"
# include "Resource.h"
# include "COMRoot.h"
# include "Map.h"
# include "Load.h"
# include "Save.h"
# include "File.h"
# include "Version.h"
# include "Globals.h"

void GetApplication (IApplication ** _ppApplication) {
	if (g_pComRoot) {
		g_pComRoot->QueryInterface (__uuidof (IApplication), reinterpret_cast<void **> (_ppApplication));
		return;
	}
	// NOTE: Create_CComRoot() is implicitly defined by DECLARE_COM_CLASS().
	CComRoot * const g_pComRoot = new CComRoot;
	g_pComRoot->QueryInterface (__uuidof (IApplication), reinterpret_cast<void **> (_ppApplication));
}

////////////////////////////////////////////////////////

static
IUnknown * _GetApplication () {
	CComPtr<IApplication> pApplication;
	GetApplication (& pApplication);
	return pApplication.Detach ();
}

DECLARE_COM_CLASS_F ("GPSMapEdit.Application.1", "{81588F76-D296-4e3f-9433-829EC3561F57}", CComRoot, _GetApplication, REGCLS_SINGLEUSE, false);

// NOTE: there may be only single instance of CComRoot due to REGCLS_SINGLEUSE flag.
CComRoot * g_pComRoot;

CComRoot::CComRoot () :
	m_dwAdviseCookie (0)
{
	assert (g_pComRoot == NULL);
	g_pComRoot = this;
}

CComRoot::~CComRoot () {
	g_pComRoot = NULL;
}

# include "COMRoot.inl"

STDMETHODIMP CComRoot::get_Version (BSTR * _ret) {
	USES_CONVERSION;
	* _ret = ::SysAllocString (A2OLE (GME_VER_S));
	return S_OK;
}

STDMETHODIMP CComRoot::MinimizeWindow () {
	::ShowWindow (g_hWnd, SW_MINIMIZE);
	return S_OK;
}

STDMETHODIMP CComRoot::RestoreWindow () {
	::ShowWindow (g_hWnd, SW_RESTORE);
	return S_OK;
}

STDMETHODIMP CComRoot::Exit () {
	::PostMessage (g_hWnd, WM_CLOSE, 0, 0);
	return S_OK;
}

STDMETHODIMP CComRoot::Close () {
	::SendMessage (g_hWnd, WM_COMMAND, ID_FILE_CLOSE, 0);
	return S_OK;
}

STDMETHODIMP CComRoot::Open (BSTR _bstrFileName, VARIANT_BOOL _vbAdd, VARIANT_BOOL _vbWait) {
	if (_bstrFileName == NULL || ::SysStringLen (_bstrFileName) == 0) {
		CFileTypeInfo::ShowOpenFile (_vbAdd == VARIANT_FALSE);
		return S_OK;
	}

	USES_CONVERSION;
	if (! DoesFileExist (OLE2A (_bstrFileName)))
		return E_INVALIDARG;

	LoadMap (OLE2A (_bstrFileName), 0, _vbAdd == VARIANT_FALSE, true, g_map);

	if (_vbWait != VARIANT_FALSE) {
		if (WaitLoadThread () != 0)
			return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CComRoot::Attach (BSTR _bstrFileName, VARIANT_BOOL _vbWait) {
	if (_bstrFileName == NULL || ::SysStringLen (_bstrFileName) == 0) {
		CFileTypeInfo::ShowOpenFile (false);
		return S_OK;
	}

	USES_CONVERSION;
	if (! DoesFileExist (OLE2A (_bstrFileName)))
		return E_INVALIDARG;

	LoadMap (OLE2A (_bstrFileName), 0, false, true, g_map);

	if (_vbWait != VARIANT_FALSE) {
		if (WaitLoadThread () != 0)
			return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CComRoot::SaveAs (BSTR _bstrFileName, BSTR _bstrFormat) {
	USES_CONVERSION;

	const CFileTypeInfo * const pSaverInfo = ::SysStringLen (_bstrFormat) > 0 ? CFileTypeInfo::FindSaverInfoByClass (OLE2A (_bstrFormat)) : g_pSaverInfo;
	if (pSaverInfo == NULL)
		return E_INVALIDARG;

	if (! SaveMap (pSaverInfo, OLE2A (_bstrFileName), true))
		return E_FAIL;

	return S_OK;
}

STDMETHODIMP CComRoot::get_FileName (BSTR * _ret) {
	USES_CONVERSION;
	* _ret = ::SysAllocString (A2OLE (g_strFileName));
	return S_OK;
}

STDMETHODIMP CComRoot::get_FileOpenFilter (BSTR * _ret) {
	string_t strFilter;
	CFileTypeInfo::GetOpenFileFilter (strFilter, 0);

	const size_t cFilterLen2 = strFilter.size ()*3;
	wchar_t * const wstrFilter = (wchar_t *) alloca (sizeof (wchar_t)*cFilterLen2);
	const size_t cFilterLen3 = ::MultiByteToWideChar (CP_ACP, 0, strFilter.c_str (), strFilter.size (), wstrFilter, cFilterLen2);

	* _ret = ::SysAllocStringLen (wstrFilter, cFilterLen3);
	return S_OK;
}

STDMETHODIMP CComRoot::get_LastOpenedFiles (long * _ret) {
	* _ret = g_last_file_names.size ();
	return S_OK;
}

STDMETHODIMP CComRoot::LastOpenedFile (long _lIndex, BSTR * _ret) {
	if (_lIndex < 0 || (size_t) _lIndex >= g_last_file_names.size ())
		return E_INVALIDARG;

	file_names_t::const_iterator i = g_last_file_names.begin ();
	std::advance (i, _lIndex);

	USES_CONVERSION;
	* _ret = ::SysAllocString (A2OLE (i->c_str ()));
	return S_OK;
}

///////////////////////////////////////////////////////////////

# include "MapSkins.h"

STDMETHODIMP CComRoot::LoadMapSkin (BSTR _strPath, VARIANT_BOOL _Day) {
	USES_CONVERSION;
	return ::LoadMapSkin (OLE2A (_strPath), _Day == VARIANT_FALSE ? "night" : "day") ? S_OK : S_FALSE;
}

///////////////////////////////////////////////////////////////

# include "About.h"

STDMETHODIMP CComRoot::ShowAboutBox (long _hWndParent) {
	::ShowAboutBox (reinterpret_cast<HWND> (_hWndParent));
	return S_OK;
}

# include "Activation.h"

STDMETHODIMP CComRoot::ShowActivationBox (long _hWndParent) {
	ActivateLicense (reinterpret_cast<HWND> (_hWndParent));
	return S_OK;
}

STDMETHODIMP CComRoot::get_IsRegistered (VARIANT_BOOL * _ret) {
	* _ret = g_bRegistered ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

///////////////////////////////////////////////////////////////

STDMETHODIMP CComRoot::get_Visible (VARIANT_BOOL * _ret) {
	* _ret = ::IsWindowVisible (g_hWnd) ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CComRoot::put_Visible (VARIANT_BOOL _Val) {
	::ShowWindow (g_hWnd, _Val != VARIANT_FALSE ? SW_SHOWNA : SW_HIDE);
	return S_OK;
}

///////////////////////////////////////////////////////////////

# include "UndoRedo.h"

STDMETHODIMP CComRoot::ResetModified () {
	ResetUndoRedo ();
	g_bModified = false;
	return S_OK;
}

///////////////////////////////////////////////////////////////

# include "COMStatus.h"

STDMETHODIMP CComRoot::get_StatusBar (IStatusBar ** _ret) {
	return g_ComStatusBar.QueryInterface (__uuidof (IStatusBar), reinterpret_cast<void **> (_ret));
}

///////////////////////////////////////////////////////////////

# include "COMStatistics.h"

STDMETHODIMP CComRoot::get_Statistics (long _lZoomLevel, IStatistics ** _ret) {
	return GetStatistics (_lZoomLevel, _ret);
}

///////////////////////////////////////////////////////////////

# include "Log.h"

STDMETHODIMP CComRoot::get_MessageLog (BSTR * _ret) {
	CCritSec::CLock lock (g_csMessageLog);
	USES_CONVERSION;
	* _ret = ::SysAllocStringLen (A2OLE (g_strMessageLog), g_cMessageLogSize);
	return S_OK;
}

///////////////////////////////////////////////////////////////

# include "PrintUnits.h"

STDMETHODIMP CComRoot::PositionToString (double _Latitude, double _Longitude, PositionFormat _Fmt, BSTR * _ret) {
	* _ret = NULL;

	char strPosition [64];
	PrintPosition (strPosition, point_t (_Longitude, _Latitude));

	USES_CONVERSION;
	* _ret = ::SysAllocString (A2OLE (strPosition));

	return S_OK;
}

# include "ParsePosition.h"

STDMETHODIMP CComRoot::StringToPosition (BSTR _bstrPosition, double * _pLatitude, double * _pLongitude, VARIANT_BOOL * _ret) {
	* _ret = VARIANT_FALSE;

	point_t pt;

	USES_CONVERSION;
	* _ret = ParsePosition (pt, OLE2A (_bstrPosition)) ? VARIANT_TRUE : VARIANT_FALSE;

	* _pLongitude = pt.x;
	* _pLatitude  = pt.y;

	return S_OK;
}

///////////////////////////////////////////////////////////////

STDMETHODIMP CComRoot::_Advise (IApplicationEventsSink * _pSink) {
	if (m_dwAdviseCookie != 0)
		Unadvise (m_dwAdviseCookie);

	if (_pSink != NULL)
		return Advise (_pSink, & m_dwAdviseCookie);
	else
		return S_OK;
}
