//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2006
//
// Change zip code dialog.
//

# include "StdAfx.h"
# include "Resource.h"
# include "ChangeZip.h"
# include "UndoRedo.h"
# include "ListView.h"
# include "ParseHelpers.h"
# include "Globals.h"

static int g_iSortColumn = -1;
static bool g_bAccent = true;

static
int CALLBACK _CompareItems (LPARAM _lParam1, LPARAM _lParam2, LPARAM _lParamSort) {
	const HWND hList = reinterpret_cast<HWND> (_lParamSort);

	char str1 [64], str2 [64];
	ListView_GetItemText (hList, _lParam1, g_iSortColumn, str1, 64); 
	ListView_GetItemText (hList, _lParam2, g_iSortColumn, str2, 64);

	// Compare as strings.
	return g_bAccent ? ::strcmp (str1, str2) : ::strcmp (str2, str1);
}

static
INT_PTR CALLBACK _ChangeZipProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_INITDIALOG: {
			zip_t ** const ppZip = reinterpret_cast<zip_t **> (_lParam);
			::SetWindowLong (_hWnd, GWL_USERDATA, reinterpret_cast<LONG> (ppZip));

			// Prepare list view control for cities.			
			const HWND hList = ::GetDlgItem (_hWnd, IDC_LIST);

			ListView_SetExtendedListViewStyleEx (hList, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);

			AddColumn (hList, 0, "Zip code");

			CCritSec::CLock lock (g_cs);

			size_t cItem = 0;
			for (zips_t::iterator i = g_map.Zips.begin (); i != g_map.Zips.end (); ++ i) {
				const zip_t & zip = * i;
				if (zip.IsDeleted ())
					continue;

				InsertItem (hList, cItem, reinterpret_cast<LPARAM> (& zip), "%s", zip.strName.c_str ());

				if (& zip == * ppZip)
					SetSelection (hList, cItem);

				++ cItem;
			}

			// Set the best width.
			AutoSizeColumns (hList);

			if (g_iSortColumn != -1)
				ListView_SortItemsEx (hList, _CompareItems, hList);

			// Make sure that current zip is visible.
			EnsureSelectionVisible (hList);

			return TRUE;
		}

		case WM_NOTIFY: {
			const NMHDR * const pInfo = reinterpret_cast<const NMHDR *> (_lParam);
			switch (pInfo->code) {
				case LVN_COLUMNCLICK: {
					const NMLISTVIEW * const pInfo2 = reinterpret_cast<const NMLISTVIEW *> (_lParam);

					if (g_iSortColumn != pInfo2->iSubItem)
						g_iSortColumn = pInfo2->iSubItem;
					else
						g_bAccent = ! g_bAccent;
					
					ListView_SortItemsEx (pInfo->hwndFrom, _CompareItems, pInfo->hwndFrom);

					break;
				}

				case NM_CLICK:
				case LVN_KEYDOWN:
					::EnableWindow (::GetDlgItem (_hWnd, IDOK), TRUE);
					::EnableWindow (::GetDlgItem (_hWnd, IDC_APPLY), TRUE);
					break;

				case NM_RETURN:
				case NM_DBLCLK:
					::SendMessage (_hWnd, WM_COMMAND, IDOK, 0);
					break;
			}
			break;
		}

		case WM_COMMAND:
			switch (LOWORD (_wParam)) {
				case IDOK: {
					const HWND hList = ::GetDlgItem (_hWnd, IDC_LIST);
					// Get new zip code.
					const size_t cSel = ListView_GetSelectionMark (hList);
					if (cSel == -1)
						break;
					zip_t * const pZip = reinterpret_cast<zip_t *> (GetLParam (hList, cSel));

					zip_t ** const ppZip = reinterpret_cast<zip_t **> (::GetWindowLong (_hWnd, GWL_USERDATA));
					* ppZip = pZip;

					::EndDialog (_hWnd, 0);
					return TRUE;
				}

				case IDCANCEL:
					// On closing dialog.
					::EndDialog (_hWnd, 1);
					return TRUE;
			}
	}

	return FALSE;
}

//////////////////////////////////////////////////

bool ShowChangeZip (HWND _hWnd, zip_t * & _pZip) {
	return DlgBox (IDD_SELECT_ZIP, _hWnd, & _ChangeZipProc, reinterpret_cast<LPARAM> (& _pZip)) == 0;
}
