//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2007-2009
//
// 'Create POI by address' dialog.
//

# include "StdAfx.h"
# include "Resource.h"
# include "CreatePOIByAddress.h"
# include "Map.h"
# include "MapView.h"
# include "NewMap.h"
# include "UndoRedo.h"
# include "BackupAction.h"
# include "Position.h"
# include "ShowRgnOnMap.h"
# include "PAHelpers.h"
# include "NearestRgnsList.h"
# include "TypeHelpers.h"
# include "CombinedAction.h"
# include "CreateObject.h"
# include "ChangeType.h"
# include "Rename.h"
# include "ChangePAI.h"
# include "ChangeCity.h"
# include "ChangeZip.h"
# include "ChangeType.h"
# include "SetRgnExtra.h"
# include "Status.h"
# include "DatabaseTable.h"
# include "Dialog.h"
# include "Button.h"
# include "ListView.h"
# include "Menu.h"
# include "PtAlgo.h"
# include "PrintUnits.h"
# include "Str.h"
# include "ParseHelpers.h"
# include "Settings.h"
# include "Globals.h"

struct attr2field_t {
	const TCHAR * strAttribute;
	bool bMandatory;
};

const attr2field_t g_Attributes [] = {
	{"Label",       true},
	{"Number",      true},
	{"Street",      true},
	{"Zip code",    false},
	{"Type",        false},
	{"Phone",       false},
	{"Fax",         false},
	{"Email",       false},
	{"Web",         false},
	{"Description", false},
};

enum input_attr_t {
	iaLabel       = 0,
	iaNumber      = 1,
	iaStreet      = 2,
	iaZipCode     = 3,
	iaType        = 4,
	iaPhone       = 5,
	iaFax         = 6,
	iaEmail       = 7,
	iaWebPage     = 8,
	iaDescription = 9,
	iaLast
};

# include "CreatePOIByAddress.inl"

/////////////////////////////////////////////////////////////////

static DWORD g_dwCancelTime;

void ShowAllStreetsInCity (HWND _hWnd, HWND _hwndButton, UINT _uiEditBox, const city_t * _pCity) {
	if (::GetTickCount () - g_dwCancelTime < 200)
		// NOTE: this click to arrow button is to close previous pop-up menu.
		return;

	CCritSec::CLock lock (g_cs);

	//
	// Prepare list of streets.
	//
	typedef std::set<string_t, LessString> streets_t;
	streets_t streets;

	float fD2Min = -1;
	for (size_t cKind = 0; cKind < rkMax; ++ cKind) {
		const layer_t & layer = g_map.layers [cKind];
		const rgns_t & rgns = layer.rgns;
		for (rgns_t::const_iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
			const rgn_t & rgn = * iRgn;
			if (rgn.IsDeleted () || rgn.IsNotMapObject ())
				continue;

			if (! rgn.pExtra || rgn.pExtra->pCity != _pCity)
				continue;

			if (! rgn.pExtra->strStreetDesc.empty ())
				streets.insert (rgn.pExtra->strStreetDesc);
		}
	}
	if (streets.empty ())
		return;

	//
	// Prepare pop-up menu.
	//
	CPopupMenu pm;
	size_t cItem = 0;
	for (streets_t::const_iterator i = streets.begin (); i != streets.end (); ++ i) {
		pm.AppendItem (1 + cItem, i->c_str ());
		++ cItem;
	}
	const size_t cStreets = cItem;

	RECT rect;
	::GetWindowRect (_hwndButton, & rect);

	//
	// Show popup menu.
	//
	const int nCmd = pm.ShowAsPopup (_hWnd, rect);

	pa_item_t * pRet = NULL;
	if (nCmd > 0 && nCmd - 1 < (int) cStreets) {
		// Set the selected text to the desired edit box.
		char strText [256];
		::GetMenuString (pm, nCmd, strText, ARRAY_SIZE (strText), MF_BYCOMMAND);
		::SetDlgItemText (_hWnd, _uiEditBox, strText);
	} else
		g_dwCancelTime = ::GetTickCount ();
}

void ShowAllNumbersInStreet (HWND _hWnd, HWND _hwndButton, UINT _uiEditBox, const city_t * _pCity, const string_t & _strStreet) {
	if (::GetTickCount () - g_dwCancelTime < 200)
		// NOTE: this click to arrow button is to close previous pop-up menu.
		return;

	CCritSec::CLock lock (g_cs);

	//
	// Prepare list of streets.
	//
	typedef std::set<string_t, LessString> hnumbers_t;
	hnumbers_t numbers;

	float fD2Min = -1;
	for (size_t cKind = 0; cKind < rkMax; ++ cKind) {
		const layer_t & layer = g_map.layers [cKind];
		const rgns_t & rgns = layer.rgns;
		for (rgns_t::const_iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
			const rgn_t & rgn = * iRgn;
			if (rgn.IsDeleted () || rgn.IsNotMapObject ())
				continue;

			if (! rgn.pExtra || rgn.pExtra->pCity != _pCity || ! (rgn.pExtra->strStreetDesc == _strStreet))
				continue;

			if (! rgn.pExtra->strNumber.empty ())
				numbers.insert (rgn.pExtra->strNumber);
		}
	}
	if (numbers.empty ())
		return;

	//
	// Prepare pop-up menu.
	//
	CPopupMenu pm;
	size_t cItem = 0;
	for (hnumbers_t::const_iterator i = numbers.begin (); i != numbers.end (); ++ i) {
		pm.AppendItem (1 + cItem, i->c_str ());
		++ cItem;
	}
	const size_t cStreets = cItem;

	RECT rect;
	::GetWindowRect (_hwndButton, & rect);

	//
	// Show popup menu.
	//
	const int nCmd = pm.ShowAsPopup (_hWnd, rect);

	pa_item_t * pRet = NULL;
	if (nCmd > 0 && nCmd - 1 < (int) cStreets) {
		// Set the selected text to the desired edit box.
		char strText [256];
		::GetMenuString (pm, nCmd, strText, ARRAY_SIZE (strText), MF_BYCOMMAND);
		::SetDlgItemText (_hWnd, _uiEditBox, strText);
	} else
		g_dwCancelTime = ::GetTickCount ();
}

//////////////////////////////////////////////////////////////////

bool ThereAreNumbersAlongRoads () {
	CCritSec::CLock lock (g_cs);

	const layer_t & layer = g_map.layers [rkPolyline];
	const rgns_t & rgns = layer.rgns;
	for (rgns_t::const_iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
		const rgn_t & rgn = * iRgn;
		if (rgn.IsDeleted () || rgn.IsNotMapObject ())
			continue;

		const size_t cElements = rgn.elements.size ();
		for (size_t cElement = 0; cElement < cElements; ++ cElement) {
			const rgn_element_t & element = rgn.elements [cElement];
			if (element.bDeleted || ! element.pExtra)
				continue;

			const point_extras_t & PointExtras = element.pExtra->PointExtras;
			const size_t cPoints = PointExtras.size ();

			for (size_t cPoint = 0; cPoint < cPoints; ++ cPoint) {
				const numbers_t & Numbers = PointExtras [cPoint].Numbers;
				if (Numbers.bHasNumbers)
					return true;
			}
		}
	}

	return false;
}

static
bool _IsInRange (unsigned long _ul, unsigned long _ul1, unsigned long _ul2) {
	if (_ul1 < _ul2)
		return _ul1 <= _ul && _ul <= _ul2;
	else
		return _ul2 <= _ul && _ul <= _ul1;
}

static
void _GetMiddlePoint (
	unsigned long _ul, unsigned long _ul1, unsigned long _ul2,
	const points_t & _points, size_t _cPoint1, size_t _cPoint2,
	point_t & _point
) {
	assert (_cPoint1 < _points.size ());
	assert (_cPoint2 < _points.size ());
	assert (_cPoint1 < _cPoint2);

	const float fCosY = ::cos (_points [0].y*c_PI/180);

	// Measure full length.
	float L = 0;
	size_t c;
	for (c = _cPoint1; c < _cPoint2; ++ c)
		L += ::hypot ((_points [c + 1].x - _points [c].x)*fCosY, _points [c + 1].y - _points [c].y);
	const float l = L/(::labs (_ul1 - _ul2) + 1);

	// Find middle segment.
	float M = _ul1 < _ul2 ? l/2 + (_ul - _ul1)*l : L - l/2 - (_ul - _ul2)*l;
	for (c = _cPoint1; c < _cPoint2; ++ c) {
		const float fDL = ::hypot ((_points [c + 1].x - _points [c].x)*fCosY, _points [c + 1].y - _points [c].y);
		if (M > fDL)
			M -= fDL;
		else {
			_point.x = _points [c].x + (_points [c + 1].x - _points [c].x)*M/fDL;
			_point.y = _points [c].y + (_points [c + 1].y - _points [c].y)*M/fDL;
			return;
		}
	}
}

static
bool _FindPositionByAddress (const city_t * _pCity, const string_t & _strStreet, const string_t & _strNumber, point_t & _point) {
	CCritSec::CLock lock (g_cs);

	// Get numeric value of the Number.
	unsigned long ulNumber = 0;
	bool bNumberIsNumeric = false;
	const char * const pNumberEnd = _strNumber.c_str () + _strNumber.size ();
	const char * p = NULL;
	if (ParseDecU (_strNumber.c_str (), pNumberEnd, & p, ulNumber))
		bNumberIsNumeric = p == pNumberEnd;
	const bool bNumberIsOdd = (ulNumber & 0x1) != 0;

	for (int nKind = rkPolygon; nKind >= rkPoint; -- nKind) {
		const layer_t & layer = g_map.layers [nKind];
		const rgns_t & rgns = layer.rgns;
		for (rgns_t::const_iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
			const rgn_t & rgn = * iRgn;
			if (rgn.IsDeleted () || rgn.IsNotMapObject ())
				continue;

			if (rgn.pExtra) {
				if (rgn.pExtra->pCity == _pCity && rgn.pExtra->strStreetDesc == _strStreet && rgn.pExtra->strNumber == _strNumber) {
					_point.x = rgn.rectBound.MeanX ();
					_point.y = rgn.rectBound.MeanY ();
					return true;
				}
			}

			if (nKind == rkPolyline && bNumberIsNumeric) {
				if (! (rgn.strLabel == _strStreet))
					continue;

				const city_t * const pCityOfRgn = rgn.pExtra ? rgn.pExtra->pCity : NULL;

				const size_t cElements = rgn.elements.size ();
				for (size_t cElement = 0; cElement < cElements; ++ cElement) {
					const rgn_element_t & element = rgn.elements [cElement];
					if (element.bDeleted || ! element.pExtra)
						continue;
					const points_t & points = element.points;

					const point_extras_t & PointExtras = element.pExtra->PointExtras;
					const size_t cPoints = __min (PointExtras.size (), points.size ());

					const numbers_t * pNumbersPrev = NULL;
					size_t cPointPrev = -1;
					for (size_t cPoint = 0; cPoint < cPoints; ++ cPoint) {
						const numbers_t & Numbers = PointExtras [cPoint].Numbers;
						if (! Numbers.bHasNumbers)
							continue;

						if (pNumbersPrev != NULL) {
							// Check left side.
							if (pNumbersPrev->wLeftParity != 0 && _IsInRange (ulNumber, pNumbersPrev->wLeftAfter, Numbers.wLeftBefore)) {
								if (
									(pNumbersPrev->wLeftParity == 1 && ! bNumberIsOdd) ||
									(pNumbersPrev->wLeftParity == 2 &&   bNumberIsOdd) ||
									pNumbersPrev->wLeftParity == 3
								) {
									const city_t * const pCity = pNumbersPrev->pCityLeftAfter ? pNumbersPrev->pCityLeftAfter : pCityOfRgn;
									if (pCity == _pCity) {
										_GetMiddlePoint (ulNumber, pNumbersPrev->wLeftAfter, Numbers.wLeftBefore, points, cPointPrev, cPoint, _point);
										return true;
									}
								}
							}

							// Check right side.
							if (pNumbersPrev->wRightParity != 0 && _IsInRange (ulNumber, pNumbersPrev->wRightAfter, Numbers.wRightBefore)) {
								if (
									(pNumbersPrev->wRightParity == 1 && ! bNumberIsOdd) ||
									(pNumbersPrev->wRightParity == 2 &&   bNumberIsOdd) ||
									pNumbersPrev->wRightParity == 3
								) {
									const city_t * const pCity = pNumbersPrev->pCityRightAfter ? pNumbersPrev->pCityRightAfter : pCityOfRgn;
									if (pCity == _pCity) {
										_GetMiddlePoint (ulNumber, pNumbersPrev->wRightAfter, Numbers.wRightBefore, points, cPointPrev, cPoint, _point);
										return true;
									}
								}
							}
						}

						pNumbersPrev = & Numbers;
						cPointPrev = cPoint;
					}
				}
			}
		}
	}

	return false;
}

////////////////////////////////////////////////////////////////

class CCreatePOIByAddress : public CDialog {
  public:
	CCreatePOIByAddress () :
		CDialog (IDD_CREATE_POI_BY_ADDRESS),
		m_dwType (0),
		m_bTypeDefined (false),
		m_pCity (NULL),
		m_pZip (NULL),
		m_bPtDefined (false),
		m_bPtPrevDefined (false),
		m_pDatabaseTable (NULL),
		m_hPos (NULL),
		m_hStatus (NULL),
		m_hPosition (NULL),
		m_bProcessed (false),
		m_bSuppressEnableButtons (false),
		m_cRecords (0)
	{}
	~CCreatePOIByAddress () {
		delete m_pDatabaseTable;
	}

	static CCreatePOIByAddress * s_pCreatePOIByAddress;

	// Override.
	virtual bool OnInit (HWND _hwndFocus);
	virtual void OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl);
	virtual HBRUSH OnCtlColor (HDC _hDC, HWND _hwndCtl);

  private:
	friend class CPOITable;

	DWORD m_dwType;
	bool m_bTypeDefined;
	city_t * m_pCity;
	zip_t  * m_pZip;

	point_t m_pt;
	bool m_bPtDefined;

	point_t m_ptPrev;
	bool m_bPtPrevDefined;
	string_t m_strLabelPrev;
	DWORD m_dwTypePrev;

	void _DoCreate ();

	bool m_bSuppressEnableButtons;
	void _EnableButtons ();

	CDatabaseTable * m_pDatabaseTable;
	size_t m_cRecords;
	size_t m_cPos;
	size_t m_cFields [iaLast];
	HWND m_hPos;
	bool m_bProcessed;
	void _NextPos ();
	void _SetNewPos (size_t _cNewPos);
	void _UpdatePos ();
	void _UpdateField (input_attr_t _attr, UINT _uiCtrlID);

	HWND m_hPosition;
	HWND m_hStatus;
	void _UpdateCreateStatus ();

	struct poi_t {
		rgn_t * pRgn;
		poi_t () : pRgn (NULL) {}

		bool Processed () const;
	};
	typedef std::vector<poi_t> POIs_t;
	POIs_t m_POIs;

  public:
	void Reset ();
	void Clean ();
};

bool CCreatePOIByAddress::poi_t::Processed () const {
	return pRgn && ! pRgn->IsDeleted ();
}

////////////////////////////////////////////////

enum {
	tcOrdinal  = 0,
	tcPosition = 1,

	tcLast
};

class CPOITable : public CDialog {
  public:
	CPOITable () : CDialog (IDD_POI_TABLE), m_pMain (NULL) {}

	void Init (CCreatePOIByAddress * _pMain) {
		m_pMain = _pMain;
		_FillTable ();
	}

	void Update () {
		_FillTable ();
	}
	void UpdatePos () {
		if (m_pMain == NULL)
			return;

		SetSelection (m_Table, m_pMain->m_cPos);
		EnsureSelectionVisible (m_Table);
	}

	static CPOITable * s_pPOITable;

	// Override.
	virtual bool OnInit (HWND _hwndFocus) {
		// Table.
		m_Table.Init (m_hWnd, IDC_TABLE, "POITable");
		m_Table.SetExtStyle (LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP | LVS_EX_GRIDLINES);

		AddColumn (m_Table, tcOrdinal,  "#");
		AddColumn (m_Table, tcPosition, "Position");

		for (size_t cCol = 0; cCol < ARRAY_SIZE (g_Attributes); ++ cCol)
			AddColumn (m_Table, cCol + tcLast, g_Attributes [cCol].strAttribute);

		AutoSizeColumns (m_Table);
		m_Table.LoadHeaders ();

		return true;
	}
	virtual void OnDestroy () {
		m_Table.SaveHeaders ();
		m_Table.Destroy ();
	}

	virtual void OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
		switch (_wCmd) {
			case IDC_LOCATE:
				if (m_pMain && m_pMain->m_cPos < m_pMain->m_POIs.size ()) {
					const rgn_t * const pRgn = m_pMain->m_POIs [m_pMain->m_cPos].pRgn;
					if (pRgn)
						ShowRgnOnMap (* pRgn);
				}
				break;

			case IDCANCEL:
				Hide ();
				break;
		}
	}

	virtual bool OnNotify (int _idCtrl, NMHDR * _pInfo) {
		switch (_pInfo->code) {
			case LVN_GETDISPINFO: {
				NMLVDISPINFO * const pInfo2 = reinterpret_cast<NMLVDISPINFO *> (_pInfo);

				if (m_pMain == NULL || m_pMain->m_pDatabaseTable == NULL)
					break;

				const size_t cItem = (size_t) pInfo2->item.iItem;
				assert (cItem < m_pMain->m_cRecords);
				if (cItem >= m_pMain->m_cRecords && cItem >= m_pMain->m_POIs.size ())
					break;
				rgn_t * const pRgn = m_pMain->m_POIs [cItem].pRgn;
				const bool bProcessed = m_pMain->m_POIs [cItem].Processed ();

				if (pInfo2->item.mask & LVIF_TEXT) {
					switch (pInfo2->item.iSubItem) {
						case tcOrdinal:
							::sprintf (pInfo2->item.pszText, "%d", cItem);
							break;

						case tcPosition:
							if (bProcessed && ! pRgn->origins.empty ())
								PrintPosition (pInfo2->item.pszText, pRgn->origins [0].point);
							else
								::sprintf (pInfo2->item.pszText, "-");

							break;

						default:
							const size_t cCol = pInfo2->item.iSubItem - tcLast;
							if (cCol >= iaLast)
								break;

							const size_t cField = m_pMain->m_cFields [cCol];
							if (cField == -1) {
								::_tcsncpy (pInfo2->item.pszText, "-", pInfo2->item.cchTextMax);
								break;
							}

							const size_t cLen = m_pMain->m_pDatabaseTable->GetFieldS (cItem, cField, pInfo2->item.pszText);
							pInfo2->item.pszText [cLen] = '\0';

							break;
					}
				}
				break;
			}

			case NM_CUSTOMDRAW: {
				NMLVCUSTOMDRAW * const pInfo2 = reinterpret_cast<NMLVCUSTOMDRAW *> (_pInfo);
				if (pInfo2->nmcd.dwDrawStage == CDDS_PREPAINT) {
					SetMsgResult (CDRF_NOTIFYITEMDRAW);
					return true;
				} else if (pInfo2->nmcd.dwDrawStage == CDDS_ITEMPREPAINT) {
					if (m_pMain == NULL)
						break;
					const size_t cItem = (size_t) pInfo2->nmcd.dwItemSpec;
					if (cItem >= m_pMain->m_POIs.size ())
						break;
					const bool bProcessed = m_pMain->m_POIs [cItem].Processed ();

					pInfo2->clrTextBk = bProcessed ? c_colRed : ::GetSysColor (COLOR_WINDOW);
					SetMsgResult (CDRF_NEWFONT);
					return true;
				}
				break;
			}

			case NM_RCLICK:
				m_Table.PopupMenu_OnHeaderRClick (_pInfo);
				break;

			case LVN_ITEMCHANGED: {
				const NMLISTVIEW * const pInfo2 = reinterpret_cast<const NMLISTVIEW *> (_pInfo);
				if (
					(pInfo2->uOldState & LVIS_SELECTED) == 0 &&
					(pInfo2->uNewState & LVIS_SELECTED) != 0
				) {
					_UpdateButtons (pInfo2->iItem);

					if (m_pMain)
						m_pMain->_SetNewPos (pInfo2->iItem);
				}

				break;
			}
		}

		return false;
	}

  private:
	CCreatePOIByAddress * m_pMain;

	CListView m_Table;

	void _FillTable () {
		// Table.
		ListView_SetItemCount (m_Table, m_pMain ? m_pMain->m_cRecords : 0);
		SetSelection (m_Table, m_pMain ? m_pMain->m_cPos : 0);
		EnsureSelectionVisible (m_Table);

		// Total.
		size_t cProcessed = 0;
		if (m_pMain)
			for (size_t c = 0; c < m_pMain->m_POIs.size (); ++ c) {
				if (m_pMain->m_POIs [c].Processed ())
					++ cProcessed;
			}
		SetDlgItemTextF (IDC_TOTAL, "Total: %d, Non-processed: %d", m_pMain->m_cRecords, m_pMain->m_cRecords - cProcessed);
	}

	void _UpdateButtons (size_t _cSelItem) {
		EnableDlgItem (IDC_LOCATE, _cSelItem < m_pMain->m_POIs.size () && m_pMain->m_POIs [_cSelItem].Processed ());
	}
};

CPOITable * CPOITable::s_pPOITable;

bool CCreatePOIByAddress::OnInit (HWND _hwndFocus) {
	// Import.
	SetDlgItemUInt  (IDC_POS, 0);
	SetDlgItemStyle (IDC_POS, WS_EX_RIGHT);
	SetDlgItemText  (IDC_RECORDS, "/0");

	m_hPos = GetDlgItem (IDC_POS);

	// Number.
	SetButtonIcon (m_hWnd, IDC_NEAREST_NUMBER, IDI_DOWN);

	// Street.
	SetAutoCompleteForStreet (GetDlgItem (IDC_STREET), NULL);
	SetButtonIcon (m_hWnd, IDC_NEAREST_STREET, IDI_DOWN);

	// City.
	SetButtonIcon (m_hWnd, IDC_REMOVE_CITY,    IDI_CROSS);
	SetButtonIcon (m_hWnd, IDC_NEAREST_CITY,   IDI_DOWN);

	// Zip code.
	SetButtonIcon (m_hWnd, IDC_REMOVE_ZIP,     IDI_CROSS);
	SetButtonIcon (m_hWnd, IDC_NEAREST_ZIP,    IDI_DOWN);

	// Position.
	m_hPosition = GetDlgItem (IDC_POSITION);
	SetButtonIcon (m_hWnd, IDC_LOCATE,         IDI_LOCATE);

	// Contact.
	SetButtonIcon (m_hWnd, IDC_REMOVE_PHONE,   IDI_CROSS);
	SetButtonIcon (m_hWnd, IDC_REMOVE_FAX,     IDI_CROSS);
	SetButtonIcon (m_hWnd, IDC_REMOVE_EMAIL,   IDI_CROSS);
	SetButtonIcon (m_hWnd, IDC_REMOVE_WEBPAGE, IDI_CROSS);
	SetButtonIcon (m_hWnd, IDC_NEAREST_PHONE,  IDI_DOWN);
	SetButtonIcon (m_hWnd, IDC_NEAREST_FAX,    IDI_DOWN);

	// Status.
	m_hStatus = GetDlgItem (IDC_STATUS);

	_EnableButtons ();

	return true;
}

void CCreatePOIByAddress::OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
	switch (_wCmd) {
		case ID_CONNECT_TABLE: {
			CDatabaseTable * const pDatabaseTable = ImportTable (m_hWnd, m_cFields);
			if (pDatabaseTable) {
				delete m_pDatabaseTable;
				m_pDatabaseTable = pDatabaseTable;

				if (CPOITable::s_pPOITable)
					CPOITable::s_pPOITable->Update ();

				m_cPos = 0;
				m_cRecords = m_pDatabaseTable->GetRecords ();

				m_POIs.clear ();
				m_POIs.resize (m_cRecords);

				_UpdatePos ();
			}
			break;
		}

		case ID_DISCONNECT_TABLE: 
			if (m_pDatabaseTable) {
				delete m_pDatabaseTable;
				m_pDatabaseTable = NULL;

				if (CPOITable::s_pPOITable)
					CPOITable::s_pPOITable->Update ();

				m_cPos = 0;
				m_cRecords = 0;
				m_POIs.clear ();

				_UpdatePos ();
			}
			break;

		case ID_VIEW_TABLE:
			ShowModeless (CPOITable::s_pPOITable, g_hWnd);
			if (CPOITable::s_pPOITable)
				CPOITable::s_pPOITable->Init (this);
			break;

		case ID_GO_NEXT_NONPROCESSED: {
			bool bFound = false;
			while (m_cPos + 1 < m_cRecords && m_cPos < m_POIs.size () && m_POIs [m_cPos].Processed ()) {
				++ m_cPos;
				bFound = true;
			}
			if (bFound)
				_UpdatePos ();
			else
				ErrorMsgBox ("Non-processed record is not found.");
			break;
		}

		case ID_GO_PREV_NONPROCESSED: {
			bool bFound = false;
			while (m_cPos > 0 && m_cPos < m_POIs.size () && m_POIs [m_cPos].Processed ()) {
				-- m_cPos;
				bFound = true;
			}
			if (bFound)
				_UpdatePos ();
			else
				ErrorMsgBox ("Non-processed record is not found.");
			break;
		}

		case IDC_BEGIN:
			if (m_cPos != 0) {
				m_cPos = 0;
				_UpdatePos ();
			}
			break;
		case IDC_END: {
			const size_t cNewPos = m_cRecords != 0 ? m_cRecords - 1 : 0;
			if (m_cPos != cNewPos) {
				m_cPos = cNewPos;
				_UpdatePos ();
			}
			break;
		}
		case IDC_NEXT:
			_NextPos ();
			break;
		case IDC_PREV:
			if (m_cPos > 0) {
				-- m_cPos;
				_UpdatePos ();
			}
			break;

		case IDC_POS:
			if (_wNotifyCode == EN_UPDATE) {
				const size_t cNewPos = GetDlgItemUInt (IDC_POS) - 1;
				_SetNewPos (cNewPos);
			}
			break;

		case IDC_LABEL:
		case IDC_NUMBER:
		case IDC_STREET:
		case IDC_WEBPAGE:
		case IDC_POSITION:
			if (_wNotifyCode == EN_UPDATE)
				_EnableButtons ();
			break;

		case IDC_CHANGE_TYPE:
			if (ShowChangeType (m_hWnd, rkPoint, m_dwType)) {
				FillType (m_hWnd, IDC_TYPE, rkPoint, m_dwType);
				m_bTypeDefined = true;
				_EnableButtons ();
			}
			break;

		case IDC_SELECT_CITY:
			if (SelectCity (m_hWnd, true, m_pCity)) {
				FillCity (m_hWnd, IDC_CITY, m_pCity);
				_EnableButtons ();
			}
			break;
		case IDC_REMOVE_CITY:
			m_pCity = NULL;
			FillCity (m_hWnd, IDC_CITY, NULL);
			_EnableButtons ();
			break;

		case IDC_SELECT_ZIP:
			if (ShowChangeZip (m_hWnd, m_pZip)) {
				FillZip (m_hWnd, IDC_ZIP, m_pZip);
				_EnableButtons ();
			}
			break;
		case IDC_REMOVE_ZIP:
			m_pZip = NULL;
			FillZip (m_hWnd, IDC_ZIP, NULL);
			_EnableButtons ();
			break;

		case IDC_NEAREST_NUMBER: {
			string_t strStreet;
			GetDlgItemText (IDC_STREET, strStreet);
			if (m_pCity == NULL || strStreet.empty ()) {
				rect_t rectVisible;
				GetVisibleRect (rectVisible);

				ShowNearestRgnsList (m_hWnd, _hwndCtl, IDC_NUMBER, nrNumber, point_t (rectVisible.MeanX (), rectVisible.MeanY ())); break;
			} else {
				ShowAllNumbersInStreet (m_hWnd, _hwndCtl, IDC_NUMBER, m_pCity, strStreet);
			}
			break;
		}

		case IDC_NEAREST_STREET: {
			if (m_pCity == NULL) {
				rect_t rectVisible;
				GetVisibleRect (rectVisible);

				ShowNearestRgnsList (m_hWnd, _hwndCtl, IDC_STREET, nrStreet, point_t (rectVisible.MeanX (), rectVisible.MeanY ())); break;
			} else {
				ShowAllStreetsInCity (m_hWnd, _hwndCtl, IDC_STREET, m_pCity);
			}
			break;
		}

		case IDC_NEAREST_CITY: {
			rect_t rectVisible;
			GetVisibleRect (rectVisible);

			city_t * const pCity = static_cast<city_t *> (ShowNearestRgnsList (m_hWnd, _hwndCtl, IDC_CITY, nrCity, point_t (rectVisible.MeanX (), rectVisible.MeanY ())));
			if (pCity) {
				m_pCity = pCity;
				FillCity (m_hWnd, IDC_CITY, m_pCity);
				_EnableButtons ();
			}
			break;
		}

		case IDC_NEAREST_ZIP: {
			rect_t rectVisible;
			GetVisibleRect (rectVisible);

			zip_t * const pZip = static_cast<zip_t *> (ShowNearestRgnsList (m_hWnd, _hwndCtl, IDC_ZIP, nrZip, point_t (rectVisible.MeanX (), rectVisible.MeanY ())));
			if (pZip) {
				m_pZip = pZip;
				FillZip (m_hWnd, IDC_ZIP, m_pZip);
				_EnableButtons ();
			}
			break;
		}

		case IDC_NEAREST_PHONE: {
			rect_t rectVisible;
			GetVisibleRect (rectVisible);

			ShowNearestRgnsList (m_hWnd, _hwndCtl, IDC_PHONE, nrPhone, point_t (rectVisible.MeanX (), rectVisible.MeanY ()));
			break;
		}

		case IDC_NEAREST_FAX: {
			rect_t rectVisible;
			GetVisibleRect (rectVisible);

			ShowNearestRgnsList (m_hWnd, _hwndCtl, IDC_FAX,   nrPhone, point_t (rectVisible.MeanX (), rectVisible.MeanY ()));
			break;
		}

		case IDC_GO_WEB: {
			string_t strURL;
			GetDlgItemText (IDC_WEBPAGE, strURL);
			if (! strURL.empty ())
				::ShellExecute (m_hWnd, NULL, strURL.c_str (), NULL, NULL, SW_SHOWNORMAL);
			break;
		}

		case IDC_RESET_CONTACT:
			SetDlgItemText (IDC_PHONE,       NULL);
			SetDlgItemText (IDC_FAX,         NULL);
			SetDlgItemText (IDC_EMAIL,       NULL);
			SetDlgItemText (IDC_WEBPAGE,     NULL);
			SetDlgItemText (IDC_DESCRIPTION, NULL);
			break;

		case IDC_REMOVE_PHONE:   SetDlgItemText (IDC_PHONE,   NULL); break;
		case IDC_REMOVE_FAX:     SetDlgItemText (IDC_FAX,     NULL); break;
		case IDC_REMOVE_EMAIL:   SetDlgItemText (IDC_EMAIL,   NULL); break;
		case IDC_REMOVE_WEBPAGE: SetDlgItemText (IDC_WEBPAGE, NULL); break;

		case IDC_FIND: {
			if (m_cPos < m_POIs.size () && m_POIs [m_cPos].Processed ()) {
				const rgn_t * const pRgn = m_POIs [m_cPos].pRgn;
				if (pRgn && ! pRgn->origins.empty ())
					m_pt = pRgn->origins [0].point;
			} else {
				string_t strStreet;
				GetDlgItemText (IDC_STREET, strStreet);

				string_t strNumber;
				GetDlgItemText (IDC_NUMBER, strNumber);

				if (! _FindPositionByAddress (m_pCity, strStreet, strNumber, m_pt)) {
					SetDlgItemText (IDC_POSITION, "(Not found.)");

					m_bPtDefined = false;
					_EnableButtons ();
					break;
				}
			}

			char strPos [64];
			PrintPosition (strPos, m_pt);

			SetDlgItemText (IDC_POSITION, strPos);

			m_bPtDefined = true;
			_EnableButtons ();

			break;
		}

		case IDC_LOCATE:
			if (m_bPtDefined) {
				//SetScale (g_map.Levels (0).);
				SetToCenter (m_pt);
				Refresh ();
			}
			break;

		case IDC_CREATE: {
			if (! m_bPtDefined || ! m_bTypeDefined)
				break;

			_DoCreate ();

			Refresh ();

			_UpdatePos ();

			// Reset position.
			SetDlgItemText (IDC_POSITION, NULL);
			m_bPtDefined = false;

			if (m_pDatabaseTable) {
				_NextPos ();

				if (CPOITable::s_pPOITable)
					CPOITable::s_pPOITable->Update ();
			}

			break;
		}

		case ID_CREATE_AUTO: {
			SetStatus ("Creating POIs...");
			SetProgressStatus (0);

			size_t cCreated = 0;
			{
				CWaiter w;

				for (m_cPos = 0; m_cPos < m_cRecords; ++ m_cPos) {
					if (m_cPos < m_POIs.size () && m_POIs [m_cPos].Processed ())
						continue;

					_UpdatePos ();
					if (! IsDlgItemEnabled (IDC_CREATE))
						continue;

					_DoCreate ();
					++ cCreated;

					SetProgressStatus (m_cPos*100/m_cRecords);
				}
			}

			SetStatusReady ();

			Refresh ();

			m_cPos = 0;
			_UpdatePos ();

			if (CPOITable::s_pPOITable)
				CPOITable::s_pPOITable->Update ();

			InfoMsgBox ("%d POIs created.", cCreated);

			break;
		}

		case IDCANCEL:
			Hide ();
			break;
	}
}

void CCreatePOIByAddress::_DoCreate () {
	const int nDepth = BeginCombinedAction ();

	// Create POI.
	points_t points (1);
	points [0] = m_pt;
	rgn_t & rgn = CreateObject (points, rkPoint, 0);

	// Set type.
	ChangeType (rgn, m_dwType);

	// Set label.
	string_t strLabel;
	GetDlgItemText (IDC_LABEL, strLabel);
	RenameObject (rgn, strLabel);

	// Set city.
	ChangeCity (rgn, m_pCity);

	// Set zip.
	ChangeZip (rgn, m_pZip);

	// Set street.
	string_t strStreet;
	GetDlgItemText (IDC_STREET, strStreet);
	SetObjectPAStreet (rgn, strStreet);

	// Set number.
	string_t strNumber;
	GetDlgItemText (IDC_NUMBER, strNumber);
	SetObjectPANumber (rgn, strNumber);

	// Set phone.
	string_t strPhone;
	GetDlgItemText (IDC_PHONE, strPhone);
	SetObjectPhone (rgn, strPhone);

	// Set fax.
	string_t strFax;
	GetDlgItemText (IDC_FAX, strFax);
	SetObjectFax (rgn, strFax);

	// Set e-mail.
	string_t strEmail;
	GetDlgItemText (IDC_EMAIL, strEmail);
	SetObjectEmail (rgn, strEmail);

	// Set URL.
	string_t strWebPage;
	GetDlgItemText (IDC_WEBPAGE, strWebPage);
	SetObjectWebPage (rgn, strWebPage);

	// Set description.
	string_t strDescription;
	GetDlgItemText (IDC_DESCRIPTION, strDescription);
	SetObjectDescription (rgn, strDescription);

	string_t strName = "create POI";
	if (! strLabel.empty ()) {
		strName.append (" \'");
		strName.append (strLabel);
		strName.append ("\'");
	}
	EndCombinedAction (nDepth, strName.c_str ());

	if (m_cPos < m_POIs.size ())
		m_POIs [m_cPos].pRgn = & rgn;

	// Disable 'Create'.
	m_dwTypePrev = m_dwType;
	m_strLabelPrev = strLabel;
	m_ptPrev = m_pt;
	m_bPtPrevDefined = true;
}

HBRUSH CCreatePOIByAddress::OnCtlColor (HDC _hDC, HWND _hwndCtl) {
	if (_hwndCtl == m_hPos) {
		::SetBkColor (_hDC, m_bProcessed ? c_colRed : ::GetSysColor (COLOR_WINDOW));
		return m_bProcessed ? g_hbrRed : ::GetSysColorBrush (COLOR_WINDOW);
	}

	if (_hwndCtl == m_hStatus) {
		::SetBkMode (_hDC, TRANSPARENT);
		::SetTextColor (_hDC, ! IsDlgItemEnabled (IDC_CREATE) ? c_colRed : ::GetSysColor (COLOR_BTNTEXT));
		return ::GetSysColorBrush (COLOR_BTNFACE);
	}

	if (_hwndCtl == m_hPosition) {
		::SetBkMode (_hDC, TRANSPARENT);
		::SetTextColor (_hDC, ! m_bPtDefined ? c_colRed : ::GetSysColor (COLOR_BTNTEXT));
		return ::GetSysColorBrush (COLOR_BTNFACE);
	}

	return NULL;
}

void CCreatePOIByAddress::Reset () {
	m_pCity = NULL;
	FillCity (m_hWnd, IDC_CITY, NULL);

	m_pZip = NULL;
	FillZip (m_hWnd, IDC_ZIP, NULL);

	m_POIs.clear ();
	m_POIs.resize (m_cRecords);

	_UpdatePos ();
}

void CCreatePOIByAddress::Clean () {
	const size_t cPOIs = m_POIs.size ();
	for (size_t c = 0; c < cPOIs; ++ c) {
		if (m_POIs [c].pRgn && m_POIs [c].pRgn->IsDeleted ())
			m_POIs [c].pRgn = NULL;
	}
}

void CCreatePOIByAddress::_EnableButtons () {
	if (m_bSuppressEnableButtons)
		return;

	const bool bImport = m_pDatabaseTable != NULL;

	const bool bBegin      = bImport && m_cPos != 0;
	const bool bPrev       = bImport && m_cPos > 0;
	const bool bNext       = bImport && m_cPos + 1 < m_cRecords;
	const bool bEnd        = bImport && m_cPos + 1 != m_cRecords;
	const bool bCreateAuto = bImport && m_pCity != NULL && m_bTypeDefined;

	// Menu.
	const HMENU hMenu = GetMenu (m_hWnd);
	::EnableMenuItem (hMenu, ID_DISCONNECT_TABLE,     MF_BYCOMMAND | (bImport     ? MF_ENABLED : MF_GRAYED));
	::EnableMenuItem (hMenu, ID_VIEW_TABLE,           MF_BYCOMMAND | (bImport     ? MF_ENABLED : MF_GRAYED));
	::EnableMenuItem (hMenu, ID_GO_NEXT_NONPROCESSED, MF_BYCOMMAND | (bImport     ? MF_ENABLED : MF_GRAYED));
	::EnableMenuItem (hMenu, ID_GO_PREV_NONPROCESSED, MF_BYCOMMAND | (bImport     ? MF_ENABLED : MF_GRAYED));
	::EnableMenuItem (hMenu, IDC_BEGIN,               MF_BYCOMMAND | (bBegin      ? MF_ENABLED : MF_GRAYED));
	::EnableMenuItem (hMenu, IDC_PREV,                MF_BYCOMMAND | (bPrev       ? MF_ENABLED : MF_GRAYED));
	::EnableMenuItem (hMenu, IDC_NEXT,                MF_BYCOMMAND | (bNext       ? MF_ENABLED : MF_GRAYED));
	::EnableMenuItem (hMenu, IDC_END,                 MF_BYCOMMAND | (bEnd        ? MF_ENABLED : MF_GRAYED));
	::EnableMenuItem (hMenu, ID_CREATE_AUTO,          MF_BYCOMMAND | (bCreateAuto ? MF_ENABLED : MF_GRAYED));

	// Table.
	EnableDlgItem (IDC_BEGIN,   bBegin);
	EnableDlgItem (IDC_PREV,    bPrev);
	EnableDlgItem (IDC_POS,     bImport);
	EnableDlgItem (IDC_RECORDS, bImport);
	EnableDlgItem (IDC_NEXT,    bNext);
	EnableDlgItem (IDC_END,     bEnd);

	// City.
	const bool bCity = ! g_map.Cities.empty ();
	EnableDlgItem (IDC_CITY,        bCity);
	EnableDlgItem (IDC_SELECT_CITY, bCity);
	EnableDlgItem (IDC_REMOVE_CITY, bCity);

	// Zip code.
	const bool bZip = ! g_map.Zips.empty ();
	EnableDlgItem (IDC_ZIP,         bZip);
	EnableDlgItem (IDC_SELECT_ZIP,  bZip);
	EnableDlgItem (IDC_REMOVE_ZIP,  bZip);
	EnableDlgItem (IDC_NEAREST_ZIP, ! bZip || ! ThereAreNearestRgns (nrZip));

	// Web page.
	string_t strURL;
	GetDlgItemText (IDC_WEBPAGE, strURL);
	EnableDlgItem (IDC_GO_WEB, ! strURL.empty ());

	// Position.
	string_t strStreet;
	GetDlgItemText (IDC_STREET, strStreet);
	string_t strNumber;
	GetDlgItemText (IDC_NUMBER, strNumber);
	EnableDlgItem (IDC_FIND,   m_pCity != NULL && ! strStreet.empty () && ! strNumber.empty ());
	EnableDlgItem (IDC_LOCATE, m_bPtDefined);

	// Create.
	if (m_bProcessed) {
		EnableDlgItem (IDC_CREATE, false);
	} else {
		string_t strLabel;
		GetDlgItemText (IDC_LABEL, strLabel);
		const bool bEqual = m_bPtDefined && m_bPtPrevDefined && m_ptPrev == m_pt && m_strLabelPrev == strLabel && m_dwTypePrev == m_dwType;
		EnableDlgItem (IDC_CREATE, ! bEqual && m_bPtDefined && m_bTypeDefined);
	}
	_UpdateCreateStatus ();
}

void CCreatePOIByAddress::_UpdateCreateStatus () {
	if (m_bProcessed)
		SetDlgItemText (IDC_STATUS, "Status: current record is already processed.");
	else if (m_pCity == NULL)
		SetDlgItemText (IDC_STATUS, "Status: city is not defined.");
	else if (! m_bPtDefined)
		SetDlgItemText (IDC_STATUS, "Status: position is not defined.");
	else if (! m_bTypeDefined)
		SetDlgItemText (IDC_STATUS, "Status: type is not defined.");
	else {
		string_t strLabel;
		GetDlgItemText (IDC_LABEL, strLabel);
		const bool bEqual = m_bPtDefined && m_bPtPrevDefined && m_ptPrev == m_pt && m_strLabelPrev == strLabel && m_dwTypePrev == m_dwType;
		if (bEqual)
			SetDlgItemText (IDC_STATUS, "Status: POI with current label, type and position is already created.");
		else
			SetDlgItemText (IDC_STATUS, "Status: OK");
	}
}

static
bool _ParseType (const char * _pValue, const char * _pEnd, const char ** _ppEnd, unsigned long & _ulType) {
	const char * p = _pValue;

	// Omit spaces.
	while (p < _pEnd && * p == ' ')
		++ p;

	if (p + 1 < _pEnd && p [0] == '0' && p [1] == 'x')
		return ParseHex (p + 2, _pEnd, _ppEnd, _ulType);
	else
		return ParseDecU (p, _pEnd, _ppEnd, _ulType);
}

void CCreatePOIByAddress::_UpdateField (input_attr_t _attr, UINT _uiCtrlID) {
	const size_t cField = m_cFields [_attr];
	if (cField == -1)
		return;

	char * const str = (char *) alloca (sizeof (char)*(m_pDatabaseTable->GetFieldMaxLength (m_cPos, cField) + 1));
	const size_t cLen = m_pDatabaseTable->GetFieldS (m_cPos, cField, str);
	str [cLen] = '\0';

	switch (_attr) {
		case iaZipCode: {
			m_pZip = NULL;
			if (cLen != 0) {
				while (true) {
					// Search for existing zip code.
					for (zips_t::iterator zi = g_map.Zips.begin (); zi != g_map.Zips.end (); ++ zi) {
						zip_t & zip = * zi;
						if (zip.IsDeleted ())
							continue;

						if (zip.strName == str) {
							m_pZip = & zip;
							break;
						}
					}
					if (m_pZip != NULL)
						break;

					// Create new zip code.
					AddZip (str);
				}
			}
			FillZip (m_hWnd, _uiCtrlID, m_pZip);
			_EnableButtons ();
			break;
		}

		case iaType: {
			unsigned long ulType = 0;
			const char * pEnd = NULL;
			_ParseType (str, str + cLen, & pEnd, ulType);

			FillType (m_hWnd, _uiCtrlID, rkPoint, ulType);
			m_dwType = ulType;
			m_bTypeDefined = true;
			break;
		}

		default:
			SetDlgItemText (_uiCtrlID, str);
			break;
	}
}

void CCreatePOIByAddress::_SetNewPos (size_t _cNewPos) {
	if (_cNewPos < m_cRecords && m_cPos != _cNewPos) {
		m_cPos = _cNewPos;
		_UpdatePos ();
	}
}

void CCreatePOIByAddress::_NextPos () {
	if (m_cPos + 1 < m_cRecords) {
		++ m_cPos;
		_UpdatePos ();
	}
}

void CCreatePOIByAddress::_UpdatePos () {
	if (CPOITable::s_pPOITable)
		CPOITable::s_pPOITable->UpdatePos ();

	m_bProcessed = m_cPos < m_POIs.size () && m_POIs [m_cPos].Processed ();

	SetDlgItemUInt  (IDC_POS,     m_pDatabaseTable ? m_cPos + 1 : 0);
	SetDlgItemTextF (IDC_RECORDS, "/%d", m_cRecords);
	
	if (m_pDatabaseTable != NULL) {
		m_bSuppressEnableButtons = true;

		_UpdateField (iaLabel,       IDC_LABEL);
		_UpdateField (iaStreet,      IDC_STREET);
		_UpdateField (iaNumber,      IDC_NUMBER);
		_UpdateField (iaZipCode,     IDC_ZIP);
		_UpdateField (iaType,        IDC_TYPE);
		_UpdateField (iaPhone,       IDC_PHONE);
		_UpdateField (iaFax,         IDC_FAX);
		_UpdateField (iaEmail,       IDC_EMAIL);
		_UpdateField (iaWebPage,     IDC_WEBPAGE);
		_UpdateField (iaDescription, IDC_DESCRIPTION);

		SendCommand (IDC_FIND);

		m_bSuppressEnableButtons = false;
	}

	_EnableButtons ();
}

CCreatePOIByAddress * CCreatePOIByAddress::s_pCreatePOIByAddress;

void CreatePOIByAddress () {
	bool bNoAddressData = false;
	if (g_map.Cities.empty () || ! ThereAreNearestRgns (nrCity))
		bNoAddressData = true;
	else if (! ThereAreNumbersAlongRoads ()) {
		if (! ThereAreNearestRgns (nrStreet) || ! ThereAreNearestRgns (nrNumber))
			bNoAddressData = true;
	}
	if (bNoAddressData) {
		ErrorMsgBox ("The map does not contain address data required to restore positions of POIs.");
		return;
	}

	ShowModeless (CCreatePOIByAddress::s_pCreatePOIByAddress, g_hWnd);
}

# include "HoluxTypes.h"

bool CreatePOIByAddressIsAvailable () {
	return g_map.pTypeSet != & g_tsHolux;
}

static
void _ResetCreatePOIByAddress () {
	if (CCreatePOIByAddress::s_pCreatePOIByAddress)
		CCreatePOIByAddress::s_pCreatePOIByAddress->Reset ();
}

INVOKE_AFTER_REMOVE_MAP (_ResetCreatePOIByAddress);
INVOKE_ON_BACKUP_MAP    (_ResetCreatePOIByAddress);

static
void _CleanCreatePOIByAddress () {
	if (CCreatePOIByAddress::s_pCreatePOIByAddress)
		CCreatePOIByAddress::s_pCreatePOIByAddress->Clean ();
}

INVOKE_BEFORE_GC (_CleanCreatePOIByAddress);