# ifndef __Cursors_H_
# define __Cursors_H_

# include "String.h"
# include "PtAlgo.h"

struct CCursor_t {
	point_t pt;

	float fHeading;
	bool bHasHeading;
	bool bHidden;
	bool bSelected;

	COLORREF col, colArrow, colContour;
	int nR;
	int nArrowShape;

	string_t strLabel;
	COLORREF colLabel;
	int xLabel0, yLabel0;
	UINT uiAlignMask;

	CComPtr<IPicture> pIcon;
	int xIcon0, yIcon0;
	void SetCursorShift (HDC _hdc);

	CComVariant vTag;
	long lID;

	CCursor_t () :
		pt (0, 0),
		fHeading (0), bHasHeading (false), bSelected (false),
		bHidden (false),
		col (0), colArrow (0xFFFFFF), colContour (-1),
		nR (7),
		nArrowShape (0),
		xIcon0 (0), yIcon0 (0),
		colLabel (0),
		xLabel0 (3), yLabel0 (3), uiAlignMask (TA_TOP | TA_LEFT),
		lID (0)
	{}
};

typedef std::list<CCursor_t> Cursors_t;
extern Cursors_t g_Cursors;

extern COLORREF g_colDefaultCursorColor;

void DrawCursors          (HDC _hDC, const rect_t & _rectVisible);
void DrawCursorsSelection (HDC _hDC);

CCursor_t * FindCursorByID (long _lID);
CCursor_t * FindTopmostCursor (int _x, int _y);
void DeleteCursor (CCursor_t & _cursor);

void ShowCursors   (long _lIDFrom, long _lIDTo, bool _bShow);
void ShowCursors   (std::set<long> & _setIds,   bool _bShow);
void SelectCursors (std::set<long> & _setIds,   bool _bSelect);
void DeleteCursors (long _lIDFrom, long _lIDTo);

void SetCursorLabelFont (const char * _strFont, int _nSize, bool _bBold, bool _bItalics);
void DestroyCursors ();

extern rect_t g_CursorsSelectionRect;

# endif // __Cursors_H_