//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2008
//
// Customize menu.
//

# include "StdAfx.h"
# include "Resource.h"
# include "CustomizeMenu.h"
# include "Menu.h"
# include "Dialog.h"
# include "ListView.h"
# include "Settings.h"
# include "Str.h"
# include "Globals.h"

class CCustomizeMenu : public CDialog {
  public:
	CCustomizeMenu () : CDialog (IDD_MENU) {}

	// Override.
	virtual bool OnInit (HWND _hwndFocus) {
		// List.
		m_hList = GetDlgItem (IDC_LIST);
		ListView_SetExtendedListViewStyleEx (m_hList, LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES, LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES);
		AddColumn (m_hList, 0, "Command");

		_UpdateList ();
		_UpdateState (false);

		// OK.
		EnableDlgItem (IDOK, false);

		return true;
	}

	virtual void OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
		switch (_wCmd) {
			case IDC_RESTORE_DEFAULTS: {
				_UpdateState (true);

				EnableDlgItem (IDOK, true);

				break;
			}

			case IDOK: {
				hidden_menu_items_t HiddenMenuItems;
				const size_t cItems = ListView_GetItemCount (m_hList);
				for (size_t c = 0; c < cItems; ++ c) {
					const bool bHidden = ListView_GetCheckState (m_hList, c) == FALSE;
					if (! bHidden)
						continue;

					const size_t cIdx = GetLParam (m_hList, c);
					if (cIdx < g_menu_items.size ()) {
						const WORD wID = g_menu_items [cIdx].uiCmd;
						HiddenMenuItems.insert (wID);
					}
				}

				SetHiddenMenuItems (HiddenMenuItems);

				EndDialog (1);
				break;
			}

			case IDCANCEL:
				EndDialog (0);
				break;
		}
	}

	virtual bool OnNotify (int _idCtrl, NMHDR * _pInfo) {
		switch (_pInfo->code) {
			case LVN_ITEMCHANGED: {
				const NM_LISTVIEW * const pInfo2 = reinterpret_cast<const NM_LISTVIEW *> (_pInfo);
				if ((pInfo2->uNewState & LVIS_STATEIMAGEMASK) != 0 && (pInfo2->uOldState & LVIS_STATEIMAGEMASK) != 0) {
//					const bool bChecked = ListView_GetCheckState (_pInfo->hwndFrom, pInfo2->iItem) != FALSE;

					EnableDlgItem (IDOK, true);
				}
				break;
			}

			case NM_CUSTOMDRAW: {
				NMLVCUSTOMDRAW * const pInfo2 = reinterpret_cast<NMLVCUSTOMDRAW *> (_pInfo);
				if (pInfo2->nmcd.dwDrawStage == CDDS_PREPAINT) {
					SetMsgResult (CDRF_NOTIFYITEMDRAW);
					return true;
				} else if (pInfo2->nmcd.dwDrawStage == CDDS_ITEMPREPAINT) {
					const size_t cItem = (size_t) pInfo2->nmcd.dwItemSpec;
					const size_t cIdx = GetLParam (m_hList, cItem);
					if (cIdx >= g_menu_items.size ())
						break;
					const size_t cSubMenu = g_menu_items [cIdx].cSubMenu;

					pInfo2->clrTextBk = (cSubMenu & 0x1) != 0 ? ::GetSysColor (COLOR_WINDOW) : RGB (224, 224, 224);
					SetMsgResult (CDRF_NEWFONT);
					return true;
				}
				break;
			}
		}
		return false;
	}

  private:
	HWND m_hList;

	void _GetCmdName (UINT _uiCmd, char * _strName, size_t _cNameMax) const;
	void _UpdateList (UINT _uiCmdSel = -1);
	void _UpdateState (bool _bReset);
};

void CCustomizeMenu::_UpdateState (bool _bReset) {
	std::set<WORD> setHiddenItems;
	GetHiddenMenuItems (_bReset, setHiddenItems);

	const size_t cItems = ListView_GetItemCount (m_hList);
	for (size_t c = 0; c < cItems; ++ c) {
		const size_t cIdx = GetLParam (m_hList, c);
		if (cIdx < g_menu_items.size ()) {
			const WORD wID = g_menu_items [cIdx].uiCmd;
			const bool bHidden = setHiddenItems.find (wID) != setHiddenItems.end ();
			ListView_SetCheckState (m_hList, c, bHidden ? FALSE : TRUE);
		}
	}
}

void CCustomizeMenu::_GetCmdName (UINT _uiCmd, char * _strName, size_t _cNameMax) const {
	const size_t cItems = ListView_GetItemCount (m_hList);
	for (size_t c = 0; c < cItems; ++ c) {
		const size_t cIdx = GetLParam (m_hList, c);
		if (cIdx < g_menu_items.size ()) {
			const UINT uiCmd = g_menu_items [cIdx].uiCmd;
			if (uiCmd == _uiCmd) {
				ListView_GetItemText (m_hList, c, 0, _strName, _cNameMax);
				return;
			}
		}
	}

	_strName [0] = '\0';
}

void CCustomizeMenu::_UpdateList (UINT _uiCmdSel) {
	ListView_DeleteAllItems (m_hList);

	// Add commands from main menu items.
	size_t cItem = 0;
	const size_t cMenuItems = g_menu_items.size ();
	for (size_t c = 0; c < cMenuItems; ++ c) {
		const UINT uiCmd           = g_menu_items [c].uiCmd;
		const char * const strText = g_menu_items [c].strText.c_str ();

		if (ID_FILE_RECENT_0        <= uiCmd && uiCmd <  ID_FILE_RECENT_0        + 100)
			continue;
		if (ID_VIEW_SCALE_0         <= uiCmd && uiCmd <  ID_VIEW_SCALE_0         + 100)
			continue;
		if (ID_VIEW_LEVEL0          <= uiCmd && uiCmd <= ID_VIEW_LEVEL9)
			continue;
		if (ID_FAVORITES_0          <= uiCmd && uiCmd <  ID_FAVORITES_0          + 100)
			continue;
		if (ID_FILE_IMPORT_EXPORT_0 <= uiCmd && uiCmd <  ID_FILE_IMPORT_EXPORT_0 + 200)
			continue;
		if (ID_MAP_SKINS_0          <= uiCmd && uiCmd <  ID_MAP_SKINS_0          + 100)
			continue;

		// Command.
		InsertItem (m_hList, cItem, c, "%s", strText);

		if (_uiCmdSel != -1 && uiCmd == _uiCmdSel)
			SetSelection (m_hList, cItem);

		++ cItem;
	}

	AutoSizeColumns (m_hList);
	EnsureSelectionVisible (m_hList);
}

void CustomizeMenu (HWND _hWnd) {
	CCustomizeMenu cm;
	cm.ShowModal (_hWnd);
}
