//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2004
//
// dBASE database file parser stuff.
//

# include "StdAfx.h"
# include "DbfFile.h"
# include "ParseHelpers.h"

CDbfFile::CDbfFile (const char * _strFileName) :
	m_hFile (_strFileName),
	m_hFileMapping (NULL),
	m_pData (NULL),
	m_cSize (0),
	m_pRecords (NULL),
	m_cRecordSize (0)
{
	if (! m_hFile.IsValid ())
		return;

	m_cSize = ::GetFileSize (m_hFile, NULL);

	m_hFileMapping = ::CreateFileMapping (m_hFile, NULL, PAGE_READONLY, 0, 0, NULL);
	m_pData = reinterpret_cast<char *> (::MapViewOfFile (m_hFileMapping, FILE_MAP_READ, 0, 0, 0));
	if (! m_pData)
		return;

	const CDbfHdr * const pHdr = reinterpret_cast<const CDbfHdr *> (m_pData);
	m_pRecords = m_pData + pHdr->wHdrSize;
	m_cRecordSize = pHdr->wRecordSize;
	m_cRecords = m_cRecordSize ? (m_cSize - pHdr->wHdrSize)/m_cRecordSize : 0;

	const size_t cFields = GetFields ();
	m_vecFieldOffsets.resize (cFields);
	size_t cOffset = 0;
	for (size_t cField = 0; cField < cFields; ++ cField) {
		if (pHdr->FieldDescriptors [cField].strName [0] == 0xD)
			break;

		m_vecFieldOffsets [cField] = cOffset;
		cOffset += pHdr->FieldDescriptors [cField].btLen;
	}
}

CDbfFile::~CDbfFile () {
	if (m_pData)
		::UnmapViewOfFile (m_pData);
}

int CDbfFile::GetFieldN (size_t _cRecord, size_t _cField) const {
	const size_t cFieldLen = GetFieldLength (_cField);
	const char * strField = GetRawField (_cRecord, _cField);
	return ParseDecU (strField, strField + cFieldLen);
}
