//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// GDI drawing helpers.
//

# include "StdAfx.h"
# include "DrawHelpers.h"
# include "fast_floor.h"
# include "Globals.h"

void GetCursorHotSpot (HCURSOR _hcur, POINT & _ptShift) {
	ICONINFO ii;
	::memset (& ii, 0, sizeof (ii));
	::GetIconInfo (_hcur, & ii);

	_ptShift.x = ii.xHotspot;
	_ptShift.y = ii.yHotspot;

	if (ii.hbmColor)
		::DeleteObject (ii.hbmColor);
	if (ii.hbmMask)
		::DeleteObject (ii.hbmMask);
}

void DrawLine (HDC _hDC, int _x0, int _y0, int _x1, int _y1) {
	const POINT pts [2] = {{_x0, _y0}, {_x1, _y1}};
	::Polyline (_hDC, pts, ARRAY_SIZE (pts));
}

void DrawSquare (HDC _hDC, int _x, int _y, int _nSize) {
	::Rectangle (_hDC, _x - _nSize, _y - _nSize, _x + _nSize + 1, _y + _nSize + 1);
}

void DrawDiamond (HDC _hDC, int _x, int _y, int _nSize) {
	const POINT pts [4] = {
		{_x - _nSize, _y},
		{_x,          _y + _nSize},
		{_x + _nSize, _y},
		{_x,          _y - _nSize},
	};
	::Polygon (_hDC, pts, ARRAY_SIZE (pts));
}

void DrawTriangle1 (HDC _hDC, int _x, int _y, int _nSize) {
	_x -= _nSize/2;
	_y += _nSize/2;
	const POINT pts [3] = {
		{_x,          _y},
		{_x + _nSize, _y - _nSize},
		{_x + _nSize, _y},
	};
	::Polygon (_hDC, pts, ARRAY_SIZE (pts));
}

void DrawTriangle2 (HDC _hDC, int _x, int _y, int _nSize) {
	_x -= _nSize/2;
	_y -= _nSize/2;
	const POINT pts [3] = {
		{_x,          _y},
		{_x,          _y + _nSize},
		{_x + _nSize, _y},
	};
	::Polygon (_hDC, pts, ARRAY_SIZE (pts));
}

void DrawCross (HDC _hDC, int _x, int _y) {
	DrawCross (_hDC, _x, _y, c_iCrossSize);
}

void DrawCross (HDC _hDC, int _x, int _y, int _nSize) {
	DrawLine (_hDC, _x - _nSize, _y - _nSize, _x + _nSize + 1, _y + _nSize + 1);
	DrawLine (_hDC, _x - _nSize, _y + _nSize, _x + _nSize + 1, _y - _nSize - 1);
}

void DrawCrossSerif (HDC _hDC, int _x, int _y, int _nSize1, int _nSize2) {
	// Top T.
	DrawLine (_hDC, _x,           _y - 1,       _x,               _y - _nSize1 - 1);
	DrawLine (_hDC, _x - _nSize2, _y - _nSize1, _x + _nSize2 + 1, _y - _nSize1);

	// Bottom T.
	DrawLine (_hDC, _x,           _y + 1,       _x,               _y + _nSize1 + 1);
	DrawLine (_hDC, _x - _nSize2, _y + _nSize1, _x + _nSize2 + 1, _y + _nSize1);

	// Left T.
	DrawLine (_hDC, _x - 1,       _y,           _x - _nSize1 - 1, _y);
	DrawLine (_hDC, _x - _nSize1, _y - _nSize2, _x - _nSize1,     _y + _nSize2 + 1);

	// Right T.
	DrawLine (_hDC, _x + 1,       _y,           _x + _nSize1 + 1, _y);
	DrawLine (_hDC, _x + _nSize1, _y - _nSize2, _x + _nSize1,     _y + _nSize2 + 1);
}

void DrawText2 (HDC _hDC, int _x, int _y, const char * _strText, COLORREF _colText, COLORREF _colHL) {
	const size_t cLen = ::strlen (_strText);

	::SetBkMode (_hDC, TRANSPARENT);

	if (g_bNT) {
		::SetTextColor (_hDC, _colHL);

		POLYTEXT pt [4];
		pt [0].x = _x - 1;
		pt [0].y = _y;
		pt [1].x = _x + 1;
		pt [1].y = _y;
		pt [2].x = _x;
		pt [2].y = _y + 1;
		pt [3].x = _x;
		pt [3].y = _y - 1;
		for (size_t c = 0; c < ARRAY_SIZE (pt); ++ c) {
			pt [c].n       = cLen;
			pt [c].lpstr   = _strText;
			pt [c].uiFlags = 0;
			pt [c].pdx     = NULL;
		}
		::PolyTextOut (_hDC, pt, ARRAY_SIZE (pt));
	}

	::SetTextColor (_hDC, _colText);
	::TextOut (_hDC, _x, _y, _strText, cLen);
}

void DrawCompassArrow (HDC _hDC, int _x0, int _y0, float _fAzimuth, int _l) {
	const float fCosA = ::cos (_fAzimuth);
	const float fSinA = ::sin (_fAzimuth);

	//
	// Draw north arrow.
	//

	const int h1 = 5, h2 = _l;

	POINT pts [3];
	pts [0].x = _x0 + fast_floor (h1*fCosA + .5f);
	pts [0].y = _y0 + fast_floor (h1*fSinA + .5f);
	pts [1].x = _x0 - fast_floor (h1*fCosA + .5f);
	pts [1].y = _y0 - fast_floor (h1*fSinA + .5f);
	pts [2].x = _x0 + fast_floor (h2*fSinA + .5f);
	pts [2].y = _y0 - fast_floor (h2*fCosA + .5f);

	::SelectObject (_hDC, g_hbrBlue);
	::Polygon (_hDC, pts, ARRAY_SIZE (pts));

	//
	// Draw south arrow.
	//

	pts [0].x = _x0 - fast_floor (h1*fCosA + .5f);
	pts [0].y = _y0 - fast_floor (h1*fSinA + .5f);
	pts [1].x = _x0 + fast_floor (h1*fCosA + .5f);
	pts [1].y = _y0 + fast_floor (h1*fSinA + .5f);
	pts [2].x = _x0 - fast_floor (h2*fSinA + .5f);
	pts [2].y = _y0 + fast_floor (h2*fCosA + .5f);

	::SelectObject (_hDC, g_hbrRed);
	::Polygon (_hDC, pts, ARRAY_SIZE (pts));
}

void DrawArrow (HDC _hDC, int _x, int _y, float _fAngleInDegrees, int _l, int _w0, int _w1) {
	assert (_w0 < _w1);

	POINT points [7] = {
		{_l/2,       0},
		{_l/2 - _w1, _w1},
		{_l/2 - _w1, _w0},
		{-_l/2,      _w0},
		{-_l/2,      -_w0},
		{_l/2 - _w1, -_w0},
		{_l/2 - _w1, -_w1}
	};

	const float fSin = ::sin (_fAngleInDegrees*c_PI/180);
	const float fCos = ::cos (_fAngleInDegrees*c_PI/180);

	for (size_t c = 0; c < ARRAY_SIZE (points); ++ c) {
		const long x = points [c].x;
		const long y = points [c].y;
		points [c].x = _x + fast_floor (x*fCos - y*fSin + .5f);
		points [c].y = _y + fast_floor (x*fSin + y*fCos + .5f);
	}

	::Polygon (_hDC, points, ARRAY_SIZE (points));
}

void DrawTriangleArrow (HDC _hDC, int _x0, int _y0, float _fAngleInDegrees, int _r) {
	POINT points [3];
	points [0].x = _r + 7; points [0].y = 0;
	points [1].x = _r;     points [1].y = -3;
	points [2].x = _r;     points [2].y = 3;

	const float fSin = ::sin (_fAngleInDegrees*c_PI/180);
	const float fCos = ::cos (_fAngleInDegrees*c_PI/180);

	for (size_t c = 0; c < ARRAY_SIZE (points); ++ c) {
		const long x = points [c].x;
		const long y = points [c].y;
		points [c].x = _x0 + fast_floor (x*fCos - y*fSin + .5f);
		points [c].y = _y0 + fast_floor (x*fSin + y*fCos + .5f);
	}

	::Polygon (_hDC, points, ARRAY_SIZE (points));
}
