//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Generic EH filter.
//

# include "StdAfx.h"
# include "EHFilter.h"
# include "Log.h"
# include "Version.h"

# ifdef WIN32
DWORD EHFilter (void * _ei) {
	if (::IsDebuggerPresent ())
		return EXCEPTION_CONTINUE_SEARCH;

	const EXCEPTION_POINTERS * const pe = reinterpret_cast<EXCEPTION_POINTERS *> (_ei);

	const char * strType = NULL;
	switch (pe->ExceptionRecord->ExceptionCode) {
		case EXCEPTION_ACCESS_VIOLATION:    strType = "access violation"; break;
		case EXCEPTION_STACK_OVERFLOW:      strType = "stack overlow"; break;
		case EXCEPTION_FLT_DIVIDE_BY_ZERO:  strType = "divide by zero (floating-point)"; break;
		case EXCEPTION_ILLEGAL_INSTRUCTION: strType = "illegal instruction"; break;
		case EXCEPTION_INT_DIVIDE_BY_ZERO:  strType = "divide by zero (integer)"; break;

		case EXCEPTION_BREAKPOINT:
			return EXCEPTION_CONTINUE_SEARCH;

		default:                            strType = "application-specific exception"; break;
	}

	ReportWarning ("Internal error.");
	ReportText ("Please report the error log to the developer (kg@geopainting.com):");
	ReportText ("------------------------------");
	ReportText ("Instruction at [0x%08x] caused %s (exception code is 0x%08x).", pe->ExceptionRecord->ExceptionAddress, strType, pe->ExceptionRecord->ExceptionCode);

	if (
		pe->ExceptionRecord->ExceptionCode == EXCEPTION_ACCESS_VIOLATION &&
		pe->ExceptionRecord->NumberParameters >= 2
	) {
		ReportText (
			"Attempt to %s to [0x%08x] which is illegal address.",
			pe->ExceptionRecord->ExceptionInformation [0] == 0 ? "read" : "write",
			pe->ExceptionRecord->ExceptionInformation [1]
		);
	}
/*
	// Report stack.
	STACKFRAME sf;
	const size_t cStackDepth = 5;
	DWORD dwPC [cStackDepth];
	for (size_t cFrame = 0; cFrame < cStackDepth; ++ cFrame) {
		::StackWalk (IMAGE_FILE_MACHINE_I386, ::GetCurrentProcess (), ::GetCurrentThread (), & sf, NULL, NULL, NULL, NULL, NULL);
		dwPC [cFrame] = sf.AddrPC.Offset;
	}
	ReportText ("Stack: 0x%08x 0x%08x 0x%08x 0x%08x", dwPC [0], dwPC [1], dwPC [2], dwPC [3]);
*/

	// Report program version.
	ReportText ("GPSMapEdit version: " GME_VER_S);

	// Report OS version.
	OSVERSIONINFO oi = {sizeof (OSVERSIONINFO)};
	::GetVersionEx (& oi);
	ReportText (
		"OS version: %s %d.%d %s",
		oi.dwPlatformId == VER_PLATFORM_WIN32_NT ? "NT" : "9x",
		oi.dwMajorVersion, oi.dwMinorVersion, oi.szCSDVersion 
	);

	ReportText ("------------------------------");

	return EXCEPTION_EXECUTE_HANDLER;
}
# endif // WIN32
