//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2008
//
// Dialog for editing point coordinates.
//

# include "StdAfx.h"
# include "Resource.h"
# include "EditPoint.h"
# include "PtAlgo.h"
# include "PrintUnits.h"
# include "ParsePosition.h"
# include "ParseHelpers.h"
# include "Dialog.h"
# include "Globals.h"

class CEditPoint : public CDialog {
  public:
	CEditPoint (point_t & _point) : CDialog (IDD_EDIT_POINT), m_point (_point) {}

	// Override.
	virtual bool OnInit (HWND _hwndFocus) {
		char strX [20], strY [20];
		PrintLon (strX, m_point.x);
		PrintLat (strY, m_point.y);

		SetDlgItemTextF (IDC_POSITION, "%s %s", strY, strX);

		EnableDlgItem (IDOK, false);
		return true;
	}

	virtual void OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
		switch (_wCmd) {
			case IDC_POSITION: {
				if (_wNotifyCode == EN_CHANGE) {
					point_t pt;
					const bool bOK = _ParsePos (pt);
					EnableDlgItem (IDOK, bOK);
				}
				break;
			}

			case IDOK: {
				point_t pt;
				const bool bOK = _ParsePos (pt);
				if (bOK)
					m_point = pt;

				EndDialog (0);
				break;
			}

			case IDCANCEL:
				EndDialog (1);
				break;
		}
	}

  private:
	point_t & m_point;

	bool _ParsePos (point_t & _pt) const;
};

bool CEditPoint::_ParsePos (point_t & _pt) const {
	const HWND hPos = ::GetDlgItem (m_hWnd, IDC_POSITION);

	const size_t cLen = ::GetWindowTextLength (hPos);
	char * const str = reinterpret_cast<char *> (alloca ((cLen + 1)*sizeof (char)));
	::GetWindowText (hPos, str, cLen + 1);

	return ParsePosition (_pt, str, str + cLen);
}

bool EditPoint (HWND _hWnd, point_t & _point) {
	CEditPoint epd (_point);
	return epd.ShowModal (_hWnd) == 0;
}