# ifndef __FileTypeInfo_H_
# define __FileTypeInfo_H_

struct map_t;

# include "LoadStatus.h"
# include "SaveStatus.h"

class CMapLoader;
class CMapSaver;

enum {
	ftiDefault        = 0x0001, // Saver by default.
	ftiHideByDefault  = 0x0002, // Don't show in the list of formats to save or export.
	ftiImportExport   = 0x0004, // Show saver/loader in 'export'/'import' menu but not in 'open'/'save as' one.

	ftiRenderer       = 0x0008, // Loader which does not pre-load map objects but renders them on-the-fly from file to current viewport.

	ftiDontMapFile    = 0x0010, // For loaders based on 3rd party APIs.
	ftiMapFileByParts = 0x0020, // For forward-scanning loaders (e.g. text files). By default, files up to 4 GB may be supported.
	ftiLoadOver4GB    = 0x0040, // For loaders by parts: files over 4 GB are supported.

	ftiWpts           = 0x0080, // Export of attached waypoints.
	ftiTracks         = 0x0100, // Export of attached tracks.
	ftiRoutes         = 0x0200, // Export of attached routes.
	ftiWTR            = (ftiWpts | ftiTracks | ftiRoutes),

	ftiBookmarks      = 0x0400, // Export of bookmarks.

	ftiSaveCopyOfMap  = 0x0800, // Export only: pass copy of map to CMapSaver::Save().
	ftiSaveThunk      = 0x1000, // Export only: provide dummy saver in 'Save As' list redirecting user to 'File | Export'.
	ftiShowSaveAs     = 0x2000, // Export only: automatically show 'Save As' dialog to select file.
};

class CFileTypeInfo {
  public:
  	static void Init ();

	typedef CMapLoader * (* pfCreateLoader_t) ();
	typedef CMapSaver  * (* pfCreateSaver_t)  ();

	CFileTypeInfo (
		pfCreateLoader_t _pfCreateLoader, pfCreateSaver_t _pfCreateSaver,
		const char * _strClass,	const char * _strExt, const char * _strFilter,
		DWORD _dwFlags
	);

	const pfCreateLoader_t m_pfCreateLoader;
	const pfCreateSaver_t  m_pfCreateSaver;
	const char * const m_strClass;
	std::vector<string_t> m_vecExts;
	string_t m_strFilter;

	const DWORD m_dwFlags;

	size_t m_cImportExportCmd;

	// The linked list of creators.
	CFileTypeInfo * m_pNext;
	static CFileTypeInfo * s_pFirst;

  private:
	bool m_bEnableSave;

	friend class CMapLoader;

	CFileTypeInfo * FindSaverInfo () const;
	static const CFileTypeInfo * GetDefaultSaverInfo ();

  public:
	bool IsSaveEnabled () const {return m_bEnableSave;}
	void EnableSave (bool _bEnable);

	//
	// The utilities for map loaders/savers (registration, search etc).
	//

	static void GetOpenFileFilter (string_t & _strFilter, DWORD _dwRequiredFlagsFilter);

	static CMapLoader * CreateLoader (
		const char * _pData, size_t _cFileSize, const char * _pFileName,
		const CFileTypeInfo ** _ppSaverInfo, DWORD & _dwFlags,
		const CFileTypeInfo ** _ppFileTypeInfo = NULL
	);
	//
	static void ShowOpenFile (bool _bRemoveOldMap);
	static void ShowOpenFile (bool _bRemoveOldMap, bool _bWait, map_t & _map, DWORD _dwRequiredFlagsFilter, const char * _strDefaultName = NULL);

	static void ShowSaveFile ();

	static void  InitImportMenu (HMENU _hMenu);
	static HMENU InitExportMenu (HMENU _hMenu);
	static void UpdateExportMenu (HMENU _hMenu);

	static void ImportExportFile (size_t _cCmd, const char * _strFileName = NULL);
	void ExportFile (const char * _strFileName) const;
	void _ExportFileImpl (const char * _strFileName, bool _bOpenFile) const;

	static CFileTypeInfo * FindSaverInfoByClass  (const char * _strClass);
	static CFileTypeInfo * FindLoaderInfoByClass (const char * _strClass);
};

SaveStatus_t ExecuteSaveFileSafely (CMapSaver * _pSaver, HANDLE _hFile, const char * _strFileName, const map_t & _map);

# endif // __FileTypeInfo_H_