//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Object filter.
//

# include "StdAfx.h"
# include "Filter.h"
# include "Map.h"
# include "MapView.h"
# include "Selection.h"
# include "Menu.h"

void SelectByFilter (const CFilter & _filter) {
	ClearSelPoints ();

	CCritSec::CLock lock (g_cs);

	// NOTE: temp flag is used to avoid dublicated selections.	
	g_map.ClearTemporaryFlags ();
	// Set temp flag for already selected rgns.
	if (! g_Selection.empty ()) {
		const size_t cSelItems = g_Selection.size ();
		for (size_t cSel = 0; cSel < cSelItems; ++ cSel) {
			find_info_t & fi = g_Selection [cSel];
			if (fi.pLayer) {
		        rgn_t & rgn = * fi.iRgn;
				rgn.bTemporary = true;
			}
		}
	}

	for (size_t cKind = 0; cKind < rkMax; ++ cKind) {
		layer_t & layer = g_map.layers [cKind];
		rgns_t & rgns = layer.rgns;
		const size_t cRgns = layer.cRgns;

		for (rgns_t::iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
			rgn_t & rgn = * iRgn;
			if (rgn.IsImmutable ())
				continue;

			if (rgn.bTemporary == false && _filter.DoesFit (rgn)) {
				find_info_t fi;
				fi.pLayer           = & layer;
				fi.iRgn             = iRgn;
				fi.cElementOrOrigin = -1;

				g_Selection.push_back (fi);
			}
		}
	}

	Redraw ();
	UpdateMenu ();
}
