//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Find routing node by ID.
//

# include "StdAfx.h"
# include "FindNodeID.h"
# include "EditNumber.h"
# include "Map.h"
# include "MapView.h"
# include "Position.h"
# include "Wrappers.h"
# include "Globals.h"

void FindNodeID () {
	size_t cNodeID = 0;
	if (! EditNumber (g_hWnd, cNodeID, "Find routing node by ID:"))
		return;

	{
		CCritSec::CLock lock (g_cs);

		const layer_t & polylines = g_map.layers [rkPolyline];
		const rgns_t & rgns = polylines.rgns;
		for (rgns_t::const_iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
			const rgn_t & rgn = * iRgn;
			if (rgn.IsImmutable () || ! rgn.HasRGNodes ())
				continue;

			const size_t cElements = rgn.elements.size ();
			for (size_t cElement = 0; cElement < cElements; ++ cElement) {
				const rgn_element_t & element = rgn.elements [cElement];
				if (element.bDeleted || ! element.pExtra)
					continue;

				const point_extras_t & PointExtras = element.pExtra->PointExtras;
				const size_t cPoints = PointExtras.size ();
				for (size_t c = 0; c < cPoints; ++ c) {
					const rgnode_t * const pRGNode = PointExtras [c].GetRGNode ();
					if (pRGNode == NULL)
						continue;

					if (pRGNode->dwNodeID == cNodeID && c < element.points.size ()) {
						const point_t & point = element.points [c];

						SetToCenter (point);
						Refresh ();

						return;
					}
				}
			}
		}
	}

	ErrorMsgBox ("Node ID is not found.");
}