//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2008
//
// Flip object.
//

# include "StdAfx.h"
# include "FlipObject.h"
# include "UndoRedo.h"
# include "RefreshHoles.h"
# include "Label.h"
# include "Globals.h"

class CFlipObjectAction : public CEditAction {
  public:
	CFlipObjectAction (rgn_t * _pRgn, size_t _cElementOrOrigin, bool _bVertical) :
		m_bVertical (_bVertical),
		m_pRgn (_pRgn),
		m_cElementOrOrigin (_cElementOrOrigin),
		m_bWasModified ((_pRgn->dwFlags & rgn_t::fModified) != 0)
	{}

	virtual void GetName (string_t & _str) const;
	virtual cu_t CanUndo () const {return cuYes;}
	virtual bool Do ();
	virtual void Undo ();
	virtual void Redo ();

  private:
	const bool    m_bVertical;
	rgn_t * const m_pRgn;
	const size_t  m_cElementOrOrigin;
	const bool    m_bWasModified;

	void Flip ();
};

void CFlipObjectAction::GetName (string_t & _str) const {
	CCritSec::CLock lock (g_cs);

	assert (m_pRgn);
	if (m_cElementOrOrigin == -1)
		_str.append ("flip [");
	else
		_str.append ("flip element of [");
	GetHintText (* m_pRgn, _str);
	_str.append ("]");
}

void CFlipObjectAction::Flip () {
	CCritSec::CLock lock (g_cs);

	assert (m_pRgn);

	if (m_pRgn->IsWTR () || m_pRgn->IsPoint ())
		return;

	if (m_cElementOrOrigin == -1) {
		const point_t pt0 (m_pRgn->rectBound.MeanX (), m_pRgn->rectBound.MeanY ());

		const size_t cElements = m_pRgn->elements.size ();
		for (size_t cElement = 0; cElement < cElements; ++ cElement) {
			rgn_element_t & element = m_pRgn->elements [cElement];
			if (element.bDeleted)
				continue;

			points_t & points = element.points;
			const size_t cPoints = points.size ();
			for (size_t cPoint = 0; cPoint < cPoints; ++ cPoint) {
				point_t & pt = points [cPoint];

				if (m_bVertical)
					pt.y = pt0.y - (pt.y - pt0.y);
				else
					pt.x = pt0.x - (pt.x - pt0.x);
			}
		}
	} else {
		assert (m_cElementOrOrigin < m_pRgn->elements.size ());
		rgn_element_t & elem = m_pRgn->elements [m_cElementOrOrigin];

		const point_t pt0 (elem.rectBound.MeanX (), elem.rectBound.MeanY ());

		points_t & points = elem.points;
		const size_t cPoints = points.size ();
		for (size_t cPoint = 0; cPoint < cPoints; ++ cPoint) {
			point_t & pt = points [cPoint];

			if (m_bVertical)
				pt.y = pt0.y - (pt.y - pt0.y);
			else
				pt.x = pt0.x - (pt.x - pt0.x);
		}
	}

	m_pRgn->RefreshAttributes (g_map, false);
}

bool CFlipObjectAction::Do () {
	Flip ();
	m_pRgn->dwFlags |= rgn_t::fModified;
	return true;
}

void CFlipObjectAction::Undo () {
	Flip ();

	if (! m_bWasModified)
		m_pRgn->dwFlags &= ~rgn_t::fModified;
}

void CFlipObjectAction::Redo () {
	CFlipObjectAction::Do ();
}

///////////////////////////////////////////

void FlipObject (rgn_t * _pRgn, size_t _cElementOrOrigin, bool _bVertical) {
	PerformEditAction (new CFlipObjectAction (_pRgn, _cElementOrOrigin, _bVertical));

	if (_pRgn->IsPolygon () && _cElementOrOrigin != -1)
		RefreshHoles (_pRgn, true);
}
