//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
// Garmin and MapSource are registered trademarks or trademarks of Garmin Ltd. or one of its subsidiaries.
//
// The dialod to select items in Garmin IMG container.
//

# include "StdAfx.h"
# include "Resource.h"
# include "GarminImgContainer.h"
# include "Dialog.h"
# include "ListView.h"
# include "PrintUnits.h"
# include "Globals.h"

enum {
	lvID          = 0,
	lvSections    = 1,
	lvSize        = 2,
	lvDescription = 3
};

class CSelectGarminImgContainerItems : public CDialog {
  public:
	CSelectGarminImgContainerItems (CGarminImgContainerItems & _items) :
		CDialog (IDD_IMG_LIST),
		m_items (_items)
	{}

	// Override.
	virtual bool OnInit (HWND _hwndFocus);
	virtual void OnDestroy ();
	virtual void OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl);
	virtual bool OnNotify (int _idCtrl, NMHDR * _pInfo);

  private:
	CGarminImgContainerItems & m_items;

	CListView m_List;

	static int CALLBACK _CompareItems (LPARAM _lItem1, LPARAM _lItem2, LPARAM _lData);
};

int CALLBACK CSelectGarminImgContainerItems::_CompareItems (LPARAM _lItem1, LPARAM _lItem2, LPARAM _lData) {
	const CSelectGarminImgContainerItems * const pThis = reinterpret_cast<const CSelectGarminImgContainerItems *> (_lData);
	const size_t cSortingColumn = pThis->m_List.GetSortingColumn ();

	const CGarminImgContainerItem * const pItem1 = reinterpret_cast<CGarminImgContainerItem *> (GetLParam (pThis->m_List, _lItem1));
	const CGarminImgContainerItem * const pItem2 = reinterpret_cast<CGarminImgContainerItem *> (GetLParam (pThis->m_List, _lItem2));

	switch (cSortingColumn) {
		default:
		case lvID:          return ::lstrcmpi (pItem1->strID         .c_str (), pItem2->strID         .c_str ());
		case lvSections:    return ::lstrcmpi (pItem1->strSections   .c_str (), pItem2->strSections   .c_str ());
		case lvSize:        return             pItem1->cSize                  - pItem2->cSize;
		case lvDescription: return ::lstrcmpi (pItem1->strDescription.c_str (), pItem2->strDescription.c_str ());
	}
}

bool CSelectGarminImgContainerItems::OnInit (HWND _hwndFocus) {
	m_List.Init (m_hWnd, IDC_LIST, "GarminImgContainerList");
	m_List.SetExtStyle (LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES | LVS_EX_HEADERDRAGDROP);

	AddColumn (m_List, lvID,          "ID");
	AddColumn (m_List, lvSections,    "Sections");
	AddColumn (m_List, lvSize,        "Size",        true);
	AddColumn (m_List, lvDescription, "Description");

	m_List.SetSorting (_CompareItems, reinterpret_cast<LPARAM> (this));

	size_t cItem = 0;
	for (CGarminImgContainerItems::const_iterator i = m_items.begin (); i != m_items.end (); ++ i) {
		const string_t & strID               = i->first; 
		const CGarminImgContainerItem & item = i->second;

		char strSize [32];
		PrintInt (strSize, item.cSize / 1024);

		InsertItem     (m_List, cItem, reinterpret_cast<DWORD> (& item), "%s",   strID.c_str ());
		SetSubItemText (m_List, cItem, lvSections,                       "%s",   item.strSections.c_str ());
		SetSubItemText (m_List, cItem, lvSize,                           "%s K", strSize);
		SetSubItemText (m_List, cItem, lvDescription,                    "%s",   item.strDescription.c_str ());

		ListView_SetCheckState (m_List, cItem, item.bSelected ? TRUE : FALSE);

		++ cItem;
	}

	AutoSizeColumns (m_List);
	m_List.LoadHeaders ();

	return true;
}

void CSelectGarminImgContainerItems::OnDestroy () {
	m_List.SaveHeaders ();
	m_List.Destroy ();
}

void CSelectGarminImgContainerItems::OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
	switch (_wCmd) {
		case IDC_SELECT_ALL:
		case IDC_UNSELECT_ALL: {
			const BOOL bSelect = _wCmd == IDC_SELECT_ALL ? TRUE : FALSE;
			const size_t cItems = ListView_GetItemCount (m_List);
			for (size_t cItem = 0; cItem < cItems; ++ cItem)
				ListView_SetCheckState (m_List, cItem, bSelect);
			break;
		}

		case IDOK: {
			const size_t cItems = ListView_GetItemCount (m_List);
			for (size_t cItem = 0; cItem < cItems; ++ cItem) {
				const bool bChecked = ListView_GetCheckState (m_List, cItem) != FALSE;
				CGarminImgContainerItem * const pItem = reinterpret_cast<CGarminImgContainerItem *> (GetLParam (m_List, cItem));
				pItem->bSelected = bChecked;
			}

			EndDialog (0);
			break;
		}

		case IDCANCEL:
			EndDialog (1);
			break;
	}
}

bool CSelectGarminImgContainerItems::OnNotify (int _idCtrl, NMHDR * _pInfo) {
	switch (_pInfo->code) {
		case NM_RCLICK:
			m_List.PopupMenu_OnHeaderRClick (_pInfo);
			break;

		case LVN_COLUMNCLICK: {
			// Save selection.
			const size_t cSel = ListView_GetSelectionMark (m_List);
			const LPARAM lParamSel = cSel == -1 ? -1 : GetLParam (m_List, cSel);

			m_List.Sort_OnColumnClick (_pInfo);

			// Restore selection.
			if (lParamSel != -1)
				m_List.SelectByLParam (lParamSel);

			break;
		}
	}

	return false;
}

bool SelectGarminImgContainerItems (CGarminImgContainerItems & _items) {
	CSelectGarminImgContainerItems sgici (_items);
	return sgici.ShowModal (g_hWnd) == 0;
}
