//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2008
//
// Common protocol scenario.
//

# include "StdAfx.h"
# include "GpsProtocol.h"
# include "ComPort.h"
# include "Positioning.h"

bool CGpsProtocol::Run (CComPort * _pPort, size_t _cPort, HANDLE _hStopEvent, HANDLE _hReadyEvent) {
	// Report protocol name.
	string_t strProtocolName;
	this->GetName (strProtocolName);
	char strStatus [256];
	::sprintf (strStatus, "Trying %s...", strProtocolName.c_str ());
	SetPositioningStatus (strStatus);

	COMMTIMEOUTS ct = {MAXDWORD, 0, 0, 10, 10};
	::SetCommTimeouts (* _pPort, & ct);

	// Clean the input buffer.
	::PurgeComm (* _pPort, PURGE_TXCLEAR | PURGE_RXCLEAR);

	// Start the protocol.
	this->Init (_pPort, _cPort);
	CGpsProtocol::CStarter ps (this);

	const bool bAggressiveMethod = UseAggressiveMethod ();

	bool bTry = true;

	while (true) {
		// Wait for data.
		DWORD dwDataSize = 0;
		while (true) {
			if (bTry) {
				const HANDLE events [2] = {_hStopEvent, _hReadyEvent};
				if (::WaitForMultipleObjects (2, events, FALSE, 20) != WAIT_TIMEOUT)
					return true;
			} else {
				if (::WaitForSingleObject (_hStopEvent, 20) == WAIT_OBJECT_0)
					return true;
			}

			if (! bAggressiveMethod)
				dwDataSize = _pPort->GetInQueueSize ();
			else
				dwDataSize = RecommendedDataSizeForReadFile ();

			if (dwDataSize > 0)
				break;

			const CGpsProtocol::Status_t s = this->OnDataReceived (NULL, 0);
    		if (s == CGpsProtocol::stSureError) {
				if (! bTry) {
					::ResetEvent (_hReadyEvent);
					SetPositioningStatus ("No data detected.");
					::WaitForSingleObject (_hStopEvent, 2000);
				}
				return false;
			}
		}

		while (true) {
			// Read the data.
			BYTE pData [4096];
			DWORD dwRead = 0;
			::ReadFile (* _pPort, pData, dwDataSize < sizeof (pData) ? dwDataSize : sizeof (pData), & dwRead, NULL);

			// Analyze the data.
			const CGpsProtocol::Status_t s = this->OnDataReceived (dwRead ? pData : NULL, dwRead);
			if (s == stSureError) {
				if (! bTry) {
					::ResetEvent (_hReadyEvent);
					SetPositioningStatus ("Protocol failure.");
					::WaitForSingleObject (_hStopEvent, 2000);
				}
				return false;
			}

			if (dwRead == 0)
				break;

			if (bTry && s == stSureOK) {
				bTry = false;

				::SetEvent (_hReadyEvent);

				::sprintf (strStatus, "%s over COM%d.", strProtocolName.c_str (), _cPort);
				SetPositioningStatus (strStatus);
			}

			if (! bAggressiveMethod)
				break;
			if (::WaitForSingleObject (_hStopEvent, 1) != WAIT_TIMEOUT)
				break;
		}
	}
}