//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2008-2009
//
// GPX format helpers.
//

# include "StdAfx.h"
# include "GpxHelpers.h"
# include "FileWriter.h"
# include "fast_floor.h"

void WriteGPXDateTime (CFileWriter & _fw, DATE _dt, bool _bHasMilliseconds) {
	// ISO 8601 'dateTime' type
	// Sample: 2001-11-28T21:05:28Z

	int nMilliseconds = 0;
	if (_bHasMilliseconds) { 
		const int c_nSecondsPerDay = 24*60*60;
		const double dfSeconds = _dt*c_nSecondsPerDay;
		double dfSecondsInt = ::floor (dfSeconds); 
		nMilliseconds = fast_floor_d ((dfSeconds - dfSecondsInt)*1000 + .5);
		if (nMilliseconds >= 1000) {
			dfSecondsInt += 1;
			nMilliseconds -= 1000;
		}

		// NOTE: round _dt to integer seconds to make ::VariantTimeToSystemTime() to work good.
		_dt = (dfSecondsInt + 0.0001)/c_nSecondsPerDay;
	} 

	SYSTEMTIME st;
	::VariantTimeToSystemTime (_dt, & st);

	_fw.WriteU (st.wYear);
	_fw.WriteC ('-');

	if (st.wMonth < 10)
		_fw.WriteC ('0');
	_fw.WriteU (st.wMonth);
	_fw.WriteC ('-');

	if (st.wDay < 10)
		_fw.WriteC ('0');
	_fw.WriteU (st.wDay);
	_fw.WriteC ('T');

	if (st.wHour < 10)
		_fw.WriteC ('0');
	_fw.WriteU (st.wHour);
	_fw.WriteC (':');

	if (st.wMinute < 10)
		_fw.WriteC ('0');
	_fw.WriteU (st.wMinute);
	_fw.WriteC (':');

	if (st.wSecond < 10)
		_fw.WriteC ('0');
	_fw.WriteU (st.wSecond);
	if (_bHasMilliseconds) {
		_fw.WriteC ('.');
		if (nMilliseconds == 0) {
			_fw.WriteS ("000");
		} else if (nMilliseconds < 10) {
			_fw.WriteC ('0');
			_fw.WriteC ('0');
		} else if (10 <= nMilliseconds && nMilliseconds < 100)
			_fw.WriteC ('0');
		_fw.WriteU (nMilliseconds);
	}

	_fw.WriteC ('Z');
}
