//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2008
//
// GPX track writer.
//

# include "StdAfx.h"
# include "Version.h"
# include "Tracking.h"
# include "Positioning.h"
# include "FileWriter.h"
# include "GpxHelpers.h"

class CGpxTrackWriter : public CTrackWriter {
  public:
	virtual void Start ();
	virtual void Stop ();
	virtual void StartNewTrack ();
	virtual void AddPointToTrack (const PositionInfo_t & _pi);

  private:
	size_t m_cTrackNumber;
	size_t m_cPoints;
};

DECLARE_TRACK_WRITER (CGpxTrackWriter, "gpx", "GPS eXchange format (*.gpx)");

void CGpxTrackWriter::Start () {
	m_pFileWriter->WriteS (
		"<\?xml version=\"1.0\" encoding=\"UTF-8\" \?>\r\n"
		"<gpx\r\n"
		" version=\"1.0\"\r\n"
		" creator=\"GPSMapEdit " GME_VER_S " - http://www.geopainting.com\"\r\n"
		" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\r\n"
		" xmlns=\"http://www.topografix.com/GPX/1/0\"\r\n"
		" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd\">\r\n"
	);

	// Date and Time.
	SYSTEMTIME stCurrent;
	::GetSystemTime (& stCurrent);
	DATE dtCurrent;
	::SystemTimeToVariantTime (& stCurrent, & dtCurrent);
	m_pFileWriter->WriteS ("<time>");
	WriteGPXDateTime (* m_pFileWriter, dtCurrent, false);
	m_pFileWriter->WriteS ("</time>\r\n");

	m_pFileWriter->WriteS (
		"<trk>\r\n"
		" <trkseg>\r\n"
	);

	m_cTrackNumber = 0;
	m_cPoints = 0;
}

void CGpxTrackWriter::Stop () {
	m_pFileWriter->WriteS (
		" </trkseg>\r\n"
		"</trk>\r\n"
		"</gpx>\r\n"
	);
}

void CGpxTrackWriter::StartNewTrack () {
	if (m_cPoints == 0)
		return;

	m_pFileWriter->WriteS (
		" </trkseg>\r\n"
		" <trkseg>\r\n"
	);

	++ m_cTrackNumber;
	m_cPoints = 0;
}

void CGpxTrackWriter::AddPointToTrack (const PositionInfo_t & _pi) {
	if ((_pi.dwDef & PositionInfo_t::defXY) == 0)
		return;

	// Position.
	m_pFileWriter->WriteFmt ("  <trkpt lat=\"%0.6f\" lon=\"%0.6f\">\r\n", _pi.fY, _pi.fX);

	// Altitude.
	if (_pi.dwDef & PositionInfo_t::defAltitude)
		m_pFileWriter->WriteFmt ("   <ele>%0.1f</ele>\r\n", _pi.fAltitude);

	// Date and Time.
	if (_pi.dwDef & PositionInfo_t::defTime) {
		m_pFileWriter->WriteS ("   <time>");
		WriteGPXDateTime (* m_pFileWriter, _pi.dtTime, true);
		m_pFileWriter->WriteS ("</time>\r\n");
	}

	// Speed.
	if (_pi.dwDef & PositionInfo_t::defSpeed)
		m_pFileWriter->WriteFmt ("   <speed>%0.1f</speed>\r\n", _pi.fSpeed/3.6f);

	// HDOP.
	if (_pi.dwDef & PositionInfo_t::defHDOP)
		m_pFileWriter->WriteFmt ("   <hdop>%0.1f</hdop>\r\n", _pi.fHDOP);
	// VDOP.
	if (_pi.dwDef & PositionInfo_t::defVDOP)
		m_pFileWriter->WriteFmt ("   <vdop>%0.1f</vdop>\r\n", _pi.fVDOP);
	// Satellites.
	if (_pi.dwDef & PositionInfo_t::defSatellites)
		m_pFileWriter->WriteFmt ("   <sat>%d</sat>\r\n", _pi.cSatellites);
	// Fix.
	if (_pi.dwDef & PositionInfo_t::defFix) {
		const char * strFix = NULL;
		switch (_pi.Fix) {
			default:
			case PositionInfo_t::fixInvalid: strFix = "   <fix>none</fix>\r\n"; break;
			case PositionInfo_t::fix2D:      strFix = "   <fix>2D</fix>\r\n";   break;
			case PositionInfo_t::fix3D:      strFix = "   <fix>3D</fix>\r\n";   break;
			case PositionInfo_t::fix3D_diff: strFix = "   <fix>dgps</fix>\r\n"; break;
			case PositionInfo_t::fixPrec:    strFix = "   <fix>pps</fix>\r\n";  break;
		}
		m_pFileWriter->WriteS (strFix);
	}

	m_pFileWriter->WriteS ("  </trkpt>\r\n");
	++ m_cPoints;

	m_pFileWriter->FlushBuffer ();
}
