//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2007
// Holux is registered trademark of Holux Technology Inc.
//
// Holux TKL track parser.
//

# include "StdAfx.h"
# include "HoluxTkl.h"
# include "Map.h"
# include "Types.h"
# include "Load.h"
# include "Log.h"
# include "Status.h"
# include "ParseHelpers.h"
# include "Wrappers.h"
# include "Globals.h"

class CTklLoader : public CMapLoader {
  public:
	// Override.
	virtual bool IsValid (const char * _pData, size_t _cFileSize) const;
	virtual LoadStatus_t Load (const char * _pData, size_t _cFileSize, const char * _strFileName, map_t * _pMap);
};

DECLARE_WPTRK_LOADER (CTklLoader, "holux-tkl", "tkl", "Holux MapShow305 tracks (*.tkl)", ftiTracks);

bool CTklLoader::IsValid (const char * _pData, size_t _cFileSize) const {
	if (_cFileSize < 0x20000 + sizeof (CTklHdr))
		return false;

	if (reinterpret_cast<const CTklHdr *> (_pData + 0x20000)->wSignature == c_wTklSignature)
		return true;

	return false;
}

LoadStatus_t CTklLoader::Load (const char * _pData, size_t _cFileSize, const char * _strFileName, map_t * _pMap) {
	const char * const pEnd  = _pData + _cFileSize;

	file_refs_t file_refs;
	file_refs.push_back (file_ref_t ());
	file_refs.back ().strFileName = _strFileName;
	file_refs.back ().bAttachment = true;

	SetProgressStatus (0);

	const CTklHdr      * const pHdr = reinterpret_cast<const CTklHdr *> (_pData + 0x20000);
	const CTklTrackLog * const pLogs [c_cTklMaxTracks] = {
		reinterpret_cast<const CTklTrackLog *> (_pData),
		reinterpret_cast<const CTklTrackLog *> (_pData + sizeof (CTklTrackLog)),
		reinterpret_cast<const CTklTrackLog *> (_pData + sizeof (CTklTrackLog)*2),
		reinterpret_cast<const CTklTrackLog *> (_pData + 0x10000),
		reinterpret_cast<const CTklTrackLog *> (_pData + 0x10000 + sizeof (CTklTrackLog)),
	};

	for (size_t cTrack = 0; /*cTrack < pHdr->wTracks &&*/ cTrack < c_cTklMaxTracks; ++ cTrack) {
		const CTklTrack & track = pHdr->tracks [cTrack];
		if (track.wPoints == 0)
			continue;
		if (track.dwOccupied == 0)
			continue;

		const CTklTrackLog & log = * (pLogs [cTrack]);

		//
		// Create track.
		//
		rgn_t & rgn = _pMap->CreateRgn (rkPolyline);
		rgn.SetTrackType ();

		rgn.pFileRef = & file_refs.back ();
		rgn.cBeginPos = reinterpret_cast<const char *> (& log) - _pData;
		rgn.cEndPos = pEnd - _pData;
		rgn.cFileLine = -1;

		rgn.strLabel.assign   (track.strName);
		rgn.strComment.assign (track.strComment, GetLineLen (track.strComment, track.strComment + sizeof (track.strComment) - 1));

		rgn.elements.push_back (rgn_element_t ());
		rgn_element_t & element = rgn.elements.back ();
		element.cLevel = 0;
		element.cFileOffset = rgn.cBeginPos;

		points_t & points = element.points;
		points.resize (track.wPoints);
		for (size_t cPoint = 0; cPoint < track.wPoints && cPoint < c_cTklMaxPoints; ++ cPoint) {
			const CTklPoint & pt = log.points [cPoint];
			point_t & point = points [cPoint];

			point.x =  pt.lX/36000.f;
			point.y = -pt.lY/36000.f;
		}

		rgn.pExtra->wtr.CreateTrack ();

		SetProgressStatus (50*cTrack/c_cTklMaxTracks);
	}

	_pMap->file_refs.swap (file_refs);

	// Suppress warning about empty Zoom etc.
	_pMap->bNoHeader = true;
	_pMap->cHeaderLen = 0;

	SetProgressStatus (50);

	return lsOK;
}
