//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Insert zoom level from the map.
//

# include "StdAfx.h"
# include "RemoveLevel.h"
# include "UndoRedo.h"
# include "Types.h"
# include "MapView.h"
# include "Status.h"
# include "Scale.h"
# include "Levels.h"

class CInsertLevelAction : public CEditAction {
  public:
	CInsertLevelAction (size_t _cLevel) : m_cLevel (_cLevel) {}

	virtual void GetName (string_t & _str) const {
		_str.append ("Insert level ");
		_str.append (1, '0' + m_cLevel);
	}
	virtual cu_t CanUndo () const {return cuBackup;}
	virtual bool Do ();
	virtual void Undo () {}
	virtual void Redo () {}

  private:
	const size_t m_cLevel;
};

bool CInsertLevelAction::Do () {
	CCritSec::CLock lock (g_cs);

	//
	// Modify header.
	//

	if (g_map.Levels.empty ()) {
		map_level_t level;

		level.SetBasicScale (g_map.pTypeSet->cMaxBits);
		level.cZoom = 0;
		g_map.Levels.push_back (level);

		level.cZoom = 1;
		level.SetBasicScale (14);
		g_map.Levels.push_back (level);

		g_map.bNoHeader = false;

		return true;
	}

	assert (m_cLevel < g_map.Levels.size ());
	const size_t cBits = (m_cLevel == 0) ? 24 : (g_map.Levels [m_cLevel].btBits + g_map.Levels [m_cLevel - 1].btBits)/2;

	// Make sure m_cLevel is not beyond new level.
	if (m_cLevel > 0) {
		map_level_t & level = g_map.Levels [m_cLevel - 1];
		if (level.btBitsRangeTo <= cBits)
			level.btBitsRangeTo = cBits + 1;
	}

	{
		assert (m_cLevel < g_map.Levels.size ());

		map_level_t level;
		level.SetBasicScale (cBits);
		level.cZoom = m_cLevel == 0 ? 0 : (g_map.Levels [m_cLevel].cZoom + g_map.Levels [m_cLevel - 1].cZoom)/2;
		g_map.Levels.insert (g_map.Levels.begin () + m_cLevel, level);
	}

	//
	// Shift level for all elements.
	//
	for (size_t cKind = 0; cKind < rkMax; ++ cKind) {
		layer_t & layer = g_map.layers [cKind];
		rgns_t & rgns = layer.rgns;
		for (rgns_t::iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
			rgn_t & rgn = * iRgn;
			if (rgn.IsImmutable () || rgn.IsNotMapObject ())
				continue;

			if (rgn.uchEndLevel != 0 && rgn.uchEndLevel >= m_cLevel)
				++ rgn.uchEndLevel;

			// Check origins.
			for (size_t cOrigin = 0; cOrigin < rgn.origins.size (); ++ cOrigin) {
				rgn_origin_t & origin = rgn.origins [cOrigin];
				if (origin.bDeleted)
					continue;

				if (origin.cLevel >= m_cLevel)
					++ origin.cLevel;
			}
			
			// Check elements of the rgn.
			for (size_t cElement = 0; cElement < rgn.elements.size (); ++ cElement) {
				rgn_element_t & element = rgn.elements [cElement]; 
				if (element.bDeleted)
					continue;

				if (element.cLevel >= m_cLevel)
					++ element.cLevel;
			}
		}
	}

	return true;
}

///////////////////////////////////////

void InsertLevelBefore (size_t _cLevel) {
	PerformEditAction (new CInsertLevelAction (_cLevel));

	Refresh ();
	SetScaleStatus (g_fScale);
	MakeLevelsMenu ();
}