//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Object labels handling.
//

# include "StdAfx.h"
# include "Map.h"
# include "Types.h"
# include "Label.h"
# include "Bookmark.h"
# include "BookmarkTypes.h"
# include "Globals.h"

void RemoveGarminSpecialCodes (string_t & _str) {
	bool bRemove = false;
	const size_t cLen = _str.size ();
	size_t c;
	for (c = 0; c < cLen; ++ c) {
		if (_str [c] == '~') {
			bRemove = true;
			break;
		}
	}
	if (! bRemove)
		// Nothing to do!
		return;

	char * const pBuf = (char *) alloca ((cLen + 1)*sizeof (char));
	size_t cOut = 0, cIn = 0;

	while (cIn < cLen) {
		bool bTrim = false;
		if (cIn + 4 <= cLen && ::_strnicmp (_str.c_str () + cIn, "~[0x", 4) == 0) {
			// Trim the special code.
			if (       cIn + 6 < cLen && _str [cIn + 6] == ']') {
				cIn += 7;
				bTrim = true;
			} else if (cIn + 8 < cLen && _str [cIn + 8] == ']') {
				cIn += 9;
				bTrim = true;
			}
		}

		if (! bTrim)
			pBuf [cOut ++] = _str [cIn ++];
	}

	pBuf [cOut ++] = '\0';
	assert (cOut <= cLen + 1);

	if (cOut < cLen + 1)
		_str.assign (pBuf);
}

bool Is6bitLabelCorrect (const char * _str) {
	for (const char * p = _str; * p; ++ p)
		if (::strchr (" abcdefghijklmnopqrstuvwxyz0123456789@!\"#$%&'()*+,-./:;<=>?[\\]^_`", ::tolower (* p)) == NULL) {
			if (
				p [0] == '~' &&
				p [1] == '[' &&
				p [2] == '0' &&
				(p [3] == 'x' || p [3] == 'X') &&
				::isxdigit (p [4]) &&
				::isxdigit (p [5]) &&
				(
					p [6] == ']' ||
					(
						::isxdigit (p [6]) &&
						::isxdigit (p [7]) &&
						p [8] == ']'
					)
				)
			)
				continue;
			return false;
		}
	return true;
}

bool IsPhoneCorrect (const char * _str) {
	for (const char * p = _str; * p; ++ p)
		if (::strchr ("0123456789 -", * p) == NULL)
			return false;
	return true;
}

void GetHintText (const rgn_t & _rgn, string_t & _strText) {
	const type_desc_t * const pTD = _rgn.GetTypeDesc ();
	assert (pTD);
	if (! pTD)
		return;

	const char * strTypeName = pTD->strTypeName;
	if (pTD->bUnknown) {
		if (_rgn.IsWaypoint ())
			strTypeName = "Waypoint";
		else if (_rgn.IsTrack ())
			strTypeName = "Track";
		else if (_rgn.IsRoute ())
			strTypeName = "Route";
	}

	if (_rgn.IsBookmark ()) {
		_strText.append ("Bookmark: ");
		string_t strType;
		GetBookmarkDisplayType (* _rgn.pExtra->pBookmark, strType, _strText);
		return;
	}
	if (_rgn.IsNote ()) {
		_strText.append ("Note drawing");
		return;
	}

	if (! _rgn.strLabel.empty ()) {
		string_t strLabel = _rgn.strLabel;
		RemoveGarminSpecialCodes (strLabel);
		_strText.append (strLabel);
		_strText.append (" (");
		_strText.append (strTypeName);
		_strText.append (")");
	} else
		_strText.append (strTypeName);
}
