//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Main module.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Globals.h"
# include "Version.h"

# include "Menu.h"
# include "Keyboard.h"
# include "Status.h"
# include "Toolbar.h"
# include "MapView.h"
# include "MiniMap.h"
# include "WebMap.h"

# include "ResMan.h"
# include "Settings.h"
# include "FileAssoc.h"
# include "COMBase.h"
# include "COMRootEvents.h"

# include "GarminImgRegister.h"

# include "Map.h"
# include "Types.h"
# include "Scale.h"
# include "Position.h"
# include "Levels.h"
# include "PrintUnits.h"
# include "DrawNodes.h"
# include "DrawRoadClass.h"
# include "DrawAddress.h"
# include "Tool.h"
# include "Selection.h"

# include "Clipboard.h"
# include "Copy.h"
# include "Paste.h"
# include "TypeFilter.h"

# include "NewMap.h"
# include "Load.h"
# include "Save.h"
# include "Print.h"
# include "Log.h"
# include "RecentFiles.h"

# include "MapProperties.h"
# include "RgnProperties.h"
# include "RasterProperties.h"
# include "ShowRgnOnMap.h"
# include "FindNodeID.h"
# include "FindRoadID.h"

# include "ToolCreate.h"
# include "ToolTransform.h"

# include "UndoRedo.h"
# include "CombinedAction.h"
# include "DeleteObject.h"
# include "DeleteElement.h"
# include "DeleteAttachment.h"
# include "SetEntryPoint.h"
# include "Bookmark.h"

# include "GeneralizeRgn.h"
# include "RemoveHoles.h"
# include "SliceMap.h"
# include "JoinRgnLevels.h"
# include "SplitRgnLevels.h"
# include "RemoveDuplicates.h"
# include "ConvertLabels.h"
# include "VerifyMap.h"
# include "GenerateRGNodes.h"
# include "RemoveRGNodes.h"
# include "CleanUpExternalNodes.h"
# include "JoinRoads.h"
# include "RemoveSelfIntersections.h"
# include "RemoveJitters.h"

# include "GPSPanel.h"
# include "Positioning.h"
# include "Tracking.h"

# include "Cursors.h"
# include "Paths.h"

# include "EditPoint.h"
# include "Options.h"
# include "Favorites.h"

# include "PAItems.h"
# include "CreatePOIByAddress.h"
# include "FindByAddress.h"

# include "BookmarksList.h"
# include "NotesList.h"
# include "EditWTR.h"
# include "Attachments.h"
# include "TurnRestrictions.h"
# include "MapSkins.h"
# include "ViewPlugins.h"

# include "OrderKey.h"
# include "Activation.h"
# include "About.h"

# include "UnitTests.h"
# include "ParseHelpers.h"
# include "System.h"
# include "Str.h"

# define _MEASURE_START_TIME

//////////////////////////////////////////

bool g_bRegistered = false;

char g_strFileName [MAX_PATH + 1];
const CFileTypeInfo * g_pSaverInfo;
const CFileTypeInfo * g_pLoaderInfo;
bool g_bHaveWaypointsOrTracks;
bool g_bHaveBookmarks;

bool g_bDialogMode = false;

HINSTANCE g_hInst;
HWND g_hWnd;
DWORD g_dwGuiThreadId;

HCURSOR g_hWaitCursor;
HCURSOR g_hArrowCursor;

bool g_bNT;
bool g_bWindows7;

//////////////////////////////////////////

char g_strDegreeSign [5] = "";
size_t g_cDegreeSignLen = 1;

void InitDegreeSign () {
	// NOTE: degree sign has code 176 dec (> 127), so it may not be correctly displayed
	// in some MBCS languages like Chinese.
	g_cDegreeSignLen = ::WideCharToMultiByte (CP_ACP, 0, L"", -1, g_strDegreeSign, sizeof (g_strDegreeSign)/sizeof (g_strDegreeSign [0]), NULL, NULL);
	if (g_cDegreeSignLen != 0)
		-- g_cDegreeSignLen;
	else {
		::strcpy (g_strDegreeSign, "*");
		g_cDegreeSignLen = 1;
	}
}

//////////////////////////////////////////

bool g_bEnableSnapToGrid   = true;
bool g_bEnableSticking     = true;
bool g_bZoomToFullAfterAdd = true;
bool g_bDrawWTROnTop       = false;
bool g_bMSelByShift        = true;

//////////////////////////////////////////

HFONT g_hFont;
HFONT g_hFixedFont;

HBRUSH g_hbrNull;
HBRUSH g_hbrBlack;
HBRUSH g_hbrDkRed;
HBRUSH g_hbrDkGreen;
HBRUSH g_hbrDkYellow;
HBRUSH g_hbrDkBlue;
HBRUSH g_hbrDkMagenta;
HBRUSH g_hbrDkCyan;
HBRUSH g_hbrGray;
HBRUSH g_hbrLtGray;
HBRUSH g_hbrRed;
HBRUSH g_hbrGreen;
HBRUSH g_hbrYellow;
HBRUSH g_hbrBlue;
HBRUSH g_hbrMagenta;
HBRUSH g_hbrCyan;
HBRUSH g_hbrWhite;
HBRUSH g_hbrWhiteHatched;

HBRUSH g_hbrDkRed_special;
HBRUSH g_hbrDkGreen_special;
HBRUSH g_hbrDkYellow_special;
HBRUSH g_hbrDkBlue_special;
HBRUSH g_hbrDkMagenta_special;
HBRUSH g_hbrDkCyan_special;
HBRUSH g_hbrRed_special;
HBRUSH g_hbrGreen_special;
HBRUSH g_hbrYellow_special;
HBRUSH g_hbrBlue_special;
HBRUSH g_hbrMagenta_special;
HBRUSH g_hbrCyan_special;

HPEN g_hpenNull;
HPEN g_hpenBlackThin;
HPEN g_hpenBlackThick;
HPEN g_hpenGrayThin;
HPEN g_hpenGrayThick;
HPEN g_hpenWhiteThick;
HPEN g_hpenRedThin;
HPEN g_hpenRedDot;
HPEN g_hpenRedThick;
HPEN g_hpenGreenThick;

HBRUSH g_hbrTmp;
HBRUSH g_hbrEmptyMap;

HPEN g_hpenGrid;

static HBRUSH g_hbrActiveColor;

static
void _InitCommonGDI () {
	LOGFONT lf;
	GetFont (lf);
	g_hFont = ::CreateFontIndirect (& lf);

	g_hFixedFont = ::CreateFont (14, 0, 0, 0, FW_DONTCARE, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_RASTER_PRECIS,  CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH   | FF_MODERN,   "Courier New");

	g_hpenNull        = (HPEN) ::GetStockObject (NULL_PEN);
	g_hpenBlackThin   = ::CreatePen (PS_SOLID, 1, c_colBlack);
	g_hpenBlackThick  = ::CreatePen (PS_SOLID, 3, c_colBlack);
	g_hpenGrayThin    = ::CreatePen (PS_SOLID, 1, c_colGray);
	g_hpenGrayThick   = ::CreatePen (PS_SOLID, 3, c_colGray);
	g_hpenWhiteThick  = ::CreatePen (PS_SOLID, 3, c_colWhite);
	g_hpenRedThin     = ::CreatePen (PS_SOLID, 1, c_colRed);
	g_hpenRedDot      = ::CreatePen (PS_DOT,   1, c_colRed);
	g_hpenRedThick    = ::CreatePen (PS_SOLID, 3, c_colRed);
	g_hpenGreenThick  = ::CreatePen (PS_SOLID, 3, c_colDkGreen);

	g_hbrNull         = (HBRUSH) ::GetStockObject (NULL_BRUSH);
	g_hbrBlack        = ::CreateSolidBrush (c_colBlack);
	g_hbrDkRed        = ::CreateSolidBrush (c_colDkRed);
	g_hbrDkGreen      = ::CreateSolidBrush (c_colDkGreen);
	g_hbrDkYellow     = ::CreateSolidBrush (c_colDkYellow);
	g_hbrDkBlue       = ::CreateSolidBrush (c_colDkBlue);
	g_hbrDkMagenta    = ::CreateSolidBrush (c_colDkMagenta);
	g_hbrDkCyan       = ::CreateSolidBrush (c_colDkCyan);
	g_hbrGray         = ::CreateSolidBrush (c_colDkGray);
	g_hbrLtGray       = ::CreateSolidBrush (c_colLtGray);
	g_hbrRed          = ::CreateSolidBrush (c_colRed);
	g_hbrGreen        = ::CreateSolidBrush (c_colGreen);
	g_hbrYellow       = ::CreateSolidBrush (c_colYellow);
	g_hbrBlue         = ::CreateSolidBrush (c_colBlue);
	g_hbrMagenta      = ::CreateSolidBrush (c_colMagenta);
	g_hbrCyan         = ::CreateSolidBrush (c_colCyan);
	g_hbrWhite        = ::CreateSolidBrush (c_colWhite);
	g_hbrWhiteHatched = ::CreateHatchBrush (HS_DIAGCROSS, c_colWhite);

	g_hbrDkRed_special     = ::CreateSolidBrush (RGB (149,  47,  12));
	g_hbrDkGreen_special   = ::CreateSolidBrush (RGB ( 57, 126,  67));
	g_hbrDkYellow_special  = ::CreateSolidBrush (RGB (184, 152,   0));//220, 119,  79)) - brown;
	g_hbrDkBlue_special    = ::CreateSolidBrush (RGB (  5,  67, 131));
	g_hbrDkMagenta_special = ::CreateSolidBrush (RGB (164,   7,  88));
	g_hbrDkCyan_special    = ::CreateSolidBrush (RGB (  2, 141, 139));
	g_hbrRed_special       = ::CreateSolidBrush (RGB (247,  89,  81));
	g_hbrGreen_special     = ::CreateSolidBrush (RGB ( 34, 207,  58));
	g_hbrYellow_special    = ::CreateSolidBrush (RGB (249, 186,  62));
	g_hbrBlue_special      = ::CreateSolidBrush (RGB ( 35, 140, 247));
	g_hbrMagenta_special   = ::CreateSolidBrush (RGB (250, 109, 182));
	g_hbrCyan_special      = ::CreateSolidBrush (RGB (150, 224, 208));

	g_hbrEmptyMap     = ::CreateSolidBrush (c_colGray);
	g_hbrTmp          = ::CreateHatchBrush (HS_DIAGCROSS, c_colBlack);

	g_hpenGrid        = ::CreatePen (PS_SOLID, 1, c_colLtGray);

	g_hbrActiveColor  = ::GetSysColorBrush (COLOR_ACTIVECAPTION);
}

static
void _DestroyCommonGDI () {
	::DeleteObject (g_hFont);
	::DeleteObject (g_hFixedFont);

	::DeleteObject (g_hpenBlackThin);
	::DeleteObject (g_hpenBlackThick);
	::DeleteObject (g_hpenGrayThin);
	::DeleteObject (g_hpenGrayThick);
	::DeleteObject (g_hpenWhiteThick);
	::DeleteObject (g_hpenRedThin);
	::DeleteObject (g_hpenRedDot);
	::DeleteObject (g_hpenRedThick);
	::DeleteObject (g_hpenGreenThick);

	::DeleteObject (g_hbrBlack);
	::DeleteObject (g_hbrDkRed);
	::DeleteObject (g_hbrDkGreen);
	::DeleteObject (g_hbrDkYellow);
	::DeleteObject (g_hbrDkBlue);
	::DeleteObject (g_hbrDkMagenta);
	::DeleteObject (g_hbrDkCyan);
	::DeleteObject (g_hbrGray);
	::DeleteObject (g_hbrLtGray);
	::DeleteObject (g_hbrRed);
	::DeleteObject (g_hbrGreen);
	::DeleteObject (g_hbrYellow);
	::DeleteObject (g_hbrBlue);
	::DeleteObject (g_hbrMagenta);
	::DeleteObject (g_hbrCyan);
	::DeleteObject (g_hbrWhite);
	::DeleteObject (g_hbrWhiteHatched);

	::DeleteObject (g_hbrDkRed_special);
	::DeleteObject (g_hbrDkGreen_special);
	::DeleteObject (g_hbrDkYellow_special);
	::DeleteObject (g_hbrDkBlue_special);
	::DeleteObject (g_hbrDkMagenta_special);
	::DeleteObject (g_hbrDkCyan_special);
	::DeleteObject (g_hbrRed_special);
	::DeleteObject (g_hbrGreen_special);
	::DeleteObject (g_hbrYellow_special);
	::DeleteObject (g_hbrBlue_special);
	::DeleteObject (g_hbrMagenta_special);
	::DeleteObject (g_hbrCyan_special);

	::DeleteObject (g_hbrEmptyMap);
	::DeleteObject (g_hbrTmp);

	::DeleteObject (g_hpenGrid);
}

inline BYTE _InvertByte (BYTE _bt) {return _bt < 128 ? 255 : 0;}

COLORREF GetContrastColor (COLORREF _col) {
	return RGB (
		_InvertByte (GetRValue (_col)),
		_InvertByte (GetGValue (_col)),
		_InvertByte (GetBValue (_col))
	);
}

/////////////////////////////////////////////////////

static
void _EnforceSetupUnderMSVista (bool _bInstall) {
	char strPath [MAX_PATH];
	::GetModuleFileName (NULL, strPath, sizeof (strPath));

	SHELLEXECUTEINFO shex;
	::memset (& shex, 0, sizeof (shex));
	shex.cbSize       = sizeof (SHELLEXECUTEINFO);
	shex.fMask        = SEE_MASK_NOCLOSEPROCESS;
	shex.hwnd         = NULL;
	shex.lpVerb       = "runas"; // NOTE: this elevates privileges to write HKLM registry.
	shex.lpFile       = strPath;
	shex.lpParameters = _bInstall ? "/install" : "/uninstall";
	shex.lpDirectory  = NULL;
	shex.nShow        = SW_NORMAL;
	if (! ::ShellExecuteEx (& shex))
		return;

	if (shex.hProcess != NULL) {
		// Wait for setup completion.
		::WaitForSingleObject (shex.hProcess, INFINITE);
		::CloseHandle (shex.hProcess);
	} else
		::Sleep (1000);
}

static
bool _IsRunningFirstTimeFromThisFolder () {
	//
	// Check if this is the first run from this folder and for that version.
	//
	char strPath [MAX_PATH + 1];
	::GetModuleFileName (NULL, strPath, MAX_PATH);
	char strPath2 [MAX_PATH + 1];
	GetSetting ("Path", strPath2, MAX_PATH, "");
	const bool bEqualPath = ::stricmp (strPath, strPath2) == 0;

	char strVer [32];
	GetSetting ("Version", strVer, sizeof (strVer), "");
	const bool bEqualVersion = ::stricmp (strVer, GME_VER_S) == 0;

	if (bEqualVersion && bEqualPath)
		return false;

	SaveSetting ("Version", GME_VER_S);
	SaveSetting ("Path", strPath);

	return true;
}

///////////////////////////////////////////////////////////////////////

class CCmdLine {
  public:
	CCmdLine () :
		m_bRunHidden   (false),
		m_bRegisterIMG (false),
		m_bInstall     (false),
		m_bUninstall   (false),
		m_bCOMServer   (false)
	{}

	void Parse (const char * _strCmdLine);
	bool Execute ();

	bool RunHidden () const {return m_bRunHidden;}
	bool COMServer () const {return m_bCOMServer;}

  private:
	file_names_t m_file_names;
	bool m_bRunHidden;
	bool m_bRegisterIMG;
	bool m_bInstall;
	bool m_bUninstall;
	bool m_bCOMServer;
};

void CCmdLine::Parse (const char * _strCmdLine) {
	if (_strCmdLine == NULL || _strCmdLine [0] == '\0')
		return;

	const size_t cCmdLineLen = ::strlen (_strCmdLine);

	const char * p = _strCmdLine;
	while (* p) {
		if (* p == ' ') {
			++ p;
			continue;
		}

		if (HasPrefix (p, p + cCmdLineLen, "-Embedding", 10)) {
			p += 10;
			// NOTE: this is COM activation for the process.
			m_bRunHidden = true;
			m_bCOMServer = true;
			continue;
		}

		if (HasPrefix (p, p + cCmdLineLen, "/registerimg", 12)) {
			p += 12;
			m_bRegisterIMG = true;
			continue;
		}

		if (HasPrefix (p, p + cCmdLineLen, "/install", 8)) {
			p += 8;
			m_bInstall   = true;
			m_bRunHidden = true;
			continue;
		}

		if (HasPrefix (p, p + cCmdLineLen, "/uninstall", 10)) {
			p += 10;
			m_bUninstall = true;
			m_bRunHidden = true;
			continue;
		}

		const char * pEnd = lstrchr (p + 1, * p == '\"' ? '\"' : ' ');
		if (pEnd == NULL) {
			string_t strFileName (* p == '\"' ? p + 1 : p);
			m_file_names.push_back (strFileName);
			break;
		} else {
			string_t strFileName (* p == '\"' ? p + 1 : p, pEnd);
			m_file_names.push_back (strFileName);
			p = pEnd + 1;
		}
	}

	if (! m_file_names.empty ())
		m_bRunHidden = false;
}

bool CCmdLine::Execute () {
	if (m_bRegisterIMG) {
		if (! m_file_names.empty ())
			RegisterIMG (* m_file_names.begin ());
		return true;
	}

	if (m_bInstall) {
		// NOTE: installation is already done in the invoking code, so exit immediately.
		return true;
	}

	if (m_bUninstall) {
		// NOTE: under MS Vista, even administrators have no direct write access to HKLM registry.
		if (IsRunningUnderMSVistaWithStdPrivileges ())
			_EnforceSetupUnderMSVista (false);
		else {
			SaveSetting ("Path", "");
			RemoveFileAssociations ();
			UnregisterCOMClasses ();
		}
		return true;
	}

	LoadMap (m_file_names, 0, true, false, g_map);
	return false;
}

/////////////////////////////////////////////////////

static
int _cdecl _NewHandler (size_t _cSize) {
	MEMORYSTATUS ms = {sizeof (ms)};
	::GlobalMemoryStatus (& ms);

	throw COutOfMemory (_cSize, __min (ms.dwAvailVirtual, ms.dwAvailPhys), ms.dwTotalPageFile < ms.dwTotalPhys);
}

void COutOfMemory::Report () const {
	const float fRequestedMB = RequestedSize ()/(1024.f*1024);
	const float fFreeMB      = AvailableSize ()/(1024.f*1024);

	ErrorMsgBox (
		"Not enough free memory:\n"
		" %0.3f MB are requested.\n"
		" %0.3f MB are free (but may be highly fragmented).\n"
		"%s"
		"\n"
		"The application will close.",
		fRequestedMB, fFreeMB,
		WarnSmallPageFile () ?
			"\n"
			"NOTE: OS setting for paging file size is smaller than RAM size.\n"
			"This may greatly limit OS in memory allocation.\n"
		  :
			""
	);
}

void COutOfMemory::ReportToMessageLog () const {
	const float fRequestedMB = RequestedSize ()/(1024.f*1024);
	const float fFreeMB      = AvailableSize ()/(1024.f*1024);

	ReportWarning ("Not enough free memory.");
	ReportText (" %0.3f MB requested, %0.3f MB free (but may be highly fragmented).", fRequestedMB, fFreeMB);
	if (WarnSmallPageFile ()) {
		ReportText ("NOTE: current OS setting for paging file size is smaller than RAM size.");
		ReportText ("This may greatly limit OS in memory allocation.");
	}
}

/////////////////////////////////////////////////////

	// NOTE: ComCtl32.dll is loaded to 0x5Dxx0000 causing virtial address space fragmenting.
	// Remove 'static' dependance for ComCtl32.dll to avoid its automatic loading.
# pragma comment (linker, "/delayload:comctl32.dll")
# pragma comment (linker, "/delayload:comdlg32.dll")
# pragma comment (linker, "/delayload:shell32.dll")
# pragma comment (linker, "/delayload:wininet.dll")

/////////////////////////////////////////////////////

HANDLE g_hExitEvent;
static
void _CloseExitEvent () {
	if (g_hExitEvent)
		::CloseHandle (g_hExitEvent);
	g_hExitEvent = NULL;
}

LRESULT CALLBACK _WndProc (HWND, UINT, WPARAM, LPARAM);

int APIENTRY WinMain (
	HINSTANCE _hInstance,
    HINSTANCE _hPrevInstance,
    LPSTR     _strCmdLine,
    int       _nCmdShow
) {
# ifdef _DEBUG
//	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_CHECK_ALWAYS_DF | _CRTDBG_CHECK_CRT_DF | _CRTDBG_LEAK_CHECK_DF);

	//
	// Run unit tests.
	//

	UnitTests ();
# endif // _DEBUG

# ifdef _MEASURE_START_TIME
	const DWORD dwStartTime = ::GetTickCount ();
# endif // _MEASURE_START_TIME

	//
	// Rebase some DLLs to avoid virtual address space fragmentation.
	// NOTE: do it before any heap allocations!
	//
	RebaseDLL ("ComCtl32.dll");
	RebaseDLL ("Lpk.dll");
	RebaseDLL ("UxTheme.dll");
	RebaseDLL ("GdiPlus.dll");
	const DWORD dwStartTime_RebaseTime = ::GetTickCount ();

	g_hInst = _hInstance;
	g_dwGuiThreadId = ::GetCurrentThreadId ();

	g_hExitEvent = ::CreateEvent (NULL, TRUE, FALSE, NULL);
	INVOKE_AT_EXIT (_CloseExitEvent);

	//
	// Determine OS facilities.
	//

	OSVERSIONINFO oi = {sizeof (OSVERSIONINFO)};
	::GetVersionEx (& oi);
	g_bNT = oi.dwPlatformId == VER_PLATFORM_WIN32_NT;
	g_bWindows7 = g_bNT && ((oi.dwMajorVersion << 8) + oi.dwMinorVersion) >= 0x601;

	// NOTE: shell version 5.80 is assumed.
	const HMODULE hShell = ::LoadLibrary ("Shell32.dll");
	typedef HRESULT (CALLBACK * pfDllGetVersion_t) (DLLVERSIONINFO * _pdvi);
	const pfDllGetVersion_t pfDllGetVersion = (pfDllGetVersion_t) ::GetProcAddress (hShell, "DllGetVersion");
	DLLVERSIONINFO dvi;
	::memset (& dvi, 0, sizeof (dvi));
	if (pfDllGetVersion) {
		dvi.cbSize = sizeof (DLLVERSIONINFO);
		pfDllGetVersion (& dvi);
	}
	if (! pfDllGetVersion || dvi.dwMajorVersion < 5)
		ErrorMsgBox ("MS Internet Explorer 5.80 or higher is required.");

	//
	// Set up common out-of-memory handler.
	//

	_set_new_handler (_NewHandler);
	_set_new_mode (1);

	try {
		//
		// Initialize registry access.
		//

		InitRegistryAccess ();

		g_bEnableSnapToGrid   = GetSetting ("OptionSnapToGrid",   true);
		g_bEnableSticking     = GetSetting ("OptionStick",        true);
		g_bZoomToFullAfterAdd = GetSetting ("ZoomToFullAfterAdd", true);
		g_bDrawWTROnTop       = GetSetting ("DrawWTROnTop",       false);
		g_bMSelByShift        = GetSetting ("MSelByShift",        false);

		InitDegreeSign ();
		InitPrintUnits ();

		//
		// Initialize localization.
		//

		InitResMan ();

		const bool bFirstRun = _IsRunningFirstTimeFromThisFolder ();
		if (bFirstRun || ! AreCOMClassesRegistered ()) {
			// NOTE: under MS Vista, even administrators have no direct write access to HKLM registry.
			if (IsRunningUnderMSVistaWithStdPrivileges ())
				_EnforceSetupUnderMSVista (true);
		}

		//
		// Define supported file type associations.
		//

		CFileTypeInfo::Init ();

		UpdateFileAssociations (bFirstRun);

		//
		// Register COM classes.
		//

		CCom com;
		RegisterCOMClasses (bFirstRun);

		//
		// Initialize the clipboard access.
		//

		InitClipboard ();
		
		//
		// Initialize shared GDI objects.
		//

		_InitCommonGDI ();
		INVOKE_AT_EXIT (_DestroyCommonGDI);

		InitSelection ();
		INVOKE_AT_EXIT (DestroySelection);

		InitDrawNodes ();
		INVOKE_AT_EXIT (DestroyDrawNodes);

		InitRoadClasses ();
		INVOKE_AT_EXIT (DestroyRoadClasses);

		InitDrawAddress ();
		INVOKE_AT_EXIT (DestroyDrawAddress);


# ifdef _MEASURE_START_TIME
		const DWORD dwStartTime_InitGDI = ::GetTickCount ();
# endif // _MEASURE_START_TIME

		//
		// Initialize type sets.
		//

		InitTypes ();
		INVOKE_AT_EXIT (DestroyTypes);

		InitMapSkins ();
		INVOKE_AT_EXIT (DestroyMapSkins);

# ifdef _MEASURE_START_TIME
		const DWORD dwStartTime_InitTypes = ::GetTickCount ();
# endif // _MEASURE_START_TIME

		//
		// Initialize cursors.
		//

		g_hArrowCursor = ::LoadCursor (NULL, IDC_ARROW);
		g_hWaitCursor  = ::LoadCursor (NULL, IDC_WAIT);

		INVOKE_AT_EXIT (DestroyCursors);
		INVOKE_AT_EXIT (DestroyPaths);

		//
		// Create the main window.
		//

		WNDCLASSEX wcex;
		::memset (& wcex, 0, sizeof (wcex));
		wcex.cbSize        = sizeof (WNDCLASSEX);
		wcex.style         = CS_HREDRAW | CS_VREDRAW;
		wcex.lpfnWndProc   = (WNDPROC) _WndProc;
		wcex.hInstance     = g_hResInst;
		wcex.hIcon         = ::LoadIcon (g_hInst, (LPCTSTR) IDI_MAIN);
		wcex.hCursor       = g_hArrowCursor;
		wcex.lpszMenuName  = (LPCSTR) IDM_MAIN;
		wcex.lpszClassName = "mapedit";
		::RegisterClassEx (& wcex);

		g_hWnd = ::CreateWindow (
			"mapedit", MAIN_TITLE,
			WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
			CW_USEDEFAULT, 0, GetSetting ("MainWidth", DWORD (800)), GetSetting ("MainHeight", DWORD (600)),
			NULL, NULL, g_hResInst, NULL
		);
		if (! g_hWnd)
			return TRUE;

		//
		// Initialize main menu.
		//

		InitMenu ();

		//
		// Create the status window & toolbar.
		//

		INITCOMMONCONTROLSEX icc;
		icc.dwSize = sizeof (icc);
		icc.dwICC  = ICC_WIN95_CLASSES | ICC_USEREX_CLASSES;
		::InitCommonControlsEx (& icc);

		if (! InitStatusWindow ())
			return TRUE;
		INVOKE_AT_EXIT (DestroyStatusWindow);

		if (! InitToolbar ())
			return TRUE;
		INVOKE_AT_EXIT (DestroyToolbar);

		//
		// Create the map window.
		//

		if (! InitMapWindow ())
			return TRUE;
		INVOKE_AT_EXIT (DestroyMapWindow);

# ifdef _MEASURE_START_TIME
		const DWORD dwStartTime_InitMapWindow = ::GetTickCount ();
# endif // _MEASURE_START_TIME

		//
		// Prepare data.
		//

		CTool::_Init ();
		CDestroyer mmdtr (CTool::_Destroy);

		InitWebMap ();
		INVOKE_AT_EXIT (DestroyWebMap);

		RemoveMap ();

		//
		// Parse the command line.
		//

		CCmdLine cmd_line;
		cmd_line.Parse (_strCmdLine);

		//
		// Activate COM interface.
		//

		CRegisterClassFactories rcf (cmd_line.COMServer ());

		//
		// Check license activation.
		//
		InitActivation ();

		//
		// Initialize Plug-in's
		//

		InitViewPlugins ();
		INVOKE_AT_EXIT (DestroyViewPlugins);

# ifdef _MEASURE_START_TIME
		const DWORD dwStartTime_X = ::GetTickCount ();
# endif // _MEASURE_START_TIME

		//
		// Show the main window.
		//

		if (cmd_line.RunHidden ()) {
			::ShowWindow (g_hWnd, SW_HIDE);
		} else {
			::ShowWindow (g_hWnd, GetSetting ("MainMaximized", false) ? SW_SHOWMAXIMIZED : _nCmdShow);
//			::UpdateWindow (g_hWnd);
			::SetFocus (g_hWnd);
		}

		::DragAcceptFiles (g_hWnd, TRUE);

		//
		// Execute the command line.
		//

		if (_strCmdLine && _strCmdLine [0]) {
			const bool bExit = cmd_line.Execute ();
			if (bExit)
				return 0;
		}

		InitKeyboard ();
		INVOKE_AT_EXIT (CloseKeyboard);
		UpdateMenuAccelerators (g_MainMenu);

# ifdef _MEASURE_START_TIME
		if (GetSetting ("MeasureStartTime", false)) {
			const DWORD dwStartTime_End = ::GetTickCount ();
			ReportText ("* Rebase DLLs:   %d ms", dwStartTime_RebaseTime    - dwStartTime);
			ReportText ("* InitGDI:       %d ms", dwStartTime_InitGDI       - dwStartTime);
			ReportText ("* InitTypes:     %d ms", dwStartTime_InitTypes     - dwStartTime);
			ReportText ("* InitMapWindow: %d ms", dwStartTime_InitMapWindow - dwStartTime);
			ReportText ("* X:             %d ms", dwStartTime_X             - dwStartTime);
			ReportText ("* Start time:    %d ms", dwStartTime_End           - dwStartTime);
			UpdateMenu ();
		}
# endif // _MEASURE_START_TIME

		//
		// Main message loop.
		//

		MSG msg;
		while (::GetMessage (& msg, NULL, 0, 0)) {
			if (g_bDialogMode) {
				// NOTE: this is support for in-place label editor.
				if (
					(msg.message == WM_KEYDOWN || msg.message == WM_KEYUP) &&
					(msg.wParam == VK_RETURN || msg.wParam == VK_ESCAPE)
				) {
					::TranslateMessage (& msg);
					::DispatchMessage (& msg);
					continue;
				}
				if (::IsDialogMessage (g_hWnd, & msg))
					continue;
			}

			if (CDialog::IsDialogMessage (& msg))
				continue;

			if (TranslateAccelerator (msg))
				continue;

			::TranslateMessage (& msg);
			::DispatchMessage (& msg);
		}

		SaveRecentMapPosition ();

		StopGPSUnitCommunication ();

		return msg.wParam;
	} catch (const COutOfMemory & _oom) {
		_oom.Report ();

		return -1;
	} catch (const std::exception & _e) {
		ErrorMsgBox ("%s\nThe application will close.", _e.what ());

		return -1;
	}
}

void UpdateTitle () {
	const char * const strMainTitle = g_bRegistered ? MAIN_TITLE : MAIN_TITLE " [not registered]";

	if (g_strFileName [0] == '\0')
		::SetWindowText (g_hWnd, strMainTitle);
	else {
		char strFileTitle [MAX_PATH + 1];
		::GetFileTitle (g_strFileName, strFileTitle, MAX_PATH);

		char strTitle [MAX_PATH + sizeof (MAIN_TITLE) + 50];
		::sprintf (strTitle, "%s - %s", strFileTitle, strMainTitle);

		::SetWindowText (g_hWnd, strTitle);
	}
}

///////////////////////////

static
BOOL CALLBACK EnumChildProc (HWND _hChildWnd, LPARAM _lParam) {
	std::vector<HWND> & vecChildren = * reinterpret_cast<std::vector<HWND> *> (_lParam);
	vecChildren.push_back (_hChildWnd);
	return TRUE;
}

static
void EnumChildWindows (HWND _hParentWnd, std::vector<HWND> & _vecChildren) {
	::EnumChildWindows (_hParentWnd, EnumChildProc, reinterpret_cast<LPARAM> (& _vecChildren));
}

///////////////////////////

static
LRESULT CALLBACK _WndProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_SIZING:
			return TRUE;

		case WM_SIZE: {
			// Save size and maximization status.
			if (_wParam == SIZE_RESTORED) {
				RECT rect;
				::GetWindowRect (_hWnd, & rect);
				SaveSetting ("MainWidth", DWORD (rect.right  - rect.left));
				SaveSetting ("MainHeight", DWORD (rect.bottom - rect.top));
				SaveSetting ("MainMaximized", false);
			} else if (_wParam == SIZE_MAXIMIZED)
				SaveSetting ("MainMaximized", true);

			// Calculate client area size.
			RECT rectClient;
			::GetClientRect (_hWnd, & rectClient);
			const long lMainW = rectClient.right - rectClient.left;
			const long lMainH = rectClient.bottom - rectClient.top;

			RECT rectStatus;
			::GetWindowRect (g_hStatusWnd, & rectStatus);
			const long lStatusH = rectStatus.bottom - rectStatus.top;

			RECT rectToolbar;
			// NOTE: GetWindowRect() returns incorrect size for toolbar.
			::GetClientRect (g_hToolbarWnd, & rectToolbar);
			const long lToolbarH = rectToolbar.bottom - rectToolbar.top - 1;

			// Set the map window size.
			long lW = lMainW;
			if (lW < 100)
				lW = 100;
			long lH = lMainH - lStatusH - lToolbarH;
			if (lH < 100)
				lH = 100;
			::SetWindowPos (g_hMapWnd, NULL, 0, 0, lW, lH, SWP_NOZORDER | SWP_NOMOVE | SWP_DEFERERASE);

			std::vector<HWND> vecChildren;
			EnumChildWindows (g_hMapWnd, vecChildren);

			// Set map view's child windows position to ensure them being visible.
			for (size_t cWnd = 0; cWnd < vecChildren.size (); ++ cWnd) {
				const HWND hChild = vecChildren [cWnd];
				const HWND hParent = ::GetParent (hChild);
				if (hParent != g_hMapWnd)
					continue;

				RECT rectChild;
				::GetWindowRect (hChild, & rectChild);
				POINT p0 = {rectChild.left, rectChild.top};
				::ScreenToClient (g_hMapWnd, & p0);
				POINT p1 = {rectChild.right, rectChild.bottom};
				::ScreenToClient (g_hMapWnd, & p1);
				RECT rectMap;
				::GetClientRect (g_hMapWnd, & rectMap);
				if (p1.x > rectMap.right) {
					p0.x += rectMap.right - p1.x;
					if (p0.x < 0)
						p0.x = 0;
				}
				if (p1.y > rectMap.bottom) {
					p0.y += rectMap.bottom - p1.y;
					if (p0.y < 0)
						p0.y = 0;
				}
				::MoveWindow (
					hChild,
					p0.x, p0.y,	rectChild.right - rectChild.left, rectChild.bottom - rectChild.top,
					TRUE
				);
			}

			// NOTE: toolbar & status bar set size & position automaticaly.
			::SetWindowPos (g_hStatusWnd,  NULL, 0, 0, 0, 0, SWP_NOZORDER | SWP_DEFERERASE);
			::SetWindowPos (g_hToolbarWnd, NULL, 0, 0, 0, 0, SWP_NOZORDER | SWP_DEFERERASE);

			break;
		}

		case WM_NOTIFY: {
			const NMHDR * const pHdr = reinterpret_cast<const NMHDR *> (_lParam);
			switch (pHdr->code) { 
				case TTN_GETDISPINFO: {
					TOOLTIPTEXT * const pttt = reinterpret_cast <TOOLTIPTEXT *> (_lParam);
					pttt->hinst = NULL;
					if (pHdr->hwndFrom == g_hToolbarTipWnd)
						//
						// Provide tooltip text for toolbar buttons.
						//
						::GetMenuString (::GetMenu (g_hWnd), pttt->hdr.idFrom, pttt->szText, sizeof (pttt->szText), MF_BYCOMMAND);

					break;
				}

				case NM_CLICK:
					if (pHdr->hwndFrom == g_hStatusWnd) {
						const NMMOUSE * const pMs = reinterpret_cast<const NMMOUSE *> (_lParam);
						OnStatusBarClick (pMs->dwItemSpec);
					}
					break;

				case NM_RCLICK:
					if (pHdr->hwndFrom == g_hToolbarWnd) {
						ToolbarCtxMenu ();
						break;
					}

					// Return keyboard input from the label edit box.
					if (g_bDialogMode)
						::SetFocus (g_hWnd);
					break;

				case TBN_DROPDOWN: {
					//
					// Provide dropdown lists for Undo/Redo.
					//
					NMTOOLBAR * const pNMT = reinterpret_cast<NMTOOLBAR *> (_lParam);
					if (pHdr->hwndFrom == g_hToolbarWnd) {
						CPopupMenu hPopupMenu;
						if (pNMT->iItem == ID_EDIT_UNDO) {
							size_t c = 0;
							for (actions_t::const_iterator i = g_Undo.end (); i != g_Undo.begin (); -- i, ++ c) {
								actions_t::const_iterator ic = i;
								-- ic;
								string_t strName;
								(* ic)->GetName (strName);

								hPopupMenu.AppendItem (ID_EDIT_UNDO_0 + c, strName.c_str ());

								if (c > 20)
									break;
							}
						} else if (pNMT->iItem == ID_EDIT_REDO) {
							size_t c = 0;
							for (actions_t::const_iterator i = g_Redo.end (); i != g_Redo.begin (); -- i, ++ c) {
								actions_t::const_iterator ic = i;
								-- ic;
								string_t strName;
								(* ic)->GetName (strName);

								hPopupMenu.AppendItem (ID_EDIT_REDO_0 + c, strName.c_str ());

								if (c > 20)
									break;
							}
						} else if (pNMT->iItem == ID_TOOL_CREATE_RGN) {
							InitToolsCreateObjectMenu  (hPopupMenu, true);
							CheckToolsCreateObjectMenu (hPopupMenu);
						} else if (pNMT->iItem == ID_TOOL_TRANSFORM) {
							InitToolsTransformMenu  (hPopupMenu, true);
							CheckToolsTransformMenu (hPopupMenu);
						}

						RECT rect;
						::SendMessage (g_hToolbarWnd, TB_GETRECT, (WPARAM) pNMT->iItem, (LPARAM) & rect);
						::MapWindowPoints (g_hToolbarWnd, HWND_DESKTOP, (LPPOINT) & rect, 2);

						hPopupMenu.ShowAsPopup (_hWnd, rect, false);

						return TBDDRET_DEFAULT;
					}
				}
			}
			break;
		}

		case WM_ACTIVATE:
			if (_wParam == WA_ACTIVE || _wParam == WA_CLICKACTIVE)
				UpdateMenu ();
			break;

		case WM_COMMAND: {
			const WORD wCmd = LOWORD (_wParam);

			// First, get chance for current mouse mode to process popup menu commands.
			if (! g_bLoading && wCmd != ID_REPEAT_LAST_CTX_COMMAND) {
				const bool bFromMenu = HIWORD (_wParam) == 0;
				if (CTool::_OnCommand (wCmd, bFromMenu))
					break;
			}

			switch (wCmd) {
				//
				// File menu.
				//

				case ID_FILE_EXIT:      ::SendMessage (_hWnd, WM_CLOSE, 0, 0); break;
				case ID_FILE_CLOSE:     if (! g_bLoading) RemoveMap (); break;

				case ID_FILE_ADD:
				case ID_FILE_OPEN:      if (! g_bLoading) CFileTypeInfo::ShowOpenFile (wCmd == ID_FILE_OPEN); break;

				case ID_FILE_SAVE:      if (! g_bLoading) SaveMap (); break;
				case ID_FILE_SAVE_AS:   if (! g_bLoading) SaveMapAs (); break;

				case ID_FILE_PAGE_SETUP:if (! g_bLoading) PrintSetup (_hWnd); break;
				case ID_FILE_PRINT:     if (! g_bLoading) PrintMap (_hWnd); break;

				case ID_MAP_PROPERTIES:	if (! g_bLoading) ShowMapProperties (_hWnd); break;
				case ID_RGN_PROPERTIES: if (! g_bLoading && g_Selection.size () == 1) ShowRgnProperties (_hWnd, g_Selection [0]); break;
				case ID_RASTER_PROPERTIES:
				                        if (! g_bLoading && g_Selection.size () == 1) ShowRasterProperties (_hWnd, g_Selection [0]); break;

				case ID_ERROR_LOG:      if (! g_bLoading) ShowMessageLog (); break;

				//
				// 'Edit' menu.
				//

				case ID_EDIT_UNDO:  ::SendMessage (_hWnd, WM_COMMAND, ID_EDIT_UNDO_0, 0); break;
				case ID_EDIT_REDO:  ::SendMessage (_hWnd, WM_COMMAND, ID_EDIT_REDO_0, 0); break;

				case ID_EDIT_PASTE: if (! g_bLoading) EditPaste (); break;
				case ID_EDIT_COPY:  if (! g_bLoading) EditCopy ();  break;

				case ID_EDIT_CUT:   
				case ID_EDIT_DELETE:
					if (g_bLoading)
						break;

					if (wCmd == ID_EDIT_CUT)
						EditCopy ();

					if (! g_Selection.empty ()) {
						const int iDepth = BeginCombinedAction ();

						const size_t cSelItems = g_Selection.size ();
						for (size_t cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
							const find_info_t & SelItem = g_Selection [cSelItem];

							if (SelItem.pLayer) {
								if (SelItem.iRgn->IsDeleted ())
									continue;

								if (SelItem.iRgn->IsWTR ())
									DeleteAttachment (SelItem.iRgn->pFileRef, false);
								else if (SelItem.cEntryPoint != -1)
									RemoveEntryPoint (& * SelItem.iRgn, SelItem.cEntryPoint);
								else if (SelItem.cElementOrOrigin != -1) {
									if (SelItem.iRgn->origins.empty ()) {
										assert (SelItem.cElementOrOrigin < SelItem.iRgn->elements.size ());
										if (SelItem.iRgn->elements [SelItem.cElementOrOrigin].bDeleted)
											continue;
									} else {
										assert (SelItem.cElementOrOrigin < SelItem.iRgn->origins.size ());
										if (SelItem.iRgn->origins [SelItem.cElementOrOrigin].bDeleted)
											continue;
									}
									DeleteElement (& * SelItem.iRgn, SelItem.cElementOrOrigin);
								} else
									DeleteObject (& * SelItem.iRgn);
							} else if (SelItem.pRenderer) {
								if (SelItem.pRenderer->bDeleted)
									continue;
								DeleteAttachment (SelItem.pRenderer, true);
							}
						}

						EndCombinedAction (iDepth, GetStr (IDS_DELETE_MULTIPLE));

						g_Selection.clear ();
						ClearSelPoints ();

						Refresh ();
						UpdateMenu ();
					}
					break;

				case ID_SELECT_ALL_OBJECTS:     if (! g_bLoading) SelectAllObjects (true,  false, false, false, false, false, false); break;
				case ID_SELECT_ALL_BOOKMARKS:   if (! g_bLoading) SelectAllObjects (false, true,  false, false, false, false, false); break;
				case ID_SELECT_ALL_NOTES:       if (! g_bLoading) SelectAllObjects (false, false, true,  false, false, false, false); break;
				case ID_SELECT_ALL_TRACKS:      if (! g_bLoading) SelectAllObjects (false, false, false, true,  false, false, false); break;
				case ID_SELECT_ALL_WPTS:        if (! g_bLoading) SelectAllObjects (false, false, false, false, true,  false, false); break;
				case ID_SELECT_ALL_ROUTES:      if (! g_bLoading) SelectAllObjects (false, false, false, false, false, true,  false); break;
				case ID_SELECT_ALL_RASTERS:     if (! g_bLoading) SelectAllObjects (false, false, false, false, false, false, true); break;
				case ID_SELECT_ALL_ATTACHMENTS: if (! g_bLoading) SelectAllObjects (false, false, false, true,  true,  true,  true); break;
				case ID_SELECT_ALL_POINTS:      if (! g_bLoading) SelectByKind (rkPoint);    break;
				case ID_SELECT_ALL_POLYLINES:   if (! g_bLoading) SelectByKind (rkPolyline); break;
				case ID_SELECT_ALL_POLYGONS:    if (! g_bLoading) SelectByKind (rkPolygon);  break;
				case ID_SELECT_ALL_ROADS:       if (! g_bLoading) SelectAllRoads ();  break;
				case ID_CUSTOM_FILTER:          if (! g_bLoading) SelectByType (_hWnd); break;
				case ID_UNSELECT:               if (! g_bLoading) UnselectObjects (); break;
				case ID_INVERT_SELECTION:       if (! g_bLoading) InvertSelection (); break;

				case ID_FIND_NODEID:            if (! g_bLoading) FindNodeID (); break;
				case ID_FIND_ROADID:            if (! g_bLoading) FindRoadID (); break;

				case ID_CANCEL_ACTION:
					if (CTool::_OnEsc ())
						break;

					if (g_bLoading)
						StopLoadThread ();
					else
						UnselectObjects ();
					break;

				case ID_BACK_ACTION:   CTool::_OnBack  (); break;
				case ID_ENTER_ACTION:  CTool::_OnEnter (); break;
				case ID_INSERT_ACTION: CTool::_OnIns   (); break;
				case ID_DELETE_ACTION:
					if (CTool::_OnDel ())
						break;
					::SendMessage (_hWnd, WM_COMMAND, ID_EDIT_DELETE, 0);
					break;

				case ID_LEFT:          CTool::_OnArrow (-1, 0);  break;
				case ID_RIGHT:         CTool::_OnArrow (1,  0);  break;
				case ID_UP:            CTool::_OnArrow (0,  -1); break;
				case ID_DOWN:          CTool::_OnArrow (0,  1);  break;

				//
				// View menu.
				//

				case ID_REFRESH:     if (! g_bLoading) Refresh (); UpdateMenu (); break;

				case ID_ZOOM_IN:     if (! g_bLoading) SetScale (g_fScale/c_Sqrt4); break;
				case ID_ZOOM_OUT:    if (! g_bLoading) SetScale (g_fScale*c_Sqrt4); break;

				case ID_ZOOM_FULL: {
					if (g_bLoading)
						break;

					bool bRectBoundInvalid;
					{
						CCritSec::CLock lock (g_cs);
						bRectBoundInvalid = g_map.rectBound.Invalid ();
					}
					if (! bRectBoundInvalid)
						SetFullMapScale ();
					break;
				}

				case ID_ZOOM_TO_SELECTION:
					if (! g_bLoading)
						ShowSelectionOnMap (false);
					break;

				case ID_VIEW_LEVELS_AUTO:
					if (g_bLoading)
						break;

					SetVisibleLevelAuto ();

					Refresh ();
					UpdateMenu ();
					SetScaleStatus (g_fScale);
					break;

				case ID_VIEW_LEVEL0: if (SetVisibleLevel (0)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL1: if (SetVisibleLevel (1)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL2: if (SetVisibleLevel (2)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL3: if (SetVisibleLevel (3)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL4: if (SetVisibleLevel (4)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL5: if (SetVisibleLevel (5)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL6: if (SetVisibleLevel (6)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL7: if (SetVisibleLevel (7)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL8: if (SetVisibleLevel (8)) {Refresh (); UpdateMenu ();} break;
				case ID_VIEW_LEVEL9: if (SetVisibleLevel (9)) {Refresh (); UpdateMenu ();} break;

				case ID_MANAGE_MAP_SKINS: MapSkins_EditList (_hWnd); break;

				case ID_SHOW_MINI:
					if (::IsWindowVisible (g_hMiniMapWnd)) {
						::ShowWindow (g_hMiniMapWnd, SW_HIDE);
						SaveSetting ("ShowMiniMap", false);
					} else {
						::ShowWindow (g_hMiniMapWnd, SW_SHOW);
						SaveSetting ("ShowMiniMap", true);
					}
					UpdateMenu ();
					break;

				case ID_SHOW_SS:
					if (::IsWindowVisible (g_hScaleSelectorWnd)) {
						::ShowWindow (g_hScaleSelectorWnd, SW_HIDE);
						SaveSetting ("ShowScaleSelector", false);
					} else {
						::ShowWindow (g_hScaleSelectorWnd, SW_SHOW);
						SaveSetting ("ShowScaleSelector", true);
					}
					UpdateMenu ();
					break;

				case ID_SHOW_RULER:
					g_bShowRuler = ! g_bShowRuler;
					SaveSetting ("ShowRuler", g_bShowRuler);
					UpdateMenu ();
					Redraw ();
					break;

				case ID_SHOW_GRID:
					g_bShowGridLines = ! g_bShowGridLines;
					SaveSetting ("ShowGrid", g_bShowGridLines);
					UpdateMenu ();
					Redraw ();
					break;

				case ID_SHOW_LABELS:
					g_bShowLabels = ! g_bShowLabels;
					SaveSetting ("ShowLabels", g_bShowLabels);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_POLYGON_CONTOURS:
					g_bShowPolygonContours = ! g_bShowPolygonContours;
					SaveSetting ("ShowPolygonContours", g_bShowPolygonContours);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_TRANSPARENT_POLYGONS:
					g_bTransparentPolygons = ! g_bTransparentPolygons;
					SaveSetting ("TransparentPolygons", g_bTransparentPolygons);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_MAP:
					g_bShowMap = ! g_bShowMap;
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_COVERAGE:
					g_bShowCoverageArea = ! g_bShowCoverageArea;
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_BOOKMARKS:
					g_bShowBookmarksAndNotes = ! g_bShowBookmarksAndNotes;
					SaveSetting ("ShowBookmarks", g_bShowBookmarksAndNotes);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_ATTACHMENTS:
					g_bShowAttachments = ! g_bShowAttachments;
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_RGNODES:
					g_bShowRGNodes = ! g_bShowRGNodes;
					SaveSetting ("ShowRGNodes", g_bShowRGNodes);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_ROAD_CLASSES:
					g_bShowRoadClasses = ! g_bShowRoadClasses;
					SaveSetting ("ShowRoadClasses", g_bShowRoadClasses);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_HIGHLIGHT_ADDRESSES:
					g_bHighlightAddresses = ! g_bHighlightAddresses;
					SaveSetting ("HighlightAddresses", g_bHighlightAddresses);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_NUMBERS:
					g_bShowNumbers = ! g_bShowNumbers;
					SaveSetting ("ShowNumbers", g_bShowNumbers);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_HLEVEL:
					g_bShowHLevel = ! g_bShowHLevel;
					SaveSetting ("ShowHLevel", g_bShowHLevel);
					UpdateMenu ();
					Refresh ();
					break;

				case ID_SHOW_GOOGLEMAPS:
					g_bShowGoogleMaps = ! g_bShowGoogleMaps;
					ShowWebMap (g_bShowGoogleMaps);
					SaveSetting ("ShowGoogleMaps", g_bShowGoogleMaps);
					UpdateMenu ();
					Refresh ();
					SetScaleStatus (g_fScale);
					break;

				case ID_GPS_TRACKING:
					if (IsGPSUnitActive ()) {
						StopGPSUnitCommunication ();
						HideGPSPanel ();
					} else {
						StartGPSUnitCommunication ();
						ShowGPSPanel ();
					}
					UpdateMenu ();
					Refresh ();
					break;

				case ID_CREATE_BOOKMARK_BY_GPS:
					if (IsGPSUnitActive ()) {
						const PositionInfo_t pi = GetPositionInfo ();
						if ((pi.dwDef & PositionInfo_t::defXY) != 0) {
							g_Selection.clear ();

							const int nDepth = BeginCombinedAction ();

							rgn_t & rgn = CreateBookmark (point_t (pi.fX, pi.fY));
							bookmark_t & bm = * rgn.pExtra->pBookmark;

							// Heading.
							if ((pi.dwDef & PositionInfo_t::defHeading) != 0) {
								bm.bHasHeading = true;
								bm.fHeading    = pi.fHeading;
							}

							// Date/time.
							if ((pi.dwDef & PositionInfo_t::defTime) != 0) {
								bm.dt = pi.dtTime;
							} else {
								SYSTEMTIME st;
								::GetSystemTime (& st);
								::SystemTimeToVariantTime (& st, & bm.dt);
							}

							Refresh ();

							// Show 'Properties' dialog.
							bool bDeleteApplied = false;
							const bool bOK = ShowBookmarkProperties (g_hWnd, rgn, true, bDeleteApplied);

							EndCombinedAction (nDepth, "create bookmark");
							if (! bOK)
								UndoEditAction ();

							Refresh ();
						}
					}
					break;

				case ID_BOOKMARKS:     ShowBookmarks (); break;
				case ID_NOTE_DRAWINGS: ShowNoteDrawings (); break;
				case ID_WTR:           EditWTR (); break;
				case ID_ATTACHMENTS:   ShowAttachments (); break;
				case ID_TURN_RESTRICTIONS: ShowTurnRestrictions (); break;

				case ID_GOTO_POSITION: {
					point_t point;
					ScreenToPt (g_lWidth/2, g_lHeight/2, point);
					if (EditPoint (_hWnd, point)) {
						SetToCenter (point);
						Refresh ();
					}
					break;
				}

				//
				// 'Favorites' menu.
				//

				case ID_ADD_TO_FAVORITES: if (! g_bLoading) AddToFavorites (); break;
				case ID_REMOVE_FAVORITE:  if (! g_bLoading) RemoveFavorite (); break;


				//
				// 'Tools' menu.
				//

				case ID_TOOL_DRAG:        CTool::SetCurrentTool (toolDrag);      break;
				case ID_TOOL_ZOOM:        CTool::SetCurrentTool (toolZoom);      break;
				case ID_TOOL_SELECT:      CTool::SetCurrentTool (toolSelect);    break;
				case ID_TOOL_ROTATE:      CTool::SetCurrentTool (toolRotate);    break;
				case ID_TOOL_EDIT_NODES:  CTool::SetCurrentTool (toolEditNodes); break;
				case ID_TOOL_CREATE_RGN:  CTool::SetCurrentTool (toolCreate);    break;
				case ID_TOOL_TRIM:        CTool::SetCurrentTool (toolTrim);      break;
				case ID_TOOL_TRANSFORM:   CTool::SetCurrentTool (toolTransform); break;
				case ID_TOOL_ROUTING:     CTool::SetCurrentTool (toolRouting);   break;
				case ID_TOOL_DISTANCE:    CTool::SetCurrentTool (toolDistance);  break;

				case ID_REPEAT_LAST_CTX_COMMAND: CTool::RepeatLastCtxMenuCmd (); break;

				case ID_GENERALIZE_POLY:   if (! g_bLoading) GeneralizePoints (); break;
				case ID_GENERALIZE_ADJACENT:if (! g_bLoading) GeneralizeAdjacent (); break;
				case ID_JOIN_RGN_LEVELS:   if (! g_bLoading) JoinRgnLevels (); break;
				case ID_SPLIT_RGN_LEVELS:  if (! g_bLoading) SplitRgnLevels (); break;
				case ID_REMOVE_DUPLICATES: if (! g_bLoading) RemoveDuplicates (); break;
				case ID_CONVERT_LABELS:    if (! g_bLoading) ConvertLabels (g_hWnd); break;
				case ID_REMOVE_HOLES:      if (! g_bLoading) RemoveHoles (); break;
				case ID_REMOVE_SELFINTERSECTIONS: if (! g_bLoading) RemoveSelfIntersectionsInPolygons (); break;
				case ID_REMOVE_JITTERS:    if (! g_bLoading) RemoveJittersInPolygons (); break;
				case ID_SLICE_MAP:         if (! g_bLoading) SliceMap (g_hWnd); break;
				case ID_SPLIT_MAP:         if (! g_bLoading) SplitMapToFiles (g_hWnd); break;

				case ID_VERIFY_MAP:        if (! g_bLoading) VerifyMap (); break;

				case ID_GENERATE_RG:       if (! g_bLoading) GenerateRGNodes (false); break;
				case ID_GENERATE_RG_END:   if (! g_bLoading) GenerateRGNodes (true); break;
				case ID_REMOVE_RG:         if (! g_bLoading) RemoveRGNodes (); break;
				case ID_CLEAN_UP_EXT_NODES:if (! g_bLoading) CleanUpExternalNodes (); break;
				case ID_CLEAR_EXT_NODES:   if (! g_bLoading) ClearExternalNodes (); break;
				case ID_FIX_ROUTE_PARAMS:  if (! g_bLoading) GenerateRouteParams (); break;
				case ID_MERGE_ROADS:       if (! g_bLoading) MergeRoads (false); break;

				case ID_EDIT_PAI:          if (! g_bLoading) ShowPostalAddressItems (); break;
				case ID_CREATE_POI_BY_ADDRESS: if (! g_bLoading) CreatePOIByAddress (); break;
				case ID_FIND_BY_ADDRESS:   if (! g_bLoading) FindByAddress (); break;

				case ID_OPTIONS:  ShowOptions (); break;

				case ID_STICK_TOGGLE:
					g_bEnableSticking = ! g_bEnableSticking;
					SaveSetting ("OptionStick", g_bEnableSticking);
					break;

				//
				// 'Help' menu.
				//

				case ID_GO_WEB: {
					string_t strWebPage;
					strWebPage.LoadFromResource (IDS_WEB_PAGE);
					::ShellExecute (g_hWnd, NULL, strWebPage.c_str (), NULL, NULL, SW_SHOWNORMAL);
					break;
				}

				case ID_LICENSE_AGREEMENT: {
					char strPath [MAX_PATH];
					::GetModuleFileName (NULL, strPath, MAX_PATH);
					char * const pFN = ::PathFindFileName (strPath);

//					const LANGID lang = ::GetUserDefaultLangID ();
//					if (lang == 1049)
//						::strcpy (pFN, "License.html#ru");
//					else 
						::strcpy (pFN, "License.html");

					::ShellExecute (g_hWnd, NULL, strPath, NULL, NULL, SW_SHOWNORMAL);
					break;
				}

				case ID_REGISTER_ONLINE: OrderKey (_hWnd); break;
				case ID_ACTIVATE:        ActivateLicense (_hWnd); break;
				case ID_ABOUT:           ShowAboutBox (_hWnd); break;

				default:
					if (ID_FILE_RECENT_0 <= wCmd && wCmd < ID_FILE_RECENT_0 + 100) {
						if (! g_bLoading)
							LoadRecentFile (wCmd - ID_FILE_RECENT_0);
					} else if (ID_EDIT_UNDO_0 <= wCmd && wCmd < ID_EDIT_UNDO_0 + 100) {
						if (! g_bLoading) {
							// Save visible rectangle.
							rect_t rectVisible;
							GetVisibleRect (rectVisible);

							const size_t cTimes = wCmd - ID_EDIT_UNDO_0 + 1;
							for (size_t c = 0; c < cTimes; ++ c)
								UndoEditAction ();

							// Restore visible rectangle.
							SetToCenter (point_t (rectVisible.MeanX (), rectVisible.MeanY ()));
							Refresh ();

							UpdateMenu ();
						}
					} else if (ID_EDIT_REDO_0 <= wCmd && wCmd < ID_EDIT_REDO_0 + 100) {
						if (! g_bLoading) {
							// Save visible rectangle.
							rect_t rectVisible;
							GetVisibleRect (rectVisible);

							const size_t cTimes = wCmd - ID_EDIT_REDO_0 + 1;
							for (size_t c = 0; c < cTimes; ++ c)
								RedoEditAction ();

							// Restore visible rectangle.
							SetToCenter (point_t (rectVisible.MeanX (), rectVisible.MeanY ()));
							Refresh ();

							UpdateMenu ();
						}
					} else if (ID_TOOLS_CREATE_0 <= wCmd && wCmd < ID_TOOLS_CREATE_0 + 100) {
						size_t cKind = wCmd - ID_TOOLS_CREATE_0;
						if (cKind >= tcMax)
							cKind = tcPoint;
						SetCreateKind (static_cast<TC_KIND> (cKind));
						UpdateMenu ();
						if (CTool::GetCurrentToolId () != toolCreate)
							::SendMessage (_hWnd, WM_COMMAND, ID_TOOL_CREATE_RGN, 0);
					} else if (ID_FILE_IMPORT_EXPORT_0 <= wCmd && wCmd < ID_FILE_IMPORT_EXPORT_0 + 200) {
						CFileTypeInfo::ImportExportFile (wCmd - ID_FILE_IMPORT_EXPORT_0);
					} else if (ID_TOOLS_TRANSFORM_0 <= wCmd && wCmd < ID_TOOLS_TRANSFORM_0 + 100) {
						size_t cOrder = wCmd - ID_TOOLS_TRANSFORM_0;
						if (cOrder >= 3)
							cOrder = 3;
						SetTransformationOrder (cOrder);
						UpdateMenu ();
						if (CTool::GetCurrentToolId () != toolTransform)
							::SendMessage (_hWnd, WM_COMMAND, ID_TOOL_TRANSFORM, 0);
					} else if (ID_VIEW_SCALE_0 <= wCmd && wCmd < ID_VIEW_SCALE_0 + 100) {
						if (! g_bLoading)
							SetScale (GetStdScale (wCmd - ID_VIEW_SCALE_0));
					} else if (ID_FAVORITES_0 <= wCmd && wCmd < ID_FAVORITES_0 + 100) {
						if (! g_bLoading)
							GoToFavorite (wCmd - ID_FAVORITES_0);
					} else if (ID_MAP_SKINS_0 <= wCmd && wCmd < ID_MAP_SKINS_0 + 100) {
						SelectMapSkin (wCmd == ID_MAP_SKINS_0 ? -1 : wCmd - ID_MAP_SKINS_0 - 1);
					} else if (ID_PLUGINS_SWITCHES_0 <= wCmd && wCmd < ID_PLUGINS_SWITCHES_0 + 100) {
						const HWND hCtrl = reinterpret_cast<HWND> (_lParam);
						ViewPlugins_OnSwitch (wCmd, hCtrl != g_hToolbarWnd);
					}

					break;

				case ID_SCALES_CB: {
					switch (HIWORD (_wParam)) {
						case CBN_SELCHANGE: {
							if (! g_hScaleCB)
								break;
							const size_t cIndex = ::SendMessage (g_hScaleCB, CB_GETCURSEL, 0, 0);
							const size_t cCmd = ::SendMessage (g_hScaleCB, CB_GETITEMDATA, cIndex, 0);
							::SetFocus (g_hWnd);
							::SendMessage (g_hWnd, WM_COMMAND, MAKELONG (cCmd, 0), 0);
							break;
						}

						case CBN_SETFOCUS:
							g_bDialogMode = true;
							break;	

						case CBN_KILLFOCUS:
							g_bDialogMode = false;
							break;	
					}
					break;
				}
			}
			break;
		}

		case WM_CONTEXTMENU: {
			const HWND hWnd = reinterpret_cast<HWND> (_wParam);
			if (hWnd != g_hMapWnd)
				break;

			int x = GET_X_LPARAM (_lParam);
			int y = GET_Y_LPARAM (_lParam);
			if (x == -1 && y == -1) {
				POINT p;
				::GetCursorPos (& p);
				::ScreenToClient (g_hMapWnd, & p);
				x = p.x;
				y = p.y;
			}
			CTool::_OnCtxMenu (x, y);
			break;
		}

		case WM_DROPFILES: {
			const HDROP hDrop = reinterpret_cast <HDROP> (_wParam);

			file_names_t file_names;
			size_t c = 0;
			while (true) {
				char strFileName [MAX_PATH + 1];
				if (::DragQueryFile (hDrop, c ++, strFileName, MAX_PATH) == 0)
					break;

				file_names.push_back (strFileName);
			}
			::DragFinish (hDrop);

			LoadMap (file_names, 0, false, false, g_map);

			break;
		}

		case WM_MOUSEWHEEL: {
			const WORD  wKeys = LOWORD (_wParam); // key flags
			const short zDelta = GET_WHEEL_DELTA_WPARAM (_wParam); // wheel rotation

			if (wKeys & MK_CONTROL) {
				POINT p;
				p.x = GET_X_LPARAM (_lParam); // horizontal position of pointer
				p.y = GET_Y_LPARAM (_lParam); // vertical position of pointer
				::ScreenToClient (g_hMapWnd, & p);

				// Zoom in/out.
//				::SendMessage (_hWnd, WM_COMMAND, zDelta > 0 ? ID_ZOOM_IN : ID_ZOOM_OUT, 0);
				if (! g_bLoading) {
					if (zDelta > 0)
						SetScale (g_fScale/c_Sqrt4, p.x, p.y);
					else
						SetScale (g_fScale*c_Sqrt4, p.x, p.y);
				}
			} else {
				if (wKeys & MK_SHIFT) {
					// Horizontal scroll.
					for (int i = 0; i < abs (zDelta/WHEEL_DELTA); i ++)
						ScrollMap (zDelta < 0 ? SB_LINELEFT : SB_LINERIGHT, 0, -1, 0);
					ScrollMap (SB_ENDSCROLL, 0, SB_ENDSCROLL, 0);
				} else {
				    // Vertical scroll.
					for (int i = 0; i < abs (zDelta/WHEEL_DELTA); i ++)
						ScrollMap (-1, 0, zDelta < 0 ? SB_LINEDOWN : SB_LINEUP, 0);
					ScrollMap (SB_ENDSCROLL, 0, SB_ENDSCROLL, 0);
				}
			}
			break;
		}

			// Emulate scrolling by keyboard.
		case WM_KEYDOWN:
			switch (_wParam) { 
				case VK_LEFT:  ScrollMap (SB_LINELEFT,  0, -1,          0); break;
				case VK_RIGHT: ScrollMap (SB_LINERIGHT, 0, -1,          0); break;
				case VK_UP:    ScrollMap (-1,           0, SB_LINEUP,   0);  break;
				case VK_DOWN:  ScrollMap (-1,           0, SB_LINEDOWN, 0);  break;

 				case VK_PRIOR: ScrollMap (SB_LINERIGHT, 0, SB_LINEUP,   0); break;
 				case VK_NEXT:  ScrollMap (SB_LINERIGHT, 0, SB_LINEDOWN, 0); break;
 				case VK_HOME:  ScrollMap (SB_LINELEFT,  0, SB_LINEUP,   0); break;
 				case VK_END:   ScrollMap (SB_LINELEFT,  0, SB_LINEDOWN, 0); break;
			} 
 			break; 

		case WM_KEYUP:
			ScrollMap (SB_ENDSCROLL, 0, SB_ENDSCROLL, 0);
			break;

		case WM_LBUTTONDOWN:
		case WM_RBUTTONDOWN:
			// Return keyboard input from label edit box.
			//if (g_bDialogMode)
				::SetFocus (_hWnd);
			break;

		case WM_CLOSE:
			if (g_bLoading)
				StopLoadThread ();

			if (! RequestSave ())
				return 0;

			StopTracking ();

			::DestroyWindow (_hWnd);  
			return 0;

		case WM_DESTROY:
			::SetEvent (g_hExitEvent);
			::PostQuitMessage (0);
			break;

		case WM_MEASUREITEM:
//			if (! CPopupMenu::OnMenuMeasureItem (_lParam))
				g_MainMenu.OnMenuMeasureItem (_lParam);
			return TRUE;

		case WM_DRAWITEM: {
			const DWORD dwCtrlID = _wParam;
			const DRAWITEMSTRUCT * const pDI = reinterpret_cast<DRAWITEMSTRUCT *> (_lParam);
			if (dwCtrlID == 0/* pDI->CtlType == ODT_MENU*/) {
//				if (! CPopupMenu::OnMenuDrawItem (_lParam))
					g_MainMenu.OnMenuDrawItem (_lParam);
			} else {
				//if (dwCtrlID == sbProgress) {

				//
				// Draw progress bar (requested by status bar).
				//

				if (pDI->itemData == 0)
					break;

				const int x0 = pDI->rcItem.left + 2;

				RECT rect;
				rect.left = x0;
				rect.top = pDI->rcItem.top + 2;
				rect.right = x0 + pDI->itemData;
				rect.bottom = pDI->rcItem.bottom - 2;
				::FillRect (pDI->hDC, & rect, g_hbrActiveColor);
			}

			return TRUE;
		}

		case WM_USER_FILE_LOADED:
			if (_wParam == 0)
				Fire_OnFileLoaded ();
			else
				Fire_OnFileLoadFailed ();
			break;

		default:
			return ::DefWindowProc (_hWnd, _uMsg, _wParam, _lParam);
	}

	return FALSE;
}

