//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2008
//
// 'Map properties' dialog, 'cgpsmapper' page.
//

# include "StdAfx.h"
# include "Resource.h"
# include "MapProperties.h"
# include "Map.h"
# include "ChangeMapHeader.h"
# include "ParseHelpers.h"
# include "ComboBox.h"
# include "LinkDlg.h"
# include "Globals.h"

# pragma const_seg (".text")

static
const CComboBoxData g_cbdPreprocess [] = {
	{IDS_PREPROCESS_NO, map_header_t::ppNo},
	{IDS_PREPROCESS_GO, map_header_t::ppGeneralization},
	{IDS_PREPROCESS_PO, map_header_t::ppPolygons},
	{IDS_PREPROCESS_GI, map_header_t::ppFull},
};

static
const CComboBoxData g_cbdTransparent [] = {
	{IDS_TRANSPARENT_N, map_header_t::trN},
	{IDS_TRANSPARENT_Y, map_header_t::trY},
	{IDS_TRANSPARENT_S, map_header_t::trS},
};

# pragma const_seg ()

INT_PTR CALLBACK _MapCGPSMapperProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	const INT_PTR r = LinkDlgProc (_hWnd, _uMsg, _wParam, _lParam);
	if (r)
		return r;

	switch (_uMsg) {
		case WM_INITDIALOG: {
			CCritSec::CLock lock (g_cs);

			char buf [64];

			// TreSize.
			::sprintf (buf, "%d", g_map.Header.cTreSize);
			::SetDlgItemText (_hWnd, IDC_MAP_TRESIZE, buf);

			// TreMargin.
			::sprintf (buf, "%0.3f", g_map.Header.fTreMargin);
			::SetDlgItemText (_hWnd, IDC_MAP_TREMARGIN, buf);

			// RgnLimit.
			::sprintf (buf, "%d", g_map.Header.cRgnLimit);
			::SetDlgItemText (_hWnd, IDC_MAP_RGNLIMIT, buf);

			// Preprocess.
			FillComboBox (_hWnd, IDC_MAP_PREPROCESS, g_cbdPreprocess, ARRAY_SIZE (g_cbdPreprocess), g_map.Header.Preprocess);

			// Transparent.
			FillComboBox (_hWnd, IDC_MAP_TRANSPARENT, g_cbdTransparent, ARRAY_SIZE (g_cbdTransparent), g_map.Header.Transparent);

			// POI index.
			::CheckDlgButton (_hWnd, IDC_MAP_POIINDEX,       g_map.Header.bPOIIndex       ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_MAP_POINUMBERFIRST, g_map.Header.bPOINumberFirst ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_MAP_POIZIPFIRST,    g_map.Header.bPOIZipFirst    ? BST_CHECKED : BST_UNCHECKED);

			if (! g_map.Header.bPOIIndex) {
				::EnableWindow (::GetDlgItem (_hWnd, IDC_MAP_POINUMBERFIRST), FALSE);
				::EnableWindow (::GetDlgItem (_hWnd, IDC_MAP_POIZIPFIRST),    FALSE);
			}

			// Settings for commercial version of cgpsmapper.exe
			::CheckDlgButton (_hWnd, IDC_MAP_MG,             g_map.Header.bMG             ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_MAP_NUMBERING,      g_map.Header.bNumbering      ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_MAP_ROUTING,        g_map.Header.bRouting        ? BST_CHECKED : BST_UNCHECKED);

			return TRUE;
		}

		case WM_COMMAND: {
			switch (LOWORD (_wParam)) {
				case IDC_MAP_TRESIZE:
				case IDC_MAP_TREMARGIN:
				case IDC_MAP_RGNLIMIT:
					if (HIWORD (_wParam) == EN_CHANGE)
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					break;

				case IDC_MAP_PREPROCESS:
				case IDC_MAP_TRANSPARENT:
					if (HIWORD (_wParam) == CBN_SELCHANGE)
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					break;

				case IDC_POLISH_074:
				case IDC_POLISH_RGNx0:
				case IDC_MAP_POIINDEX:
				case IDC_MAP_POINUMBERFIRST:
				case IDC_MAP_POIZIPFIRST:
				case IDC_MAP_MG:
				case IDC_MAP_NUMBERING:
				case IDC_MAP_ROUTING:
					if (HIWORD (_wParam) == BN_CLICKED)
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);

					if (LOWORD (_wParam) == IDC_MAP_POIINDEX) {
						if (::IsDlgButtonChecked (_hWnd, IDC_MAP_POIINDEX) == BST_CHECKED) {
							::EnableWindow (::GetDlgItem (_hWnd, IDC_MAP_POINUMBERFIRST), TRUE);
							::EnableWindow (::GetDlgItem (_hWnd, IDC_MAP_POIZIPFIRST),    TRUE);
						} else {
							::EnableWindow (::GetDlgItem (_hWnd, IDC_MAP_POINUMBERFIRST), FALSE);
							::EnableWindow (::GetDlgItem (_hWnd, IDC_MAP_POIZIPFIRST),    FALSE);
						}
					}
					break;
			}
			break;
		}

		case WM_NOTIFY: {
            const NMHDR * const pInfo = reinterpret_cast<NMHDR *> (_lParam);
            switch (pInfo->code) {
				case PSN_APPLY: {
					map_header_t Header = g_map.Header;

					char buf [256];

					// TreSize.
					::GetDlgItemText (_hWnd, IDC_MAP_TRESIZE,   buf, sizeof (buf)/sizeof (buf [0]));
					Header.cTreSize = ParseDecU (buf, buf + ::strlen (buf));

					// TreMargin.
					::GetDlgItemText (_hWnd, IDC_MAP_TREMARGIN, buf, sizeof (buf)/sizeof (buf [0]));
					Header.fTreMargin = ParseReal (buf, buf + ::strlen (buf));

					// RgnLimit.
					::GetDlgItemText (_hWnd, IDC_MAP_RGNLIMIT,  buf, sizeof (buf)/sizeof (buf [0]));
					Header.cRgnLimit = ParseDecU (buf, buf + ::strlen (buf));

					// Preprocess.
					int nPreprocess = 0;
					if (GetComboBoxItemData (_hWnd, IDC_MAP_PREPROCESS, nPreprocess))
						Header.Preprocess = static_cast<map_header_t::preprocess_t> (nPreprocess);

					// Transparent.
					int nTransparent = 0;
					if (GetComboBoxItemData (_hWnd, IDC_MAP_TRANSPARENT, nTransparent))
						Header.Transparent = static_cast<map_header_t::transparency_t> (nTransparent);

					// POI index.
					Header.bPOIIndex       = ::IsDlgButtonChecked (_hWnd, IDC_MAP_POIINDEX)       == BST_CHECKED;
					Header.bPOINumberFirst = ::IsDlgButtonChecked (_hWnd, IDC_MAP_POINUMBERFIRST) == BST_CHECKED;
					Header.bPOIZipFirst    = ::IsDlgButtonChecked (_hWnd, IDC_MAP_POIZIPFIRST)    == BST_CHECKED;

					Header.bMG             = ::IsDlgButtonChecked (_hWnd, IDC_MAP_MG)             == BST_CHECKED;
					Header.bNumbering      = ::IsDlgButtonChecked (_hWnd, IDC_MAP_NUMBERING)      == BST_CHECKED;
					Header.bRouting        = ::IsDlgButtonChecked (_hWnd, IDC_MAP_ROUTING)        == BST_CHECKED;

					if (! (Header == g_map.Header))
						ChangeMapHeader (Header);

					break;
				}
			}
			break;
		}
	}

	return FALSE;
}
