//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// 'Map properties' dialog, 'Statistics' tab.
//

# include "StdAfx.h"
# include "Resource.h"
# include "MapProperties.h"
# include "Map.h"
# include "Types.h"
# include "ListView.h"
# include "ParseHelpers.h"
# include "PrintUnits.h"
# include "Globals.h"

enum {
	lvType             = 0,
	lvKind             = 1,
	lvTypeName         = 2,
	lvCount            = 3,
	lvCountWithAddress = 4,
	lvCountWithContact = 5,
	lvLength           = 6,
	lvArea             = 7,
	lvOccupiedSize     = 8
};

static HWND g_hTypes;
static int g_iSortColumn_Stat = lvType;
static bool g_bAccent_Stat = true;

static
int CALLBACK _CompareItems_Stat (LPARAM _lParam1, LPARAM _lParam2, LPARAM) {
	char str1 [64], str2 [64];
	ListView_GetItemText (g_hTypes, _lParam1, g_iSortColumn_Stat, str1, ARRAY_SIZE (str1)); 
	ListView_GetItemText (g_hTypes, _lParam2, g_iSortColumn_Stat, str2, ARRAY_SIZE (str2));

	if (
		g_iSortColumn_Stat == lvCount            ||
		g_iSortColumn_Stat == lvCountWithAddress ||
		g_iSortColumn_Stat == lvCountWithContact ||
		g_iSortColumn_Stat == lvOccupiedSize
	) {
		// Compare as integers.
		return g_bAccent_Stat ?
			ParseDecU (str1, str1 + ARRAY_SIZE (str1)) - ParseDecU (str2, str2 + ARRAY_SIZE (str2)) :
			ParseDecU (str2, str2 + ARRAY_SIZE (str2)) - ParseDecU (str1, str1 + ARRAY_SIZE (str1));
	} else {
		// Compare as strings.
		return g_bAccent_Stat ? ::strcmp (str1, str2) : ::strcmp (str2, str1);
	}
}

struct ut_data_t {
	size_t cCount;
	size_t cCountWithAddress;
	size_t cCountWithContact;
	double fLengthInM;
	double fAreaInM2;
	size_t cOccupiedSizeInB;

	ut_data_t () {::memset (this, 0, sizeof (ut_data_t));}
};

static
void operator += (ut_data_t & _1, const ut_data_t & _2) {
	_1.cCount            += _2.cCount;
	_1.cCountWithAddress += _2.cCountWithAddress;
	_1.cCountWithContact += _2.cCountWithContact;
	_1.fLengthInM        += _2.fLengthInM;
	_1.fAreaInM2         += _2.fAreaInM2;
	_1.cOccupiedSizeInB  += _2.cOccupiedSizeInB;
}

static
void _CountRgn (ut_data_t & _ts, const rgn_t & _rgn, float _fLengthInM, float _fAreaInM2) {
	++ _ts.cCount;

	if (_rgn.pExtra) {
		if (! _rgn.pExtra->strNumber.empty () && ! _rgn.pExtra->strStreetDesc.empty () && _rgn.pExtra->pCity != NULL && ! _rgn.pExtra->pCity->IsDeleted ())
			++ _ts.cCountWithAddress;

		if (
			! _rgn.pExtra->strPhone      .empty () ||
			! _rgn.pExtra->strFax        .empty () ||
			! _rgn.pExtra->strEmail      .empty () ||
			! _rgn.pExtra->strWebPage    .empty () ||
			! _rgn.pExtra->strDescription.empty ()
		)
			++ _ts.cCountWithContact;
	}

	_ts.fLengthInM += _fLengthInM;
	_ts.fAreaInM2  += _fAreaInM2;

	if (_rgn.cEndPos > _rgn.cBeginPos)
		// NOTE: _rgn.cEndPos may be incorrect for damaged files (w/o correct end).
		_ts.cOccupiedSizeInB += _rgn.cEndPos - _rgn.cBeginPos;
}

static
void _FillTypeStat (size_t & _cItem, const ut_data_t & _ts, int _nKind, DWORD _dwType, size_t _cImageIdx, const char * _strTypeName) {
	// Code.
	InsertItem     (g_hTypes, _cItem, 0,                  _dwType != (DWORD) -1 ? "0x%04x" : "-", _dwType);
	SetItemIcon    (g_hTypes, _cItem, _cImageIdx);

	// Kind.
	SetSubItemText (g_hTypes, _cItem, lvKind,             "%s", _nKind != rkMax ? g_strKinds [_nKind] : "-");

	// Type Name.
	SetSubItemText (g_hTypes, _cItem, lvTypeName,         "%s", _strTypeName);

	// Count.
	SetSubItemText (g_hTypes, _cItem, lvCount,            "%d", _ts.cCount);
	SetSubItemText (g_hTypes, _cItem, lvCountWithAddress, "%d", _ts.cCountWithAddress);
	SetSubItemText (g_hTypes, _cItem, lvCountWithContact, "%d", _ts.cCountWithContact);

	// Length.
	if (_ts.fLengthInM != 0) {
		char strLength [32], strUnits [16];
		PrintDistance (strLength, strUnits, _ts.fLengthInM);
		SetSubItemText (g_hTypes, _cItem, lvLength, "%s %s", strLength, strUnits);
	}
	// Area.
	if (_ts.fAreaInM2 != 0) {
		char strArea [32], strUnits [16];
		PrintArea (strArea, strUnits, _ts.fAreaInM2);
		SetSubItemText (g_hTypes, _cItem, lvArea,   "%s %s", strArea,   strUnits);
	}

	// Memory.
	SetSubItemText (g_hTypes, _cItem, lvOccupiedSize,     "%d", _ts.cOccupiedSizeInB);

	++ _cItem;
}

static
void _FillStatistics (HWND _hWnd, size_t _cLevel) {
	CCritSec::CLock lock (g_cs);

	const size_t cTypes = g_map.pTypeSet->Types ().size ();
	const type_desc_t * const pUnknownTD = g_map.pTypeSet->GetUnknownTD ();

	//
	size_t cMaxNodes = 0, cMaxElements = 0, cTotalNodes = 0;
	size_t cRGLinks = 0;

	size_t cLabels   = 0, cLabelsSize   = 0;
	size_t cComments = 0, cCommentsSize = 0;
	size_t cRgnsSize = 0;
	size_t cElementsTotal = 0, cElementsSize = 0;
	size_t cOriginsSize   = 0;
	size_t cPointsSize    = 0;
	size_t cPointsSize2   = 0;
	
	// The array of counters being parallel to types list.
	ut_data_t * const pCounters = reinterpret_cast<ut_data_t *> (alloca (sizeof (ut_data_t)*cTypes));
	::memset (pCounters, 0, sizeof (ut_data_t)*cTypes);

	typedef std::map<DWORD, ut_data_t> mapTypes_t;
	mapTypes_t mapUnknownTypes;
	mapTypes_t mapSpecificTypes;

	ut_data_t utdRoads;
	ut_data_t utdCities;
	ut_data_t utdWater;

	// Count types.
	size_t cKind;
	for (cKind = 0; cKind < rkMax; ++ cKind) {
		const rgns_t & rgns = g_map.layers [cKind].rgns;
		for (rgns_t::const_iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
			const rgn_t & rgn = * iRgn;
			if (rgn.IsDeleted () || rgn.IsNotMapObject ())
				continue;

			bool bHasElements = false;

			double fLength = 0;
			double fArea   = 0;

			// Elements.
			const size_t cElements = rgn.elements.size ();
			for (size_t cElement = 0; cElement < cElements; ++ cElement) {
				const rgn_element_t & element = rgn.elements [cElement];
				if (element.bDeleted)
					continue;

				if (
					_cLevel == -1 ||
					element.cLevel == _cLevel ||
					(rgn.uchEndLevel != 0 && element.cLevel <= _cLevel && _cLevel <= rgn.uchEndLevel)
				) {
					bHasElements = true;

					++ cElementsTotal;
					cElementsSize += sizeof (element);
					cPointsSize  += element.points.capacity ()*sizeof (point_t);
					cPointsSize2 += element.points.size ()*sizeof (point_t);

					if (cKind == rkPolyline)
						fLength += element.fLengthOrArea;
					else if (cKind == rkPolygon) {
						if (element.bHole)
							fArea -= element.fLengthOrArea;
						else
							fArea += element.fLengthOrArea;
					}

					const size_t cNodes = element.points.size ();
					cTotalNodes += cNodes;

					if (cNodes > cMaxNodes)
						cMaxNodes = cNodes;

					if (element.pExtra) {
						const point_extras_t & PointExtras = element.pExtra->PointExtras;
						const size_t cNodes2 = PointExtras.size ();
						size_t cRGNodesInElement = 0;
						for (size_t cNode = 0; cNode < cNodes2 && cNode < cNodes; ++ cNode) {
							const rgnode_t * const pNode = PointExtras [cNode].GetRGNode ();
							if (pNode != NULL)
								++ cRGNodesInElement;
						}
						if (cRGNodesInElement != 0)
							cRGLinks += cRGNodesInElement - 1;
					}
				}
			}

			// Origins.
			const size_t cOrigins = rgn.origins.size ();
			for (size_t cOrigin = 0; cOrigin < cOrigins; ++ cOrigin) {
				const rgn_origin_t & origin = rgn.origins [cOrigin];
				if (origin.bDeleted)
					continue;

				if (
					_cLevel == -1 ||
					origin.cLevel == _cLevel ||
					(rgn.uchEndLevel != 0 && origin.cLevel <= _cLevel && _cLevel <= rgn.uchEndLevel)
				) {
					bHasElements = true;

					++ cTotalNodes;

					cOriginsSize += sizeof (rgn_origin_t);
				}
			}

			if (! bHasElements)
				continue;

			if (cElements > cMaxElements)
				cMaxElements = cElements;

			// Count labels.
			if (! rgn.strLabel.empty ()) {
				++ cLabels;
				cLabelsSize += rgn.strLabel.size ();
			}

			// Count comments.
			if (! rgn.strComment.empty ()) {
				++ cComments;
				cCommentsSize += rgn.strComment.size ();
			}

			cRgnsSize += sizeof (rgn);

			fLength *= 40e6/360;
			fArea   *= 40e6*40e6/(360*360)*g_map.fCosY;

			// Count types.
			const type_desc_t * const pTD = rgn.GetTypeDesc ();
			assert (pTD);
			if (pTD && ! pTD->bUnknown) {
				if (pTD->Type == rgn.GetType ()) {
					_CountRgn (pCounters [pTD->cOrdinal], rgn, fLength, fArea);
				} else {
					const DWORD dwType = rgn.GetType () + (DWORD (rgn.GetKind ()) << 24);
					mapTypes_t::iterator i = mapSpecificTypes.find (dwType);
					if (i == mapSpecificTypes.end ())
						i = mapSpecificTypes.insert (mapTypes_t::value_type (dwType, ut_data_t ())).first;
					_CountRgn (i->second, rgn, fLength, fArea);
				}
			} else {
				const DWORD dwType = rgn.GetType () + (DWORD (rgn.GetKind ()) << 24);
				mapTypes_t::iterator i = mapUnknownTypes.find (dwType);
				if (i == mapUnknownTypes.end ())
					i = mapUnknownTypes.insert (mapTypes_t::value_type (dwType, ut_data_t ())).first;
				_CountRgn (i->second, rgn, fLength, fArea);
			}

			if (rgn.IsRoad ())
				_CountRgn (utdRoads, rgn, fLength, fArea);

			if (pTD && pTD->Category == catCities)
				_CountRgn (utdCities, rgn, fLength, fArea);

			if (pTD && pTD->Category == catWater)
				_CountRgn (utdWater, rgn, fLength, fArea);
		}
	}

	// Count RG nodes.
	g_map.CountRGNodes ();

	size_t cRGNodes = 0;
	size_t cExternalNodes = 0;
	for (rgnodes_t::const_iterator in = g_map.rgnodes.begin (); in != g_map.rgnodes.end (); ++ in) {
		const rgnode_t & node = * in;
		if (! node.IsUsed ())
			continue;

		++ cRGNodes;

		if (node.bIsExternal)
			++ cExternalNodes;
	}

	// Count turn restictions.
	size_t cTurnRestrictions = 0;
	for (rgrestrictions_t::const_iterator ir = g_map.rgrestrictions.begin (); ir != g_map.rgrestrictions.end (); ++ ir) {
		const rgrestriction_t & restriction = ir->second;
		if (restriction.IsUsed ())
			++ cTurnRestrictions;
	}

	ListView_DeleteAllItems (g_hTypes);

	ut_data_t utdKind [rkMax];

	char strBuf [64];
	size_t cItem = 0;

	// For each type.
	// NOTE: omit 0-th type ("unknown").
	size_t cType = 1;
	CTypeSet::types_t::const_iterator it = g_map.pTypeSet->Types ().begin ();
	for (++ it; it != g_map.pTypeSet->Types ().end () && cType < cTypes; ++ it, ++ cType) {
		if (pCounters [cType].cCount == 0)
			continue;
		const type_desc_t & td = * it;

		utdKind [td.Kind] += pCounters [cType];

		_FillTypeStat (cItem, pCounters [cType], td.Kind, td.ulType & 0xFFFFFF, td.cImageListIndex, td.strTypeName);
	}

	// Specific types.
	mapTypes_t::iterator i;
	for (i = mapSpecificTypes.begin (); i != mapSpecificTypes.end (); ++ i) {
		const DWORD dwType = i->first & 0xFFFFFF;
		const BYTE Kind    = i->first >> 24;
		assert (Kind < rkMax);

		const type_desc_t * pTD = g_map.pTypeSet->GetTypeDesc (dwType, Kind);
		if (pTD == NULL)
			pTD = pUnknownTD;

		utdKind [Kind] += i->second;

		_FillTypeStat (cItem, i->second, Kind, dwType, pTD->cImageListIndex, pTD->strTypeName);
	}

	// Unknown types.
	for (i = mapUnknownTypes.begin (); i != mapUnknownTypes.end (); ++ i) {
		const DWORD dwType = i->first & 0xFFFFFF;
		const BYTE Kind    = i->first >> 24;
		assert (Kind < rkMax);

		utdKind [Kind] += i->second;

		_FillTypeStat (cItem, i->second, Kind, dwType, pUnknownTD->cImageListIndex, pUnknownTD->strTypeName);
	}

	// Totals.
	ut_data_t utdAll;
	utdAll += utdKind [rkPoint];
	utdAll += utdKind [rkPolyline];
	utdAll += utdKind [rkPolygon];

	_FillTypeStat (cItem, utdAll,               rkMax,      -1, -1, "Total");
	_FillTypeStat (cItem, utdKind [rkPoint],    rkPoint,    -1, -1, "All points");
	_FillTypeStat (cItem, utdKind [rkPolyline], rkPolyline, -1, -1, "All polylines");
	_FillTypeStat (cItem, utdKind [rkPolygon],  rkPolygon,  -1, -1, "All polygons");
	_FillTypeStat (cItem, utdRoads,             rkPolyline, -1, -1, "All polyline roads");
	_FillTypeStat (cItem, utdCities,            rkMax,      -1, -1, "All cities");
	_FillTypeStat (cItem, utdWater,             rkMax,      -1, -1, "All hydrographic");

	ListView_SortItemsEx (g_hTypes, _CompareItems_Stat, 0);
	AutoSizeColumns (g_hTypes);

	::sprintf (strBuf, "%d", cMaxNodes);
	::SetDlgItemText (_hWnd, IDC_MAX_NODES,        strBuf);
	::sprintf (strBuf, "%d", cMaxElements);
	::SetDlgItemText (_hWnd, IDC_MAX_ELEMENTS,     strBuf);
	::sprintf (strBuf, "%d", cTotalNodes);
	::SetDlgItemText (_hWnd, IDC_TOTAL_NODES,      strBuf);

	::sprintf (strBuf, "%d", cRGNodes);
	::SetDlgItemText (_hWnd, IDC_RGNODES,          strBuf);
	::sprintf (strBuf, "%d", cRGLinks);
	::SetDlgItemText (_hWnd, IDC_RGLINKS,          strBuf);
	::sprintf (strBuf, "%d", cExternalNodes);
	::SetDlgItemText (_hWnd, IDC_EXTERNALNODES,    strBuf);
	::sprintf (strBuf, "%d", cTurnRestrictions);
	::SetDlgItemText (_hWnd, IDC_TURNRESTRICTIONS, strBuf);
}

INT_PTR CALLBACK _MapStatisticsProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_INITDIALOG: {
			CCritSec::CLock lock (g_cs);

			// Levels.
			const HWND hLevel = ::GetDlgItem (_hWnd, IDC_MAP_LEVEL);
			::SendMessage (hLevel, CB_INSERTSTRING, 0, (LPARAM) "All");
			::SendMessage (hLevel, CB_SETITEMDATA,  0, -1);
			for (size_t cLevel = 0; cLevel < g_map.Levels.size (); ++ cLevel) {
				char strText [64];
				::sprintf (strText, "%d", cLevel);
				::SendMessage (hLevel, CB_INSERTSTRING, cLevel + 1, (LPARAM) strText);
				::SendMessage (hLevel, CB_SETITEMDATA,  cLevel + 1, cLevel);
			}
			::SendMessage (hLevel, CB_SETCURSEL, 0, 0);

			// Statistics.			
			g_hTypes = ::GetDlgItem (_hWnd, IDC_TYPES);

			ListView_SetExtendedListViewStyleEx (g_hTypes, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);

			::SetWindowLong (g_hTypes, GWL_STYLE, ::GetWindowLong (g_hTypes, GWL_STYLE) | LVS_SHAREIMAGELISTS);
			ListView_SetImageList (g_hTypes, g_map.pTypeSet->GetTypeIcons (), LVSIL_SMALL);

			AddColumn (g_hTypes, lvType,             "Code");
			AddColumn (g_hTypes, lvKind,             "Kind");
			AddColumn (g_hTypes, lvTypeName,         "Type Name");
			AddColumn (g_hTypes, lvCount,            "Count",     true);
			AddColumn (g_hTypes, lvCountWithAddress, "w/Address", true);
			AddColumn (g_hTypes, lvCountWithContact, "w/Contact", true);
			AddColumn (g_hTypes, lvLength,           "Length",    true);
			AddColumn (g_hTypes, lvArea,             "Area",      true);
			AddColumn (g_hTypes, lvOccupiedSize,     "Memory",    true);

			_FillStatistics (_hWnd, -1);
			
			return TRUE;
		}

		case WM_COMMAND:
			switch (LOWORD (_wParam)) {
				case IDC_MAP_LEVEL: {
					if (HIWORD (_wParam) == CBN_SELCHANGE) {
						const HWND hLevel = ::GetDlgItem (_hWnd, IDC_MAP_LEVEL);
						const size_t cSelIndex = ::SendMessage (hLevel, CB_GETCURSEL, 0, 0);
						const size_t cLevel = static_cast<size_t> (::SendMessage (hLevel, CB_GETITEMDATA, cSelIndex, 0));

						_FillStatistics (_hWnd, cLevel);

						return TRUE;
					}
					break;
				}
			}
			break;

		case WM_NOTIFY: {
			NMHDR * const pInfo = reinterpret_cast<NMHDR *> (_lParam);
			switch (pInfo->code) {
				case LVN_COLUMNCLICK: {
					const NMLISTVIEW * const pInfo2 = reinterpret_cast<const NMLISTVIEW *> (_lParam);

					if (g_iSortColumn_Stat != pInfo2->iSubItem)
						g_iSortColumn_Stat = pInfo2->iSubItem;
					else
						g_bAccent_Stat = ! g_bAccent_Stat;
					
					ListView_SortItemsEx (g_hTypes, _CompareItems_Stat, 0);

					break;
				}

				case NM_CUSTOMDRAW: {
					NMLVCUSTOMDRAW * const pInfo2 = reinterpret_cast<NMLVCUSTOMDRAW *> (pInfo);
					if (pInfo2->nmcd.dwDrawStage == CDDS_PREPAINT) {
						//SetMsgResult (CDRF_NOTIFYITEMDRAW);
						::SetWindowLong (_hWnd, DWL_MSGRESULT, CDRF_NOTIFYITEMDRAW);
						return true;
					} else if (pInfo2->nmcd.dwDrawStage == CDDS_ITEMPREPAINT) {
						const size_t cItem = (size_t) pInfo2->nmcd.dwItemSpec;

						char strCode [16];
						ListView_GetItemText (g_hTypes, cItem, lvType, strCode, ARRAY_SIZE (strCode));
						const bool bHeader = strCode [0] == '-';

						if (bHeader)
							pInfo2->clrTextBk = (cItem & 0x01) != 0 ? RGB (200, 255, 200) : RGB (224, 255, 224);
						else
							pInfo2->clrTextBk = (cItem & 0x01) != 0 ? RGB (224, 224, 224) : ::GetSysColor (COLOR_WINDOW);
						//SetMsgResult (CDRF_NEWFONT);
						::SetWindowLong (_hWnd, DWL_MSGRESULT, CDRF_NEWFONT);
						return true;
					}
					break;
				}
			}
			break;
		}
	}

	return FALSE;
}
