# ifndef __MapSkins_H_
# define __MapSkins_H_

void InitMapSkins ();
void DestroyMapSkins ();

void SelectMapSkin (size_t _cOrdinal, bool _bRefresh = true);
size_t GetSelectedMapSkin ();
void UpdateMapSkin ();
bool LoadMapSkin (const char * _strFileName, const char * _strMode);

void MapSkins_InitMenu   (HMENU _hmenu);
void MapSkins_UpdateMenu (HMENU _hmenu);
bool MapSkins_EditList (HWND _hWnd);
void MapSkins_UpdateGUI ();

//////////////////////////////////

void MapSkin_DrawPOI (HDC _hDC, size_t _cIconIdx, int _x, int _y, bool _bCenter);

enum CMapColor {
	spcBgColor,
	spcTextColor,
	spcOutlineColor,

	spcHWSign1Text,
	spcHWSign1Bg,
	spcHWSign2Text,
	spcHWSign2Bg,
	spcHWSign3Text,
	spcHWSign3Bg,

	spcLast
};

class CMapSkin;
COLORREF MapSkin_GetColor (CMapColor _mc, CMapSkin * _pMapSkin = NULL);
HBRUSH   MapSkin_GetBrush (CMapColor _mc);

# include "Wrappers.h"

class CMapSkin {
  public:
	CMapSkin ();
	~CMapSkin ();

	HPEN    & AddPen   (HPEN    _hpen) {m_pens   .push_back (_hpen); return m_pens   .back ();}
	HBRUSH  & AddBrush (HBRUSH  _hbr)  {m_brushes.push_back (_hbr);  return m_brushes.back ();}
	HCURSOR & AddIcon  (HCURSOR _hcur) {m_icons  .push_back (_hcur); return m_icons  .back ();}
	const char * AddLabel (const string_t & _str) {m_labels.push_back (_str); return m_labels.back ().c_str ();}

	void SetPOIs (HBITMAP _hbmp, size_t _cW, size_t _cH);

	bool DrawPOI (HDC _hDC, size_t _cIconIdx, int _x, int _y, bool _bCenter) const;

	void SetColor (CMapColor _mc, COLORREF _col) {m_colSetting [_mc] = _col; m_bColorDefined [_mc] = true;}
	COLORREF GetColor (CMapColor _mc) const {return m_colSetting [_mc];}
	HBRUSH   GetBrush (CMapColor _mc) const {
		assert (IsColorDefined (_mc));
		if (m_hbrSetting [_mc] == NULL)
			m_hbrSetting [_mc] = ::CreateSolidBrush (GetColor (_mc));
		return m_hbrSetting [_mc];
	}
	bool IsColorDefined (CMapColor _mc) const {return m_bColorDefined [_mc];}

  private:
	bool m_bColorDefined [spcLast];
	COLORREF m_colSetting [spcLast];
	mutable HBRUSH m_hbrSetting [spcLast];

	typedef std::list<HPEN>     pens_t;
	typedef std::list<HBRUSH>   brushes_t;
	typedef std::list<HCURSOR>  icons_t;
	typedef std::list<string_t> labels_t;
	pens_t    m_pens;
	brushes_t m_brushes;
	icons_t   m_icons;
	labels_t  m_labels;

	HBITMAP m_hbmpPOIs, m_hbmpPrev;
	size_t m_cPOIsW;
	size_t m_cPOIsH;

//	typedef std::map<DWORD, size_t> mapPOIType2IconIndex_t;
//	mapPOIType2IconIndex_t m_mapPOIType2IconIndex;

	CDCHolder m_hdcMem;
};

class CMapSkinTypeInfo {
  public:
	typedef bool (* pfParseSkin_t) (const char * _strFileName, const char * _strMode, CMapSkin * _pMapSkin);

	CMapSkinTypeInfo (pfParseSkin_t _pfParseSkin, const char * _strTypeset, const char * _strExt, const char * _strFilter);

	const char * const m_strTypeset;
	const char * const m_strExt;
	const char * const m_strFilter;
	const pfParseSkin_t m_pfParseSkin;
  
	// The linked list of parsers.
	CMapSkinTypeInfo * m_pNext;
	static CMapSkinTypeInfo * s_pFirst;

	static const CMapSkinTypeInfo * FindByFileExt (const char * _strFileName);
};

# define DECLARE_MAP_SKIN(_pfParser,_strTypeset,_strExt,_strFilter) CMapSkinTypeInfo Info_##_pfParser (_pfParser, _strTypeset, _strExt, _strFilter);

# endif // __MapSkins_H_