# ifndef __Menu_H_
# define __Menu_H_

# include "fvector.h"

void InitMenu ();
void UpdateMenu ();
void UpdateImportExportMenu ();
void UpdateMapSkinsMenu ();

typedef std::set<WORD> hidden_menu_items_t;
void GetHiddenMenuItems (bool _bDefault, hidden_menu_items_t & _HiddenItems);
void SetHiddenMenuItems (const hidden_menu_items_t & _HiddenItems);

void InitToolsCreateObjectMenu  (HMENU _hMenu, bool _bHideMenuItems);
void InitToolsTransformMenu     (HMENU _hMenu, bool _bHideMenuItems);
void CheckToolsCreateObjectMenu (HMENU _hMenu);
void CheckToolsTransformMenu    (HMENU _hMenu);

void PopupMenu (HMENU _hMenu);

bool IsSeparator (HMENU _hMenu, int _nPos);

void DeleteDuplicatedMenuSeparators (HMENU _hMenu, bool _bRecursive = false, bool _bDeleteEmptySubMenus = false);
void DeleteParentMenu  (HMENU _hMenu, UINT _uiCmd);
void DisableParentMenu (HMENU _hMenu, UINT _uiCmd);

void NormalizeMenuText (char * _str);

class CMenuHolder {
	// Disable copy semantics.
	const CMenuHolder & operator= (const CMenuHolder &);
	CMenuHolder (const CMenuHolder &);

  public:
	CMenuHolder (HMENU _hMenu) : m_hMenu (_hMenu) {}
	~CMenuHolder ();

  private:
	HMENU m_hMenu;
};

class CMenuBase {
  public:
	CMenuBase ();
	~CMenuBase ();

	int ShowAsPopup (HWND _hwndParent, RECT & _rectExclude, bool _bRetCmd = true) const;

	void AppendItem     (WORD _wCmdId, const char * _strItem);
	void RemoveItem     (WORD _wCmdId);

	void EnableItem     (WORD _wCmdId, bool _bEnable);
	void CheckItem      (WORD _wCmdId, bool _bCheck);
	void SetDefaultItem (WORD _wCmdId);

	operator HMENU () const {return m_hMenu;}

	HMENU GetMenu    () const {return m_hMenu;}
	HWND  GetToolbar () const {return m_hwndToolbar;}

	// Handler of WM_MEASUREITEM.
	void OnMenuMeasureItem (LPARAM _lParam) const;
	// Handler of WM_DRAWITEM.
	void OnMenuDrawItem    (LPARAM _lParam) const;

	struct menuitem_data_t {
		WORD wCmdId;
		WORD wImage;
	};

//	void SetIcons (UINT _uiIconsBmp, const menuitem_data_t * _pMenuItems, size_t _cMenuItems);
	// Update owner-draw flags for menu items.
	// NOTE: To be invoked after each menu item inserting.
	void UpdateODItems ();

	void RemoveDuplicatedMenuSeparators ();

  protected:
	bool m_bToDestroy;
	bool m_bHorzMenu;
	HMENU m_hMenu;
	HWND  m_hwndToolbar;

	HIMAGELIST m_hilIcons;
	const menuitem_data_t * m_pMenuItems;
	size_t m_cMenuItems;
	size_t _FindImageForCmd (WORD _wCmdId) const;

	fvector<TCHAR *> m_MenuStrings;
	void _DestroyMenuStrings ();

	void _MakeMenuItemsOD (HMENU _hMenu, bool _bHorzMenu);
};

class CPopupMenu : public CMenuBase {
  public:
	CPopupMenu ();
	CPopupMenu (HMENU _hMenu);
};

class CMainMenu : public CMenuBase {
  public:
	void Init (HMENU _hMenu);
	void SetToolbar (HWND _hwndToolbar);
};

extern CMainMenu g_MainMenu;
extern HMENU g_hmenuWTR;
extern HMENU g_hmenuBookmarks;
extern HMENU g_hmenuFile;
extern HMENU g_hmenuView;
extern HMENU g_hmenuViewLevels;
extern HMENU g_hmenuViewScales;
extern HMENU g_hmenuFavorites;
extern HMENU g_hmenuTools;
extern HMENU g_hmenuToolsCreateObject;
extern HMENU g_hToolsTransform;

/////////////////////////////

struct menu_item_t {
	UINT    uiCmd;
	fstring strText;
	size_t  cSubMenu;
};

typedef std::vector<menu_item_t> menu_items_t;
extern menu_items_t g_menu_items;

/////////////////////////////

enum {
	// Popup submenus.
	smpObject      = 0,
	smpRaster      = 1,
	smpWTR         = 2,
	smpEditNode    = 3,
	smpEditLine    = 4,
	smpEditPolygon = 5,
	smpTrim        = 6,
	smpCreate      = 7,
	smpGoogle      = 8,
	smpRoutes      = 9,
	smpRotate      = 10,
	smpDistance    = 11,
	smpEntryPoint  = 12,
	smpToolbar     = 13,
	smpBookmark    = 14,
};

# endif // __Menu_H_