//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// 'File|New' command.
//

# include "StdAfx.h"
# include "NewMap.h"
# include "Map.h"
# include "UndoRedo.h"
# include "Log.h"
# include "Menu.h"
# include "RecentFiles.h"
# include "Status.h"
# include "Save.h"
# include "Scale.h"
# include "Position.h"
# include "MapView.h"
# include "MiniMap.h"
# include "Position.h"
# include "Selection.h"
# include "Globals.h"

CInvokeOnRemoveMap * CInvokeOnRemoveMap::s_pFirst;

CInvokeOnRemoveMap::CInvokeOnRemoveMap (pvfv_t _pf, bool _bBefore) :
	m_pf (_pf),
	m_bBefore (_bBefore)
{
	// Insert to the head of the list.
	m_pNext = CInvokeOnRemoveMap::s_pFirst;
	CInvokeOnRemoveMap::s_pFirst = this;
}

/////////////////////////////////////////////////

bool RemoveMap () {
	if (! RequestSave ())
		return false;

	SaveRecentMapPosition ();

	ResetUndoRedo (false);
	g_bModified = false;
	g_bHaveWaypointsOrTracks = false;
	g_bHaveBookmarks = false;

	// Update title.
	g_strFileName [0] = '\0';
	UpdateTitle ();

	// Disable mini-window.
	g_lMiniW = 0;
	g_lMiniH = 0;
	::SetWindowPos (g_hMiniMapWnd, NULL, 0, 0, g_lMiniW, g_lMiniH, SWP_NOMOVE | SWP_NOZORDER);

	// NOTE: this ResetXXX() should stop parallel threads before closing map.
	// Invoke all 'before' resetters.
	const CInvokeOnRemoveMap * pRM;
	for (pRM = CInvokeOnRemoveMap::s_pFirst; pRM != NULL; pRM = pRM->m_pNext) {
		if (pRM->m_bBefore)
			pRM->m_pf ();
	}

	ResetStatus ();
	{
		CWaiter w;
		CCritSec::CLock lock (g_cs);
		g_map.clear ();
		g_Selection.clear ();
		ClearSelPoints ();
	}

	// Invoke all 'after' resetters.
	for (pRM = CInvokeOnRemoveMap::s_pFirst; pRM != NULL; pRM = pRM->m_pNext) {
		if (! pRM->m_bBefore)
			pRM->m_pf ();
	}

	RefreshScales ();
	Refresh ();

	ClearLog ();

	UpdateMenu ();

	return true;
}
