//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2008-2009
//
// RG node properties dialog.
//

# include "StdAfx.h"
# include "Resource.h"
# include "NodeProperties.h"
# include "PropertyPages.h"
# include "SetHLevel.h"
# include "Map.h"
# include "Types.h"
# include "MapView.h"
# include "View3D.h"
# include "Settings.h"
# include "Wrappers.h"
# include "Globals.h"

void CNodeHLevelPage::_EnableCtrls (bool _bHasHLevel) {
	EnableDlgItem (IDC_HLEVEL,      _bHasHLevel);
	EnableDlgItem (IDC_HLEVEL_SPIN, _bHasHLevel);
	EnableDlgItem (IDC_SHOW_3D,     _bHasHLevel);
}

void CNodeHLevelPage::_NormalizeHLevel (int & _nHLevel) {
	if (_nHLevel < -2)
		_nHLevel = -2;
	else if (_nHLevel > 15)
		_nHLevel = 15;
}

bool CNodeHLevelPage::OnInit (HWND _hwndFocus) {
	CNodeProperties * const pData = static_cast<CNodeProperties *> (GetPP ());

	// 3D level.
	if (pData->cElement < pData->pRgn->elements.size ()) {
		const rgn_element_t & element = pData->pRgn->elements [pData->cElement];
		if (pData->cPoint < element.points.size ()) {
			if (element.pExtra) {
				const point_extras_t & PointExtras = element.pExtra->PointExtras;
				const point_extra_t * const pPointExtra = pData->cPoint < PointExtras.size () ? & PointExtras [pData->cPoint] : NULL;
				if (pPointExtra && pPointExtra->HasHLevel ()) {
					if (pPointExtra->HasHLevel ()) {
						CheckDlgButton (IDC_HAS_HLEVEL, true);
						_EnableCtrls (true);

						const int nHLevel = pPointExtra->GetHLevel ();
						SetDlgItemInt (IDC_HLEVEL, nHLevel);

						return true;
					}
				}
			}
		}
	}

	CheckDlgButton (IDC_HAS_HLEVEL, false);
	SetDlgItemText (IDC_HLEVEL, "0");

	_EnableCtrls (false);

	return true;
}

void CNodeHLevelPage::OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
	switch (_wCmd) {
		case IDC_HAS_HLEVEL: {
			const bool bHasHLevel = IsDlgButtonChecked (IDC_HAS_HLEVEL);
			_EnableCtrls (bHasHLevel);
			PropSheetChanged ();
			break;
		}

		case IDC_HLEVEL: {
			if (_wNotifyCode == EN_CHANGE) {
//				EnableDlgItem (IDOK, bValid);
			} else if (_wNotifyCode == EN_UPDATE)
				PropSheetChanged ();
			break;
		}

		case IDC_SHOW_3D: {
			CNodeProperties * const pData = static_cast<CNodeProperties *> (GetPP ());
			if (pData->cElement < pData->pRgn->elements.size ()) {
				rgn_element_t & element = pData->pRgn->elements [pData->cElement];
				if (pData->cPoint < element.points.size ()) {
					const point_t & pt = element.points [pData->cPoint];

					ViewIn3D (pt);
				}
			}
			break;
		}
	}
}

bool CNodeHLevelPage::OnNotify (int _idCtrl, NMHDR * _pInfo) {
	if (_pInfo->code == UDN_DELTAPOS && _idCtrl == IDC_HLEVEL_SPIN) {
		const NMUPDOWN * const pUD = reinterpret_cast<const NMUPDOWN *> (_pInfo);
		const bool bUp = pUD->iDelta < 0;

		int nHLevel = GetDlgItemInt (IDC_HLEVEL);

		if (bUp)
			++ nHLevel;
		else
			-- nHLevel;
		_NormalizeHLevel (nHLevel);

		SetDlgItemInt (IDC_HLEVEL, nHLevel);
		PropSheetChanged ();
	}

	return CPage::OnNotify (_idCtrl, _pInfo);
}

void CNodeHLevelPage::OnActivatePage (const PSHNOTIFY *) {
	SaveSetting ("NodePropertiesPage", (DWORD) 3);
}

void CNodeHLevelPage::OnApply (const PSHNOTIFY *) {
	CNodeProperties * const pData = static_cast<CNodeProperties *> (GetPP ());

	// 3D level.
	if (pData->cElement < pData->pRgn->elements.size ()) {
		rgn_element_t & element = pData->pRgn->elements [pData->cElement];
		if (pData->cPoint < element.points.size ()) {
			const bool bHasHLevel = IsDlgButtonChecked (IDC_HAS_HLEVEL);
			if (bHasHLevel) {
				int nHLevelOld = 0;
				if (element.pExtra) {
					point_extras_t & PointExtras = element.pExtra->PointExtras;
					const point_extra_t * const pPointExtra = pData->cPoint < PointExtras.size () ? & PointExtras [pData->cPoint] : NULL;
					if (pPointExtra && pPointExtra->HasHLevel ())
						nHLevelOld = pPointExtra->GetHLevel ();
				}

				int nHLevel = GetDlgItemInt (IDC_HLEVEL);
				_NormalizeHLevel (nHLevel);
				SetHLevel (pData->pRgn, pData->cElement, pData->cPoint, nHLevel, true);
			} else
				SetHLevel (pData->pRgn, pData->cElement, pData->cPoint, 0, false);

			if (g_bShowHLevel)
				Refresh ();
			RefreshViewIn3D ();
		}
	}
}
