//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2006-2009
//
// RG node properties dialog.
//

# include "StdAfx.h"
# include "Resource.h"
# include "NodeProperties.h"
# include "PropertyPages.h"
# include "Map.h"
# include "Types.h"
# include "NavitelTypes.h"
# include "MapView.h"
# include "Settings.h"
# include "Wrappers.h"
# include "Globals.h"

CNodeProperties::CNodeProperties (rgn_t * _pRgn, size_t _cElement, size_t _cPoint) :
	pExtra (NULL),
	pRgn (_pRgn), cElement (_cElement), cPoint (_cPoint),
	pElement (_pRgn && _cElement < _pRgn->elements.size () ? & (_pRgn->elements [_cElement]) : NULL)
{}

/////////////////////////////////////////////////////

void ShowNodeProperties (HWND _hWnd, rgn_t & _rgn, size_t _cElement, size_t _cPoint) {
	CCritSec::CLock lock (g_cs);

	bool bAddress = false;

	CNodeProperties np (& _rgn, _cElement, _cPoint);

	np.AddPage (new CNodePage);
	if (_rgn.IsRoad ()) {
		np.AddPage (new CNodeRoutingPage);

		bAddress = (g_map.pTypeSet->dwFeatures & CTypeSet::tsfHasRoadAddress) != 0;
		if (bAddress)
			np.AddPage (new CNodeNumbersPage);

		if (g_map.pTypeSet == & g_tsNavitel)
			np.AddPage (new CNodeHLevelPage);
	}

	size_t cPage = GetSetting ("NodePropertiesPage", (DWORD) 0);
	if (cPage == 3 && ! bAddress)
		-- cPage;
	np.ShowModal (_hWnd, IDS_NODE_PROPERTIES, cPage);
}

/////////////////////////////////////////////////////

# include "MovePoint.h"
# include "Types.h"
# include "PrintUnits.h"
# include "EditHelpers.h"

bool CNodePage::OnInit (HWND _hwndFocus) {
	CNodeProperties * const pData = static_cast<CNodeProperties *> (GetPP ());

	if (pData->cElement < pData->pRgn->elements.size ()) {
		const rgn_element_t & element = pData->pRgn->elements [pData->cElement];
		if (pData->cPoint < element.points.size ()) {
			const point_t & point = element.points [pData->cPoint];

			char buf [20];

			// Latitude.
			PrintLat (buf, point.y);
			SetDlgItemText (IDC_LAT, buf);

			// Longitude.
			PrintLon (buf, point.x);
			SetDlgItemText (IDC_LON, buf);
		}

		// Ordinal.
		SetDlgItemUInt (IDC_ORDINAL, pData->cPoint);
	}

	return true;
}

void CNodePage::OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
	switch (_wCmd) {
		case IDC_LAT:
		case IDC_LON: {
			if (_wNotifyCode == EN_CHANGE) {
				const bool bValid = 
					ValidateAsArc (GetDlgItem (IDC_LON)) &&
					ValidateAsArc (GetDlgItem (IDC_LAT));

				EnableDlgItem (IDOK, bValid);
			} else if (_wNotifyCode == EN_UPDATE)
				PropSheetChanged ();
			break;
		}
	}
}

void CNodePage::OnActivatePage (const PSHNOTIFY *) {
	SaveSetting ("NodePropertiesPage", (DWORD) 0);
}

void CNodePage::OnApply (const PSHNOTIFY *) {
	CNodeProperties * const pData = static_cast<CNodeProperties *> (GetPP ());

	if (pData->cElement < pData->pRgn->elements.size ()) {
		rgn_element_t & element = pData->pRgn->elements [pData->cElement];
		if (pData->cPoint < element.points.size ()) {
			point_t point;
			point.x = GetTextAsArc (GetDlgItem (IDC_LON));
			point.y = GetTextAsArc (GetDlgItem (IDC_LAT));

			g_map.pTypeSet->pfRoundPoint (g_map, element.cLevel, point);

			MovePoint (pData->pRgn, pData->cElement, pData->cPoint, point);

			Refresh ();
		}
	}
}
