# ifndef __NodesExtra_H_
# define __NodesExtra_H_

# include "FVector.h"

//
// Extra attributes for polyline nodes.
//

struct rgnode_t;
struct city_t;
struct zip_t;

	// House numbering data.
struct numbers_t {
	numbers_t () :
		bHasNumbers (0),
		wLeftBefore  (0), wLeftAfter (0), wLeftParity (0),
		wRightBefore (0), wRightAfter (0), wRightParity (0),
		pCityLeftAfter (NULL), pCityRightAfter (NULL),
		pZipLeftAfter  (NULL), pZipRightAfter (NULL)
	{}

	bool operator == (const numbers_t & _other) const;

	DWORD wLeftBefore  : 31,
	      bHasNumbers  : 1,
	      wLeftAfter   : 30,
	      wLeftParity  : 2;
	DWORD wRightBefore,
	      wRightAfter  : 30,
	      wRightParity : 2;
	city_t * pCityLeftAfter, * pCityRightAfter;
	zip_t * pZipLeftAfter, * pZipRightAfter;

	void ReverseSides (numbers_t * _pPrev, bool _bLast);
};

struct point_extra_t {
	point_extra_t () : pRGNode (NULL), bHasHLevel (false), nHLevel (0) {}
	explicit point_extra_t (rgnode_t * _pRGNode) : pRGNode (_pRGNode), bEndNode (false) {}

	// True if neither RG node nor house numbers nor 3D-level.
	bool empty () const;

	bool operator == (const point_extra_t & _other) const;

	const rgnode_t * GetRGNode () const {return pRGNode;}
	      rgnode_t * GetRGNode ()       {return pRGNode;}
	void SetRGNode (rgnode_t * _pRGNode, bool _bEndNode);
	void SetRGNode (rgnode_t * _pRGNode, size_t _cPoint, size_t _cPoints) {
		SetRGNode (_pRGNode, _cPoint == 0 || _cPoint + 1 == _cPoints);
	}

	// Set node reference w/o changing counters inside node.
	void AttachRGNode (rgnode_t * _pRGNode, size_t _cPoint, size_t _cPoints) {
		pRGNode = _pRGNode;
		bEndNode = _cPoint == 0 || _cPoint + 1 == _cPoints;
	}

	void SetEndNode (bool _bEndNode) {bEndNode = _bEndNode;}

	bool HasHLevel () const {return bHasHLevel;}
	int GetHLevel () const {return nHLevel;}
	void SetHLevel (int _nHLevel, bool _bHasHLevel = true) {
		bHasHLevel = _bHasHLevel;
		nHLevel = _nHLevel;
	}
	bool EqualHLevel (const point_extra_t & _other) const {
		if (bHasHLevel != _other.bHasHLevel)
			return false;
		if (! bHasHLevel)
			return true;
		return nHLevel == _other.nHLevel;
	}

  private:
	// Routing graph node.
	rgnode_t * pRGNode;
	// NOTE: this affect cardinality.
	bool bEndNode;

	// 3D level.
	bool bHasHLevel;
	signed char nHLevel;

  public:
	// House numbering data.
	numbers_t Numbers;
};

typedef fvector<point_extra_t> point_extras_t;

bool IsNotEmpty (const point_extras_t & _PointExtras);
bool HasRGNodes (const point_extras_t & _PointExtras);
bool HasNumbers (const point_extras_t & _PointExtras);
bool HasHLevel (const point_extras_t & _PointExtras);

# endif // __NodesExtra_H_