//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Note Drawing class.
//

# include "StdAfx.h"
# include "Resource.h"
# include "NoteDrawing.h"
# include "SetNoteDrawing.h"
# include "Map.h"
# include "MapView.h"
# include "CreateObject.h"
# include "ParseHelpers.h"
# include "Window.h"
# include "Dialog.h"
# include "DrawHelpers.h"
# include "Settings.h"
# include "Globals.h"

# include "NoteDrawing.inl"

bool note_drawing_t::IsDuplicate (const note_drawing_t & _other) const {
	return
		this->cWidth == _other.cWidth &&
		this->col    == _other.col;
}

void note_drawing_t::Save (const rgn_t & _rgn, string_t & _str) {
	const note_drawing_t & note = * _rgn.pExtra->pNoteDrawing;

	char strBuf [64];
	::sprintf (strBuf, "%d,%d", note.cWidth, note.col);
	_str.append (strBuf);

	const size_t cElements = _rgn.elements.size ();
	for (size_t cElement = 0; cElement < cElements; ++ cElement) {
		const rgn_element_t & element = _rgn.elements [cElement];
		if (element.bDeleted)
			continue;

		const points_t & points = element.points;
		const size_t cPoints = points.size ();
		if (cPoints <= 1)
			continue;

		::sprintf (strBuf, ";%d", cPoints);
		_str.append (strBuf);

		for (size_t cPoint = 0; cPoint < cPoints; ++ cPoint) {
			const point_t & point = points [cPoint];

			::sprintf (strBuf, ",(%.6f,%.6f)", point.y, point.x);
			_str.append (strBuf);
		}
	}
}

bool note_drawing_t::Load (rgn_t & _rgn, const char * _pStart, const char * _pEnd, const char ** _ppEnd) {
	const char * & p = * _ppEnd;
	p = _pStart;
	OmitSpaces (& p, _pEnd);

	// Width.
	unsigned long ulWidth = 0;
	if (! ParseDecU (p, _pEnd, & p, ulWidth))
		return false;
	OmitSpaces (& p, _pEnd);
	if (* p != ',')
		return false;
	++ p;
	OmitSpaces (& p, _pEnd);

	// Color.
	unsigned long ulColor = 0;
	if (! ParseDecU (p, _pEnd, & p, ulColor))
		return false;
	OmitSpaces (& p, _pEnd);

	if (! _rgn.IsNote ())
		_rgn.pExtra->pNoteDrawing = new note_drawing_t;
	note_drawing_t & nd = * _rgn.pExtra->pNoteDrawing;
	nd.col    = ulColor;
	nd.cWidth = ulWidth;

	bool bOK = false;

	while (p < _pEnd) {
		if (* p != ';')
			break;
		++ p;
		OmitSpaces (& p, _pEnd);

		// Points.
		unsigned long ulPoints = 0;
		if (! ParseDecU (p, _pEnd, & p, ulPoints))
			return false;
		OmitSpaces (& p, _pEnd);
		if (* p != ',')
			return false;
		++ p;
		OmitSpaces (& p, _pEnd);

		// Points.
		points_t points;
		points.reserve (ulPoints);
		while (p < _pEnd) {
			if (* p == ',')
				++ p;
			if (* p != '(')
				break;
			++ p;

			double fY = 0;
			if (! ParseReal (p, _pEnd, & p, fY))
				break;
			if (* p != ',')
				break;
			++ p;

			double fX = 0;
			if (! ParseReal (p, _pEnd, & p, fX))
				break;
			if (* p != ')')
				break;
			++ p;

			points.push_back (point_t (fX, fY));
		}
		if (points.size () < 2)
			continue;

		_rgn.elements.push_back (rgn_element_t ());
		rgn_element_t & element = _rgn.elements.back ();
		element.cLevel = 0;
		element.points.swap (points);

		bOK = true;
	}

	return bOK;
}
