//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// 'Options' dialog, 'Edit' page.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Options.h"
# include "MapView.h"
# include "Selection.h"
# include "PropertyPages.h"
# include "Keyboard.h"
# include "CustomizeMenu.h"
# include "ViewPlugins.h"
# include "ResMan.h"
# include "Settings.h"
# include "Globals.h"

INT_PTR CALLBACK _OptionsCommonProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_INITDIALOG: {
			// Language.
			const HWND hLang = ::GetDlgItem (_hWnd, IDC_LANGUAGE);
			for (size_t l = 0; l < GetLanguagesNum (); ++ l) {
				::SendMessage (hLang, CB_INSERTSTRING, l, reinterpret_cast<LPARAM> (GetLanguageName (l).c_str ()));
				::SendMessage (hLang, CB_SETITEMDATA,  l, l);
			
				if (l == GetCurrentLanguage ())
					::SendMessage (hLang, CB_SETCURSEL, l, 0);
			}

			// Buttons for multiple selection.
			const HWND hMSel = ::GetDlgItem (_hWnd, IDC_MSEL);
			::SendMessage (hMSel, CB_INSERTSTRING, 0, reinterpret_cast<LPARAM> ("SHIFT for multiple selection, CTRL for elements selection"));
			::SendMessage (hMSel, CB_INSERTSTRING, 1, reinterpret_cast<LPARAM> ("CTRL for multiple selection, SHIFT for elements selection"));
			::SendMessage (hMSel, CB_SETCURSEL, g_bMSelByShift ? 0 : 1, 0);

			// View plug-ins.
			const HWND hVP = ::GetDlgItem (_hWnd, IDC_PLUGINS);
			if (! ViewPlugins_CanCustomize ())
				EnableWindow (hVP, FALSE);
		
			return TRUE;
		}

		case WM_COMMAND:
			switch (LOWORD (_wParam)) {
				case IDC_LANGUAGE:
				case IDC_MSEL:
					if (HIWORD (_wParam) == CBN_SELCHANGE) {
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
						return TRUE;
					}
					break;

				case IDC_KEYBOARD:
					CustomizeKeyboard (_hWnd);
					break;

				case IDC_MENU:
					CustomizeMenu (_hWnd);
					break;

				case IDC_PLUGINS:
					ViewPlugins_Customize (_hWnd);
					break;
			}
			break;

		case WM_NOTIFY: {
			const NMHDR * const pInfo = reinterpret_cast<const NMHDR *> (_lParam);
			switch (pInfo->code) {
				case PSN_APPLY: {
					// Language.
					const HWND hLang = ::GetDlgItem (_hWnd, IDC_LANGUAGE);
					const size_t cSelIndex = ::SendMessage (hLang, CB_GETCURSEL, 0, 0);
					if (cSelIndex == -1)
						break;
					const size_t l = ::SendMessage (hLang, CB_GETITEMDATA, cSelIndex, 0);

					ActivateLanguage (l);

					// Buttons for multiple selection.
					{
						const HWND hMSel = ::GetDlgItem (_hWnd, IDC_MSEL);
						const size_t cSelIndex = ::SendMessage (hMSel, CB_GETCURSEL, 0, 0);
						g_bMSelByShift = cSelIndex == 0;
						SaveSetting ("MSelByShift", g_bMSelByShift);
					}

					break;
				}
			}
		}
	}

	return FALSE;
}
