//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// 'Options' dialog, 'Edit' page.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Options.h"
# include "MapView.h"
# include "Selection.h"
# include "ToolEditNodes.h"
# include "PropertyPages.h"
# include "Types.h"
# include "SelectColor.h"
# include "Button.h"
# include "ComboBox.h"
# include "Settings.h"
# include "Globals.h"

const CComboBoxData c_cbdNodeShapes [] = {
	{IDS_NODESHAPE_STD,   0},
	{IDS_NODESHAPE_SERIF, 1},
};

INT_PTR CALLBACK _OptionsEditProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_INITDIALOG: {
			CenterPropertyPages (_hWnd);

			// Snap to grid.
			::CheckDlgButton (_hWnd, IDC_ENABLE_GRID,  GetSetting ("OptionSnapToGrid", true) ? BST_CHECKED : BST_UNCHECKED);

			// Stick to neighbours.
			::CheckDlgButton (_hWnd, IDC_ENABLE_STICK, GetSetting ("OptionStick",      true) ? BST_CHECKED : BST_UNCHECKED);

			// New object color & width.
			ListView_SetBkColor (::GetDlgItem (_hWnd, IDC_COLOR_NEW), GetNewObjectColor ());
			::SetDlgItemInt (_hWnd, IDC_WIDTH_NEW, GetNewObjectWidth (), FALSE);

			// Default type set.
			const HWND hTypeSet = ::GetDlgItem (_hWnd, IDC_TYPESET);
			const CTypeSet * const pDefaultTypeSet = GetDefaultTypeSet ();
			size_t cTypeSet = 0;
			for (const CTypeSet * pTypeSet = CTypeSet::s_pFirst; pTypeSet; pTypeSet = pTypeSet->m_pNext) {
				if (pTypeSet == & g_tsNull)
					continue;

				::SendMessage (hTypeSet, CB_INSERTSTRING, cTypeSet, reinterpret_cast<LPARAM> (pTypeSet->strDisplayName));
				::SendMessage (hTypeSet, CB_SETITEMDATA,  cTypeSet, reinterpret_cast<LPARAM> (pTypeSet));

				if (pTypeSet == pDefaultTypeSet)
					::SendMessage (hTypeSet, CB_SETCURSEL, cTypeSet, 0);

				++ cTypeSet;
			}

			// Node shapes.
			FillComboBox (_hWnd, IDC_NODE_SHAPE, c_cbdNodeShapes, ARRAY_SIZE (c_cbdNodeShapes), GetNodeShape ());

			// Notify if paste results in objects duplicates.
			::CheckDlgButton (_hWnd, IDC_NOTIFY_PASTE_DUPLICATE,         GetSetting_NotifyPasteDuplicate    () ? BST_CHECKED : BST_UNCHECKED);

			// Notify id creating results in object out of 0-th level.
			::CheckDlgButton (_hWnd, IDC_NOTIFY_CREATING_OUT_OF_0_LEVEL, GetSetting_NotifyCreatingNon0Level () ? BST_CHECKED : BST_UNCHECKED);

			return TRUE;
		}

		case WM_COMMAND: {
			switch (LOWORD (_wParam)) {
				case IDC_ENABLE_GRID:
				case IDC_ENABLE_STICK:
				case IDC_NOTIFY_PASTE_DUPLICATE:
				case IDC_NOTIFY_CREATING_OUT_OF_0_LEVEL:
					PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					return TRUE;

				case IDC_TYPESET:
					if (HIWORD (_wParam) == CBN_SELCHANGE) {
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
						return TRUE;
					}
					return TRUE;

				case IDC_SELECT_COLOR_NEW: {
					const HWND hColor = ::GetDlgItem (_hWnd, IDC_COLOR_NEW);
					COLORREF cr = ListView_GetBkColor (hColor);
					if (SelectColor (_hWnd, cr)) {
						ListView_SetBkColor (hColor, cr);
						::InvalidateRect (hColor, NULL, TRUE);
						::UpdateWindow (hColor);

						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					}

					return TRUE;
				}

				case IDC_WIDTH_NEW:
					switch (HIWORD (_wParam)) {
						case EN_CHANGE:
							PropSheet_Changed (::GetParent (_hWnd), _hWnd);
							return TRUE;
					}
					break;

				case IDC_COLORS_SELECTION:
					if (AdjustColorsForSelection (_hWnd))
						Redraw ();
					break;

				case IDC_COLORS_NODES:
					if (AdjustColorsForNodes (_hWnd))
						Refresh ();
					break;

				case IDC_NODE_SHAPE:
					if (HIWORD (_wParam) == CBN_SELCHANGE)
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					break;
			}
			break;
		}

		case WM_NOTIFY: {
            const NMHDR * const pInfo = reinterpret_cast<NMHDR *> (_lParam);
            switch (pInfo->code) {
				case PSN_APPLY: {
					g_bEnableSnapToGrid = ::IsDlgButtonChecked (_hWnd, IDC_ENABLE_GRID)  == BST_CHECKED;
					g_bEnableSticking   = ::IsDlgButtonChecked (_hWnd, IDC_ENABLE_STICK) == BST_CHECKED;

					SaveSetting ("OptionSnapToGrid", g_bEnableSnapToGrid);
					SaveSetting ("OptionStick",      g_bEnableSticking);

					// Color.
					SaveSetting ("Color_New", ListView_GetBkColor (::GetDlgItem (_hWnd, IDC_COLOR_NEW)));

					// Width.
					size_t cWidth = ::GetDlgItemInt (_hWnd, IDC_WIDTH_NEW, NULL, FALSE);
					if (cWidth == 0)
						cWidth = 1;
					if (cWidth > 10)
						cWidth = 10;
					SaveSetting ("Width_New", (DWORD) cWidth);

					// Node shape.
					int nNodeShape = 0;
					if (GetComboBoxItemData (_hWnd, IDC_NODE_SHAPE, nNodeShape))
						SaveSetting ("NodeShape", (DWORD) nNodeShape);

					// Default type set.
					{
						const HWND hTypeSet = ::GetDlgItem (_hWnd, IDC_TYPESET);
						const size_t cSelIndex = ::SendMessage (hTypeSet, CB_GETCURSEL, 0, 0);
						const CTypeSet * const pTypeSet = reinterpret_cast<const CTypeSet *> (::SendMessage (hTypeSet, CB_GETITEMDATA, cSelIndex, 0));
						SaveSetting ("DefaultTypeSet", pTypeSet->strID);
					}

					// Notify if paste results in objects duplicates.
					const bool bNotifyPasteDuplicate    = ::IsDlgButtonChecked (_hWnd, IDC_NOTIFY_PASTE_DUPLICATE)         == BST_CHECKED;
					SaveSetting ("NotifyPasteDuplicate",    bNotifyPasteDuplicate);

					const bool bNotifyCreatingNon0Level = ::IsDlgButtonChecked (_hWnd, IDC_NOTIFY_CREATING_OUT_OF_0_LEVEL) == BST_CHECKED;
					SaveSetting ("NotifyCreatingNon0Level", bNotifyCreatingNon0Level);

					Redraw ();

					break;
				}
			}
			break;
		}
	}

	return FALSE;
}

COLORREF GetNewObjectColor () {
	return GetSetting ("Color_New", (DWORD) c_colGray);
}

size_t GetNewObjectWidth () {
	return GetSetting ("Width_New", (DWORD) 1);
}

DWORD GetNodeShape () {
	return GetSetting ("NodeShape", (DWORD) 0);
}

const CTypeSet * GetDefaultTypeSet () {
	char strTypeSet [32];
	GetSetting ("DefaultTypeSet", strTypeSet, sizeof (strTypeSet), "Garmin");
	return CTypeSet::FindTypeSet (strTypeSet);
}

bool GetSetting_NotifyPasteDuplicate () {
	return GetSetting ("NotifyPasteDuplicate", true);
}

bool GetSetting_NotifyCreatingNon0Level () {
	return GetSetting ("NotifyCreatingNon0Level", false);
}
