//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2008
//
// 'Options' dialog, 'GPS Tracking' page.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Options.h"
# include "MapView.h"
# include "Tracking.h"
# include "ComboBox.h"
# include "Settings.h"
# include "Globals.h"

INT_PTR CALLBACK _OptionsGpsTrackingProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_INITDIALOG: {
			// Track format.
			const HWND hFormat = ::GetDlgItem (_hWnd, IDC_TRACK_FORMAT);

			char strExt [16];
			GetSetting ("TrackWriter", strExt, ARRAY_SIZE (strExt), "mps");

			size_t cItem = 0;
			for (CTrackWriterTypeInfo * p = CTrackWriterTypeInfo::s_pFirst; p != NULL; p = p->m_pNext) {
				AddComboBox (hFormat, cItem, (LPARAM) p->m_strExt, p->m_strDisplayName);
				if (::strcmp (p->m_strExt, strExt) == 0)
					SelectComboBox (hFormat, cItem);

				++ cItem;
			}

			return TRUE;
		}

		case WM_COMMAND: {
			switch (LOWORD (_wParam)) {
				case IDC_TRACK_FORMAT:
					if (HIWORD (_wParam) == CBN_SELCHANGE)
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					return TRUE;			
			}
			
			break;
		}
		
		case WM_NOTIFY: {
			const NMHDR * const pInfo = reinterpret_cast<const NMHDR *> (_lParam);
			switch (pInfo->code) {
				case PSN_APPLY: {
					// Track format.
					const HWND hFormat = ::GetDlgItem (_hWnd, IDC_TRACK_FORMAT);
					int nExt = 0;
					if (GetComboBoxItemData (_hWnd, IDC_TRACK_FORMAT, nExt)) {
						const char * const strExt = (const char *) nExt;
						SaveSetting ("TrackWriter", strExt);
					}
					
					break;
				}
			}

			break;
		}	
	}

	return FALSE;
}

