//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// 'Options' dialog, 'View' page.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Options.h"
# include "Settings.h"
# include "MapView.h"
# include "MapSkins.h"
# include "SelectColor.h"
# include "SelectFont.h"
# include "TypesDraw.h"
# include "DrawAddress.h"
# include "DrawNodes.h"
# include "Button.h"
# include "Globals.h"

# pragma const_seg (".text")

const LOGFONT c_lfDefault = {
	12, 0, 0, 0,
	FW_NORMAL, FALSE, FALSE, FALSE,
	DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
	DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Arial"
};

# pragma const_seg ()

////////////////////////////////////////////////////////////////

static
void _SaveCheckbox (HWND _hWnd, UINT _uiCmd, bool & _b, const char * _strSetting) {
	_b = ::IsDlgButtonChecked (_hWnd, _uiCmd) == BST_CHECKED;
	SaveSetting (_strSetting, _b);
}

INT_PTR CALLBACK _OptionsViewProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	switch (_uMsg) {
		case WM_INITDIALOG:
			// Map background.
			ListView_SetBkColor (::GetDlgItem (_hWnd, IDC_COLOR_BG), GetBackgroundColor ());

			// Transparent polygons.
			SetButtonIcon (_hWnd, IDC_TPV_HATCHED, IDI_HATCHED);
			SetButtonIcon (_hWnd, IDC_TPV_HOLLOW,  IDI_HOLLOW);
			if (GetTransparentPolygonFill ())
				::CheckRadioButton (_hWnd, IDC_TPV_HATCHED, IDC_TPV_HATCHED, IDC_TPV_HATCHED);
			else
				::CheckRadioButton (_hWnd, IDC_TPV_HOLLOW,  IDC_TPV_HOLLOW,  IDC_TPV_HOLLOW);

			// Show tracks/routes above map.
			::CheckDlgButton (_hWnd, IDC_SHOW_WTR_ON_TOP,  g_bDrawWTROnTop          ? BST_CHECKED : BST_UNCHECKED);

			// Track.
			ListView_SetBkColor (::GetDlgItem (_hWnd, IDC_TRACK_COLOR), GetDefaultTrackColor ());
			::SetDlgItemInt (_hWnd, IDC_TRACK_WIDTH, GetDefaultTrackWidth (), FALSE);

			// Route.
			ListView_SetBkColor (::GetDlgItem (_hWnd, IDC_ROUTE_COLOR), GetDefaultRouteColor ());
			::SetDlgItemInt (_hWnd, IDC_ROUTE_WIDTH, GetDefaultRouteWidth (), FALSE);

			// Map infotips.
			::CheckDlgButton (_hWnd, IDC_SHOW_TYPE,        g_bTTShowType            ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_COMMENT,     g_bTTShowComment         ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_STREET,      g_bTTShowStreetAndNumber ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_CITY,        g_bTTShowCity            ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_ZIPCODE,     g_bTTShowZipCode         ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_PHONE,       g_bTTShowPhone           ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_DESCRIPTION, g_bTTShowDescription     ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_ROUTING,     g_bTTShowRouting         ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_TIME,        g_bTTShowTime            ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_SPEED,       g_bTTShowSpeed           ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_HDOP,        g_bTTShowHDOP            ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_VDOP,        g_bTTShowVDOP            ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_SATS,        g_bTTShowSatellites      ? BST_CHECKED : BST_UNCHECKED);
			::CheckDlgButton (_hWnd, IDC_SHOW_FIX,         g_bTTShowFix             ? BST_CHECKED : BST_UNCHECKED);

			return TRUE;

		case WM_COMMAND: {
			switch (LOWORD (_wParam)) {
				case IDC_MAP_SKINS:
					if (MapSkins_EditList (_hWnd))
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					return TRUE;

				case IDC_FONT: {
					LOGFONT lf;
					GetFont (lf);

					if (
						lf.lfHeight == c_lfDefault.lfHeight &&
						lf.lfWeight == c_lfDefault.lfWeight &&
						lf.lfItalic == c_lfDefault.lfItalic &&
						::stricmp (lf.lfFaceName, c_lfDefault.lfFaceName) == 0
					)
						lf.lfHeight = -9;

					if (SelectFontDlg (_hWnd, lf)) {
						if (
							lf.lfHeight == -9 &&
							lf.lfWeight == c_lfDefault.lfWeight &&
							lf.lfItalic == c_lfDefault.lfItalic &&
							::stricmp (lf.lfFaceName, c_lfDefault.lfFaceName) == 0
						)
							lf.lfHeight = c_lfDefault.lfHeight;

						SaveSetting ("Font", & lf, sizeof (lf));

						if (g_hFont)
							::DeleteObject (g_hFont);
						g_hFont = ::CreateFontIndirect (& lf);

						UpdateCommonTypesFonts ();

						Refresh ();
					}
					return TRUE;
				}

				case IDC_SELECT_COLOR_BG: {
					const HWND hColor = ::GetDlgItem (_hWnd, IDC_COLOR_BG);
					COLORREF cr = ListView_GetBkColor (hColor);
					if (SelectColor (_hWnd, cr)) {
						ListView_SetBkColor (hColor, cr);
						::InvalidateRect (hColor, NULL, TRUE);
						::UpdateWindow (hColor);

						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					}

					return TRUE;
				}

				case IDC_TPV_HATCHED:
					if (! GetTransparentPolygonFill ())
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					break;

				case IDC_TPV_HOLLOW:
					if (GetTransparentPolygonFill ())
						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					break;

				case IDC_TRACK_SELECT_COLOR: {
					const HWND hColor = ::GetDlgItem (_hWnd, IDC_TRACK_COLOR);
					COLORREF cr = ListView_GetBkColor (hColor);
					if (SelectColor (_hWnd, cr)) {
						ListView_SetBkColor (hColor, cr);
						::InvalidateRect (hColor, NULL, TRUE);
						::UpdateWindow (hColor);

						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					}
					return TRUE;
				}

				case IDC_SHOW_WTR_ON_TOP:
					PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					return TRUE;

				case IDC_ROUTE_SELECT_COLOR: {
					const HWND hColor = ::GetDlgItem (_hWnd, IDC_ROUTE_COLOR);
					COLORREF cr = ListView_GetBkColor (hColor);
					if (SelectColor (_hWnd, cr)) {
						ListView_SetBkColor (hColor, cr);
						::InvalidateRect (hColor, NULL, TRUE);
						::UpdateWindow (hColor);

						PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					}
					return TRUE;
				}

				case IDC_TRACK_WIDTH:
				case IDC_ROUTE_WIDTH:
					switch (HIWORD (_wParam)) {
						case EN_CHANGE:
							PropSheet_Changed (::GetParent (_hWnd), _hWnd);
							return TRUE;
					}
					break;

				case IDC_SHOW_TYPE:
				case IDC_SHOW_COMMENT:
				case IDC_SHOW_STREET:
				case IDC_SHOW_CITY:
				case IDC_SHOW_ZIPCODE:
				case IDC_SHOW_PHONE:
				case IDC_SHOW_DESCRIPTION:
				case IDC_SHOW_ROUTING:
				case IDC_SHOW_TIME:
				case IDC_SHOW_SPEED:
				case IDC_SHOW_HDOP:
				case IDC_SHOW_VDOP:
				case IDC_SHOW_SATS:
				case IDC_SHOW_FIX:
					PropSheet_Changed (::GetParent (_hWnd), _hWnd);
					return TRUE;

				case IDC_COLORS_ADDRESS:
					if (AdjustColorsForAddressHighlighting (_hWnd)) {
						Refresh ();
					}
					break;

				case IDC_COLORS_NODES:
					if (AdjustColorsForRoutingNodes (_hWnd)) {
						Refresh ();
					}
					break;
			}
			break;
		}

		case WM_NOTIFY: {
			const NMHDR * const pInfo = reinterpret_cast<const NMHDR *> (_lParam);
			switch (pInfo->code) {
				case PSN_APPLY: {
					// Map background.
					SaveSetting ("Color_BG",   ListView_GetBkColor (::GetDlgItem (_hWnd, IDC_COLOR_BG)));

					// Transparent polygons.
					SaveSetting ("TransparentPolygonFill", ::IsDlgButtonChecked (_hWnd, IDC_TPV_HATCHED) == BST_CHECKED);

					// Track color.
					SaveSetting ("TrackColor", ListView_GetBkColor (::GetDlgItem (_hWnd, IDC_TRACK_COLOR)));

					// Track width.
					size_t cTrackWidth = ::GetDlgItemInt (_hWnd, IDC_TRACK_WIDTH, NULL, FALSE);
					if (cTrackWidth == 0)
						cTrackWidth = 3;
					if (cTrackWidth > 10)
						cTrackWidth = 10;
					SaveSetting ("TrackWidth", (DWORD) cTrackWidth);

					// Show tracks/routes above map.
					_SaveCheckbox (_hWnd, IDC_SHOW_WTR_ON_TOP,  g_bDrawWTROnTop,          "DrawWTROnTop");

					// Route color.
					SaveSetting ("RouteColor", ListView_GetBkColor (::GetDlgItem (_hWnd, IDC_ROUTE_COLOR)));

					// Route width.
					size_t cRouteWidth = ::GetDlgItemInt (_hWnd, IDC_ROUTE_WIDTH, NULL, FALSE);
					if (cRouteWidth == 0)
						cRouteWidth = 3;
					if (cRouteWidth > 10)
						cRouteWidth = 10;
					SaveSetting ("RouteWidth", (DWORD) cRouteWidth);

					// Map infotips.
					_SaveCheckbox (_hWnd, IDC_SHOW_TYPE,        g_bTTShowType,            "TTShowType");
					_SaveCheckbox (_hWnd, IDC_SHOW_COMMENT,     g_bTTShowComment,         "TTShowComment");
					_SaveCheckbox (_hWnd, IDC_SHOW_STREET,      g_bTTShowStreetAndNumber, "TTShowStreetAndNumber");
					_SaveCheckbox (_hWnd, IDC_SHOW_CITY,        g_bTTShowCity,            "TTShowCity");
					_SaveCheckbox (_hWnd, IDC_SHOW_ZIPCODE,     g_bTTShowZipCode,         "TTShowZip");
					_SaveCheckbox (_hWnd, IDC_SHOW_PHONE,       g_bTTShowPhone,           "TTShowPhone");
					_SaveCheckbox (_hWnd, IDC_SHOW_DESCRIPTION, g_bTTShowDescription,     "TTShowDescription");
					_SaveCheckbox (_hWnd, IDC_SHOW_ROUTING,     g_bTTShowRouting,         "TTShowRouting");
					_SaveCheckbox (_hWnd, IDC_SHOW_TIME,        g_bTTShowTime,            "TTShowTime");
					_SaveCheckbox (_hWnd, IDC_SHOW_SPEED,       g_bTTShowSpeed,           "TTShowSpeed");
					_SaveCheckbox (_hWnd, IDC_SHOW_HDOP,        g_bTTShowHDOP,            "TTShowHDOP");
					_SaveCheckbox (_hWnd, IDC_SHOW_VDOP,        g_bTTShowVDOP,            "TTShowVDOP");
					_SaveCheckbox (_hWnd, IDC_SHOW_SATS,        g_bTTShowSatellites,      "TTShowSatellites");
					_SaveCheckbox (_hWnd, IDC_SHOW_FIX,         g_bTTShowFix,             "TTShowFix");

					Refresh ();

					break;
				}
			}
			break;
		}
	}

	return FALSE;
}

////////////////////////////////////////////////////////////

COLORREF GetBackgroundColor () {
	return GetSetting ("Color_BG", (DWORD) c_colWhite);
}

bool GetTransparentPolygonFill () {
	return GetSetting ("TransparentPolygonFill", true);
}

COLORREF GetDefaultTrackColor () {
	return GetSetting ("TrackColor", (DWORD) c_colGray);
}

size_t GetDefaultTrackWidth () {
	return GetSetting ("TrackWidth", (DWORD) 3);
}

COLORREF GetDefaultRouteColor () {
	return GetSetting ("RouteColor", (DWORD) c_colBlue);
}

size_t GetDefaultRouteWidth () {
	return GetSetting ("RouteWidth", (DWORD) 3);
}

void GetFont (LOGFONT & _lf) {
	if (IsSettingDefined ("Font"))
		GetSetting ("Font", & _lf, sizeof (_lf));
	else
		_lf = c_lfDefault;
}
