//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
// Panorama is trademarks of ZAO KB Panorama.
//
// Panorama SXF format helpers.
//

# include "StdAfx.h"
# include "PanoramaSXF.h"
# include "FileWriter.h"
# include "Globals.h"

void SXFWriteAttributeInt (CFileWriter & _fw, WORD _wKey, int _nValue) {
	_fw.WriteWORD (_wKey);
	if (0 <= _nValue && _nValue <= 0x7F) {
		_fw.WriteBYTE (1);
		_fw.WriteBYTE (0);
		_fw.WriteBYTE ((BYTE) _nValue);
	} else if (0x80 <= _nValue && _nValue <= 0x7FFF) {
		_fw.WriteBYTE (2);
		_fw.WriteBYTE (0);
		_fw.WriteWORD (_nValue);
	} else {
		_fw.WriteBYTE (4);
		_fw.WriteBYTE (0);
		_fw.WriteDWORD (_nValue);
	}
}

void SXFWriteAttributeString (CFileWriter & _fw, WORD _wKey, const char * _strValue) {
	size_t cLen = ::strlen (_strValue);
	assert (cLen <= 255);
	if (cLen > 255)
		cLen = 255;

	_fw.WriteWORD (_wKey);
	_fw.WriteBYTE (126); // ANSI (Windows)
	_fw.WriteBYTE (cLen);
	_fw.WriteS    (_strValue, cLen);
	_fw.WriteBYTE ('\0');
}

void SXFWriteAttributeFloat (CFileWriter & _fw, WORD _wKey, char * _strFmt, double _fValue) {
	char strBuf [16];
	::sprintf (strBuf, _strFmt, _fValue);

	SXFWriteAttributeString (_fw, _wKey, strBuf);
}

//////////////////////////////////////////////////////////

struct sxf_attribute_t {
	const char * strId;
	WORD wCode;
};

static const sxf_attribute_t g_sxf_Attributes [] = {
	{"WIDTH_M",   11},
	{"HEIGHT_M",  1},
	{"LENGTH_M",  2},
	{"NAME",      9},

	{"ROADID",    1013},
	{"ROADNUMB",  1010},
	{"KMNUMBER",  1011},
	{"KMNUMBERB", 1051},
	{"DIST",      1012},
	{"MVALUE",    1020},
	{"COMMENT",   250},
	{"CONTIN",    1021},

	{"PHONE",     1031},
	{"FAX",       1053},
	{"EMAIL",     1054},

	{"OPERMODE",  1055},
	{"WORKSTA",   1033},
	{"WORKEND",   1034},
	{"DATESTA",   1036},
	{"DATEEND",   1037},

	{"PAYMENT",   1038},
	{"PAYDESK",   1056},

	{"PLAN",      1032},
	{"INFOREL",   1039},
	{"FUEL_TYP",  1040},
	{"WAVE",      1035},

	{"STREET_N",  31201},
	{"HOUSE_N",   31202},
	{"TRUNK_N",   31203},
	{"BUILD_N",   31204},
	{"ESTATE_N",  31205},
	{"STR_CODE",  31206},
	{"TOWN_N",    31207},
	{"TOW_CODE",  31208},
	{"POST_C",    31209},
};

WORD GetSXFAttributeCode (const char * _strAttributeId) {
	for (size_t c = 0; c < ARRAY_SIZE (g_sxf_Attributes); ++ c) {
		if (::stricmp (g_sxf_Attributes [c].strId, _strAttributeId) == 0)
			return g_sxf_Attributes [c].wCode;
	}

	assert (false);
	return 0;
}

const char * GetSXFAttributeId (WORD _wAttributeCode) {
	for (size_t c = 0; c < ARRAY_SIZE (g_sxf_Attributes); ++ c) {
		if (g_sxf_Attributes [c].wCode == _wAttributeCode)
			return g_sxf_Attributes [c].strId;
	}

	assert (false);
	return 0;
}
