# ifndef __PanoramaSXF_H_
# define __PanoramaSXF_H_

const DWORD c_dwPanoramaSXF_passport_signature = 0x00465853;

struct CSxfPassport {
	DWORD dwSignature;
	DWORD dwSize;
	DWORD dwFormatVer;
	DWORD dwCheckSum;
	char strCreationDate [12];
	char strNomenclature [32];
	DWORD dwScale;
	char strName [32];
	DWORD fDataStatus     : 2,
	      bProjOK         : 1,
	      fTruePosition   : 2,
	      fTypeCoding     : 2,
	      bGenTable       : 1,
	      btLblCoding     : 8,
	      btPrecPosition  : 8,
	      bSpecialSorting : 1,
	      fReserved       : 7;
	DWORD dwClassifier;

	double dfX_sw;
	double dfY_sw;
	double dfX_nw;
	double dfY_nw;
	double dfX_ne;
	double dfY_ne;
	double dfX_se;
	double dfY_se;

	double dfB_sw;
	double dfL_sw;
	double dfB_nw;
	double dfL_nw;
	double dfB_ne;
	double dfL_ne;
	double dfB_se;
	double dfL_se;

	BYTE btEllipsoid;
	BYTE btVerticalDatum;
	BYTE btProjection;
	BYTE btCoordinateSystem;
	BYTE btPlaneUnits;
	BYTE btElevationUnit;
	BYTE btFrameType;
	BYTE btMapKind;

	char strUpdateDate [12];
	BYTE btDataSource;
	BYTE btDataKind2;
	WORD wReserved;
	double dfMagneticVariation;
	double dfCovergenceOfMeridians;
	double dfMagneticVariationChangePerYear;
	char strMagneticVariationDate [12];
	DWORD dwReserved;
	double dfReliefStep;
	double dfReserved;

	DWORD dwResolution;

	LONG lScannerFrame [8];

	DWORD dwFrameClass;

	double dfParallel1;
	double dfParallel2;
	double dfCentralMeridian;
	double dfMainParallel;
	double dfLatitude0;
	double dfLongitude0;
};

const DWORD c_dwPanoramaSXF_dd_signature = 0x00544144;

struct CSxfDataDescriptor {
	DWORD dwSignature;
	DWORD dwSize;

	char strNomenclature [32];

	DWORD dwRecordsNum;

	DWORD fDataStatus     : 2,
	      bProjOK         : 1,
	      fTruePosition   : 2,
	      fCoding         : 2,
	      bGenTable       : 1,
	      fLblCoding      : 8,
	      fReserved       : 16;

	WORD wClassifierOfObjects;
	WORD wClassifierOfSemantics;
};

const DWORD c_dwPanoramaSXF_rec_signature = 0x7FFF7FFF;

struct CSxfRecord {
	DWORD dwSignature;
	DWORD dwSize;

	DWORD dwMetricsLen;
	DWORD dwTypeCode;
	WORD wOrdinal;
	WORD wGroup;

	BYTE fKind          : 4,
	     bNFrameCrossed : 1,
	     bEFrameCrossed : 1,
	     bSFrameCrossed : 1,
	     bWFrameCrossed : 1;
	BYTE bClosed        : 1,
	     bHasSemantics  : 1,
	     bDoubleSize    : 1,
	     bRefVector     : 1,
	     bTextInUnicode : 1,
	     fReserved      : 3;
	BYTE bVector        : 1,
	     b3D            : 1,
	     bFloat         : 1,
	     bHasText       : 1,
	     bHasIcon       : 1,
	     bScalableIcon  : 1,
	     fSpline        : 2;

	BYTE fScaleLo : 4,
	     fScaleHi : 4;

	DWORD dwPoints;

	WORD wSubObjects;
	WORD wPoints;
};

const DWORD c_dwPanoramaSXF_r3d_signature = 0x7FFF7FFD;

struct CSxfRecord3D {
	DWORD dwSignature;
	DWORD dwSize;

	double dfDX;
	double dfDy;
	double dfDH;
	double dfDA;

	DWORD dwIdxInLib;
};

class CFileWriter;
void SXFWriteAttributeInt    (CFileWriter & _fw, WORD _wKey, int _nValue);
void SXFWriteAttributeString (CFileWriter & _fw, WORD _wKey, const char * _strValue);
void SXFWriteAttributeFloat  (CFileWriter & _fw, WORD _wKey, char * _strFmt, double _fValue);

//////////////////////////////////

WORD GetSXFAttributeCode (const char * _strAttributeId);
const char * GetSXFAttributeId (WORD _wAttributeCode);

# endif // __PanoramaSXF_H_