//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Parsing helper utilities.
//

# include "StdAfx.h"
# include "ParseHelpers.h"
# include "Log.h"
# include "Globals.h"

char * FindIStr (const char * _str, const char * _strSub) {
	const size_t cLen    = ::strlen (_str);
	const size_t cLenSub = ::strlen (_strSub);
	for (size_t c = 0; c < cLen; ++ c) {
		if (HasPrefix (_str + c, _str + cLen, _strSub, cLenSub))
			return const_cast<char *> (_str + c);
	}
	return NULL;
}

bool IsBinary (const char * _pBegin, const char * _pEnd) {
	assert (_pBegin <= _pEnd);

	size_t cBin = 0, cTxt = 0;
	for (const char * p = _pBegin ; p < _pEnd; ++ p) {
		const char ch = * p;
		if ((unsigned) ch >= 32)
			continue;

		if (ch == 0)
			return true;

		if (ch == '\r' || ch == '\n')
			++ cTxt;
		else
			++ cBin;
	}
	return cBin >= size_t (_pEnd - _pBegin)/8;
}

bool CheckAnsiFile (const char * _pData, size_t _cFileSize, bool _bSilent) {
	if (_cFileSize < 2)
		return true;

	if (_cFileSize >= 2 && _pData [0] == '\xFF' && _pData [1] == '\xFE') {
		if (! _bSilent)
			ReportText ("ERROR: Input file is in Unicode.");
		return false;
	}
	if (_cFileSize >= 3 && _pData [0] == '\xEF' && _pData [1] == '\xBB' && _pData [2] == '\xBF') {
		if (! _bSilent)
			ReportText ("ERROR: Input file is in UTF-8.");
		return false;
	}

	return true;
}

/////////////////////////////////////

bool __fastcall HasPrefix (const char * _pBegin, const char * _pEnd, const char * _pPrefix, size_t _cPrefixLen) {
	assert (_pBegin <= _pEnd);
	assert (_cPrefixLen > 0);
	assert (_cPrefixLen == ::strlen (_pPrefix));

	if (_pBegin + _cPrefixLen > _pEnd)
		return false;

	if (ToLower (_pBegin [0]) != ToLower (_pPrefix [0]))
		return false;

	++ _pPrefix;
	for (const char * p = _pBegin + 1; p < _pBegin + _cPrefixLen; ++ p, ++ _pPrefix) {
		if (* p == * _pPrefix)
			continue;

		if (ToLower (* p) == ToLower (* _pPrefix))
			continue;

		return false;
	}

	return true;
}

const char * __fastcall FindNextLine (const char * _pBegin, const char * _pEnd) {
	assert (_pBegin <= _pEnd);

	for (const char * p = _pBegin; p < _pEnd; ++ p)
		if (* p == '\n')
			return p + 1;

	return _pEnd;
}

size_t GetLineLen (const char * _pBegin, const char * _pNextLine) {
	assert (_pNextLine >= _pBegin);
	if (_pNextLine < _pBegin)
		return 0;
	size_t cLen = _pNextLine - _pBegin;

	while (
		cLen > 0 && (
			_pBegin [cLen - 1] == '\r' ||
			_pBegin [cLen - 1] == '\n' ||
			_pBegin [cLen - 1] == ' '  ||
			_pBegin [cLen - 1] == '\t'
		)
	)
		-- cLen;

	return cLen;	
}

void OmitSpaces (const char ** _pp, const char * _pEnd) {
	const char * p = * _pp;

	while (p < _pEnd && (* p == ' ' || * p == '\t' || * p == '\n' || * p == '\r'))
		++ p;

	* _pp = p;
}

void OmitNonSpaces (const char ** _pp, const char * _pEnd) {
	const char * p = * _pp;

	while (p < _pEnd && (* p != ' ' && * p != '\t' && * p != '\n' && * p != '\r'))
		++ p;

	* _pp = p;
}

void StripSpaces (char * _str) {
	if (_str == NULL)
		return;

	const char * const pEnd = _str + ::strlen (_str);
	const char * pBegin = _str;
	OmitSpaces (& pBegin, pEnd);
	const size_t cLen = GetLineLen (pBegin, pEnd);

	::memmove (_str, pBegin, cLen);
	_str [cLen] = '\0';
}

bool ParseHex (const char * _pValue, const char * _pEnd, const char ** _ppEnd, unsigned long & _ulRet) {
	unsigned long r = 0;
	bool bHasDigits = false;
	const char * p = _pValue;

	// Omit spaces.
	while (p < _pEnd && * p == ' ')
		++ p;

	while (p < _pEnd) {
		if ('0' <= * p && * p <= '9') {
			r *= 16;
			r += (* p) - '0';
		} else if ('a' <= * p && * p <= 'f') {
			r *= 16;
			r += (* p) - 'a' + 10;
		} else if ('A' <= * p && * p <= 'F') {
			r *= 16;
			r += (* p) - 'A' + 10;
		} else
			break;
		++ p;
		bHasDigits = true;
	}

	if (bHasDigits && _ppEnd)
		* _ppEnd = p;
	_ulRet = r;
	return bHasDigits;
}

bool ParseDecU (const char * _pValue, const char * _pEnd, const char ** _ppEnd, unsigned long & _ulRet) {
	unsigned long r = 0;
	bool bHasDigits = false;
	const char * p = _pValue;

	// Omit spaces.
	while (p < _pEnd && * p == ' ')
		++ p;

	while (p < _pEnd && '0' <= * p && * p <= '9') {
		r *= 10;
		r += (* p) - '0';
		++ p;
		bHasDigits = true;
	}

	if (bHasDigits && _ppEnd)
		* _ppEnd = p;
	_ulRet = r;
	return bHasDigits;
}

bool ParseDecU (const wchar_t * _pValue, const wchar_t * _pEnd, const wchar_t ** _ppEnd, unsigned long & _ulRet) {
	unsigned long r = 0;
	bool bHasDigits = false;
	const wchar_t * p = _pValue;

	// Omit spaces.
	while (p < _pEnd && * p == ' ')
		++ p;

	while (p < _pEnd && '0' <= * p && * p <= '9') {
		r *= 10;
		r += (* p) - '0';
		++ p;
		bHasDigits = true;
	}

	if (bHasDigits && _ppEnd)
		* _ppEnd = p;
	_ulRet = r;
	return bHasDigits;
}

bool ParseReal (const char * _pValue, const char * _pEnd, const char ** _ppEnd, float & _fRet) {
	float r = 0;
	bool bMinus = false;
	bool bHasDigits = false;
	const char * p = _pValue;

	// Omit spaces.
	while (p < _pEnd && * p == ' ')
		++ p;

	// Get sign.
	if (p < _pEnd) {
		if (* p == '-') {
			++ p;
			bMinus = true;
		} else if (* p == '+')
			++ p;
	}

	// Get whole part of the digit.
	while (p < _pEnd && '0' <= * p && * p <= '9') {
		r *= 10;
		r += (* p) - '0';
		++ p;
		bHasDigits = true;
	}

	// Get decimal dot.
	if (p == _pEnd || (p < _pEnd && * p != '.')) {
		if (bHasDigits && _ppEnd)
			* _ppEnd = p;
		_fRet = bMinus ? -r : r;
		return bHasDigits;
	}
	++ p;
	const char * pFraction = p;

	// Get fractional part of the digit.
	float f = 0;
	float pw = 0.1f;
	while (p < _pEnd && '0' <= * p && * p <= '9') {
		f += ((* p) - '0')*pw;
		++ p;
		pw *= .1f;
		bHasDigits = true;
	}

	if (bHasDigits && _ppEnd)
		* _ppEnd = p;
	_fRet = bMinus ? -(r + f) : (r + f);
	return bHasDigits;
}

bool ParseReal (const wchar_t * _pValue, const wchar_t * _pEnd, const wchar_t ** _ppEnd, float & _fRet) {
	float r = 0;
	bool bMinus = false;
	bool bHasDigits = false;
	const wchar_t * p = _pValue;

	// Omit spaces.
	while (p < _pEnd && * p == L' ')
		++ p;

	// Get sign.
	if (p < _pEnd) {
		if (* p == L'-') {
			++ p;
			bMinus = true;
		} else if (* p == L'+')
			++ p;
	}

	// Get whole part of the digit.
	while (p < _pEnd && L'0' <= * p && * p <= L'9') {
		r *= 10;
		r += (* p) - '0';
		++ p;
		bHasDigits = true;
	}

	// Get decimal dot.
	if (p == _pEnd || (p < _pEnd && * p != L'.')) {
		if (bHasDigits && _ppEnd)
			* _ppEnd = p;
		_fRet = bMinus ? -r : r;
		return bHasDigits;
	}
	++ p;
	const wchar_t * pFraction = p;

	// Get fractional part of the digit.
	float f = 0;
	float pw = 0.1f;
	while (p < _pEnd && L'0' <= * p && * p <= L'9') {
		f += ((* p) - L'0')*pw;
		++ p;
		pw *= .1f;
		bHasDigits = true;
	}

	if (bHasDigits && _ppEnd)
		* _ppEnd = p;
	_fRet = bMinus ? -(r + f) : (r + f);
	return bHasDigits;
}

bool ParseReal (const char * _pValue, const char * _pEnd, const char ** _ppEnd, double & _fRet) {
	double r = 0;
	bool bMinus = false;
	bool bHasDigits = false;
	const char * p = _pValue;

	// Omit spaces.
	while (p < _pEnd && * p == ' ')
		++ p;

	// Get sign.
	if (p < _pEnd) {
		if (* p == '-') {
			++ p;
			bMinus = true;
		} else if (* p == '+')
			++ p;
	}

	// Get whole part of the digit.
	while (p < _pEnd && '0' <= * p && * p <= '9') {
		r *= 10;
		r += (* p) - '0';
		++ p;
		bHasDigits = true;
	}

	// Get decimal dot.
	if (p == _pEnd || (p < _pEnd && * p != '.')) {
		if (bHasDigits && _ppEnd)
			* _ppEnd = p;
		_fRet = bMinus ? -r : r;
		return bHasDigits;
	}
	++ p;
	const char * pFraction = p;

	// Get fractional part of the digit.
	double f = 0;
	double pw = 0.1f;
	while (p < _pEnd && '0' <= * p && * p <= '9') {
		f += ((* p) - '0')*pw;
		++ p;
		pw *= .1f;
		bHasDigits = true;
	}

	if (bHasDigits && _ppEnd)
		* _ppEnd = p;
	_fRet = bMinus ? -(r + f) : (r + f);
	return bHasDigits;
}

bool ParseRealE (const char * _pValue, const char * _pEnd, const char ** _ppEnd, double & _fRet) {
	bool bMinus = false;
	bool bHasDigits = false;
	const char * p = _pValue;

	// Omit spaces.
	while (p < _pEnd && * p == ' ')
		++ p;

	// Get sign.
	if (p < _pEnd) {
		if (* p == '-') {
			++ p;
			bMinus = true;
		} else if (* p == '+')
			++ p;
	}

	double r = 0;
	// Get whole part of the digit.
	while (p < _pEnd && '0' <= * p && * p <= '9') {
		r *= 10;
		r += (* p) - '0';
		++ p;
		bHasDigits = true;
	}

	double f = 0;
	// Get fractional part of the digit.
	if (p < _pEnd && * p == '.') {
		++ p;
		const char * pFraction = p;
		double pw = 0.1f;
		while (p < _pEnd && '0' <= * p && * p <= '9') {
			f += ((* p) - '0')*pw;
			++ p;
			pw *= .1f;
			bHasDigits = true;
		}
	}

	double e = 0;
	// Get exponent part of the digit
	if (p < _pEnd && (* p == 'e' || * p == 'E')) {
		++ p;
		bool bExpMinus = false;
		bool bExpHasDigits = false;
		if (* p == '-') {
			++ p;
			bExpMinus = true;
		} else if (* p == '+')
			++ p;
		while (p < _pEnd && '0' <= * p && * p <= '9') {
			e *= 10;
			e += (* p) - '0';
			++ p;
			bExpHasDigits = true;
		}
		if (bExpHasDigits) {
			if (bExpMinus)
				e = -e;
		} else
			e = 1;
	} else
		e = 1;

	if (bHasDigits && _ppEnd)
		* _ppEnd = p;
	_fRet = bMinus ? -(r + f) : (r + f);
	if (e != 1)
		_fRet *= ::pow (10, e);
	return bHasDigits;
}

inline
bool _IsSingleQuote (char _ch) {
	return _ch == '\'' || _ch == '\x60' || _ch == '\x92' || _ch == '\xB4';
}

bool ParseArc (const char * _pValue, const char * _pEnd, const char ** _ppEnd, float & _fRet, bool _bRequireDegSign) {
	bool bMinus = false;
	const char * p = _pValue;
	
	while (p < _pEnd && * p == ' ' || * p == '\t')
		++ p;
	
	// Get sign.
	if (p < _pEnd) {
		switch (* p) {
			case 'W':
			case 'S':
			case '-':
				++ p;
				bMinus = true;
				break;

			case 'E':
			case 'N':
			case '+':
				++ p;
				break;
		}
	}
	
	float fDeg = 0;
	float fMin = 0;
	float fSec = 0;

	if (! ParseReal (p, _pEnd, & p, fDeg))
		return false;

	if (p < _pEnd) {
		OmitSpaces (& p, _pEnd);
		if (p < _pEnd) {
			const bool bDegreeSign = HasPrefix (p, _pEnd, g_strDegreeSign, g_cDegreeSignLen);
			const bool bDegreeSign2 = bDegreeSign || * p == '*' || * p == '\xBA';
			if (bDegreeSign || bDegreeSign2 || ! _bRequireDegSign) {
				if (bDegreeSign)
					p += g_cDegreeSignLen;
				else if (bDegreeSign2)
					++ p;

				if (ParseReal (p, _pEnd, & p, fMin) && p < _pEnd) {
					OmitSpaces (& p, _pEnd);
					if (p < _pEnd && _IsSingleQuote (p [0])) {
						++ p;

						if (ParseReal (p, _pEnd, & p, fSec) &&  p < _pEnd) {
							OmitSpaces (& p, _pEnd);
							if (p < _pEnd && (* p == '\"' || * p == '\x94'))
								++ p;
							else if (p + 1 < _pEnd && _IsSingleQuote (p [0]) && _IsSingleQuote (p [1]))
								p += 2;
							else
								return false;
						}
					}
				}
			}
		}
	}
	
	_fRet = fDeg + fMin/60 + fSec/3600;
	if (bMinus)
		_fRet = -_fRet;

	if (_ppEnd)
		* _ppEnd = p;
	return true;
}

# ifdef _DEBUG
void Test_ParseArc () {
	assert (ParseArc ("12") == 12);
	assert (::fabs (ParseArc ("W 12  34 ' 56 \"  12") - -(12.f + 34.f/60 + 56.f/3600)) < 1e-6);
}
# endif // _DEBUG
