//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// GPS position cursor drawing.
//

# include "StdAfx.h"
# include "PosCursor.h"
# include "Scale.h"
# include "Position.h"
# include "Positioning.h"
# include "fast_floor.h"
# include "Globals.h"

const size_t c_cH1 = 9;//3;
const size_t c_cH2 = 24;//8;

static
void _ShowPosCursor (HDC _hDC, float _x, float _y, float _fHeadingInDeg) {
	const long x0 = XToScreen (_x);
	const long y0 = YToScreen (_y);

	const float fA = _fHeadingInDeg*c_PI/180.f;
	const float fCosA = ::cos (fA);
	const float fSinA = ::sin (fA);

	POINT pts [3];
	pts [0].x = x0 + fast_floor (c_cH1*fCosA + .5f);
	pts [0].y = y0 + fast_floor (c_cH1*fSinA + .5f);
	pts [1].x = x0 - fast_floor (c_cH1*fCosA + .5f);
	pts [1].y = y0 - fast_floor (c_cH1*fSinA + .5f);
	pts [2].x = x0 + fast_floor (c_cH2*fSinA + .5f);
	pts [2].y = y0 - fast_floor (c_cH2*fCosA + .5f);

	::SelectObject (_hDC, ::GetStockObject (BLACK_BRUSH));
	::SelectObject (_hDC, ::GetStockObject (WHITE_PEN));
	::SetROP2 (_hDC, R2_COPYPEN);
	::Polygon (_hDC, pts, ARRAY_SIZE (pts));
}

////////////////////////////////////////////////////////////////////

void DrawGpsPosCursor (HDC _hDC) {
	const PositionInfo_t pi = GetPositionInfo ();

	_ShowPosCursor (_hDC, pi.fX, pi.fY, (pi.dwDef & PositionInfo_t::defHeading) != 0 ? pi.fHeading : 0);

	// Precision circle.
	if (pi.dwDef & PositionInfo_t::defXYPrec) {
		const long r = g_fScale > 0 ? ::fast_floor (pi.fXYPrec/g_fScale) : 0;
		if (r > c_cH2) {
			const long x0 = XToScreen (pi.fX);
			const long y0 = YToScreen (pi.fY);

			::SelectObject (_hDC, ::GetStockObject (BLACK_PEN));
			::SelectObject (_hDC, ::GetStockObject (NULL_BRUSH));
			::Ellipse (_hDC, x0 - r, y0 - r, x0 + r, y0 + r);
		}
	}
}

////////////////////////////////////////////////////////////////////

static float g_fVideoPosCursorX;
static float g_fVideoPosCursorY;
static float g_fVideoPosCursorHeading;
static bool  g_bVideoPosCursorShow;

void SetVideoPosCursor (
	float _fVideoPosCursorX,
	float _fVideoPosCursorY,
	float _fVideoPosCursorHeading
) {
	g_fVideoPosCursorX       = _fVideoPosCursorX;
	g_fVideoPosCursorY       = _fVideoPosCursorY;
	g_fVideoPosCursorHeading = _fVideoPosCursorHeading;
}

void ShowVideoPosCursor (bool _bShow) {
	g_bVideoPosCursorShow = _bShow;
}

void DrawVideoPosCursor (HDC _hDC) {
	if (g_bVideoPosCursorShow)
		_ShowPosCursor (_hDC, g_fVideoPosCursorX, g_fVideoPosCursorY, g_fVideoPosCursorHeading);
}
