//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Routing graph node finder.
//

# include "StdAfx.h"
# include "RGNodeFinder.h"
# include "Map.h"

void CreateRGNodeFinder (map_t & _map, CPointFinder<rgnode_t *> & _pf) {
	layer_t & polylines = _map.layers [rkPolyline];
	for (rgns_t::iterator ri = polylines.rgns.begin (); ri != polylines.rgns.end (); ++ ri) {
		rgn_t & rgn = * ri;
		if (rgn.IsDeleted ())
			continue;

		const size_t cElements = rgn.elements.size ();
		for (size_t cElement = 0; cElement < cElements; ++ cElement) {
			rgn_element_t & element = rgn.elements [cElement];
			if (element.bDeleted)
				continue;
			if (element.cLevel != 0 || ! element.pExtra)
				continue;

			point_extras_t & extras = element.pExtra->PointExtras;
			if (extras.empty ())
				continue;

			const points_t & points = element.points;
			const size_t cPoints = points.size ();
			const size_t cNodes = __min (cPoints, extras.size ());
			for (size_t cPoint = 0; cPoint < cNodes; ++ cPoint) {
				rgnode_t * const pRGNode = extras [cPoint].GetRGNode ();
				if (pRGNode == NULL)
					continue;
				if (! pRGNode->bIsExternal)
					continue;

				_pf.Add (points [cPoint], pRGNode);
			}
		}
	}

	_pf.Index ();
}
