//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Object properties dialog - 'Entry Points' page.
//

# include "StdAfx.h"
# include "Resource.h"
# include "RgnProperties.h"
# include "SetEntryPoint.h"
# include "CombinedAction.h"
# include "MapView.h"
# include "Scale.h"
# include "Position.h"
# include "Levels.h"
# include "ListView.h"
# include "EditText.h"
# include "EditPoint.h"
# include "PrintUnits.h"

enum {
	lvOrdinal  = 0,
	lvPosition = 1,
	lvLabel    = 2,
};

void CRgnEntryPointsPage::_FillList () {
	ListView_DeleteAllItems (m_hList);

	if (! m_rgn.pExtra)
		return;

	size_t cItem = 0;

	const size_t cEntryPoints = m_rgn.pExtra->entry_points.size ();
	for (size_t cEP = 0; cEP < cEntryPoints; ++ cEP) {
		const entry_point_t & ep = m_rgn.pExtra->entry_points [cEP];
		if (ep.bDeleted)
			continue;

		// #
		InsertItem     (m_hList, cItem, cEP,        "%d", cEP + 1);

		// Label.
		SetSubItemText (m_hList, cItem, lvLabel,    "%s", ep.strLabel.c_str ());

		// Position.
		char strPosition [32];
		PrintPosition (strPosition, ep.pt);
		SetSubItemText (m_hList, cItem, lvPosition, "%s", strPosition);

		++ cItem;
	}

	// Set the best columns width.
	AutoSizeColumns (m_hList);
}

void CRgnEntryPointsPage::_RefreshBtns () {
	const size_t cSelected = ListView_GetSelectedCount (m_hList);

	EnableDlgItem (IDC_REMOVE,     cSelected != 0);
	EnableDlgItem (IDC_EDIT,       cSelected == 1);
	EnableDlgItem (IDC_EDIT_LABEL, cSelected == 1);
	EnableDlgItem (IDC_LOCATE,     cSelected == 1);
}

bool CRgnEntryPointsPage::OnInit (HWND _hwndFocus) {
	m_hList = GetDlgItem (IDC_LIST);
	ListView_SetExtendedListViewStyleEx (m_hList, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);

	AddColumn (m_hList, lvOrdinal,  "#", true);
	AddColumn (m_hList, lvPosition, "Position");
	AddColumn (m_hList, lvLabel,    "Label");

	_FillList ();

	SetSelection (m_hList, 0);
	_RefreshBtns ();

	return true;
}

void CRgnEntryPointsPage::OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
	switch (_wCmd) {
		case IDC_REMOVE: {
			const int iDepth = BeginCombinedAction ();

			size_t cItem = -1;
			while (true) {
				// Enumerate all selected items.
				cItem = ListView_GetNextItem (m_hList, cItem, LVNI_SELECTED);
				if (cItem == -1)
					break;

				RemoveEntryPoint (& m_rgn, GetLParam (m_hList, cItem));
			}

			EndCombinedAction (iDepth, "Remove entry points");

			_FillList ();
			_RefreshBtns ();

			Refresh ();
			break;
		}

		case IDC_EDIT:
		case IDC_EDIT_LABEL: {
			const size_t cSel = ListView_GetSelectionMark (m_hList);
			if (cSel == -1)
				break;

			const size_t cOrdinal = GetLParam (m_hList, cSel);
			assert (cOrdinal < m_rgn.pExtra->entry_points.size ());
			if (cOrdinal >= m_rgn.pExtra->entry_points.size ())
				break;
			entry_point_t & ep = m_rgn.pExtra->entry_points [cOrdinal];

			if (_wCmd == IDC_EDIT_LABEL) {
				string_t strLabel = ep.strLabel;
				if (! EditText (m_hWnd, strLabel, "Entry Point", false))
					break;
				SetEntryPointLabel (& m_rgn, cOrdinal, strLabel);
			} else {
				point_t pt = ep.pt;
				if (! EditPoint (m_hWnd, pt))
					break;
				SetEntryPoint (& m_rgn, cOrdinal, pt);
			}

			_FillList ();
			_RefreshBtns ();

			Refresh ();

			break;
		}

		case IDC_LOCATE: {
			const size_t cSel = ListView_GetSelectionMark (m_hList);
			if (cSel == -1)
				break;

			const size_t cOrdinal = GetLParam (m_hList, cSel);
			assert (cOrdinal < m_rgn.pExtra->entry_points.size ());
			if (cOrdinal >= m_rgn.pExtra->entry_points.size ())
				break;
			const entry_point_t & ep = m_rgn.pExtra->entry_points [cOrdinal];
			const point_t & pt = ep.pt;

			SetVisibleLevel (0);
			const rect_t rect (pt.x - 1e-3f, pt.y - 1e-3f, pt.x + 1e-3f, pt.y + 1e-3f);
			SetScale (GetRectScale (rect));
			SetToCenter (ep.pt);
			Refresh ();

			break;
		}
	}
}

bool CRgnEntryPointsPage::OnNotify (int _idCtrl, NMHDR * _pInfo) {
	switch (_pInfo->code) {
		case LVN_ITEMCHANGED: {
			_RefreshBtns ();
			break;
		}
	}

	return CPage::OnNotify (_idCtrl, _pInfo);
}

//void CRgnEntryPointsPage::OnApply (const PSHNOTIFY *) {}
