# ifndef __Selection_H_
# define __Selection_H_

void InitSelection ();
void DestroySelection ();
bool AdjustColorsForSelection (HWND _hWnd);

bool IsSelMultipleOn (DWORD _dwKeys);
bool IsSelElementOn  (DWORD _dwKeys);

# include "Map.h"

typedef std::vector<find_info_t> find_info_m_t;
extern find_info_m_t g_Selection;

find_info_t FindRgn (map_t & _map, int _x, int _y, const rgn_t * _pExceptRgn = NULL, bool _bEnableStickedNodes = false);
find_info_t FindRgn (              int _x, int _y, const rgn_t * _pExceptRgn = NULL, bool _bEnableStickedNodes = false);
bool StickPointToNodeOrGrid (size_t _cLevelIdx, int & _x, int & _y, point_t & _pointStick, const rgn_t * pExceptRgn = NULL);

bool IsInSelection (const find_info_t & _SelItem);

// Return true if point is inside selection.
bool SelectObjectAt (int _x, int _y, bool _bAdd, bool _bElementOnly, bool _bEntryPointEnabled, bool _bRight, bool _bEnableRedraw = true);
void SelectObjectsInsideRect (const rect_t & _rectSelection, bool _bAdd, bool _bElementOnly);
void SelectObjectsInsidePolygon (const rgn_t & _rgn, bool _bAdd);
void SelectAllObjects (bool _bObjects, bool _bBookmarks, bool _bNotes, bool _bTracks, bool _bWaypoints, bool _bRoutes, bool _bRasters);
void UnselectObjects ();
void InvertSelection ();
bool IsInvertSelectionDisabled ();

void DrawSelection (HDC _hDC);

void SetSelectionOffset (int _iXOffset, int _iYOffset);
void SetSelectionTurn   (float _fAngle);

void GetSelItemCentroid (const find_info_t & _SelItem, point_t & _ptCentroid);

//////////////////////////////////////////////////////

struct sel_point_t {
	rgn_t * pRgn;
	size_t cElement;
	size_t cPoint;

	bool bSticked;
	int nStickedX;
	int nStickedY;

	bool operator == (const sel_point_t & _other) const {
		return pRgn == _other.pRgn && cElement == _other.cElement && cPoint == _other.cPoint;
	}
	bool operator < (const sel_point_t & _other) const {
		if (pRgn != _other.pRgn)
			return pRgn < _other.pRgn;
		else {
			if (cElement != _other.cElement)
				return cElement < _other.cElement;
			else
				return cPoint < _other.cPoint;
		}
	}

	bool IsValid () const;

	void GetPoint (point_t & _pt) const {
		_pt = pRgn->elements [cElement].points [cPoint];
	}
};

typedef std::vector<sel_point_t> sel_points_t;
extern sel_points_t g_SelPoints;

const int c_nNodeBoxSize = 4;

void ClearSelPoints ();
void SelectPoint (rgn_t * _pRgn, size_t _cElement, size_t _cPoint, bool _bAdd);
void UnselectPoint (int _x, int _y);
void SelectPointsInsideRect (const rect_t & _rectSelection, bool _bAdd);
size_t FindSelPoint (int _x, int _y);
void SetSelPointsOffset (int _nXOffset, int _nYOffset);
void DrawSelPoints (HDC _hDC);

/////////////////////////////////////////////////////

void DrawSelPointAdded (HDC _hDC, int _x, int _y);
void DrawStickPoint    (HDC _hDC, int _x, int _y, bool _bStick);

# endif // __Selection_H_