//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Set bookmark attributes.
//

# include "StdAfx.h"
# include "SetBookmark.h"
# include "Bookmark.h"
# include "UndoRedo.h"
# include "Label.h"

class CSetBookmarkAction : public CEditAction {
  public:
	CSetBookmarkAction (rgn_t * _pRgn, const bookmark_t & _bookmark) :
		m_pRgn (_pRgn),
		m_bookmark (_bookmark),
		m_bWasModified ((_pRgn->dwFlags & rgn_t::fModified) != 0)
	{}

	virtual void GetName (string_t & _str) const;
	virtual cu_t CanUndo () const {return cuYes;}
	virtual bool Do ();
	virtual void Undo ();
	virtual void Redo ();

  private:
	rgn_t * const m_pRgn;
	const bool    m_bWasModified;

	bookmark_t m_bookmark;
	bookmark_t m_bookmarkOld;

	void _Swap ();
};

void CSetBookmarkAction::GetName (string_t & _str) const {
	CCritSec::CLock lock (g_cs);

	assert (m_pRgn);
	_str.append ("set [");
	GetHintText (* m_pRgn, _str);
	_str.append ("]");
}

bool CSetBookmarkAction::Do () {
	if (m_pRgn->pExtra->pBookmark == NULL)
		m_pRgn->pExtra->pBookmark = new bookmark_t;

	m_bookmarkOld = * m_pRgn->pExtra->pBookmark;
	* m_pRgn->pExtra->pBookmark = m_bookmark;

	m_pRgn->dwFlags |= rgn_t::fModified;
	return true;
}

void CSetBookmarkAction::Undo () {
	m_bookmark = * m_pRgn->pExtra->pBookmark;
	* m_pRgn->pExtra->pBookmark = m_bookmarkOld;

	if (! m_bWasModified)
		m_pRgn->dwFlags &= ~rgn_t::fModified;
}

void CSetBookmarkAction::Redo () {
	CSetBookmarkAction::Do ();
}

//////////////////////////////////////////

void SetBookmark (rgn_t * _pRgn, const bookmark_t & _bookmark) {
	assert (_pRgn);
	assert (_pRgn->IsPoint () && ! _pRgn->IsWTR () && _pRgn->IsMutable ());

	PerformEditAction (new CSetBookmarkAction (_pRgn, _bookmark));
}
