//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2006
//
// Set 'don't find' attribute for object.
//

# include "StdAfx.h"
# include "SetIndexedCity.h"
# include "UndoRedo.h"
# include "Label.h"

class CSetDontFindAction : public CEditAction {
  public:
	CSetDontFindAction (rgn_t & _rgn, bool _bDontFind) :
		m_rgn (_rgn),
		m_bNewDontFind (_bDontFind),
		m_bOldDontFind ((_rgn.dwFlags & rgn_t::fDontFind) != 0),
		m_bWasModified ((_rgn.dwFlags & rgn_t::fModified) != 0)
	{}

	virtual void GetName (string_t & _str) const;
	virtual cu_t CanUndo () const {return cuYes;}
	virtual bool Do ();
	virtual void Undo ();
	virtual void Redo () {CSetDontFindAction::Do ();}

  private:
	rgn_t &    m_rgn;
	bool const m_bNewDontFind;
	bool const m_bOldDontFind;
	bool const m_bWasModified;
};

void CSetDontFindAction::GetName (string_t & _str) const {
	CCritSec::CLock lock (g_cs);

	_str.append ("find on/off [");
	GetHintText (m_rgn, _str);
	_str.append ("]");
}

bool CSetDontFindAction::Do () {
	CCritSec::CLock lock (g_cs);

	if (m_bNewDontFind)
		m_rgn.dwFlags |= rgn_t::fDontFind;
	else
		m_rgn.dwFlags &= ~rgn_t::fDontFind;

	m_rgn.dwFlags |= rgn_t::fModified;

	return true;
}

void CSetDontFindAction::Undo () {
	CCritSec::CLock lock (g_cs);

	if (m_bOldDontFind)
		m_rgn.dwFlags |= rgn_t::fDontFind;
	else
		m_rgn.dwFlags &= ~rgn_t::fDontFind;

	if (! m_bWasModified)
		m_rgn.dwFlags &= ~rgn_t::fModified;
}

///////////////////////////

void SetDontFind (rgn_t & _rgn, bool _bDontFind) {
	PerformEditAction (new CSetDontFindAction (_rgn, _bDontFind));
}
