//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Set note drawing attributes.
//

# include "StdAfx.h"
# include "SetBookmark.h"
# include "Bookmark.h"
# include "UndoRedo.h"
# include "Label.h"

class CSetNoteDrawingAction : public CEditAction {
  public:
	CSetNoteDrawingAction (rgn_t * _pRgn, const note_drawing_t & _NoteDrawing) :
		m_pRgn (_pRgn),
		m_NoteDrawing (_NoteDrawing),
		m_bWasModified ((_pRgn->dwFlags & rgn_t::fModified) != 0)
	{}

	virtual void GetName (string_t & _str) const;
	virtual cu_t CanUndo () const {return cuYes;}
	virtual bool Do ();
	virtual void Undo ();
	virtual void Redo ();

  private:
	rgn_t * const m_pRgn;
	const bool    m_bWasModified;

	note_drawing_t m_NoteDrawing;
	note_drawing_t m_NoteDrawingOld;

	void _Swap ();
};

void CSetNoteDrawingAction::GetName (string_t & _str) const {
	CCritSec::CLock lock (g_cs);

	assert (m_pRgn);
	_str.append ("set [");
	GetHintText (* m_pRgn, _str);
	_str.append ("]");
}

bool CSetNoteDrawingAction::Do () {
	if (m_pRgn->pExtra->pNoteDrawing == NULL)
		m_pRgn->pExtra->pNoteDrawing = new note_drawing_t;

	m_NoteDrawingOld = * m_pRgn->pExtra->pNoteDrawing;
	* m_pRgn->pExtra->pNoteDrawing = m_NoteDrawing;

	m_pRgn->dwFlags |= rgn_t::fModified;
	return true;
}

void CSetNoteDrawingAction::Undo () {
	m_NoteDrawing = * m_pRgn->pExtra->pNoteDrawing;
	* m_pRgn->pExtra->pNoteDrawing = m_NoteDrawingOld;

	if (! m_bWasModified)
		m_pRgn->dwFlags &= ~rgn_t::fModified;
}

void CSetNoteDrawingAction::Redo () {
	CSetNoteDrawingAction::Do ();
}

//////////////////////////////////////////

void SetNoteDrawing (rgn_t * _pRgn, const note_drawing_t & _NoteDrawing) {
	assert (_pRgn);
	assert (_pRgn->IsPolyline () && ! _pRgn->IsWTR () && _pRgn->IsMutable ());

	PerformEditAction (new CSetNoteDrawingAction (_pRgn, _NoteDrawing));
}
