//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2005-2009
//
// Select object and zoom in to full window.
//

# include "StdAfx.h"
# include "ShowRgnOnMap.h"
# include "MapView.h"
# include "Scale.h"
# include "Position.h"
# include "Selection.h"
# include "Levels.h"

void ShowRgnOnMap (const rgn_t & _rgn, bool _bForceLevel) {
	ClearSelPoints ();
	UnselectObjects ();

	{
		CCritSec::CLock lock (g_cs);

		for (size_t cKind = 0; cKind < rkMax; ++ cKind) {
			layer_t & layer = g_map.layers [cKind];
			rgns_t & rgns = layer.rgns;

			bool bFound = false;
			for (rgns_t::iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
				const rgn_t & rgn = * iRgn;
				if (rgn.IsDeleted () || & rgn != & _rgn)
					continue;

				find_info_t fi;
				fi.pLayer           = & layer;
				fi.iRgn             = iRgn;
				fi.cElementOrOrigin = -1;
				g_Selection.push_back (fi);

				bFound = true;
				break;
			}
			if (bFound)
				break;
		}
	}

	ShowSelectionOnMap (_bForceLevel);
}

void ShowSelectionOnMap (bool _bForceLevel) {
	CCritSec::CLock lock (g_cs);

	if (g_Selection.empty ())
		return;

	rect_t rect;
	const size_t cSelItems = g_Selection.size ();
	for (size_t c = 0; c < cSelItems; ++ c) {
		const find_info_t & fi = g_Selection [c];
		if (fi.pLayer) {
			if (! fi.iRgn->IsDeleted ())
				rect.Extend (fi.iRgn->rectBound);
		} else if (fi.pRenderer) {
			if (! fi.pRenderer->bDeleted && fi.pRenderer->bVisible)
				rect.Extend (fi.pRenderer->rect);
		}
	}

	size_t cNewLevel = -1;
	if (_bForceLevel) {
		const size_t cInitialLevel = GetVisibleLevel ();
		for (size_t c = 0; c < cSelItems; ++ c) {
			const find_info_t & fi = g_Selection [c];
			if (fi.pLayer && cNewLevel == -1)
				cNewLevel = -1;//FindMatchingLevel (* fi.iRgn, cInitialLevel);
		}

		if (cNewLevel != -1 && cNewLevel == cInitialLevel)
			cNewLevel = -1;
	}

	if (! rect.Invalid ()) {
		if (rect.Width () < 2e-3f) {
			rect.x0 -= 1e-3f;
			rect.x1 += 1e-3f;
		}
		if (rect.Height () < 2e-3f) {
			rect.y0 -= 1e-3f;
			rect.y1 += 1e-3f;
		}

		SetScale (GetRectScale (rect), false);
		SetToCenter (point_t (rect.MeanX (), rect.MeanY ()));
	}

	if (cNewLevel != -1)
		SetVisibleLevel (cNewLevel);

	Refresh ();
}
