//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2006-2009
//
// ESRI shapefile import wizard, support of extra attributes.
//

# include "StdAfx.h"
# include "Resource.h"
# include "ShpWizard.h"
# include "DatabaseTable.h"
# include "ListView.h"
# include "Globals.h"

enum {
	afAttribute = 0,
	afField     = 1
};

struct field2attr_t {
	const TCHAR * strField;
	input_attr_t attr;
};

field2attr_t g_field2attr [] = {
	{"COMMENT",       iaComment},
	{"ENDLEVEL",      iaEndLevel},
	{"END_LEVEL",     iaEndLevel},

	{"NUMBER",        iaNumber},
	{"STREET",        iaStreet},
	{"CITY",          iaCity},
	{"REGION",        iaRegion},
	{"COUNTRY",       iaCountry},
	{"ZIPCODE",       iaZipCode},
	{"ZIP_CODE",      iaZipCode},

	{"PHONE",         iaPhone},
	{"TELEPHONE",     iaPhone},
	{"FAX",           iaFax},
	{"EMAIL",         iaEmail},
	{"WEB",           iaWebPage},
	{"WEBPAGE",       iaWebPage},
	{"WEBSITE",       iaWebPage},
	{"WEBLINK",       iaWebPage},
	{"WWW",           iaWebPage},
	{"URL",           iaWebPage},
	{"DESCRIPTION",   iaDescription},
	{"DESCR",         iaDescription},
	{"TEXT",          iaDescription},

	{"DIRINDICATOR",  iaDirIndicator},
	{"DIR_INDICATOR", iaDirIndicator},
	{"SPEEDLIMIT",    iaSpeedLimit},
	{"SPEED_LIMIT",   iaSpeedLimit},
	{"ROUTECLASS",    iaRouteClass},
	{"ROUTE_CLASS",   iaRouteClass},
	{"ONEWAY",        iaOneWay},
	{"ONE_WAY",       iaOneWay},
	{"TOLL",          iaToll},
	{"NOEMERGENCY",   iaNoEmergency},
	{"NO_EMERGENCY",  iaNoEmergency},
	{"NODELIVERY",    iaNoDelivery},
	{"NO_DELIVERY",   iaNoDelivery},
	{"NOCAR",         iaNoCar},
	{"NO_CAR",        iaNoCar},
	{"NOBUS",         iaNoBus},
	{"NO_BUS",        iaNoBus},
	{"NOTAXI",        iaNoTaxi},
	{"NO_TAXI",       iaNoTaxi},
	{"NOPEDESTRIAN",  iaNoPedestrian},
	{"NO_PEDESTRIAN", iaNoPedestrian},
	{"NOBICYCLE",     iaNoBicycle},
	{"NO_BICYCLE",    iaNoBicycle},
	{"NOTRUCK",       iaNoTruck},
	{"NO_TRUCK",      iaNoTruck},

	{"HAS_ENTRYPOINT", iaHasEntryPoint},
	{"HASENTRYPOINT", iaHasEntryPoint},
	{"ENTRYPOINTX",   iaEntryPointX},
	{"ENTRYPOINT_X",  iaEntryPointX},
	{"ENTRYPOINTY",   iaEntryPointY},
	{"ENTRYPOINT_Y",  iaEntryPointY},
};

const size_t c_cField2Attrs = ARRAY_SIZE (g_field2attr);

struct attr2field_t {
	const TCHAR * strAttribute;
	bool bRoadsOnly;
};

const attr2field_t g_Attributes [iaLast] = {
	{"Comment",       false},
	{"End level",     false},

	{"Number",        false},
	{"Street",        false},
	{"City",          false},
	{"Region",        false},
	{"Country",       false},
	{"Zip code",      false},

	{"Phone",         false},
	{"Fax",           false},
	{"Email",         false},
	{"Web",           false},
	{"Description",   false},

	{"Has direction", true},
	{"Speed limit",   true},
	{"Route class",   true},
	{"One-way",       true},
	{"Toll",          true},
	{"No emergency",  true},
	{"No delivery",   true},
	{"No car",        true},
	{"No bus",        true},
	{"No taxi",       true},
	{"No pedestrian", true},
	{"No bicycle",    true},
	{"No truck",      true},

	{"Has entry point",       false},
	{"Entry point longitude", false},
	{"Entry point latitude",  false},
};

const size_t c_cAttributes = ARRAY_SIZE (g_Attributes);

void CShpWizard::_FillExtraAttributes (HWND _hList, bool _bFull) {
	m_bInFillExtraAttributes = true;

	if (_bFull)
		ListView_DeleteAllItems (_hList);

	size_t cItem = 0;
	for (size_t cAttribute = 0; cAttribute < c_cAttributes; ++ cAttribute) {
		if (GetKind () != rkPolyline && g_Attributes [cAttribute].bRoadsOnly)
			continue;
		if (_bFull)
			InsertItem (_hList, cItem, cAttribute, "%s", g_Attributes [cAttribute].strAttribute);

		const size_t cField = m_cFields [cAttribute];
		SetSubItemText (_hList, cItem, afField, "%s", cField == -1 ? "-" : m_pDbt->GetFieldName (cField));

		ListView_SetCheckState (_hList, cItem, cField != -1 ? TRUE : FALSE);

		++ cItem;
	}

	m_bInFillExtraAttributes = false;
}

void CShpWizard::_ResetExtraFields () {
	for (size_t cAttr = 0; cAttr < iaLast; ++ cAttr)
		m_cFields [cAttr] = -1;
}

void CShpWizard::_FindDefaultExtraFields () {
	_ResetExtraFields ();

	const size_t cFields = m_pDbt->GetFields ();
	for (size_t cField = 0; cField < cFields; ++ cField) {
		const char * const strFieldName = m_pDbt->GetFieldName (cField);
		for (size_t cAttr = 0; cAttr < c_cField2Attrs; ++ cAttr) {
			if (::stricmp (g_field2attr [cAttr].strField, strFieldName) == 0) {
				const input_attr_t attr = g_field2attr [cAttr].attr;
				if (GetKind () != rkPolyline && g_Attributes [attr].bRoadsOnly)
					continue;
				m_cFields [attr] = cField;
				break;
			}
		}
	}
}

void CShpWizard::_SelectDbfColumn (HWND _hWnd, size_t _cField) {
	const HWND hAttributes = ::GetDlgItem (_hWnd, IDC_ATTRIBUTES);
	const size_t cSelItem = ListView_GetSelectionMark (hAttributes);
	if (cSelItem == -1)
		return;
	const size_t cSelAttribute = GetLParam (hAttributes, cSelItem);

	m_cFields [cSelAttribute] = _cField;

	SetSubItemText (hAttributes, cSelItem, afField, _cField == -1 ? "-" : m_pDbt->GetFieldName (_cField));
	ListView_SetCheckState (hAttributes, cSelItem, _cField != -1);

	SelectColumn (::GetDlgItem (_hWnd, IDC_DBF_FIELDS), m_cFields [cSelAttribute]);
}

INT_PTR CALLBACK CShpWizard::ExtraProc (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	CShpWizard * const pThis = reinterpret_cast<CShpWizard *> (::GetWindowLong (_hWnd, GWL_USERDATA));

	switch (_uMsg) {
		case WM_INITDIALOG: {

			//
			// Prepare access to wizard's data.
			//

			CShpWizard * const pThis = reinterpret_cast<CShpWizard *> (_lParam);
			::SetWindowLong (_hWnd, GWL_USERDATA, _lParam);

			assert (pThis);

			//
			// Prepare list of attributes.
			//

			const HWND hAttributes = ::GetDlgItem (_hWnd, IDC_ATTRIBUTES);
			ListView_SetExtendedListViewStyleEx (hAttributes, LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES, LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES);

			AddColumn (hAttributes, afAttribute, "Attribute");
			AddColumn (hAttributes, afField,     "Field");

			pThis->m_bInFillExtraAttributes = false;
			pThis->_FillExtraAttributes (hAttributes);
			AutoSizeColumns (hAttributes);

			//
			// Prepare DBF fields list view.
			//

			const HWND hDbfFields = ::GetDlgItem (_hWnd, IDC_DBF_FIELDS);
			pThis->_PrepareDBFView (hDbfFields);
			pThis->_FillDBFView    (hDbfFields);
			AutoSizeColumns (hDbfFields);

			return FALSE;
		}

		case WM_COMMAND: {
			switch (LOWORD (_wParam)) {
				case IDC_RESET_FIELDS: {
					const HWND hAttributes = ::GetDlgItem (_hWnd, IDC_ATTRIBUTES);
					pThis->_ResetExtraFields ();

					pThis->_FillExtraAttributes (hAttributes, false);

					SelectColumn (::GetDlgItem (_hWnd, IDC_DBF_FIELDS), -1);
					
					return TRUE;
				}

				case IDC_BIND_DEFAULT_FIELDS: {
					const HWND hAttributes = ::GetDlgItem (_hWnd, IDC_ATTRIBUTES);
					pThis->_FindDefaultExtraFields ();
					pThis->_FillExtraAttributes (hAttributes, false);

					return TRUE;
				}

				case IDCANCEL:
				case IDOK: {
					::EndDialog (_hWnd, 0);
					return TRUE;
				}
			}
			break;
		}

	    case WM_NOTIFY: {
            const NMHDR * const pInfo = reinterpret_cast<NMHDR *> (_lParam);
			switch (pInfo->code) {
				case LVN_ITEMCHANGED: {
					const NM_LISTVIEW * const pInfo2 = reinterpret_cast<const NM_LISTVIEW *> (pInfo);
					const size_t cItem = pInfo2->iItem;
					const size_t cAttribute = GetLParam (pInfo->hwndFrom, cItem);
					if ((pInfo2->uNewState & LVIS_SELECTED) != 0 && pInfo->idFrom == IDC_ATTRIBUTES) {
						if (cItem != -1) {
							const bool bChecked = ListView_GetCheckState (pInfo->hwndFrom, cItem) != FALSE;

							SelectColumn (::GetDlgItem (_hWnd, IDC_DBF_FIELDS), bChecked ? pThis->m_cFields [cAttribute] : -1);
						}
					}
					if ((pInfo2->uNewState & LVIS_STATEIMAGEMASK) != 0 && (pInfo2->uOldState & LVIS_STATEIMAGEMASK) != 0) {
						if (! pThis->m_bInFillExtraAttributes && cItem != -1) {
							const bool bChecked = ListView_GetCheckState (pInfo->hwndFrom, cItem) != FALSE;
							if (! bChecked) {
								pThis->m_cFields [cAttribute] = -1;
								pThis->_FillExtraAttributes (pInfo->hwndFrom, false);
								SelectColumn (::GetDlgItem (_hWnd, IDC_DBF_FIELDS), -1);
							} else {
								if (pThis->m_cFields [cAttribute] == -1)
									ListView_SetCheckState (pInfo->hwndFrom, cItem, FALSE);
							}
						}
					}
					break;
				}

				case NM_CLICK:
				case LVN_ITEMACTIVATE: {
					const NMITEMACTIVATE * const pInfo2 = reinterpret_cast<const NMITEMACTIVATE *> (_lParam);
					if (pInfo->idFrom == IDC_DBF_FIELDS) {
						//
						// Another DBF field selected.
						//

						pThis->_SelectDbfColumn (_hWnd, pInfo2->iSubItem);
					}

					break;
				}

				case LVN_COLUMNCLICK: {
					const NMLISTVIEW * const pInfo2 = reinterpret_cast<const NMLISTVIEW *> (_lParam);
					if (pInfo->idFrom == IDC_DBF_FIELDS) {
						//
						// Another DBF field selected.
						//

						pThis->_SelectDbfColumn (_hWnd, pInfo2->iSubItem);
					}

					break;
				}
			}
			break;
        }
	}

	return FALSE;
}