//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Status bar control.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Status.h"
# include "Toolbar.h"
# include "Levels.h"
# include "Scale.h"
# include "Log.h"
# include "WebMap.h"
# include "MapView.h"
# include "COMStatusEvents.h"
# include "PtAlgo.h"
# include "Wrappers.h"
# include "PrintUnits.h"
# include "Globals.h"

enum {
	sbStatus      = 0,
	sbWarnings    = 1,
	sbProgress    = 2,
	sbProgressNum = 3,
	sbBlank1      = 4,
	sbScale       = 5,
	sbLevel       = 6,
	sbGoogle      = 7,
	sbBlank2      = 8,
	sbPosition    = 9,
	sbBlank3      = 10,
	sbSelection   = 11,
	sbBlank4      = 12,
	sbInfo        = 13
};

void _HideProgressStatus ();

static
void _SendMessage (HWND _hWnd, UINT _uMsg, WPARAM _wParam, LPARAM _lParam) {
	if (::GetCurrentThreadId () == g_dwGuiThreadId) {
		// NOTE: the timeout is used to avoid deadlocks.
		DWORD dwRet = 0;
		::SendMessageTimeout (_hWnd, _uMsg, _wParam, _lParam, SMTO_ABORTIFHUNG, 1000, & dwRet);
	} else
		::PostMessage (_hWnd, _uMsg, _wParam, _lParam);
}

/////////////////////////////////////////////

HWND g_hStatusWnd;

static size_t g_cSelectedItems_prev = 0;

bool InitStatusWindow () {
	assert (g_hWnd);

	g_hStatusWnd = ::CreateStatusWindow (WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | SBT_TOOLTIPS | SBARS_SIZEGRIP, GetStr (IDS_NO_FILES), g_hWnd, 101);
	if (! g_hStatusWnd)
		return false;

	int widths [] = {
		100, // Status
		35,  // Warnings
		108, // Progress bar
		40,  // Progress %
		15,
		70,  // Scale
		55,  // Level
		30,  // Google level
		15,
		155, // Position
		15,
		45,  // Selection
		15,
		500, // Info
	};

	// Get width factor for non-standard font in the status bar.	
	const HFONT hStatusFont = reinterpret_cast<HFONT> (::SendMessage (g_hStatusWnd, WM_GETFONT, (WPARAM) g_hFont, FALSE));
	CClientDC dcStatus (g_hStatusWnd);
	const HFONT hPrevFont = reinterpret_cast<HFONT> (::SelectObject (dcStatus, hStatusFont));
	TEXTMETRIC tm;
	::GetTextMetrics (dcStatus, & tm);
	const float fScale = tm.tmAveCharWidth/5.f;
	::SelectObject (dcStatus, hPrevFont);
/*
	RECT rect;
	::GetWindowRect (g_hStatusWnd, & rect);
	const float fScale = (rect.bottom - rect.top - 4)/(20. - 4);
*/
	for (size_t c = 0; c < sizeof (widths)/sizeof (widths [0]); ++ c) {
		if (c != sbProgress)
			widths [c] *= fScale;
		if (c >= 1)
			widths [c] += widths [c - 1];
	}
	::SendMessage (g_hStatusWnd, SB_SETPARTS, sizeof (widths)/sizeof (widths [0]), long (widths));
	::SendMessage (g_hStatusWnd, SB_SETTEXT, sbBlank1 | SBT_NOBORDERS, long (NULL));
	::SendMessage (g_hStatusWnd, SB_SETTEXT, sbBlank2 | SBT_NOBORDERS, long (NULL));
	::SendMessage (g_hStatusWnd, SB_SETTEXT, sbBlank3 | SBT_NOBORDERS, long (NULL));
	::SendMessage (g_hStatusWnd, SB_SETTEXT, sbBlank4 | SBT_NOBORDERS, long (NULL));

	return true;
}

void DestroyStatusWindow () {
	// TODO:
}

void ResetStatus () {
	SetStatus (GetStr (IDS_NO_FILES));
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbWarnings,    (LPARAM) "");
	SetProgressStatus (0);
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbProgressNum, (LPARAM) "");
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbScale,       (LPARAM) "");
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbLevel,       (LPARAM) "");
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbGoogle,      (LPARAM) "");
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbPosition,    (LPARAM) "");
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbSelection,   (LPARAM) "");
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbInfo,        (LPARAM) "");

	g_cSelectedItems_prev = 0;

	if (g_hScaleCB) {
		DWORD dwRet = 0;
		::SendMessageTimeout (g_hScaleCB, WM_SETTEXT, 0, (LPARAM) "", SMTO_ABORTIFHUNG, 1000, & dwRet);
	}
}

void OnStatusBarClick (size_t _cSection) {
	switch (_cSection) {
		case sbWarnings:
			ShowMessageLog ();
			break;
	}
}

/////////////////////////////////////////////

void SetStatusReady () {
	_HideProgressStatus ();

	if (GetWarningsCount () > 0)
		SetStatus (GetStr (IDS_READY_WARN));
	else
		SetStatus (GetStr (IDS_READY));
}

void _Remote_Fire_OnStatus (DWORD _dwData) {
	string_t * const pStr = reinterpret_cast<string_t *> (_dwData);
	if (pStr) {
		Fire_OnStatus (pStr->c_str ());
		delete pStr;
	}
}

static char g_strStatus [64];

void SetStatus (const char * _strStatus) {
	if (::strcmp (_strStatus, g_strStatus) == 0)
		return;

	g_strStatus [63] = '\0';
	::strncpy (g_strStatus, _strStatus, 64);

	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbStatus, (LPARAM) g_strStatus);

	if (g_dwGuiThreadId != ::GetCurrentThreadId ()) {
		string_t * const pStr = new string_t (_strStatus);
		::PostMessage (g_hMapWnd, WM_USER_INVOKE1, (DWORD) pStr, (LPARAM) _Remote_Fire_OnStatus);
	} else
		Fire_OnStatus (g_strStatus);
}

/////////////////////////////////////////////

static char g_strStatusWarnings [16];

void SetStatusWarnings (size_t _cWarnings) {
	if (_cWarnings == 0)
		::sprintf (g_strStatusWarnings, "");
	else
		::sprintf (g_strStatusWarnings, "W:%d", _cWarnings);

	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbWarnings, (LPARAM) g_strStatusWarnings);
}

/////////////////////////////////////////////

static int g_iProgress = 0;

static
void _HideProgressStatus () {
	g_iProgress = 0;
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbProgress | SBT_OWNERDRAW, g_iProgress);
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbProgressNum, (LPARAM) "");

	if (g_dwGuiThreadId != ::GetCurrentThreadId ())
		::PostMessage (g_hMapWnd, WM_USER_INVOKE1, 0, (LPARAM) Fire_OnProgress);
	else
		Fire_OnProgress (0);
}

void SetProgressStatus (int _iProgress) {
	if (_iProgress > 100)
		_iProgress = 100;
	if (_iProgress < 0)
		_iProgress = 0;

	if (_iProgress == g_iProgress)
		return;
	g_iProgress = _iProgress;

	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbProgress | SBT_OWNERDRAW, g_iProgress);

	static char strText [32];
	strText [31] = '\0';
	::sprintf (strText, "% 3d%%", g_iProgress);
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbProgressNum, (LPARAM) strText);

	if (g_dwGuiThreadId != ::GetCurrentThreadId ())
		::PostMessage (g_hMapWnd, WM_USER_INVOKE1, _iProgress, (LPARAM) Fire_OnProgress);
	else
		Fire_OnProgress (_iProgress);
}

/////////////////////////////////////////////

void SetPositionStatus (const point_t & _pt) {
	static char strText [64];
	strText [63] = '\0';
	PrintPosition (strText, _pt);
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbPosition, (LPARAM) strText);
}

/////////////////////////////////////////////

void SetScaleStatus (float _fScale) {
	static char strText [64];
	strText [63] = '\0';
	::sprintf (strText, "1 : %d", long (_fScale*c_pixels_per_cm*100));
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbScale, (LPARAM) strText);

	static char strText2 [64];
	strText2 [63] = '\0';
	GetZoomLevel (_fScale, strText2);
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbLevel, (LPARAM) strText2);

	static char strText3 [32];
	strText3 [31] = '\0';
	GetScale (_fScale, strText3);
	if (g_hScaleCB) {
		DWORD dwRet = 0;
		::SendMessageTimeout (g_hScaleCB, WM_SETTEXT, 0, (LPARAM) strText3, SMTO_ABORTIFHUNG, 1000, & dwRet);
	}

	static char strGoogle [8];
	strGoogle [0] = '\0';
	if (g_bShowGoogleMaps)
		::_snprintf (strGoogle, 8, "G:%d", GetGoogleMapsLevel (_fScale));
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbGoogle, (LPARAM) strGoogle);
}

//////////////////////////////////////////////////////////////////////////

# include "Selection.h"

void SetSelectionStatus () {
	const size_t cSelectedItems = g_Selection.size ();
/*	const size_t cItems = g_Selection.size ();
	for (size_t c = 0; c < cItems; ++ c) {
		const find_info_t & SelItem = g_Selection [c];
		if (SelItem.pRenderer) {
			if (SelItem.pRenderer->bDeleted)
				continue;
		} else if (SelItem.pLayer) {
			if (SelItem.iRgn->IsDeleted ())
				continue;
		}

		++ cSelectedItems;
	}*/

	if (cSelectedItems == g_cSelectedItems_prev)
		return;
	g_cSelectedItems_prev = cSelectedItems;

	static char strText [32];
	strText [0] = '\0';
	if (cSelectedItems != 0)
		::sprintf (strText, "S:%d", cSelectedItems);
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbSelection, (LPARAM) strText);
}

//////////////////////////////////////////////////////////////////////////

void SetStatusInfo (const char * _strStatus) {
	static char strText [256];
	::strncpy (strText, _strStatus ? _strStatus : "", ARRAY_SIZE (strText));
	strText [ARRAY_SIZE (strText) - 1] = '\0';
	_SendMessage (g_hStatusWnd, SB_SETTEXT, sbInfo, (LPARAM) strText);

	Fire_OnStatusInfo (strText);
}

//////////////////////////////////////////////////////////////////////////

CProgressBar::CProgressBar (size_t _cFrom, size_t _cTo) :
	m_cProgressFrom (_cFrom),
	m_cProgressTo (_cTo),
	m_cValueFrom (_cFrom),
	m_cValueTo (_cTo),
	m_cLastProgress (0)
{}

size_t CProgressBar::Progress (size_t _cValue) const {
	if (m_cProgressFrom >= m_cProgressTo)
		return m_cProgressTo;
	if (m_cValueFrom >= m_cValueTo)
		return m_cProgressFrom;
	
	if (_cValue < m_cValueFrom)
		_cValue = m_cValueFrom;
	if (_cValue > m_cValueTo)
		_cValue = m_cValueTo;

	return m_cProgressFrom + ::MulDiv (m_cProgressTo - m_cProgressFrom, _cValue - m_cValueFrom, m_cValueTo - m_cValueFrom);
}

void CProgressBar::SetValue (size_t _cValue) const {
	const size_t cCurrent = Progress (_cValue);
	if (cCurrent <= m_cLastProgress)
		return;

	SetProgressStatus (cCurrent);
	m_cLastProgress = cCurrent;
}
	
void CProgressBar::SetValueRange (size_t _cFromValue, size_t _cToValue) {
	m_cValueFrom       = _cFromValue;
	m_cValueTo         = _cToValue;
	m_cLastSubBarValue = _cFromValue;
}

CProgressBar CProgressBar::GetSubBar (size_t _cFromValue, size_t _cToValue) const {
	return CProgressBar (Progress (_cFromValue), Progress (_cToValue));
}

CProgressBar CProgressBar::GetNextSubBar (size_t _cLengthValue) const {
	const size_t cFirst = m_cLastSubBarValue;
	const size_t cLast  = m_cLastSubBarValue + _cLengthValue;
	
	m_cLastSubBarValue = cLast;
	return GetSubBar (cFirst, cLast);
}
