//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Current tool handling.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Tool.h"
# include "Menu.h"
# include "Map.h"
# include "Settings.h"
# include "MapView.h"
# include "NewMap.h"
# include "Position.h"
# include "Str.h"
# include "Wrappers.h"
# include "Globals.h"

HCURSOR g_hCursor;
HCURSOR g_hPermanentCursor;

CWaiter::CWaiter () {
	::SetCursor (g_hCursor = g_hWaitCursor);
}

void CWaiter::Stop () {
	::SetCursor (g_hCursor = g_hPermanentCursor);
}

///////////////////////////////////////////////

CTool * CTool::s_pFirst;

CTool::CTool (Tool_t _ToolId) :
	m_ToolId (_ToolId),
	m_bLastCtxMenuCmdDefined (false)
{
	m_pNext = CTool::s_pFirst;
	CTool::s_pFirst = this;
}

CTool * CTool::FindTool (Tool_t _ToolId) {
	if (_ToolId > toolMax)
		_ToolId = toolDefault;

	CTool * pTool;
	for (pTool = CTool::s_pFirst; pTool; pTool = pTool->m_pNext)
		if (pTool->m_ToolId == _ToolId)
			return pTool;

	for (pTool = CTool::s_pFirst; pTool; pTool = pTool->m_pNext)
		if (pTool->m_ToolId == toolDefault)
			return pTool;

	return NULL;
}

CTool * CTool::s_pTool;

bool CTool::_Init () {
	CTool::s_cXDrag = ::GetSystemMetrics (SM_CXDRAG);
	CTool::s_cYDrag = ::GetSystemMetrics (SM_CYDRAG);

	for (CTool * pTool = CTool::s_pFirst; pTool; pTool = pTool->m_pNext)
		if (! pTool->Init ())
			return false;

	CTool::SetCurrentTool (static_cast<Tool_t> (GetSetting ("Tool", (DWORD) toolDefault)));

	return true;
}

void CTool::_Destroy () {
	for (CTool * pTool = CTool::s_pFirst; pTool; pTool = pTool->m_pNext)
		pTool->Destroy ();
}

void CTool::_OnNewMap () {
	for (CTool * pTool = CTool::s_pFirst; pTool; pTool = pTool->m_pNext)
		pTool->OnNewMap ();
}

void CTool::_OnRemoveMap () {
	for (CTool * pTool = CTool::s_pFirst; pTool; pTool = pTool->m_pNext)
		pTool->OnRemoveMap ();
}

static void CTool_OnRemoveMap () {CTool::_OnRemoveMap ();}
INVOKE_AFTER_REMOVE_MAP (CTool_OnRemoveMap);

void CTool::SetCurrentTool (Tool_t _Tool, bool _bSaveSetting) {
	CTool * const pTool = FindTool (_Tool);
	if (pTool == NULL)
		return;

	if (pTool == s_pTool) {
		// NOTE: clicking toolbar group button being already in pressed state will unpress it.
		// Make sure that the toolbar button will be still pressed.
		UpdateMenu ();

		s_pTool->OnSwitchSubTool ();
		return;
	}

	if (s_pTool)
		s_pTool->OnDeactivate ();

	s_pTool = pTool;

	if (_bSaveSetting)
		SaveSetting ("Tool", (DWORD) _Tool);
	UpdateMenu ();

	s_pTool->OnActivate ();

	Redraw ();
}

Tool_t CTool::GetCurrentToolId () {
	return s_pTool ? s_pTool->m_ToolId : toolDefault;
}

size_t CTool::s_cXDrag,
       CTool::s_cYDrag;
int    CTool::s_iXDragStart,
       CTool::s_iYDragStart;
DWORD  CTool::s_dwTimeDragStart;
bool   CTool::m_bSuppressDrag = false;

bool   CTool::s_bDND = false;
long   CTool::s_lX_DND_Start;
long   CTool::s_lY_DND_Start;
point_t CTool::s_ptDND_Start;
DWORD  CTool::s_dwTime_DND_Start;
long   CTool::s_lX_DND;
long   CTool::s_lY_DND;
long   CTool::s_lX;
long   CTool::s_lY;
DWORD  CTool::s_dwKeys;

void CTool::SetPermanentCursor (HCURSOR _hcur) {
	g_hCursor = g_hPermanentCursor = _hcur;

	POINT p;
	::GetCursorPos (& p);
	if (::WindowFromPoint (p) == g_hMapWnd)
		::SetCursor (_hcur);
}

void CTool::StartDND (long _lX, long _lY) {
	if (CTool::s_bDND)
		return;

	CTool::s_bDND             = true;

	CTool::s_dwTime_DND_Start = ::GetTickCount ();
	CTool::s_lX_DND_Start     = _lX;
	CTool::s_lY_DND_Start     = _lY;
	ScreenToPt (_lX, _lY, CTool::s_ptDND_Start);

	CTool::s_lX_DND           = _lX;
	CTool::s_lY_DND           = _lY;

	::SetCapture (g_hMapWnd);
}

void CTool::StopDND () {
	if (! CTool::s_bDND)
		return;

	CTool::s_bDND = false;
	::ReleaseCapture ();
}

void CTool::_OnScroll () {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return;
	}
	if (CTool::s_pTool)
		CTool::s_pTool->OnScroll ();
}

bool CTool::_OnLDown (int _x, int _y, DWORD _dwKeys) {
	CTool::s_lX              = _x;
	CTool::s_lY              = _y;
	CTool::s_dwKeys          = _dwKeys;

	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	CTool::m_bSuppressDrag   = true;
	CTool::s_iXDragStart     = _x;
	CTool::s_iYDragStart     = _y;
	CTool::s_dwTimeDragStart = ::GetTickCount ();

	if (CTool::s_pTool)
		return CTool::s_pTool->OnLDown (_x, _y, _dwKeys);

	return false;
}

bool CTool::_OnLUp (int _x, int _y, DWORD _dwKeys) {
	CTool::s_lX              = _x;
	CTool::s_lY              = _y;
	CTool::s_dwKeys          = _dwKeys;

	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	CTool::m_bSuppressDrag   = false;
	if (CTool::s_pTool)
		return CTool::s_pTool->OnLUp (_x, _y, _dwKeys);

	return false;
}

void CTool::_OnMove (int _x, int _y, DWORD _dwKeys) {
	CTool::s_lX              = _x;
	CTool::s_lY              = _y;
	CTool::s_dwKeys          = _dwKeys;

	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return;
	}

	if (CTool::m_bSuppressDrag) {
		if (
			size_t (::labs (_x - CTool::s_iXDragStart)) <= CTool::s_cXDrag &&
			size_t (::labs (_y - CTool::s_iYDragStart)) <= CTool::s_cYDrag &&
			::GetTickCount () - CTool::s_dwTimeDragStart <= 1000
		)
			return;

		CTool::m_bSuppressDrag = false;
	}

	if (CTool::s_bDND) {
		CTool::s_lX_DND = _x;
		CTool::s_lY_DND = _y;
	}

	if (CTool::s_pTool)
		CTool::s_pTool->OnMove (_x, _y, _dwKeys);
}

bool CTool::_OnLDblClk (int _x, int _y, DWORD _dwKeys) {
	CTool::s_lX              = _x;
	CTool::s_lY              = _y;
	CTool::s_dwKeys          = _dwKeys;

	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}
	if (CTool::s_pTool)
		return CTool::s_pTool->OnLDblClk (_x, _y, _dwKeys);

	return false;
}

bool CTool::_OnRDown (int _x, int _y, DWORD _dwKeys) {
	CTool::s_lX              = _x;
	CTool::s_lY              = _y;
	CTool::s_dwKeys          = _dwKeys;

	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}
	if (CTool::s_pTool)
		return CTool::s_pTool->OnRDown (_x, _y, _dwKeys);

	return false;
}

bool CTool::_OnRUp (int _x, int _y, DWORD _dwKeys) {
	CTool::s_lX              = _x;
	CTool::s_lY              = _y;
	CTool::s_dwKeys          = _dwKeys;

	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}
	if (CTool::s_pTool)
		return CTool::s_pTool->OnRUp (_x, _y, _dwKeys);

	return false;
}

void CTool::_OnPaint (HDC _hDC, const RECT & _rect) {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return;
	}
	if (CTool::s_pTool)
		CTool::s_pTool->OnPaint (_hDC, _rect);
}

bool CTool::_OnCommand (WORD _wID, bool _bFromMenu) {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}
	if (CTool::s_pTool == NULL)
		return false;

	const bool bRet = CTool::s_pTool->OnCommand (_wID);
	CTool::s_bLastCtxMenuCmdFailed = ! bRet;

	bool bCtxMenuCmd = bRet;
	if (! bRet) {
		switch (_wID) {
			case ID_EDIT_COPY:
			case ID_EDIT_CUT:
			case ID_EDIT_DELETE:
			case ID_RGN_PROPERTIES:
			case ID_RASTER_PROPERTIES:
				// NOTE: these 'generic' commands processed in main window proc but defined in ctx menus.
				bCtxMenuCmd = true;
				break;
		}
	}
	if (bCtxMenuCmd && _bFromMenu) {
		const wd2str_t::const_iterator i = CTool::s_LastCtxMenuItems.find (_wID);
		if (i != CTool::s_LastCtxMenuItems.end ()) {
			CTool::s_pTool->m_bLastCtxMenuCmdDefined = true;
			CTool::s_pTool->m_wdLastCtxMenuCmd       = _wID;
			CTool::s_pTool->m_strLastCtxMenuCmd      = i->second;

			CTool::s_bLastCtxMenuCmdFailed           = false;

			UpdateMenu ();
		}
	}

	return bRet;
}

bool CTool::_OnCtxMenu (int _x, int _y) {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	if (CTool::s_pTool)
		return CTool::s_pTool->OnCtxMenu (_x, _y);

	return false;
}

bool CTool::_OnEsc () {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	if (CTool::s_pTool)
		return CTool::s_pTool->OnEsc ();

	return false;
}

bool CTool::_OnBack () {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	if (CTool::s_pTool)
		return CTool::s_pTool->OnBack ();

	return false;
}

bool CTool::_OnEnter () {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	if (CTool::s_pTool)
		return CTool::s_pTool->OnEnter ();

	return false;
}

bool CTool::_OnDel () {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	if (CTool::s_pTool)
		return CTool::s_pTool->OnDel ();

	return false;
}

bool CTool::_OnIns () {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	if (CTool::s_pTool)
		return CTool::s_pTool->OnIns ();

	return false;
}

bool CTool::_OnArrow (int _dx, int _dy) {
	{
		CCritSec::CLock lock (g_cs);
		if (g_map.rectBound.Invalid ())
			return false;
	}

	if (CTool::s_pTool)
		return CTool::s_pTool->OnArrow (_dx, _dy);

	return false;
}

////////////////////////////////////////////////////////////////

CTool::wd2str_t CTool::s_LastCtxMenuItems;
bool CTool::s_bLastCtxMenuCmdFailed;

bool CTool::IsLastCtxMenuCmdDefined () {
	return CTool::s_pTool && CTool::s_pTool->m_bLastCtxMenuCmdDefined;
}

const char * CTool::GetLastCtxMenuCmd () {
	return CTool::s_pTool ? CTool::s_pTool->m_strLastCtxMenuCmd.c_str () : NULL;
}

void CTool::RepeatLastCtxMenuCmd () {
	if (CTool::s_pTool && CTool::s_pTool->m_bLastCtxMenuCmdDefined) {
		::SendMessage (g_hWnd, WM_COMMAND, CTool::s_pTool->m_wdLastCtxMenuCmd, 0);
		if (CTool::s_bLastCtxMenuCmdFailed)
			::MessageBeep (MB_ICONERROR);
	}
}

void CTool::PrepareCtxMenu (HMENU _hMenu) {
	s_LastCtxMenuItems.clear ();
	_PrepareCtxMenu (_hMenu, NULL);
}

void CTool::_PrepareCtxMenu (HMENU _hMenu, const char * _strParent) {
	const size_t cItems = ::GetMenuItemCount (_hMenu);
	for (size_t cPos = 0; cPos < cItems; ++ cPos) {
		if (IsSeparator (_hMenu, cPos))
			continue;

		char strName [256] = "";
		if (::GetMenuString (_hMenu, cPos, strName, ARRAY_SIZE (strName), MF_BYPOSITION)) {
			// Trim accelerator.
			char * const pAcc = lstrchr (strName, '\t');
			if (pAcc)
				* pAcc = '\0';
		}

		string_t strFull;
		if (_strParent != NULL) {
			strFull = _strParent;
			strFull.append (" | ");
		}
		strFull.append (strName);

		const HMENU hSubMenu = ::GetSubMenu (_hMenu, cPos);
		if (hSubMenu != NULL) {
			_PrepareCtxMenu (hSubMenu, strFull.c_str ());
			continue;
		}
			
		const UINT uiCmd = ::GetMenuItemID (_hMenu, cPos);
		CTool::s_LastCtxMenuItems [uiCmd] = strFull;
	}
}
