# ifndef __Tool_H_
# define __Tool_H_

# include "PtAlgo.h"

	// Current cursor: permanent or 'wait' one.
extern HCURSOR g_hCursor;
	// Permanent cusor.
extern HCURSOR g_hPermanentCursor;

///////////////////////////////////////////

enum Tool_t {
	toolDrag      = 0,
	toolZoom      = 1,
	toolSelect    = 2,
	toolEditNodes = 3,
	toolCreate    = 4,
	toolTrim      = 5,
	toolTransform = 6,
	toolDummy     = 7,
	toolRouting   = 8,
	toolRotate    = 9,
	toolDistance  = 10,

	toolMax,
	toolDefault = toolDrag
};

class CTool {
  public:
	virtual bool Init () {return true;}
	virtual void Destroy () {}

	virtual void OnNewMap     () {}
	virtual void OnRemoveMap  () {}
	virtual void OnActivate   () {}
	virtual void OnDeactivate () {}

	virtual bool OnLDown   (int _x, int _y, DWORD _dwKeys) {return false;}
	virtual bool OnLUp     (int _x, int _y, DWORD _dwKeys) {return false;}
	virtual bool OnLDblClk (int _x, int _y, DWORD _dwKeys) {return OnLDown (_x, _y, _dwKeys);}
	virtual bool OnRDown   (int _x, int _y, DWORD _dwKeys) {return false;}
	virtual bool OnRUp     (int _x, int _y, DWORD _dwKeys) {return false;}
	virtual void OnMove    (int _x, int _y, DWORD _dwKeys) {}

	virtual void OnScroll () {OnMove (CTool::s_lX, CTool::s_lY, CTool::s_dwKeys);}

	virtual bool OnCtxMenu (int _x, int _y) {return false;}
	virtual bool OnEsc   () {return false;}
	virtual bool OnBack  () {return false;}
	virtual bool OnEnter () {return false;}
	virtual bool OnDel   () {return false;}
	virtual bool OnIns   () {return false;}
	virtual bool OnArrow (int _dx, int _dy) {return false;}
	virtual bool OnCommand (WORD _wID) {return false;}
	virtual void OnSwitchSubTool () {}

	virtual void OnPaint (HDC _hDC, const RECT & _rect) const {}

	//////////////////////////

	static CTool * GetCurrentTool () {return s_pTool;}
	static Tool_t GetCurrentToolId ();
	static void SetCurrentTool (Tool_t _Tool, bool _bSaveSetting = true);

	static bool PendingDND () {return s_bDND;}
	static void SetPermanentCursor (HCURSOR _hcur);

	static bool _Init ();
	static void _Destroy ();
	static void _OnNewMap ();
	static void _OnRemoveMap ();
	static bool _OnCtxMenu (int _x, int _y);
	static bool _OnEsc   ();
	static bool _OnBack  ();
	static bool _OnEnter ();
	static bool _OnDel   ();
	static bool _OnIns   ();
	static bool _OnArrow   (int _dx, int _dy);
	static void _OnMove    (int _x, int _y, DWORD _dwKeys);
	static bool _OnLDown   (int _x, int _y, DWORD _dwKeys);
	static bool _OnLUp     (int _x, int _y, DWORD _dwKeys);
	static bool _OnLDblClk (int _x, int _y, DWORD _dwKeys);
	static bool _OnRDown   (int _x, int _y, DWORD _dwKeys);
	static bool _OnRUp     (int _x, int _y, DWORD _dwKeys);
	static void _OnPaint   (HDC _hDC, const RECT & _rect);
	static bool _OnCommand (WORD _wID, bool _bFromMenu);
	static void _OnScroll ();

	static bool IsLastCtxMenuCmdDefined ();
	static const char * GetLastCtxMenuCmd ();
	static void RepeatLastCtxMenuCmd ();

  private:
	static CTool * s_pTool;

	// Data for unwanted drag suppression while short click.
	static size_t s_cXDrag, s_cYDrag;
	static int s_iXDragStart, s_iYDragStart;
	static DWORD s_dwTimeDragStart;
	static bool m_bSuppressDrag;

	// Data for RepeatLastCtxMenuCmd().
	bool m_bLastCtxMenuCmdDefined;
	WORD m_wdLastCtxMenuCmd;
	string_t m_strLastCtxMenuCmd;
	typedef std::map<WORD, string_t> wd2str_t;
	static wd2str_t s_LastCtxMenuItems;
	static bool s_bLastCtxMenuCmdFailed;

  protected:
	// Drag-and-drop mode flag.
	static bool s_bDND;
	// Drag-and-drop start position.
	static long s_lX_DND_Start, s_lY_DND_Start;
	static point_t s_ptDND_Start;
	// Drag-and-drop start time.
	static DWORD s_dwTime_DND_Start;
	// Drag-and-drop current position.
	static long s_lX_DND, s_lY_DND;
	// 
	static long s_lX, s_lY;
	static DWORD s_dwKeys;

	void StartDND (long _lX, long _lY);
	void StopDND ();

	static void PrepareCtxMenu (HMENU _hMenu);
  private:
	static void _PrepareCtxMenu (HMENU _hMenu, const char * _strParent);

  public:
	const Tool_t m_ToolId;

	static CTool * s_pFirst;
	CTool * m_pNext;
	static CTool * FindTool (Tool_t _ToolId);

	CTool (Tool_t _ToolId);
};


# define DECLARE_TOOL(_CTool) \
	static _CTool _CTool##_instance;


# endif // __Tool_H_