//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// 'Drag map' tool.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Tool.h"
# include "Map.h"
# include "Settings.h"
# include "Position.h"
# include "MapView.h"
# include "Globals.h"

class CDragTool : public CTool {
  public:
	CDragTool () : CTool (toolDrag) {}

	virtual bool Init ();
	virtual void OnActivate ();

	virtual bool OnLDown (int _x, int _y, DWORD _dwKeys);
	virtual bool OnLUp   (int _x, int _y, DWORD _dwKeys);
	virtual void OnMove  (int _x, int _y, DWORD _dwKeys);

	virtual void OnScroll () {}

  private:
	long m_lThumbPosX0;
	long m_lThumbPosY0;

	HCURSOR m_hModeDragCursor;
	HCURSOR m_hDragCursor;
//	HCURSOR m_hNoDragCursor;
};

DECLARE_TOOL (CDragTool);

bool CDragTool::Init () {
	assert (g_hInst);
	m_hModeDragCursor = ::LoadCursor (g_hInst, MAKEINTRESOURCE (IDC_CursorModeDrag));
	m_hDragCursor     = ::LoadCursor (g_hInst, MAKEINTRESOURCE (IDC_CursorDrag));
//	m_hNoDragCursor   = ::LoadCursor (g_hInst, MAKEINTRESOURCE (IDC_CursorNoDrag));

	return true;
}

void CDragTool::OnActivate () {
	SetPermanentCursor (m_hModeDragCursor);
}

bool CDragTool::OnLDown (int _x, int _y, DWORD _dwKeys) {
	SetPermanentCursor (m_hDragCursor);

	StartDND (_x, _y);

	SCROLLINFO si = {sizeof (si), SIF_ALL};
	::GetScrollInfo (g_hMapWnd, SB_HORZ, & si);
	m_lThumbPosX0 = si.nPos;

	::GetScrollInfo (g_hMapWnd, SB_VERT, & si);
	m_lThumbPosY0 = si.nPos;

	return true;
}

bool CDragTool::OnLUp (int _x, int _y, DWORD _dwKeys) {
	SetPermanentCursor (m_hModeDragCursor);

	if (! CTool::s_bDND)
		return false;
	StopDND ();
	ScrollMap (SB_ENDSCROLL, 0, SB_ENDSCROLL, 0);

	return true;
}

void CDragTool::OnMove (int _x, int _y, DWORD _dwKeys) {
	if (CTool::s_bDND)
		ScrollMap (
			SB_THUMBTRACK, m_lThumbPosX0 + CTool::s_lX_DND_Start - _x,
			SB_THUMBTRACK, m_lThumbPosY0 + CTool::s_lY_DND_Start - _y
		);
}

