//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// 'Select' tool.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Tool.h"
# include "Selection.h"
# include "Types.h"
# include "MapView.h"
# include "Load.h"
# include "Menu.h"
# include "RgnProperties.h"
# include "RasterProperties.h"
# include "View3D.h"
# include "TrackVideo.h"
# include "EditNumber.h"
# include "EditPoint.h"
# include "UndoRedo.h"
# include "MoveObject.h"
# include "MoveImage.h"
# include "Rename.h"
# include "SetComment.h"
# include "ChangeType.h"
# include "SetRgnLevels.h"
# include "ChangeCity.h"
# include "ChangeZip.h"
# include "SetRgnExtra.h"
# include "SetEntryPoint.h"
# include "ConvertTrack.h"
# include "CreateObject.h"
# include "DeleteObject.h"
# include "DeleteElement.h"
# include "Bookmark.h"
# include "SetBookmark.h"
# include "NoteDrawing.h"
# include "SetNoteDrawing.h"
# include "ShowHideAttachment.h"
# include "JoinObjects.h"
# include "ExtractAllElements.h"
# include "ExtractElement.h"
# include "MergeObjects.h"
# include "ClosePolyline.h"
# include "ReversePoints.h"
# include "TrimByPolygon.h"
# include "CombinedAction.h"
# include "SetNumberAction.h"
# include "WebMap.h"
# include "Position.h"
# include "Scale.h"
# include "Levels.h"
# include "SelectLevels.h"
# include "EditText.h"
# include "SelectList.h"
# include "ParseHelpers.h"
# include "DrawHelpers.h"
# include "DDBitmap.h"
# include "Paste.h"
# include "fast_floor.h"
# include "Globals.h"

class CSelectTool : public CTool {
  public:
	CSelectTool () :
		CTool (toolSelect),
		m_pDraggedBuffer (NULL),
		m_bFirstMoveInDND (false)
	{}

	virtual bool Init ();
	virtual void Destroy ();

	virtual void OnNewMap ();
	virtual void OnRemoveMap ();
	virtual void OnActivate ();

	virtual bool OnLDown   (int _x, int _y, DWORD _dwKeys);
	virtual bool OnLUp     (int _x, int _y, DWORD _dwKeys);
	virtual void OnMove    (int _x, int _y, DWORD _dwKeys);
	virtual bool OnRUp     (int _x, int _y, DWORD _dwKeys);
	virtual bool OnLDblClk (int _x, int _y, DWORD _dwKeys);

	virtual bool OnArrow (int _dx, int _dy);
	virtual bool OnEsc ();
	virtual bool OnEnter ();

	virtual void OnPaint (HDC _hDC, const RECT & _rect) const;

	virtual bool OnCtxMenu (int _x, int _y);

	virtual bool OnCommand (WORD _wID);

  private:
	HCURSOR m_hcurDrag;
	HCURSOR m_hcurSetEntryPoint;
	HPEN m_hpenGroupSelection;

	CDDBitmap * m_pDraggedBuffer;
	bool m_bFirstMoveInDND;

	void _MoveSelection (long _lXOffset, long _lYOffset);

	bool m_bFrameSelectionMode;

	rgn_t * m_pRgnToSetEntryPoint;
	size_t m_cEntryPointToSet;
	bool IsInSetEntryPoint () const {return m_pRgnToSetEntryPoint && ! m_pRgnToSetEntryPoint->IsDeleted ();}
	void _SetEntryPoint (int _x, int _y);

	void _ResetSavedPointers ();
	DWORD m_dwSavedRouteParams;
	bool m_bHasSavedRouteParams;
	bool m_bHasSavedPA;
	city_t * m_pSavedCity;
	zip_t * m_pSavedZip;
	string_t m_strSavedStreet;
	DWORD m_dwPANumber;
};

DECLARE_TOOL (CSelectTool);

bool CSelectTool::Init () {
	m_hcurDrag          = ::LoadCursor (g_hInst, MAKEINTRESOURCE (IDC_CursorDrag));
	m_hcurSetEntryPoint = ::LoadCursor (g_hInst, MAKEINTRESOURCE (IDC_CursorSetEntryPoint));

	m_hpenGroupSelection = ::CreatePen (PS_SOLID, 1, c_colGray);

	m_dwSavedRouteParams = 0;
	m_bHasSavedRouteParams = false;
	_ResetSavedPointers ();

	return true;
}

void CSelectTool::Destroy () {
	if (m_hpenGroupSelection)
		::DeleteObject (m_hpenGroupSelection);

	delete m_pDraggedBuffer;
}

void CSelectTool::_ResetSavedPointers () {
	m_pRgnToSetEntryPoint = NULL;
	m_cEntryPointToSet = -1;

	m_pSavedCity = NULL;
	m_pSavedZip = NULL;
	m_strSavedStreet.clear ();
	m_dwPANumber = 0;
	m_bHasSavedPA = false;
}

void CSelectTool::OnActivate () {
	SetPermanentCursor (g_hArrowCursor);

	m_bFrameSelectionMode = false;

	m_pRgnToSetEntryPoint = NULL;
	m_cEntryPointToSet = -1;
}

void CSelectTool::OnNewMap () {
	_ResetSavedPointers ();
}

void CSelectTool::OnRemoveMap () {
	_ResetSavedPointers ();
}

bool CSelectTool::OnLDown (int _x, int _y, DWORD _dwKeys) {
	if (IsInSetEntryPoint ()) {
		SetPermanentCursor (m_hcurSetEntryPoint);
		return true;
	}

	m_bFrameSelectionMode = ! SelectObjectAt (_x, _y, IsSelMultipleOn (_dwKeys), IsSelElementOn (_dwKeys), true, false);

	SetPermanentCursor (m_bFrameSelectionMode ? g_hArrowCursor : m_hcurDrag);

	m_bFirstMoveInDND = true;
	StartDND (_x, _y);

	return true;
}

void CSelectTool::_SetEntryPoint (int _x, int _y) {
	point_t pt;
	ScreenToPt (_x, _y, pt);
	g_map.pTypeSet->pfRoundPoint (g_map, 0, pt);

	if (m_pRgnToSetEntryPoint && ! m_pRgnToSetEntryPoint->IsDeleted ()) {
		bool bCreate = false;
		if (m_pRgnToSetEntryPoint->pExtra->entry_points.empty () || m_cEntryPointToSet == -1) {
			CreateEntryPoint (m_pRgnToSetEntryPoint, pt, NULL);
			bCreate = true;
		} else
			SetEntryPoint (m_pRgnToSetEntryPoint, m_cEntryPointToSet, pt);

		rgn_t * const pRgn = m_pRgnToSetEntryPoint;
		m_pRgnToSetEntryPoint = NULL;
		m_cEntryPointToSet = -1;
		Refresh ();

		// Set initial label.
/*		if (bCreate && pRgn->pExtra->GetEntryPoints () > 1) {
			const size_t cOrdinal = pRgn->pExtra->entry_points.size () - 1;
			entry_point_t & ep = pRgn->pExtra->entry_points [cOrdinal];
			string_t strLabel = ep.strLabel;
			if (! EditText (g_hWnd, strLabel, "Entry Point Label", false))
				return;

			SetEntryPointLabel (pRgn, cOrdinal, strLabel);
			Refresh ();
		}*/
	} else {
		m_pRgnToSetEntryPoint = NULL;
		m_cEntryPointToSet = -1;
	}
}

bool CSelectTool::OnLUp (int _x, int _y, DWORD _dwKeys) {
	SetPermanentCursor (g_hArrowCursor);

	if (IsInSetEntryPoint ()) {
		_SetEntryPoint (_x, _y);
		return true;
	}

	if (! CTool::s_bDND)
		return false;
	StopDND ();

    if (m_bFrameSelectionMode) {
		m_bFrameSelectionMode = false;

        if (
			// TODO: allow shift over double-click area?
			CTool::s_lX_DND_Start != CTool::s_lX_DND ||
            CTool::s_lY_DND_Start != CTool::s_lY_DND
		) {
			// Prepare the selection rectangle.
	        rect_t rectSelection;

            point_t pt1;
            ScreenToPt (CTool::s_lX_DND_Start, CTool::s_lY_DND_Start, pt1);

			point_t pt2;
            ScreenToPt (CTool::s_lX_DND,       CTool::s_lY_DND,       pt2);

            if (pt1.x < pt2.x) {
                rectSelection.x0 = pt1.x;
                rectSelection.x1 = pt2.x;
            } else {
                rectSelection.x0 = pt2.x;
                rectSelection.x1 = pt1.x;
            }

            if (pt1.y < pt2.y) {
                rectSelection.y0 = pt1.y;
                rectSelection.y1 = pt2.y;
            } else {
                rectSelection.y0 = pt2.y;
                rectSelection.y1 = pt1.y;
            }

            SelectObjectsInsideRect (rectSelection, IsSelMultipleOn (_dwKeys), IsSelElementOn (_dwKeys));
        }

		Redraw ();
		UpdateMenu ();
    } else {
		const long lXOffset = CTool::s_lX_DND - CTool::s_lX_DND_Start;
		const long lYOffset = CTool::s_lY_DND - CTool::s_lY_DND_Start;
		if (lXOffset == 0 && lYOffset == 0)
			return true;

        if (
			::labs (lXOffset) <= 2*::GetSystemMetrics (SM_CXDOUBLECLK) &&
			::labs (lYOffset) <= 2*::GetSystemMetrics (SM_CYDOUBLECLK) &&
			::GetTickCount () - CTool::s_dwTime_DND_Start <= ::GetDoubleClickTime ()
		) {
			SetSelectionOffset (0, 0);
			Redraw ();
			return true;
		}

		_MoveSelection (lXOffset, lYOffset);
	}

	return true;
}

void CSelectTool::_MoveSelection (long _lXOffset, long _lYOffset) {
	point_t pointShift;
	pointShift.x =  (_lXOffset)/g_kX;
	pointShift.y = -(_lYOffset)/g_kY;

	const size_t cLevelIdx = GetZoomLevel (g_fScale);
	if (cLevelIdx != -1)
		g_map.pTypeSet->pfRoundPoint (g_map, cLevelIdx, pointShift);

	SetSelectionOffset (0, 0);

	if (pointShift.x == 0 && pointShift.y == 0) {
		Redraw ();
		return;
	}
	
	const size_t cSelItems = g_Selection.size ();
	if (cSelItems == 0)
		return;

	const int iDepth = BeginCombinedAction ();

	for (size_t cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
		const find_info_t & SelItem = g_Selection [cSelItem];
		if (SelItem.pLayer) {
			rgn_t & rgn = * SelItem.iRgn;
			if (rgn.IsImmutable ())
				continue;

			MoveObject (& rgn, SelItem.cElementOrOrigin, pointShift);
		} else {
			assert (SelItem.pRenderer != NULL);
			MoveImage (SelItem.pRenderer, pointShift);
		}
	}

	EndCombinedAction (iDepth, GetStr (IDS_MOVE_MULTIPLE));

	Refresh ();
}

void CSelectTool::OnMove (int _x, int _y, DWORD _dwKeys) {
	if (! s_bDND) {
		SetPermanentCursor (IsInSetEntryPoint () ? m_hcurSetEntryPoint : g_hArrowCursor);

		if (IsInSetEntryPoint ())
			Redraw ();
		return;
	}

	SetPermanentCursor (m_bFrameSelectionMode ? g_hArrowCursor : m_hcurDrag);

    if (! m_bFrameSelectionMode) {
		// Round the offset to Garmin units.
		const size_t cLevelIdx = GetZoomLevel (g_fScale);
		if (cLevelIdx != -1 && cLevelIdx < g_map.Levels.size ()) {
			point_t pointShift ((_x - CTool::s_lX_DND_Start)/g_kX, -(_y - CTool::s_lY_DND_Start)/g_kY);
			g_map.pTypeSet->pfRoundPoint (g_map, cLevelIdx, pointShift);
			_x = CTool::s_lX_DND_Start + fast_floor (pointShift.x*g_kX + .5f);
			_y = CTool::s_lY_DND_Start - fast_floor (pointShift.y*g_kY + .5f);
		}

		SetSelectionOffset (_x - CTool::s_lX_DND_Start, _y - CTool::s_lY_DND_Start);
	}

	Redraw ();

	//
	// Redraw dragged images with semi-transparency.
	//
	if (g_bShowAttachments) {
		bool bHaveImages = false;
		size_t cSelItem;
		for (cSelItem = 0; cSelItem < g_Selection.size (); ++ cSelItem) {
			const find_info_t & SelItem = g_Selection [cSelItem];
			if (SelItem.pRenderer) {
				if (SelItem.pRenderer->bDeleted || ! SelItem.pRenderer->bVisible)
					continue;
				bHaveImages = true;
			}
		}

		if (bHaveImages) {
			const int c_nImageDragW = 200;

			if (m_bFirstMoveInDND) {
				m_bFirstMoveInDND = false;

				if (m_pDraggedBuffer == NULL)
					m_pDraggedBuffer = new CDDBitmap (c_nImageDragW*2, c_nImageDragW*2);

				// Fill background.
				RECT rect = {0, 0, c_nImageDragW*2, c_nImageDragW*2};
				::FillRect (m_pDraggedBuffer->GetDC (), & rect, g_hbrWhite);

				point_t pt0, pt1;
				ScreenToPt (CTool::s_lX_DND_Start - c_nImageDragW, CTool::s_lY_DND_Start + c_nImageDragW, pt0);
				ScreenToPt (CTool::s_lX_DND_Start + c_nImageDragW, CTool::s_lY_DND_Start - c_nImageDragW, pt1);
				const rect_t rectImageDrag (pt0.x, pt0.y, pt1.x, pt1.y);

				for (size_t cSelItem = 0; cSelItem < g_Selection.size (); ++ cSelItem) {
					const find_info_t & SelItem = g_Selection [cSelItem];
					if (SelItem.pRenderer) {
						if (SelItem.pRenderer->bDeleted)
							continue;

						SelItem.pRenderer->Draw (
							m_pDraggedBuffer->GetDC (),
							rectImageDrag,
							-(CTool::s_lX_DND_Start - c_nImageDragW),
							-(CTool::s_lY_DND_Start - c_nImageDragW)
						);
					}
				}
			}

			if (m_pDraggedBuffer) {
				CClientDC dcWindow (g_hMapWnd);
				m_pDraggedBuffer->AlphaBlend (
					dcWindow,
					_x - c_nImageDragW,	_y - c_nImageDragW, c_nImageDragW*2, c_nImageDragW*2,
					0,                  0,                  c_nImageDragW*2, c_nImageDragW*2,
					96
				);
			}
		}
	}
}

bool CSelectTool::OnArrow (int _dx, int _dy) {
	const size_t cSelItems = g_Selection.size ();
	if (cSelItems == 0)
		return false;

	_MoveSelection (_dx, _dy);
	return true;
}

bool CSelectTool::OnEsc () {
	if (IsInSetEntryPoint ()) {
		m_pRgnToSetEntryPoint = NULL;
		m_cEntryPointToSet = -1;
		Redraw ();
		return true;
	}
	return false;
}

bool CSelectTool::OnEnter () {
	if (IsInSetEntryPoint ()) {
		_SetEntryPoint (CTool::s_lX, CTool::s_lY);
		return true;
	}
	return false;
}

bool CSelectTool::OnRUp (int _x, int _y, DWORD _dwKeys) {
	SelectObjectAt (_x, _y, IsSelMultipleOn (_dwKeys), IsSelElementOn (_dwKeys), true, true);

	return CSelectTool::OnCtxMenu (_x, _y);
}

bool CSelectTool::OnCtxMenu (int _x, int _y) {
	if (g_bLoading)
		return false;

	const HMENU hPopupMenu = LoadMenu (IDM_POPUP);
	CMenuHolder hPopupMenu_Holder (hPopupMenu);

	HMENU hMenu = NULL;

	if (! g_Selection.empty ()) {
		size_t cImages              = 0;
		size_t cWaypoints           = 0;
		size_t cTracks              = 0;
		size_t cRoutes              = 0;
		size_t cObjects             = 0;
		size_t cFullObjects         = 0;
		size_t cPolygons            = 0;
		size_t cPolylines           = 0;
		size_t cPolylinesE          = 0;
		size_t cHasAddress          = 0;
		size_t cRoads               = 0;
		size_t cMultiElementObjects = 0;
		size_t cAttachments         = 0;
		size_t cEntryPoints         = 0;
		size_t cBookmarks           = 0;
		size_t cNotes               = 0;
		size_t cBookmarksProcessed  = 0;
		size_t cNotesProcessed      = 0;

		const size_t cSelItems = g_Selection.size ();
		for (size_t cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
			const find_info_t & SelItem = g_Selection [cSelItem];
			if (SelItem.pRenderer) {
				if (! SelItem.pRenderer->bDeleted)
					++ cImages;
			} else if (SelItem.pLayer && ! SelItem.iRgn->IsDeleted ()) {
				if (SelItem.iRgn->IsAttachment ()) {
					if (SelItem.iRgn->IsTrack ())
						++ cTracks;
					else if (SelItem.iRgn->IsRoute ())
						++ cRoutes;
					else if (SelItem.iRgn->IsWaypoint ())
						++ cWaypoints;
					else
						++ cAttachments;
				} else if (SelItem.iRgn->IsBookmark ()) {
					++ cBookmarks;
					if (SelItem.iRgn->pExtra->pBookmark->bProcessed)
						++ cBookmarksProcessed;
				} else if (SelItem.iRgn->IsNote ()) {
					++ cNotes;
					if (SelItem.iRgn->pExtra->pNoteDrawing->bProcessed)
						++ cNotesProcessed;
				} else {
					++ cObjects;
					if (SelItem.cElementOrOrigin == -1) {
						if (SelItem.cEntryPoint != -1)
							++ cEntryPoints;
						else {
							++ cFullObjects;
							if (SelItem.iRgn->IsPolyline ()) {
								++ cPolylines;
								if (g_map.pTypeSet->pfIsRoad (* SelItem.iRgn))
									++ cRoads;
							}
							if (SelItem.iRgn->IsPolygon ())
								++ cPolygons;
							if (g_map.pTypeSet->pfHasAddress (* SelItem.iRgn))
								++ cHasAddress;
						}

						if (SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.iRgn->GetElements () + SelItem.iRgn->GetOrigins () > 1)
							++ cMultiElementObjects;
					} else {
						if (SelItem.iRgn->IsPolyline ())
							++ cPolylinesE;
					}
				}
			}
		}

		if (cBookmarks != 0 && cObjects == 0) {
			hMenu = ::GetSubMenu (hPopupMenu, smpBookmark);
			assert (hMenu);
			::SetMenuDefaultItem (hMenu, ID_RGN_PROPERTIES, MF_BYCOMMAND);

			::CheckMenuItem (hMenu, ID_EDIT_PROCESSED, MF_BYCOMMAND | (cBookmarksProcessed == cBookmarks ? MF_CHECKED : MF_UNCHECKED));

			::EnableMenuItem (hMenu, ID_POSITION,               MF_BYCOMMAND | (cBookmarks == 1 ? MF_ENABLED : MF_GRAYED));
			::EnableMenuItem (hMenu, ID_RGN_PROPERTIES,         MF_BYCOMMAND | (cBookmarks == 1 ? MF_ENABLED : MF_GRAYED));

			::DeleteMenu (hMenu, ID_EDIT_NOTE_POLYLINE, MF_BYCOMMAND);
			::DeleteMenu (hMenu, ID_EDIT_NOTE_POLYGON,  MF_BYCOMMAND);

			DeleteDuplicatedMenuSeparators (hMenu);
		} else if (cNotes != 0 && cObjects == 0) {
			hMenu = ::GetSubMenu (hPopupMenu, smpBookmark);
			assert (hMenu);
			::SetMenuDefaultItem (hMenu, ID_RGN_PROPERTIES, MF_BYCOMMAND);

			::CheckMenuItem (hMenu, ID_EDIT_PROCESSED, MF_BYCOMMAND | (cNotesProcessed == cNotes ? MF_CHECKED : MF_UNCHECKED));

			::EnableMenuItem (hMenu, ID_RGN_PROPERTIES,         MF_BYCOMMAND | (cNotes == 1     ? MF_ENABLED : MF_GRAYED));

			::DeleteMenu (hMenu, ID_EDIT_BM_POINT, MF_BYCOMMAND);
			::DeleteMenu (hMenu, ID_POSITION,      MF_BYCOMMAND);

			DeleteDuplicatedMenuSeparators (hMenu, true);
		} else if (cEntryPoints == 1 && cObjects == 1) {
			hMenu = ::GetSubMenu (hPopupMenu, smpEntryPoint);
			assert (hMenu);
			::SetMenuDefaultItem (hMenu, ID_ENTRYPOINT_PROPERTIES, MF_BYCOMMAND);
		} else if (cImages > 0 && cWaypoints + cTracks + cRoutes + cAttachments + cObjects == 0) {
			hMenu = ::GetSubMenu (hPopupMenu, smpRaster);
			assert (hMenu);
			::SetMenuDefaultItem (hMenu, ID_RASTER_PROPERTIES,  MF_BYCOMMAND);

			const bool bHasCorrection = g_Selection.size () == 1 && g_Selection [0].pRenderer && ! (g_Selection [0].pRenderer->ptCorrectionShift == point_t (0, 0));
			::EnableMenuItem (hMenu, ID_EDIT_REVERT_CORRECTION, MF_BYCOMMAND | (bHasCorrection ? MF_ENABLED : MF_GRAYED));

			::EnableMenuItem (hMenu, ID_RASTER_PROPERTIES,      MF_BYCOMMAND | (cImages == 1   ? MF_ENABLED : MF_GRAYED));
		} else {
			if (cWaypoints + cTracks + cRoutes + cAttachments > 0 && cImages + cObjects == 0) {
				hMenu = ::GetSubMenu (hPopupMenu, smpWTR);
				assert (hMenu);
				::SetMenuDefaultItem (hMenu, ID_RGN_PROPERTIES, MF_BYCOMMAND);
				
				::EnableMenuItem (hMenu, ID_EDIT_CONVERT_TO_POI,      MF_BYCOMMAND | (cWaypoints > 0 && (cTracks + cRoutes == 0) ? MF_ENABLED : MF_GRAYED));

				::EnableMenuItem (hMenu, ID_EDIT_CONVERT_TO_POLYLINE, MF_BYCOMMAND | (cWaypoints == 0 && (cTracks + cRoutes) > 0 ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_CONVERT_TO_POLYGON,  MF_BYCOMMAND | (cWaypoints == 0 && (cTracks + cRoutes) > 0 ? MF_ENABLED : MF_GRAYED));

				::EnableMenuItem (hMenu, ID_RGN_PROPERTIES,           MF_BYCOMMAND | (cWaypoints + cTracks + cRoutes + cAttachments == 1 ? MF_ENABLED : MF_GRAYED));

				bool bTrackVideo = false;
				if (cSelItems == 1 && cTracks == 1) {
					const rgn_t & rgn = * g_Selection [0].iRgn;
					bTrackVideo = rgn.pFileRef && rgn.pFileRef->pTrackVideo;
				}
				if (! bTrackVideo)
					::DeleteMenu     (hMenu, ID_TRACK_VIDEO,         MF_BYCOMMAND);

				if (cTracks + cRoutes == 0) {
					::DeleteMenu     (hMenu, ID_VIEW_SHOW_DIRECTION, MF_BYCOMMAND);
				} else {
					::EnableMenuItem (hMenu, ID_VIEW_SHOW_DIRECTION, MF_BYCOMMAND | (cSelItems == 1 ? MF_ENABLED : MF_GRAYED));
					if (cTracks + cRoutes == 1) {
						const rgn_t & rgn = * g_Selection [0].iRgn;
						const bool bShowDirection = rgn.pFileRef && rgn.pFileRef->bShowDirection;
						::CheckMenuItem (hMenu, ID_VIEW_SHOW_DIRECTION, MF_BYCOMMAND | (bShowDirection ? MF_CHECKED : MF_UNCHECKED));
					}
				}
			} else if (cObjects > 0 && cWaypoints + cTracks + cRoutes + cImages + cAttachments == 0) {
				hMenu = ::GetSubMenu (hPopupMenu, smpObject);
				assert (hMenu);
				::SetMenuDefaultItem (hMenu, ID_RGN_PROPERTIES, MF_BYCOMMAND);

				const UINT uiFlags = (g_Selection.size () == 1 && g_Selection [0].cElementOrOrigin == -1) ? MF_BYCOMMAND | MF_ENABLED : MF_BYCOMMAND | MF_GRAYED;
				::EnableMenuItem (hMenu, ID_TOOL_EDIT_NODES, uiFlags);
				::EnableMenuItem (hMenu, ID_TOOL_ROTATE,     uiFlags);
				::EnableMenuItem (hMenu, ID_RGN_PROPERTIES,  uiFlags);

				::EnableMenuItem (hMenu, ID_EDIT_PASTE,           MF_BYCOMMAND | (! IsClipboardEmpty () ? MF_ENABLED : MF_GRAYED));

				::EnableMenuItem (hMenu, ID_EDIT_KIND_POLYLINE,   MF_BYCOMMAND | (cPolylines == 0 && cPolygons > 0  && IsChangeTypeAppliable () ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_KIND_POLYGON,    MF_BYCOMMAND | (cPolylines > 0  && cPolygons == 0 && IsChangeTypeAppliable () ? MF_ENABLED : MF_GRAYED));

				::EnableMenuItem (hMenu, ID_EDIT_LABEL,           MF_BYCOMMAND | (cFullObjects != 0             ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_COMMENT,         MF_BYCOMMAND | (cFullObjects != 0             ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_TYPE,            MF_BYCOMMAND | (IsChangeTypeAppliable ()      ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_END_LEVEL,       MF_BYCOMMAND | (cFullObjects != 0             ? MF_ENABLED : MF_GRAYED));

				::EnableMenuItem (hMenu, ID_EDIT_CITY,            MF_BYCOMMAND | (cFullObjects != 0 && ! g_map.Cities.empty () ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_STREET,          MF_BYCOMMAND | (cHasAddress != 0              ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_ZIP,             MF_BYCOMMAND | (cHasAddress != 0              ? MF_ENABLED : MF_GRAYED));

				::EnableMenuItem (hMenu, ID_EDIT_PA_SAVE,         MF_BYCOMMAND | (cFullObjects == 1             ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_PA_RESTORE,      MF_BYCOMMAND | (                     m_bHasSavedPA                      ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_PA_RESTORE_INC,  MF_BYCOMMAND | (cFullObjects == 1 && m_bHasSavedPA && m_dwPANumber != 0 ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_RA_SAVE,         MF_BYCOMMAND | (cFullObjects == 1             ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_RA_RESTORE,      MF_BYCOMMAND | (m_bHasSavedRouteParams        ? MF_ENABLED : MF_GRAYED));

				::EnableMenuItem (hMenu, ID_EDIT_JOIN_OBJECTS,    MF_BYCOMMAND | (IsJoinObjectsAppliable     () ? MF_ENABLED : MF_GRAYED));

				::EnableMenuItem (hMenu, ID_EDIT_MERGE_POLYLINES, MF_BYCOMMAND | (IsMergePolylinesAppliable  () ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_EDIT_CLOSE_POLYLINE,  MF_BYCOMMAND | (IsClosePolylineAppliable   () ? MF_ENABLED : MF_GRAYED));

				if (cPolygons + cPolylines == 0) {
					::DeleteMenu (hMenu, ID_TOOL_EDIT_NODES,      MF_BYCOMMAND);
					::DeleteMenu (hMenu, ID_TOOL_ROTATE,          MF_BYCOMMAND);
				}

				const rgn_t * const pSelRgn0 = & * g_Selection [0].iRgn;

				const size_t cEntryPoints = pSelRgn0->pExtra ? pSelRgn0->pExtra->GetEntryPoints () : 0;
				const bool bMayHaveEntryPoints = cObjects == 1 && cFullObjects == 1 && ! pSelRgn0->IsPolyline () && pSelRgn0->IsMutable () && pSelRgn0->IsMapObject ();
				const bool bAddEntryPoint    = bMayHaveEntryPoints;
				const bool bSetEntryPoint    = cEntryPoints == 1 && bMayHaveEntryPoints;
				const bool bDeleteEntryPoint = cEntryPoints == 1 && cObjects == 1 && cFullObjects == 1;
				::EnableMenuItem (hMenu, ID_ADD_ENTRYPOINT,       MF_BYCOMMAND | (bAddEntryPoint                ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_SET_ENTRYPOINT,       MF_BYCOMMAND | (bSetEntryPoint                ? MF_ENABLED : MF_GRAYED));
				::EnableMenuItem (hMenu, ID_REMOVE_ENTRYPOINT,    MF_BYCOMMAND | (bDeleteEntryPoint             ? MF_ENABLED : MF_GRAYED));
				if (g_map.pTypeSet == NULL || (g_map.pTypeSet->dwFeatures & CTypeSet::tsfHasEntryPoint) == 0 || pSelRgn0->IsPolyline ()) {
					DeleteParentMenu (hMenu, ID_ADD_ENTRYPOINT);
				}

				::EnableMenuItem (hMenu, ID_SELECT_INSIDE_POLYGON, MF_BYCOMMAND | (g_Selection.size () == 1 ? MF_ENABLED : MF_GRAYED));
				if (! pSelRgn0->IsPolygon ()) {
					::DeleteMenu (hMenu, ID_SELECT_INSIDE_POLYGON, MF_BYCOMMAND);
				}

				::EnableMenuItem (hMenu, ID_EDIT_REVERSE_NODES,   MF_BYCOMMAND | (IsReversePolylineAppliable () ? MF_ENABLED : MF_GRAYED));
				if (cPolylines == 0 && cPolylinesE == 0) {
					::DeleteMenu (hMenu, ID_EDIT_MERGE_POLYLINES, MF_BYCOMMAND);
					::DeleteMenu (hMenu, ID_EDIT_CLOSE_POLYLINE,  MF_BYCOMMAND);
					::DeleteMenu (hMenu, ID_EDIT_REVERSE_NODES,   MF_BYCOMMAND);
				}

				if (cRoads == 0) {
					if (cPolylines != 0)
						DisableParentMenu (hMenu, ID_EDIT_SPEED_LIMIT);
					else
						DeleteParentMenu  (hMenu, ID_EDIT_SPEED_LIMIT);
				}

				::EnableMenuItem (hMenu, ID_EDIT_EXTRACT_ALL, MF_BYCOMMAND | (cMultiElementObjects >= 1 ? MF_ENABLED : MF_GRAYED));

				bool bEnableExtract = false;
				if (cSelItems == 1 && g_Selection [0].pLayer && pSelRgn0->IsMutable () && pSelRgn0->IsMapObject () && g_Selection [0].cElementOrOrigin != -1) {
					if (g_Selection [0].iRgn->GetElements () + g_Selection [0].iRgn->GetOrigins () > 1)
						bEnableExtract = true;
				}
				::EnableMenuItem (hMenu, ID_EDIT_EXTRACT,     MF_BYCOMMAND | (bEnableExtract ? MF_ENABLED : MF_GRAYED));

				::DeleteMenu (hMenu, ID_EDIT_TRIM_OUTSIDE,       MF_BYCOMMAND); // NOTE: not ready yet
				::DeleteMenu (hMenu, ID_EDIT_TRIM_INSIDE,        MF_BYCOMMAND); // NOTE: not ready yet
				::DeleteMenu (hMenu, ID_EDIT_TRIM_SELECTION,     MF_BYCOMMAND); // NOTE: considered as complex to understand
				::DeleteMenu (hMenu, ID_EDIT_POLYGONS_XOR,       MF_BYCOMMAND); // NOTE: considered as useless
				if (cPolygons > 0) {
					const bool bBooleanApplicable = IsPolygonBooleanApplicable ();
					::EnableMenuItem (hMenu, ID_EDIT_POLYGONS_MERGE,     MF_BYCOMMAND | (bBooleanApplicable ? MF_ENABLED : MF_GRAYED));
					::EnableMenuItem (hMenu, ID_EDIT_POLYGONS_INTERSECT, MF_BYCOMMAND | (bBooleanApplicable ? MF_ENABLED : MF_GRAYED));
					::EnableMenuItem (hMenu, ID_EDIT_POLYGONS_SUBTRACT,  MF_BYCOMMAND | (bBooleanApplicable ? MF_ENABLED : MF_GRAYED));
//					::EnableMenuItem (hMenu, ID_EDIT_POLYGONS_XOR,       MF_BYCOMMAND | (bBooleanApplicable ? MF_ENABLED : MF_GRAYED));

//					::EnableMenuItem (hMenu, ID_EDIT_TRIM_SELECTION,     MF_BYCOMMAND | (IsTrimSelectedObjectsApplicable () ? MF_ENABLED : MF_GRAYED));
//					const bool bTrimApplicable = IsTrimMapApplicable ();
//					::EnableMenuItem (hMenu, ID_EDIT_TRIM_OUTSIDE,       MF_BYCOMMAND | (bTrimApplicable    ? MF_ENABLED : MF_GRAYED));
//					::EnableMenuItem (hMenu, ID_EDIT_TRIM_INSIDE,        MF_BYCOMMAND | (bTrimApplicable    ? MF_ENABLED : MF_GRAYED));
				} else {
					::DeleteMenu (hMenu, ID_EDIT_POLYGONS_MERGE,     MF_BYCOMMAND);
					::DeleteMenu (hMenu, ID_EDIT_POLYGONS_SUBTRACT,  MF_BYCOMMAND);
					::DeleteMenu (hMenu, ID_EDIT_POLYGONS_INTERSECT, MF_BYCOMMAND);
//					::DeleteMenu (hMenu, ID_EDIT_POLYGONS_XOR,       MF_BYCOMMAND);

//					::DeleteMenu (hMenu, ID_EDIT_TRIM_OUTSIDE,       MF_BYCOMMAND);
//					::DeleteMenu (hMenu, ID_EDIT_TRIM_INSIDE,        MF_BYCOMMAND);
				}

				const bool bCanViewIn3D = cSelItems == 1 && g_Selection [0].pLayer && CanViewIn3D (* pSelRgn0);
				if (! bCanViewIn3D)
					::DeleteMenu (hMenu, ID_VIEW_IN_3D, MF_BYCOMMAND);

				DeleteDuplicatedMenuSeparators (hMenu);
			}
		}
	}

	if (hMenu == NULL && g_bShowGoogleMaps) {
		hMenu = ::GetSubMenu (hPopupMenu, smpGoogle);
		::EnableMenuItem (hMenu, ID_GM_REFRESH_TILE, (CanRefreshWebMapTile () ? MF_ENABLED : MF_GRAYED) | MF_BYCOMMAND);
	}

	if (hMenu) {
		::EnableMenuItem (hMenu, ID_VIEW_BEST_FIT, MF_BYCOMMAND | (! g_Selection.empty () ? MF_ENABLED : MF_GRAYED));

		CTool::PrepareCtxMenu (hMenu);
		PopupMenu (hMenu);
		return true;
	} else
		return false;
}

static
void _ShowEntryPointProperties (HWND g_hWnd, const find_info_t & _fi) {
	if (! _fi.pLayer || _fi.cEntryPoint == -1)
		return;
	rgn_t & rgn = * g_Selection [0].iRgn;
	if (! rgn.pExtra)
		return;
	const size_t cOrdinal = _fi.cEntryPoint;
	if (cOrdinal >= rgn.pExtra->entry_points.size ())
		return;
	const entry_point_t & ep = rgn.pExtra->entry_points [cOrdinal];
	string_t strLabel = ep.strLabel;
	if (! EditText (g_hWnd, strLabel, "Entry Point Label", false))
		return;

	SetEntryPointLabel (& rgn, cOrdinal, strLabel);
	Refresh ();
}

bool CSelectTool::OnCommand (WORD _wID) {
	switch (_wID) {
		case ID_EDIT_TYPE:          ShowChangeTypeForSelection (g_hWnd);             return true;
		case ID_EDIT_KIND_POLYLINE: ShowChangeTypeForSelection (g_hWnd, rkPolyline); return true;
		case ID_EDIT_KIND_POLYGON:  ShowChangeTypeForSelection (g_hWnd, rkPolygon);  return true;

		case ID_EDIT_BM_POINT: {
			bool bAppliable = false;
			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsBookmark ())
					bAppliable = true;
			}
			if (! bAppliable)
				return false;

			const int iDepth = BeginCombinedAction ();

			size_t cLevel = GetZoomLevel (g_fScale);
			if (cLevel == -1)
				cLevel = 0;

			find_info_m_t SelectionNew;
			find_info_m_t SelectionOld = g_Selection;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = SelectionOld [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsBookmark ()) {
					rgn_t & rgnBookmark = * SelItem.iRgn;
					if (rgnBookmark.IsDeleted ())
						continue;

					point_t pt;
					bookmark_t::GetPos (rgnBookmark, pt);
					g_map.pTypeSet->pfRoundPoint (g_map, cLevel, pt);

					points_t points (1);
					points [0] = pt;
					rgns_t::iterator iRgnNew = CreateObjectIt (points, rkPoint, cLevel);

					find_info_t fi;
					fi.pLayer           = & g_map.layers [rkPoint];
					fi.iRgn             = iRgnNew;
					fi.cElementOrOrigin = -1;
					SelectionNew.push_back (fi);

					DeleteObject (& rgnBookmark);
				}
			}

			// Select type.
			g_Selection.swap (SelectionNew);
			const bool bOK = ShowChangeTypeForSelection (g_hWnd);

			EndCombinedAction (iDepth, GetStr (IDS_CONVERT_MULTIPLE));
			if (! bOK) {
				UndoEditAction ();
				g_Selection.swap (SelectionOld);
			}

			Refresh ();


			return true;
		}

		case ID_EDIT_NOTE_POLYLINE:
		case ID_EDIT_NOTE_POLYGON: {
			bool bAppliable = false;
			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsNote ())
					bAppliable = true;
			}
			if (! bAppliable)
				return false;

			const int iDepth = BeginCombinedAction ();

			size_t cLevel = GetZoomLevel (g_fScale);
			if (cLevel == -1)
				cLevel = 0;

			const RGN_KIND Kind = _wID == ID_EDIT_NOTE_POLYLINE ? rkPolyline : rkPolygon;

			find_info_m_t SelectionNew;
			find_info_m_t SelectionOld = g_Selection;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = SelectionOld [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsNote ()) {
					rgn_t & rgnNote = * SelItem.iRgn;
					if (rgnNote.IsDeleted ())
						continue;

					rgn_t * pRgnNew = NULL;

					const size_t cElements = rgnNote.elements.size ();
					for (size_t cElement = 0; cElement < cElements; ++ cElement) {
						rgn_element_t & element = rgnNote.elements [cElement];
						if (element.bDeleted)
							continue;
						if (SelItem.cElementOrOrigin != -1 && SelItem.cElementOrOrigin != cElement)
							continue;

						points_t points = element.points;
						g_map.pTypeSet->pfRoundPoints (g_map, cLevel, points);

						if (pRgnNew == NULL) {
							rgns_t::iterator iRgnNew = CreateObjectIt (points, Kind, cLevel);
							pRgnNew = & * iRgnNew;

							find_info_t fi;
							fi.pLayer           = & g_map.layers [Kind];
							fi.iRgn             = iRgnNew;
							fi.cElementOrOrigin = -1;
							SelectionNew.push_back (fi);
						} else {
							assert (pRgnNew != NULL);
							pRgnNew->elements.push_back (rgn_element_t ());
							rgn_element_t & element = pRgnNew->elements.back ();
							element.cLevel = cLevel;
							element.points.swap (points);
						}
					}

					if (SelItem.cElementOrOrigin == -1)
						DeleteObject (& rgnNote);
					else
						DeleteElement (& rgnNote, SelItem.cElementOrOrigin);
				}
			}

			// Select type.
			g_Selection.swap (SelectionNew);
			const bool bOK = ShowChangeTypeForSelection (g_hWnd);

			EndCombinedAction (iDepth, GetStr (IDS_CONVERT_MULTIPLE));
			if (! bOK) {
				UndoEditAction ();
				g_Selection.swap (SelectionOld);
			}

			Refresh ();

			return true;
		}

		case ID_EDIT_LABEL: {
			if (g_Selection.empty ())
				return false;

			if (g_Selection.size () == 1 && g_Selection [0].pLayer && g_Selection [0].iRgn->IsMutable () && g_Selection [0].iRgn->IsMapObject () && g_Selection [0].cElementOrOrigin == -1) {
				OnEditLabelStart ();
				return true;
			}

			CCritSec::CLock lock (g_cs);

			// Get initial value for the label.
			string_t strLabel;
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1) {
					bAppliable = true;

					if (strLabel.empty ())
						strLabel = SelItem.iRgn->strLabel;
					else if (! (strLabel == SelItem.iRgn->strLabel)) {
						strLabel = string_t ();
						break;
					}
				}
			}

			if (! bAppliable)
				return false;

			// Edit label.
			if (! EditText (g_hWnd, strLabel, GetStr (IDS_EDIT_LABEL), false))
				return false;

			// Change labels for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1)
					RenameObject (* SelItem.iRgn, strLabel);
			}

			EndCombinedAction (iDepth, GetStr (IDS_CHANGE_LABEL_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_COMMENT: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			// Get initial value for the comment.
			string_t strComment;
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1) {
					bAppliable = true;

					if (strComment.empty ())
						strComment = SelItem.iRgn->strComment;
					else if (! (strComment == SelItem.iRgn->strComment)) {
						strComment = string_t ();
						break;
					}
				}
			}

			if (! bAppliable)
				return false;

			// Edit comment.
			if (! EditText (g_hWnd, strComment, GetStr (IDS_EDIT_COMMENT), true))
				return false;

			// Change comments for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1)
					SetComment (* SelItem.iRgn, strComment);
			}

			EndCombinedAction (iDepth, GetStr (IDS_CHANGE_COMMENT_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_END_LEVEL: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			// Get initial value for levels.
			size_t cEndLevel = 0;
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1) {
					bAppliable = true;

					if (cEndLevel < SelItem.iRgn->uchEndLevel)
						cEndLevel = SelItem.iRgn->uchEndLevel;
				}
			}

			if (! bAppliable)
				return false;

			// Edit levels value.
			if (! EditNumber (g_hWnd, cEndLevel, GetStr (IDS_SET_LEVELS_REQ)))
				return false;

			// Change levels for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1)
					SetObjectEndLevel (* SelItem.iRgn, cEndLevel);
			}

			EndCombinedAction (iDepth, GetStr (IDS_SET_LEVELS_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_CITY: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			// Get initial value for cities.
			city_t * pCitySel = NULL;
			bool bIniSel = true;
			bool bFirst = true;
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1) {
					bAppliable = true;

					city_t * pCity = SelItem.iRgn->pExtra ? SelItem.iRgn->pExtra->pCity : NULL;
					if (pCity && pCity->IsDeleted ())
						pCity = NULL;

					if (bFirst) {
						pCitySel = pCity;
						bFirst = false;
					} else {
						if (pCitySel != pCity) {
							pCitySel = NULL;
							bIniSel = false;
							break;
						}
					}
				}
			}

			if (! bAppliable)
				return false;

			// Edit city value.
			if (! SelectCity (g_hWnd, bIniSel, pCitySel))
				return false;

			// Change cities for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1)
					ChangeCity (* SelItem.iRgn, pCitySel);
			}

			EndCombinedAction (iDepth, GetStr (IDS_CHANGE_CITY_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_STREET: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			// Get initial value for street.
			string_t strStreet;
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1) {
					bAppliable = true;

					if (
						g_map.pTypeSet->pfHasAddress (* SelItem.iRgn) &&
						SelItem.iRgn->pExtra && ! SelItem.iRgn->pExtra->strStreetDesc.empty ()
					) {
						if (strStreet.empty ())
							strStreet = SelItem.iRgn->pExtra->strStreetDesc;
						else if (! (strStreet == SelItem.iRgn->pExtra->strStreetDesc)) {
							strStreet = string_t ();
							break;
						}
					}
				}
			}

			if (! bAppliable)
				return false;

			// Edit street.
			if (! EditText (g_hWnd, strStreet, GetStr (IDS_EDIT_STREET), false))
				return false;

			// Change street for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1)
					if (g_map.pTypeSet->pfHasAddress (* SelItem.iRgn))
						SetObjectPAStreet (* SelItem.iRgn, strStreet);
			}

			EndCombinedAction (iDepth, GetStr (IDS_CHANGE_STREET_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_ZIP: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			// Get initial value for zip codes.
			zip_t * pZip = NULL;
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1) {
					bAppliable = true;

					if (SelItem.iRgn->pExtra && SelItem.iRgn->pExtra->pZip) {
						if (! pZip)
							pZip = SelItem.iRgn->pExtra->pZip;
						else if (pZip != SelItem.iRgn->pExtra->pZip) {
							pZip = NULL;
							break;
						}
					}
				}
			}

			if (! bAppliable)
				return false;

			// Edit zip code value.
			if (! ShowChangeZip (g_hWnd, pZip))
				return false;

			// Change zip code for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1)
					ChangeZip (* SelItem.iRgn, pZip);
			}

			EndCombinedAction (iDepth, GetStr (IDS_CHANGE_ZIP_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_PA_SAVE:
			if (g_Selection.size () == 1 && g_Selection [0].pLayer && g_Selection [0].iRgn->IsMutable () && g_Selection [0].iRgn->IsMapObject () && g_Selection [0].cElementOrOrigin == -1) {
				const rgn_t & rgn = * g_Selection [0].iRgn;
				m_pSavedCity     = rgn.pExtra ? rgn.pExtra->pCity         : NULL;
				m_pSavedZip      = rgn.pExtra ? rgn.pExtra->pZip          : NULL;
				m_strSavedStreet = rgn.pExtra ? rgn.pExtra->strStreetDesc : NULL;

				const char * strNumber = rgn.pExtra ? rgn.pExtra->strNumber.c_str () : NULL;
				if (strNumber == NULL || strNumber [0] == '\0')
					strNumber = rgn.strLabel.c_str ();
				m_dwPANumber = strNumber == NULL || strNumber [0] == '\0' ? 0 : ParseDecU (strNumber);

				m_bHasSavedPA = true;

				return true;	
			}
			return false;

		case ID_EDIT_PA_RESTORE: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1)
					bAppliable = true;
			}
			if (! bAppliable)
				return false;

			// Change postal address for multiple objects.
			const int iDepth = BeginCombinedAction ();

			size_t cApplied = 0;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () && SelItem.cElementOrOrigin == -1) {
					ChangeCity (* SelItem.iRgn, m_pSavedCity);
					ChangeZip  (* SelItem.iRgn, m_pSavedZip);
					SetObjectPAStreet (* SelItem.iRgn, m_strSavedStreet);
					++ cApplied;
				}
			}

			EndCombinedAction (iDepth, GetStr (cApplied == 1 ? IDS_CHANGE_PA : IDS_CHANGE_PA_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_PA_RESTORE_INC:
			if (g_Selection.size () == 1 && g_Selection [0].pLayer && g_Selection [0].iRgn->IsMutable () && g_Selection [0].iRgn->IsMapObject () && g_Selection [0].cElementOrOrigin == -1) {
				rgn_t & rgn = * g_Selection [0].iRgn;

				const int iDepth = BeginCombinedAction ();

				ChangeCity (rgn, m_pSavedCity);
				ChangeZip  (rgn, m_pSavedZip);
				SetObjectPAStreet (rgn, m_strSavedStreet);

				if (m_dwPANumber != 0) {
					SetNumber (m_dwPANumber, m_dwPANumber + 1);
					char strNumber [32];
					::sprintf (strNumber, "%d", m_dwPANumber);

					SetObjectPANumber (rgn, strNumber);
					RenameObject      (rgn, strNumber);
				}

				EndCombinedAction (iDepth, GetStr (IDS_CHANGE_PA_INC));

				Refresh ();
				return true;	
			}
			return false;

		case ID_EDIT_ROUTE_CLASS: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			// Get initial value for route class.
			int nRouteClassSel = -1;
			bool bFirst = true;
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (
					SelItem.pLayer && ! SelItem.iRgn->IsDeleted () && SelItem.iRgn->IsRoad () &&
					SelItem.cElementOrOrigin == -1
				) {
					bAppliable = true;

					const int nRouteClass = SelItem.iRgn->pExtra ? SelItem.iRgn->pExtra->fRouteClass : -1;

					if (bFirst) {
						nRouteClassSel = nRouteClass;
						bFirst = false;
					} else {
						if (nRouteClassSel != nRouteClass) {
							nRouteClassSel = -1;
							break;
						}
					}
				}
			}

			if (! bAppliable)
				return false;

			// Select different route class.
			static const CListBoxItem s_lbiRouteClasses [] = {
				{IDS_ROUTE_CLASS_0, 0},
				{IDS_ROUTE_CLASS_1, 1},
				{IDS_ROUTE_CLASS_2, 2},
				{IDS_ROUTE_CLASS_3, 3},
				{IDS_ROUTE_CLASS_4, 4},
			};
			if (! SelectFromList (g_hWnd, IDS_ROUTE_CLASSES_TITLE, s_lbiRouteClasses, sizeof (s_lbiRouteClasses)/sizeof (s_lbiRouteClasses [0]), nRouteClassSel))
				return false;

			// Change route class for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (
					SelItem.pLayer && ! SelItem.iRgn->IsDeleted () && SelItem.iRgn->IsRoad () &&
					SelItem.cElementOrOrigin == -1
				)
					SetObjectRouteClass (* SelItem.iRgn, nRouteClassSel);
			}

			EndCombinedAction (iDepth, GetStr (IDS_CHANGE_ROUTE_CLASS_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_SPEED_LIMIT: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			// Get initial value for speed limit.
			int nSpeedLimitSel = -1;
			bool bFirst = true;
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (
					SelItem.pLayer && ! SelItem.iRgn->IsDeleted () && SelItem.iRgn->IsRoad () &&
					SelItem.cElementOrOrigin == -1
				) {
					bAppliable = true;

					const int nSpeedLimit = SelItem.iRgn->pExtra ? SelItem.iRgn->pExtra->fSpeedLimit : -1;

					if (bFirst) {
						nSpeedLimitSel = nSpeedLimit;
						bFirst = false;
					} else {
						if (nSpeedLimitSel != nSpeedLimit) {
							nSpeedLimitSel = -1;
							break;
						}
					}
				}
			}

			if (! bAppliable)
				return false;

			// Select different speed limit.
			static const CListBoxItem s_lbiSpeedLimits [] = {
				{IDS_SPEED_LIMIT_0, 0},
				{IDS_SPEED_LIMIT_1, 1},
				{IDS_SPEED_LIMIT_2, 2},
				{IDS_SPEED_LIMIT_3, 3},
				{IDS_SPEED_LIMIT_4, 4},
				{IDS_SPEED_LIMIT_5, 5},
				{IDS_SPEED_LIMIT_6, 6},
				{IDS_SPEED_LIMIT_7, 7},
			};
			if (! SelectFromList (g_hWnd, IDS_SPEED_LIMITS_TITLE, s_lbiSpeedLimits, sizeof (s_lbiSpeedLimits)/sizeof (s_lbiSpeedLimits [0]), nSpeedLimitSel))
				return false;

			// Change speed limit for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (
					SelItem.pLayer && ! SelItem.iRgn->IsDeleted () && SelItem.iRgn->IsRoad () &&
					SelItem.cElementOrOrigin == -1
				)
					SetObjectSpeedLimit (* SelItem.iRgn, nSpeedLimitSel);
			}

			EndCombinedAction (iDepth, GetStr (IDS_CHANGE_SPEED_LIMIT_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_RA_SAVE:
			if (g_Selection.size () == 1 && g_Selection [0].pLayer && g_Selection [0].iRgn->IsMutable () && g_Selection [0].iRgn->IsMapObject () && g_Selection [0].cElementOrOrigin == -1) {
				const rgn_t & rgn = * g_Selection [0].iRgn;
				m_dwSavedRouteParams = rgn.pExtra ? rgn.pExtra->dwRouteParams : 0;
				m_bHasSavedRouteParams = true;
				return true;	
			}
			return false;

		case ID_EDIT_RA_RESTORE: {
			if (g_Selection.empty () || ! m_bHasSavedRouteParams)
				return false;

			CCritSec::CLock lock (g_cs);

			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (
					SelItem.pLayer && ! SelItem.iRgn->IsDeleted () && SelItem.iRgn->IsRoad () &&
					SelItem.cElementOrOrigin == -1
				)
					bAppliable = true;
			}
			if (! bAppliable)
				return false;

			// Change route attributes for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (
					SelItem.pLayer && ! SelItem.iRgn->IsDeleted () && SelItem.iRgn->IsRoad () &&
					SelItem.cElementOrOrigin == -1
				)
					SetObjectRouteParams (* SelItem.iRgn, m_dwSavedRouteParams);
			}

			EndCombinedAction (iDepth, GetStr (IDS_CHANGE_RA_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_JOIN_OBJECTS:
			if (IsJoinObjectsAppliable ()) {
				JoinSelectedObjects ();
				Refresh ();
				return true;
			}
			return false;

		case ID_EDIT_EXTRACT_ALL: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (
					SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () &&
					SelItem.cElementOrOrigin == -1 &&
					SelItem.iRgn->GetElements () + SelItem.iRgn->GetOrigins  () > 1
				)
					bAppliable = true;
			}
			if (! bAppliable)
				return false;

			// Change route attributes for multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (
					SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsMapObject () &&
					SelItem.cElementOrOrigin == -1 &&
					SelItem.iRgn->GetElements () + SelItem.iRgn->GetOrigins  () > 1
				)
					ExtractAllElements (* SelItem.iRgn);
			}

			EndCombinedAction (iDepth, GetStr (IDS_EXTRACT_ALL_ELEMENTS_MULTIPLE));

			Refresh ();
			return true;
		}

		case ID_EDIT_EXTRACT:
			if (g_Selection.size () == 1 && g_Selection [0].pLayer && g_Selection [0].iRgn->IsMutable () && g_Selection [0].iRgn->IsMapObject () && g_Selection [0].cElementOrOrigin != -1) {
				if (g_Selection [0].iRgn->GetElements () + g_Selection [0].iRgn->GetOrigins  () > 1) {
					ExtractElement (* g_Selection [0].iRgn, g_Selection [0].cElementOrOrigin);
					Refresh ();
					return true;
				}
			}
			return false;

		case ID_EDIT_MERGE_POLYLINES:    if (IsMergePolylinesAppliable       ()) {MergeSelectedPolylines   ();  Refresh (); return true;} else return false;
		case ID_EDIT_CLOSE_POLYLINE:     if (IsClosePolylineAppliable        ()) {CloseSelectedPolylines   ();  Refresh (); return true;} else return false;
		case ID_EDIT_REVERSE_NODES:      if (IsReversePolylineAppliable      ()) {ReverseSelectedPolylines ();  Refresh (); return true;} else return false;

		case ID_EDIT_POLYGONS_MERGE:     if (IsPolygonBooleanApplicable      ()) {MergeSelectedPolygons     (); Refresh (); return true;} else return false;
		case ID_EDIT_POLYGONS_INTERSECT: if (IsPolygonBooleanApplicable      ()) {IntersectSelectedPolygons (); Refresh (); return true;} else return false;
		case ID_EDIT_POLYGONS_SUBTRACT:  if (IsPolygonBooleanApplicable      ()) {SubtractSelectedPolygons  (); Refresh (); return true;} else return false;
		case ID_EDIT_POLYGONS_XOR:       if (IsPolygonBooleanApplicable      ()) {XorSelectedPolygons       (); Refresh (); return true;} else return false;

		case ID_EDIT_TRIM_SELECTION:     if (IsTrimSelectedObjectsApplicable ()) {TrimSelectedObjects ();       Refresh (); return true;} else return false;
		case ID_EDIT_TRIM_OUTSIDE:       if (IsTrimMapApplicable             ()) {TrimMap (taKeepInside);       Refresh (); return true;} else return false;
		case ID_EDIT_TRIM_INSIDE:        if (IsTrimMapApplicable             ()) {TrimMap (taKeepOutside);      Refresh (); return true;} else return false;

		case ID_SELECT_INSIDE_POLYGON: {
			if (g_Selection.size () != 1 || ! g_Selection [0].pLayer || ! g_Selection [0].iRgn->IsPolygon ())
				return false;
			const rgn_t & rgn = * g_Selection [0].iRgn;
			SelectObjectsInsidePolygon (rgn, true);
			Redraw ();
			return true;
		}

		case ID_EDIT_REVERT_CORRECTION:
			if (g_Selection.size () == 1 && g_Selection [0].pRenderer && ! g_Selection [0].pRenderer->bDeleted && ! (g_Selection [0].pRenderer->ptCorrectionShift == point_t (0, 0))) {
				MoveImage (
					g_Selection [0].pRenderer,
					point_t (
						- g_Selection [0].pRenderer->ptCorrectionShift.x,
						- g_Selection [0].pRenderer->ptCorrectionShift.y
					)
				);

				Refresh ();
				return true;
			}
			return false;

		case ID_EDIT_CONVERT_TO_POI:
		case ID_EDIT_CONVERT_TO_POLYLINE:
		case ID_EDIT_CONVERT_TO_POLYGON: {
			if (g_Selection.empty ())
				return false;

			CCritSec::CLock lock (g_cs);

			// Verify context.
			bool bAppliable = false;

			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && ! SelItem.iRgn->IsDeleted () && SelItem.iRgn->IsWTR ()) {
					if (_wID == ID_EDIT_CONVERT_TO_POI      && SelItem.iRgn->GetKind () != rkPoint)
						continue;
					if (_wID == ID_EDIT_CONVERT_TO_POLYLINE && SelItem.iRgn->GetKind () != rkPolyline)
						continue;
					if (_wID == ID_EDIT_CONVERT_TO_POLYGON  && SelItem.iRgn->GetKind () != rkPolyline)
						continue;

					bAppliable = true;
				}
			}

			if (! bAppliable)
				return false;

			// Prepare levels to import the track.
			levels_t Levels;
			bool bGeneralize = _wID != ID_EDIT_CONVERT_TO_POI;
			if (g_map.Levels.empty ())
				Levels.insert (0);
			else {
				// Ask user to select levels to import track (the current level is default).
				Levels.insert (GetZoomLevel (g_fScale));
				if (! SelectLevels (g_hWnd, _wID != ID_EDIT_CONVERT_TO_POI, Levels, bGeneralize))
					break;
				if (Levels.empty ())
					break;
			}

			// Prepare kind.
			RGN_KIND Kind;
			switch (_wID) {
				case ID_EDIT_CONVERT_TO_POI:      Kind = rkPoint;    break;
				case ID_EDIT_CONVERT_TO_POLYLINE: Kind = rkPolyline; break;
				case ID_EDIT_CONVERT_TO_POLYGON:  Kind = rkPolygon;  break;
				default: assert (false);
			}

			// Convert multiple objects.
			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && ! SelItem.iRgn->IsDeleted () && SelItem.iRgn->IsWTR ()) {
					if (_wID == ID_EDIT_CONVERT_TO_POI      && SelItem.iRgn->GetKind () != rkPoint)
						continue;
					if (_wID == ID_EDIT_CONVERT_TO_POLYLINE && SelItem.iRgn->GetKind () != rkPolyline)
						continue;
					if (_wID == ID_EDIT_CONVERT_TO_POLYGON  && SelItem.iRgn->GetKind () != rkPolyline)
						continue;

					ConvertWaypointOrTrack (Levels, & * SelItem.iRgn, Kind, bGeneralize);
				}
			}

			EndCombinedAction (iDepth, GetStr (IDS_CONVERT_MULTIPLE));

			Refresh ();

			// Change type.
			ShowChangeTypeForSelection (g_hWnd);

			return true;
		}

		case ID_EDIT_PROCESSED: {
			bool bProcessed = true;
			bool bAppliable = false;
			const size_t cSelItems = g_Selection.size ();
			size_t cSelItem;
			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsBookmarkOrNote ()) {
					bAppliable = true;

					if (SelItem.iRgn->IsBookmark ()) {
						rgn_t & rgnBookmark = * SelItem.iRgn;
						if (! rgnBookmark.pExtra->pBookmark->bProcessed)
							bProcessed = false;
					} else if (SelItem.iRgn->IsNote ()) {
						rgn_t & rgnNote = * SelItem.iRgn;
						if (! rgnNote.pExtra->pNoteDrawing->bProcessed)
							bProcessed = false;
					}
				}
			}
			if (! bAppliable)
				return false;

			bProcessed = ! bProcessed;

			const int iDepth = BeginCombinedAction ();

			for (cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsMutable () && SelItem.iRgn->IsBookmarkOrNote ()) {
					if (SelItem.iRgn->IsBookmark ()) {
						rgn_t & rgnBookmark = * SelItem.iRgn;

						bookmark_t bm = * rgnBookmark.pExtra->pBookmark;
						bm.bProcessed = bProcessed;
						SetBookmark (& rgnBookmark, bm);
					} else if (SelItem.iRgn->IsNote ()) {
						rgn_t & rgnNote = * SelItem.iRgn;

						note_drawing_t nd = * rgnNote.pExtra->pNoteDrawing;
						nd.bProcessed = bProcessed;
						SetNoteDrawing (& rgnNote, nd);
					}
				}
			}

			EndCombinedAction (iDepth, bProcessed ? "Mark bookmarks as processed" : "Mark bookmarks as non-processed");

			Refresh ();

			// Change type.
			ShowChangeTypeForSelection (g_hWnd);

			return true;
		}

		case ID_VIEW_SHOW_DIRECTION: {
			if (g_Selection.size () != 1 || ! g_Selection [0].pLayer)
				return false;

			rgn_t & rgn = * g_Selection [0].iRgn;
			if (! rgn.IsDeleted () && rgn.IsWTR () && rgn.GetKind () == rkPolyline && rgn.pFileRef) {
				rgn.pFileRef->bShowDirection = ! rgn.pFileRef->bShowDirection;
				Refresh ();
				return true;
			}

			return false;
		}

		case ID_VIEW_HIDE: {
			if (g_Selection.empty ())
				return false;

			bool bRet = false;

			const size_t cSelItems = g_Selection.size ();
			for (size_t cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && SelItem.iRgn->IsAttachment () && SelItem.iRgn->IsVisible () && SelItem.iRgn->pFileRef) {
					ShowAttachment (SelItem.iRgn->pFileRef, false, false);
					bRet = true;
				} else if (SelItem.pRenderer && ! SelItem.pRenderer->bDeleted && SelItem.pRenderer->bVisible) {
					ShowAttachment (SelItem.pRenderer, false, true);
					bRet = true;
				}
			}

			if (bRet)
				Refresh ();

			return bRet;
		}

		case ID_VIEW_IN_3D: {
			if (g_Selection.size () != 1 || ! g_Selection [0].pLayer || ! CanViewIn3D (* g_Selection [0].iRgn))
				return false;

			point_t pt0;
			ScreenToPt (CTool::s_lX, CTool::s_lY, pt0);

			ViewIn3D (pt0);
			return true;
		}

		case ID_TRACK_VIDEO: {
			if (g_Selection.size () != 1 || ! g_Selection [0].pLayer)
				return false;
			const rgn_t & rgn = * g_Selection [0].iRgn;

			point_t pt0;
			ScreenToPt (CTool::s_lX, CTool::s_lY, pt0);

			ShowTrackVideo (rgn, pt0);
			return true;
		}

		case ID_ADD_ENTRYPOINT:
		case ID_SET_ENTRYPOINT: {
			if (g_Selection.size () != 1 || ! g_Selection [0].pLayer)
				return false;
			rgn_t & rgn = * g_Selection [0].iRgn;
			if (rgn.IsPolyline () || rgn.IsImmutable () || rgn.IsNotMapObject ())
				return false;
			m_pRgnToSetEntryPoint = & rgn;
			if (_wID == ID_ADD_ENTRYPOINT)
				m_cEntryPointToSet = -1;
			else {
				m_cEntryPointToSet = g_Selection [0].cEntryPoint;
				if (m_cEntryPointToSet == -1)
					m_cEntryPointToSet = rgn.pExtra ? rgn.pExtra->GetEntryPointOrdinal0 () : 0;
			}
			SetPermanentCursor (m_hcurSetEntryPoint);
			return true;
		}

		case ID_REMOVE_ENTRYPOINT: {
			if (g_Selection.size () != 1 || ! g_Selection [0].pLayer)
				return false;
			rgn_t & rgn = * g_Selection [0].iRgn;
			if (! rgn.pExtra || rgn.pExtra->entry_points.empty ())
				return false;
			size_t cOrdinal = g_Selection [0].cEntryPoint;
			if (cOrdinal == -1)
				cOrdinal = rgn.pExtra->GetEntryPointOrdinal0 ();
			RemoveEntryPoint (& rgn, cOrdinal);
			Refresh ();
			return true;
		}

		case ID_ENTRYPOINT_PROPERTIES:
			if (g_Selection.size () != 1)
				return false;
			_ShowEntryPointProperties (g_hWnd, g_Selection [0]);
			return true;

		case ID_VIEW_BEST_FIT: {
			if (g_Selection.empty ())
				return false;

			rect_t rect;
			const size_t cSelItems = g_Selection.size ();
			for (size_t cSelItem = 0; cSelItem < cSelItems; ++ cSelItem) {
				const find_info_t & SelItem = g_Selection [cSelItem];
				if (SelItem.pLayer && ! SelItem.iRgn->IsDeleted ()) {
					if (SelItem.cElementOrOrigin != -1) {
						if (SelItem.iRgn->GetKind () == rkPoint)
							rect.Extend (SelItem.iRgn->origins [SelItem.cElementOrOrigin].point);
						else
							rect.Extend (SelItem.iRgn->elements [SelItem.cElementOrOrigin].rectBound);
					} else
						rect.Extend (SelItem.iRgn->rectBound);
				} else {
					if (! SelItem.pRenderer->bDeleted)
						rect.Extend (SelItem.pRenderer->rect);
				}
			}
			if (rect.Invalid ())
				return false;

			SetScale (GetRectScale (rect));
			SetToCenter (point_t (rect.MeanX (), rect.MeanY ()));
			Refresh ();

			return true;
		}

		case ID_POSITION: {
			if (g_Selection.size () != 1)
				return false;
			rgn_t & rgn = * g_Selection [0].iRgn;
			if (rgn.origins.empty () || rgn.origins [0].bDeleted)
				return false;
			const point_t & point0 = rgn.origins [0].point;
			point_t point = point0;
			if (EditPoint (g_hWnd, point)) {
				MoveObject (& rgn, 0, point_t (point.x - point0.x, point.y - point0.y));
				Refresh ();
			}
			break;
		}

		case ID_GM_REFRESH_TILE: RefreshWebMapTile (CTool::s_lX, CTool::s_lY); return true;
		case ID_GM_SHIFT:        ShiftWebMap (g_hWnd); return true;
		case ID_GM_RESET_SHIFT:  ResetWebMapShift ();  return true;
	}
	return false;
}

bool CSelectTool::OnLDblClk (int _x, int _y, DWORD _dwKeys) {
	SelectObjectAt (_x, _y, IsSelMultipleOn (_dwKeys), IsSelElementOn (_dwKeys), true, false);
	if (g_Selection.size () == 1) {
		if (g_Selection [0].pLayer) {
			if (g_Selection [0].cEntryPoint != -1)
				_ShowEntryPointProperties (g_hWnd, g_Selection [0]);
			else
				ShowRgnProperties (g_hWnd, g_Selection [0]);
		} else
			ShowRasterProperties (g_hWnd, g_Selection [0]);

		return true;
	}

	return false;
}

void CSelectTool::OnPaint (HDC _hDC, const RECT & _rect) const {
	if (IsInSetEntryPoint ()) {
		assert (m_pRgnToSetEntryPoint != NULL);

		point_t pt0;
		if (m_pRgnToSetEntryPoint->dwFlags & rgn_t::fPtLabelDefined)
			pt0 = m_pRgnToSetEntryPoint->ptLabel;
		else {
			pt0.x = m_pRgnToSetEntryPoint->rectBound.MeanX ();
			pt0.y = m_pRgnToSetEntryPoint->rectBound.MeanY ();
		}
		const int x0 = XToScreen (pt0.x);
		const int y0 = YToScreen (pt0.y);

		::SelectObject (_hDC, m_hpenGroupSelection);
		::SetROP2      (_hDC, R2_COPYPEN);
		DrawLine       (_hDC, x0, y0, CTool::s_lX, CTool::s_lY);

		return;
	}

    if (! CTool::s_bDND)
		return;

    if (m_bFrameSelectionMode) {
		::SelectObject (_hDC, m_hpenGroupSelection);
		::SetROP2      (_hDC, R2_COPYPEN);

        POINT ptSelRect [5] = {
            {CTool::s_lX_DND_Start, CTool::s_lY_DND_Start},
            {CTool::s_lX_DND_Start, CTool::s_lY_DND},
            {CTool::s_lX_DND,       CTool::s_lY_DND},
            {CTool::s_lX_DND,       CTool::s_lY_DND_Start},
            {CTool::s_lX_DND_Start, CTool::s_lY_DND_Start}
        };
        ::Polyline (_hDC, ptSelRect, ARRAY_SIZE (ptSelRect));
    }
}
