//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Generalization of tracks to draw them faster.
//

# include "StdAfx.h"
# include "TrackGeneralization.h"
# include "Map.h"
# include "Scale.h"
# include "Generalize.h"
# include "fast_floor.h"
# include "Globals.h"

	// In meters per 1 cm.
static const int c_nGeneralizationScales [] = {100, 1000, 10000, 100000};

void GeneralizePoints (rgn_element_t & _element) {
	points_t points = _element.points;
	if (points.empty ())
		return;
	const size_t cPoints0 = points.size ();
	const float fCosY = ::cos (points [0].y*c_PI/180);

	for (size_t cScale = 0; cScale < ARRAY_SIZE (c_nGeneralizationScales); ++ cScale) {
		const float d = 360*c_nGeneralizationScales [cScale]/(c_pixels_per_cm*40e6);
		const size_t cDeleted = Generalize (points, d, fCosY);

		_element.pExtra->pointsGeneralized.push_back (points);

		const size_t cPoints = points.size ();
		assert (cPoints <= cPoints0);
	}
}

const points_t & GetGeneralizedPoints (const rgn_element_t & _element) {
	const int nScaleVal = fast_floor (g_fScale*c_pixels_per_cm);
	if (nScaleVal < c_nGeneralizationScales [0])
		return _element.points;

	// Get pre-generalized polyline for current scale.
	size_t cScaleIdx = ARRAY_SIZE (c_nGeneralizationScales) - 1;
	for (size_t cScale = 1; cScale < ARRAY_SIZE (c_nGeneralizationScales); ++ cScale) {
		if (nScaleVal < c_nGeneralizationScales [cScale]) {
			cScaleIdx = cScale - 1;
			break;
		}
	}

	const size_t cMaxScaleIdx = _element.pExtra->pointsGeneralized.size ();
	if (cScaleIdx >= cMaxScaleIdx)
		cScaleIdx = cMaxScaleIdx - 1;

	polypoints_t::const_iterator ip = _element.pExtra->pointsGeneralized.begin ();
	std::advance (ip, cScaleIdx);

	return * ip;
}