//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Track video bookmarks.
//

# include "StdAfx.h"
# include "Resource.h"
# include "TrackVideoBookmark.h"
# include "Bookmark.h"
# include "Map.h"

void tv_bookmarks_t::Init (const char * _strTrackFileName) {
	Clear ();
	m_strTrackFileName = _strTrackFileName;
}

void tv_bookmarks_t::Update () {
	Clear (false);

	{
		CCritSec::CLock lock (g_cs);

		rgns_t & rgns = g_map.layers [rkPoint].rgns;
		for (rgns_t::iterator iRgn = rgns.begin (); iRgn != rgns.end (); ++ iRgn) {
			rgn_t & rgn = * iRgn;
			if (rgn.IsImmutable () || ! rgn.IsBookmark ())
				continue;

			const bookmark_t * const pBookmark = rgn.pExtra->pBookmark;
			assert (pBookmark != NULL);
			if (pBookmark->strFileName.empty ())
				continue;
			if (! (pBookmark->strFileName == m_strTrackFileName))
				continue;
			m_bookmarks.insert (offset2bookmarks_t::value_type (pBookmark->dwFilePos, & rgn));
		}
	}

	if (m_dwPositionInFile != -1)
		_UpdateHere ();
}

void tv_bookmarks_t::Clear (bool _bResetPos) {
	if (_bResetPos)
		m_dwPositionInFile = -1;

	m_bookmarks.clear ();
	m_iLowerBound = m_bookmarks.end ();
	m_iUpperBound = m_bookmarks.end ();
	m_cBookmarks = 0;
}

void tv_bookmarks_t::SetPositionPrec (DWORD _dwPositionInFile) {
	if (m_dwPositionInFile == _dwPositionInFile)
		return;
	m_dwPositionInFile = _dwPositionInFile;

	_UpdateHere ();
}

void tv_bookmarks_t::_UpdateHere () {
	const std::pair<offset2bookmarks_t::iterator, offset2bookmarks_t::iterator> er = m_bookmarks.equal_range (m_dwPositionInFile);
	m_iLowerBound = er.first;
	m_iUpperBound = er.second;

	m_cBookmarks = std::distance (er.first, er.second);
}

void tv_bookmarks_t::AddHere (rgn_t & _rgn) {
	if (m_cBookmarks == 0)
		m_bookmarks.insert (offset2bookmarks_t::value_type (m_dwPositionInFile, & _rgn));
	else
		m_bookmarks.insert (m_iUpperBound, offset2bookmarks_t::value_type (m_dwPositionInFile, & _rgn));
	_UpdateHere ();
}

void tv_bookmarks_t::DeleteHere (size_t _cOrdinal) {
	if (_cOrdinal >= m_cBookmarks)
		return;

	offset2bookmarks_t::iterator i = m_iLowerBound;
	std::advance (i, _cOrdinal);

	m_bookmarks.erase (i);
	_UpdateHere ();
}

size_t tv_bookmarks_t::GetBookmarksHere () const {
	return m_cBookmarks;
}

rgn_t * tv_bookmarks_t::GetBookmarkHere (size_t _cOrdinal) const {
	if (_cOrdinal >= m_cBookmarks)
		return NULL;

	offset2bookmarks_t::iterator i = m_iLowerBound;
	std::advance (i, _cOrdinal);
	return i->second;
}

DWORD tv_bookmarks_t::FindPrevPosition () const {
	if (! HasPrevPosition ())
		return m_dwPositionInFile;

	if (m_cBookmarks == 0) {
		if (m_dwPositionInFile > m_bookmarks.rbegin ()->first)
			return m_bookmarks.rbegin ()->first;
	}

	offset2bookmarks_t::const_iterator i = m_iLowerBound;
	if (i == m_bookmarks.end () || i == m_bookmarks.begin ())
		return m_dwPositionInFile;
	-- i;
	return i->first;
}

DWORD tv_bookmarks_t::FindNextPosition () const {
	if (! HasNextPosition ())
		return m_dwPositionInFile;

	if (m_cBookmarks == 0) {
		if (m_dwPositionInFile < m_bookmarks.begin ()->first)
			return m_bookmarks.begin ()->first;
	}

	if (m_iUpperBound == m_bookmarks.end ())
		return m_dwPositionInFile;
	return m_iUpperBound->first;
}

bool tv_bookmarks_t::HasPrevPosition () const {
	if (m_bookmarks.empty ())
		return false;
	return m_dwPositionInFile > m_bookmarks.begin ()->first;
}

bool tv_bookmarks_t::HasNextPosition () const {
	if (m_bookmarks.empty ())
		return false;
	return m_dwPositionInFile < m_bookmarks.rbegin ()->first;
}


