# ifndef __Tracking_H_
# define __Tracking_H_

struct PositionInfo_t;
struct rect_t;

void StartTracking ();
void StopTracking ();
void StartNewTrack ();
void AddPointToTrack (const PositionInfo_t & _pi);

void DrawTracks (HDC _hDC, const rect_t & _rectVisible);

void GetTracksFolder (char _strTracksFolder [MAX_PATH]);

////////////////////////////////////

class CFileWriter;

class CTrackWriter {
  public:
	CTrackWriter () :
		m_pFileWriter (NULL),
		m_hTrackFile (NULL)
	{}
	~CTrackWriter ();

	void Init (HANDLE _hTrackFile);

	virtual void Start () = 0;
	virtual void Stop () = 0;
	virtual void StartNewTrack () = 0;
	virtual void AddPointToTrack (const PositionInfo_t & _pi) = 0;

  protected:
	CFileWriter * m_pFileWriter;
	HANDLE m_hTrackFile;
};

class CTrackWriterTypeInfo {
  public:
	typedef CTrackWriter * (* pfCreateTrackWriter_t) ();

	CTrackWriterTypeInfo (
		pfCreateTrackWriter_t _pfCreateTrackWriter,
		 const char * _strExt, const char * _strDisplayName
	);

	const pfCreateTrackWriter_t m_pfCreateTrackWriter;
	const char * const m_strDisplayName;
	const char * const m_strExt;

	// The linked list of creators.
	CTrackWriterTypeInfo * m_pNext;
	static CTrackWriterTypeInfo * s_pFirst;

	static CTrackWriterTypeInfo * GetCurrent ();
};

# define DECLARE_TRACK_WRITER(_T, _strExt, _strDisplayName) \
	CTrackWriter * Create_##_T () {return new _T;} \
	CTrackWriterTypeInfo Creator_##_T (& Create_##_T, _strExt, _strDisplayName);

# endif // __Tracking_H_