//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Object filter by type.
//

# include "StdAfx.h"
# include "Resource.h"
# include "TypeFilter.h"
# include "Filter.h"
# include "SelectType.h"
# include "Types.h"
# include "Map.h"
# include "Settings.h"
# include "ParseHelpers.h"
# include "Globals.h"

class CTypeFilter : public CFilter {
  public:
	CTypeFilter ();

	virtual void GetName (string_t & _strName) const;
	virtual bool ShowSettings (HWND _hWnd);
	virtual bool DoesFit (const rgn_t & _rgn) const;

  private:
	selected_types_t m_SelectedTypes;
};

CTypeFilter::CTypeFilter () {
	const size_t cStreamLen = GetSettingLen ("FilterTypes") + 1;
	char * const strStream = (char *) alloca (sizeof (char)*cStreamLen);
	GetSetting ("FilterTypes", strStream, cStreamLen, NULL);

	const char * p = strStream;
	const char * const pEnd = strStream + cStreamLen;
	unsigned long ul = 0;
	if (! ParseDecU (p, pEnd, & p, ul))
		return;

	const size_t cTypes = ul;
	m_SelectedTypes.reserve (cTypes);

	for (size_t c = 0; c < cTypes; ++ c) {
		OmitSpaces (& p, pEnd);
		if (! ParseDecU (p, pEnd, & p, ul))
			return;
		const RGN_KIND Kind = ul < rkMax ? static_cast<RGN_KIND> (ul) : rkPoint;

		OmitSpaces (& p, pEnd);
		if (! ParseHex (p, pEnd, & p, ul))
			return;
		const DWORD dwType = ul;

		m_SelectedTypes.push_back (selected_type_t ());
		selected_type_t & st = m_SelectedTypes.back ();
		st.Kind   = Kind;
		st.dwType = dwType;
	}
}

bool CTypeFilter::ShowSettings (HWND _hWnd) {
	const CTypeSet * pTypeSet = NULL;
	{
		CCritSec::CLock lock (g_cs);
		pTypeSet = g_map.pTypeSet;
	}

	if (! SelectMultipleTypes (_hWnd, pTypeSet, m_SelectedTypes))
		return false;

	// Save selected types to registry.
	const size_t cTypes = m_SelectedTypes.size ();
	const size_t cStreamLen = cTypes*8 + 10;
	char * const strStream = (char *) alloca (sizeof (char)*(cStreamLen + 1));
	size_t cOffset = 0;

	cOffset += ::sprintf (strStream + cOffset, "%d ", cTypes);

	for (size_t c = 0; c < cTypes; ++ c) {
		assert (cOffset < cStreamLen);
		if (cOffset + 8 >= cStreamLen)
			break;
		const selected_type_t & st = m_SelectedTypes [c];
		cOffset += ::sprintf (strStream + cOffset, "%d %04x ", st.Kind, st.dwType);
	}
	if (cOffset >= cStreamLen)
		cOffset = cStreamLen;
	strStream [cOffset] = '\0';

	SaveSetting ("FilterTypes", strStream);

	return true;
}

void CTypeFilter::GetName (string_t & _strName) const {
	_strName.append ("type");
}

bool CTypeFilter::DoesFit (const rgn_t & _rgn) const {
	if (_rgn.IsNotMapObject ())
		return false;
	return IsTypeSelected (m_SelectedTypes, _rgn.GetKind (), _rgn.GetType ());
}

/////////////////////////////////////////////////////

void SelectByType (HWND _hWnd) {
	CTypeFilter f;
	if (! f.ShowSettings (_hWnd))
		return;

	SelectByFilter (f);
}

//////////////////////////////////////

class CKindFilter : public CFilter {
  public:
	CKindFilter (RGN_KIND _Kind) : m_Kind (_Kind) {}

	// Override.
	virtual void GetName (string_t & _strName) const {_strName.append ("kind");}
	virtual bool ShowSettings (HWND _hWnd) {return true;}
	virtual bool DoesFit (const rgn_t & _rgn) const {return _rgn.IsMapObject () && _rgn.IsMutable () && _rgn.GetKind () == m_Kind;}

  private:
	const RGN_KIND m_Kind;
};

void SelectByKind (RGN_KIND _Kind) {
	CKindFilter kf (_Kind);
	SelectByFilter (kf);
}

//////////////////////////////////////

class CRoadsFilter : public CFilter {
  public:
	// Override.
	virtual void GetName (string_t & _strName) const {_strName.append ("all roads");}
	virtual bool ShowSettings (HWND _hWnd) {return true;}
	virtual bool DoesFit (const rgn_t & _rgn) const {return _rgn.IsMapObject () && _rgn.IsMutable () && _rgn.IsRoad ();}
};

void SelectAllRoads () {
	CRoadsFilter rf;
	SelectByFilter (rf);
}
