//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// Object type sets stuff.
//

# include "StdAfx.h"
# include "Resource.h"
# include "Types.h"
# include "TypesDraw.h"
# include "TypesPOIDraw.h"
# include "TypeLocalization.h"
# include "MapSkins.h"
# include "GarminTypes.h"
# include "HoluxTypes.h"
# include "Map.h"
# include "DrawHelpers.h"
# include "Globals.h"

const char * g_strKinds [rkMax];
type_desc_t g_tdUnknown;

type_desc_t::type_desc_t () : strTypeNameLocalized (NULL) {
	Clear ();
}

type_desc_t::type_desc_t (const type_desc_t & _other) : strTypeNameLocalized (NULL) {
	Clear ();

	::memcpy (this, & _other, sizeof (type_desc_t));
	if (_other.strTypeNameLocalized) {
		strTypeNameLocalized = NULL;
		SetTypeNameLocalized (_other.strTypeNameLocalized);
	}
}

type_desc_t::~type_desc_t () {
	Clear ();
}

void type_desc_t::operator = (const type_desc_t & _other) {
	Clear ();

	::memcpy (this, & _other, sizeof (type_desc_t));
	if (_other.strTypeNameLocalized) {
		strTypeNameLocalized = NULL;
		SetTypeNameLocalized (_other.strTypeNameLocalized);
	}
}

void type_desc_t::Clear () {
	delete [] strTypeNameLocalized;

	::memset (this, 0, sizeof (type_desc_t));
	cIconIdx = -1;

	DestroySecondaryAttributes ();
}

void type_desc_t::SetTypeNameLocalized (const char * _str) {
	delete [] strTypeNameLocalized;
	strTypeNameLocalized = NULL;

	if (_str != NULL) {
		strTypeNameLocalized = new char [sizeof (char)*(::strlen (_str) + 1)];
		::strcpy (strTypeNameLocalized, _str);
	}
}

void type_desc_t::InitSecondaryAttributes (HDC _hdcTmp) {
	DestroySecondaryAttributes ();

	// Prepare cosmetic pen (for printing).
/*	if (phPen) {
		LOGPEN lp;
		if (::GetObject (* phPen, sizeof (lp), & lp) >= sizeof (lp)) {
			LOGBRUSH lb;
			lb.lbStyle = BS_SOLID;
			lb.lbColor = lp.lopnColor;
			lb.lbHatch = 0;
			hPenGeometric = ::ExtCreatePen (
				PS_GEOMETRIC | lp.lopnStyle | PS_ENDCAP_FLAT | PS_JOIN_ROUND,
				lp.lopnWidth.x, & lb, 0, NULL
			);
		}
	}*/

	// Create brushes for transparent polygons.
	if (Kind == rkPolygon || bUnknown) {
		assert (hpenTransparentPolygon == NULL);
		assert (hbrTransparentPolygon  == NULL);

		assert (phBrush);
		if (phBrush == NULL || * phBrush == g_hbrNull) {
			hpenTransparentPolygon = ::CreatePen (PS_SOLID, 1, c_colGray);
			hbrTransparentPolygon  = g_hbrNull;
		} else {
			LOGBRUSH lb;
			::memset (& lb, 0, sizeof (lb));
			::GetObject (* phBrush, sizeof (lb), & lb);
			if (lb.lbStyle == BS_SOLID || lb.lbStyle == BS_HATCHED) {
				hpenTransparentPolygon = ::CreatePen (PS_SOLID, 1, lb.lbColor);
				hbrTransparentPolygon  = ::CreateHatchBrush (HS_DIAGCROSS, lb.lbColor);
			} else {
				// Get color of (0,0)-th point in the brush pattern.
				::SelectObject (_hdcTmp, * phBrush);
				::PatBlt (_hdcTmp, 0, 0, 1, 1, PATCOPY);
				const COLORREF col0 = ::GetPixel (_hdcTmp, 0, 0);

				hpenTransparentPolygon = ::CreatePen (PS_SOLID, 1, col0);
				hbrTransparentPolygon  = ::CreateHatchBrush (HS_DIAGCROSS, col0);
			}
		}
	} else
		hbrTransparentPolygon = NULL;
}

void type_desc_t::DestroySecondaryAttributes () {
	if (hPenGeometric != NULL) {
		::DeleteObject (hPenGeometric);
		hPenGeometric = NULL;
	}

	if (hpenTransparentPolygon != NULL) {
		::DeleteObject (hpenTransparentPolygon);
		hpenTransparentPolygon = NULL;
	}

	if (hbrTransparentPolygon != NULL && hbrTransparentPolygon != g_hbrNull)
		::DeleteObject (hbrTransparentPolygon);
	hbrTransparentPolygon = NULL;
}

///////////////////////////////////////////////

void InitTypes () {
	g_strKinds [rkPoint]    = GetStr (IDS_POINT);
	g_strKinds [rkPolyline] = GetStr (IDS_POLYLINE);
	g_strKinds [rkPolygon]  = GetStr (IDS_POLYGON);

	g_tdUnknown.Category       = catUnknown;
	g_tdUnknown.strTypeName    = "Unknown";
	g_tdUnknown.phBrush        = & g_hbrUnknown;
	g_tdUnknown.phPen          = & g_hpenUnknown;
	g_tdUnknown.phPen2         = NULL;
	g_tdUnknown.hPenGeometric  = NULL;
	g_tdUnknown.phFont         = NULL;
	g_tdUnknown.colText        = c_colBlack;
	g_tdUnknown.phCursor       = & g_hcurUnknown;
	g_tdUnknown.pfPrepareLabel = PreparePointLabel;
	g_tdUnknown.pfDrawLabel    = DrawLabel;

	InitCommonTypesGDI ();

	ResetTypes ();
}

void ResetTypes () {
	CCritSec::CLock lock (g_cs);

	CTypeSet * pTypeSet;
	for (pTypeSet = CTypeSet::s_pFirst; pTypeSet; pTypeSet = pTypeSet->m_pNext)
		pTypeSet->DestroySecondaryAttributes ();

	for (pTypeSet = CTypeSet::s_pFirst; pTypeSet; pTypeSet = pTypeSet->m_pNext) {
		pTypeSet->m_pfInit ();
		pTypeSet->InitSecondaryAttributes ();
		pTypeSet->bLocalizationIsReady = false;
	}

	// Prepare localized names for all types.
	LoadTypeSetLocalization ();
}

void CompleteCustomTypes () {
	CCritSec::CLock lock (g_cs);

	for (CTypeSet * pTypeSet = CTypeSet::s_pFirst; pTypeSet; pTypeSet = pTypeSet->m_pNext)
		pTypeSet->CompleteCustomTypes ();
}

void DestroyTypes () {
	CCritSec::CLock lock (g_cs);

	for (CTypeSet * pTypeSet = CTypeSet::s_pFirst; pTypeSet; pTypeSet = pTypeSet->m_pNext) {
		pTypeSet->DestroySecondaryAttributes ();
		pTypeSet->m_pfDestroy ();
	}

	DestroyCommonTypesGDI ();
}

///////////////////////////////////////////////

CTypeSet * CTypeSet::s_pFirst;
CTypeSetConvertorInfo *  CTypeSetConvertorInfo::s_pFirst;

CTypeSet::CTypeSet (pfInit_t _pfInit, pfInit_t _pfDestroy) :
	m_pfInit (_pfInit),
	m_pfDestroy (_pfDestroy),
	pBasicTypeSet (NULL),
	bLocalizationIsReady (false),
	bHasLocalizedTypes (false),
	m_bSecondaryAttributes (false),
	m_bHasCustomTypes (false)
{
	// Insert to the head of the list.
	m_pNext = CTypeSet::s_pFirst;
	CTypeSet::s_pFirst = this;
}

const CTypeSet * CTypeSet::FindTypeSet (const char * _strID) {
	for (const CTypeSet * pTypeSet = CTypeSet::s_pFirst; pTypeSet; pTypeSet = pTypeSet->m_pNext) {
		if (::stricmp (_strID, pTypeSet->strID) == 0)
			return pTypeSet;
	}
	return & g_tsNull;
}

DWORD ConvertNullTypeSet (RGN_KIND _Kind, DWORD _dwType) {return _dwType;}

pfConvertTypeSet_t CTypeSet::FindConvertTypeSetTo (const CTypeSet * _pTypeSetTo) const {
	if (this == & g_tsNull)
		return ConvertNullTypeSet;

	for (const CTypeSetConvertorInfo * p = CTypeSetConvertorInfo::s_pFirst; p; p = p->m_pNext) {
		if (p->pTypeSetFrom == this && p->pTypeSetTo == _pTypeSetTo)
			return p->pfConvertTypeSet;
	}
	return NULL;
}

///////////////////////////////////////////////

void CTypeSet::SetTypes (types_t & _types) {
	_OrderTypes (_types);

	m_types.swap (_types);

	m_code2type.clear ();
	for (types_t::iterator it = m_types.begin (); it != m_types.end (); ++ it) {
		type_desc_t & td = * it;
		m_code2type [td.ulType] = & td;
	}

	m_bHasCustomTypes = false;
}

void CTypeSet::_OrderTypes (types_t & _types) {
	assert (! _types.empty ());
	type_desc_t & td = * _types.begin ();
	assert (_types.begin ()->Category == catUnknown);

	// Make sure all types are sorted by code.
	_types.sort ();
	assert (_types.begin ()->Category == catUnknown);

	// Check 'nested' type ranges for consistency.
	for (types_t::iterator it = _types.begin (); it != _types.end (); ++ it) {
		type_desc_t & td0 = * it;

		types_t::iterator it1 = it;
		++ it1;
		if (it1 == _types.end ())
			break;
		type_desc_t & td1 = * it1;

		assert (td0.ulType <= td0.ulType2);
		assert (td0.ulType < td1.ulType);

		if (td0.ulType2 > td1.ulType2) {
			types_t::iterator it2 = it1;
			++ it2;

			it2 = _types.insert (it2, type_desc_t ());
			type_desc_t & td2 = * it2;
			td2 = td0;
			td2.ulType  = td1.ulType2 + 1;
			td2.ulType2 = td0.ulType2;

			td0.ulType2 = td1.ulType - 1;
		}
	}
}

type_desc_t & CTypeSet::AddCustomTypeDesc (DWORD _dwType, BYTE _btKind) {
	const ULONG ulType = _dwType + (ULONG (_btKind) << 24);

	m_types.push_back (type_desc_t ());
	type_desc_t & td = m_types.back ();

	td.ulType   = ulType;
	td.ulType2  = ulType;
	td.Category = catCustom;

	if (_btKind == rkPolygon)
		td.phPen = & g_hpenNull;
	
	const label_desc_t * pLD = NULL;
	switch (_btKind) {
		default:
		case rkPoint:    pLD = & g_DefLabel;      break;
		case rkPolyline: pLD = & g_RoadLabel;     break;
		case rkPolygon:  pLD = & g_CenteredLabel; break;
	}
	td.phFont         = pLD->phFont;
	td.colText        = pLD->colText;
	td.pfPrepareLabel = pLD->pfPrepareLabel;
	td.pfDrawLabel    = pLD->pfDrawLabel;

	m_code2type [ulType] = & td;

	m_bHasCustomTypes = true;

	return td;
}

void CTypeSet::CompleteCustomTypes () {
	if (m_bHasCustomTypes)
		_OrderTypes (m_types);

	m_bSecondaryAttributes = false;
	InitSecondaryAttributes ();
}

const type_desc_t * CTypeSet::GetTypeDesc (DWORD _dwType, BYTE _btKind) const {
	const size_t cTypes = m_types.size ();
	if (cTypes <= 1)
		return GetUnknownTD ();

	const ULONG ulType = _dwType + (ULONG (_btKind) << 24);

	code2type_t::const_iterator ict = m_code2type.lower_bound (ulType);
	if (ict == m_code2type.end ())
		return GetUnknownTD ();

	if (ict->second->ulType != ulType) {
		if (ict == m_code2type.begin ())
			return GetUnknownTD ();

		assert (ict->second->ulType > ulType);
		-- ict;
		assert (ict->second->ulType < ulType);
	}
	const type_desc_t & td = * ict->second;

	if (td.ulType <= ulType && ulType <= td.ulType2)
		return & td;
	else
		return GetUnknownTD ();
}

void CTypeSet::InitSecondaryAttributes () {
	assert (! m_bSecondaryAttributes);
	m_bSecondaryAttributes = true;

	// Check type ranges for consistency.
	types_t::iterator it;
# ifdef _DEBUG
	for (it = m_types.begin (); it != m_types.end (); ++ it) {
		type_desc_t & td0 = * it;

		types_t::iterator it1 = it;
		++ it1;
		if (it1 == m_types.end ())
			break;
		type_desc_t & td1 = * it1;

		assert (td0.ulType <= td0.ulType2);
		assert (td0.ulType2 < td1.ulType);
	}
# endif // _DEBUG

	// Prepare tmp DC to determine color of bitmap brush.
	const CMemDC hdcTmp (CClientDC (NULL), 1, 1);

	size_t cType = 0;
	for (it = m_types.begin (); it != m_types.end (); ++ it, ++ cType) {
		type_desc_t & td = * it;

		td.bUnknown = it == m_types.begin ();

		td.pTypeSet = this;
		td.cOrdinal = cType;

		if (! td.HasSecondaryAttributes ())
			td.InitSecondaryAttributes (hdcTmp);
	}
}

void CTypeSet::DestroySecondaryAttributes () {
	if (! m_bSecondaryAttributes)
		return;

	for (types_t::iterator it = m_types.begin (); it != m_types.end (); ++ it) {
		type_desc_t & td = * it;
		td.DestroySecondaryAttributes ();
	}

	if (hilTypeIcons) {
		::ImageList_Destroy (hilTypeIcons);
		hilTypeIcons = NULL;
	}

	m_bSecondaryAttributes = false;
}

/////////////////////////////////////////////////

HIMAGELIST CTypeSet::GetTypeIcons () const {
	if (hilTypeIcons == NULL)
		const_cast<CTypeSet *> (this)->_InitTypeIcons ();
	return hilTypeIcons;
}

void CTypeSet::_InitTypeIcons () {
	if (m_types.empty ())
		return;

	//
	// Create image lists for all kinds.
	//

	const size_t cTypes = Types ().size ();
	const size_t c_cIconW = 16, c_cIconH = 16;
	hilTypeIcons = ::ImageList_Create (c_cIconW, c_cIconH, ILC_COLORDDB, cTypes, 10);
	::ImageList_SetBkColor (hilTypeIcons, ::GetSysColor (COLOR_WINDOW));

	// Prepare buffer bitmap.
	const CClientDC hdcScreen (NULL);
	const CMemDC hdcMem (hdcScreen, c_cIconW, c_cIconH);
	::SetROP2       (hdcMem, R2_COPYPEN);
	::SetBkMode     (hdcMem, TRANSPARENT);
	::SetBrushOrgEx (hdcMem, 1, 1, NULL);

	// Render the icons.
	for (types_t::iterator it = m_types.begin (); it != m_types.end (); ++ it) {
		type_desc_t & td = * it;

		// Clean background.
		const RECT rect = {0, 0, c_cIconW, c_cIconH};
		::FillRect (hdcMem, & rect, ::GetSysColorBrush (COLOR_WINDOW));

		assert (td.Kind < rkMax);
		const RGN_KIND Kind = static_cast<RGN_KIND> (td.Kind);

		switch (Kind) {
			case rkPoint:
				if (td.cIconIdx != -1) {
					MapSkin_DrawPOI (hdcMem, td.cIconIdx, 0, 0, false);
				} else {
					assert (td.phCursor);
					::DrawIcon (hdcMem, 0, 0, * td.phCursor);
				}

				break;

			case rkPolyline:
				// Render with the first pen.
				assert (td.phPen);
				::SelectObject (hdcMem, * td.phPen);
				::MoveToEx (hdcMem, 0,        c_cIconH/2, NULL);
				::LineTo   (hdcMem, c_cIconW, c_cIconH/2);

				// Render with the second pen, if any.
				if (td.phPen2) {
					::SelectObject (hdcMem, * td.phPen2);
					::MoveToEx (hdcMem, 0,        c_cIconH/2, NULL);
					::LineTo   (hdcMem, c_cIconW, c_cIconH/2);
				}

				break;

			case rkPolygon:
				// Render sample.
				::SelectObject (hdcMem, td.phPen == NULL ? g_hpenNull : * td.phPen);
				assert (td.phBrush);
				::SelectObject (hdcMem, * td.phBrush);

				::Rectangle (hdcMem, 0, 0, c_cIconW, c_cIconH);

				break;
		}

		ICONINFO ii = {TRUE, 0, 0, hdcMem.GetBitmap (), hdcMem.GetBitmap ()};
		const HICON hicon = ::CreateIconIndirect (& ii);

		td.cImageListIndex = ::ImageList_AddIcon (hilTypeIcons, hicon);

		::DestroyIcon (hicon);
	}
	GetUnknownTD ()->cImageListIndex = 0;
}

/////////////////////////////////////////////////

void type_desc_t::GetCursorShift (POINT & _ptShift) const {
	if (! bCursorShiftInitialized) {
		bCursorShiftInitialized = true;

		ptCursorShift.x = 0;
		ptCursorShift.y = 0;
		if (phCursor)
			GetCursorHotSpot (* phCursor, ptCursorShift);
	}

	_ptShift = ptCursorShift;
}

/////////////////////////////////////////////////

static
void InitNullTypes () {
	CTypeSet::types_t TypeDescsOfNullTypeSet;
	TypeDescsOfNullTypeSet.push_back (g_tdUnknown);

	g_tsNull.SetTypes (TypeDescsOfNullTypeSet);

	g_tsNull.strID            = "Null";
	g_tsNull.strDisplayName   = "Null";

	g_tsNull.pfGetLatGrid     = DontRoundGrid;
	g_tsNull.pfRoundPoint     = DontRoundPoint;
	g_tsNull.pfRoundPoints    = DontRoundPoints;
	g_tsNull.pfRoundRect      = DontRoundRect;

	g_tsNull.pfIsCity         = RgnPredicateFalse;
	g_tsNull.pfIsHwExit       = RgnPredicateFalse;
	g_tsNull.pfIsRoad         = RgnPredicateFalse;
	g_tsNull.pfIsUnpavedRoad  = RgnPredicateFalse;
	g_tsNull.pfIsRoundabout   = RgnPredicateFalse;
	g_tsNull.pfIsBackground   = RgnPredicateFalse;
	g_tsNull.pfHasAddress     = RgnPredicateFalse;
	g_tsNull.pfHasCityAndZip  = RgnPredicateFalse;
	g_tsNull.pfHasPhone       = RgnPredicateFalse;
	g_tsNull.pfHasContact     = RgnPredicateFalse;
	g_tsNull.pfHasDescription = RgnPredicateFalse;

	g_tsNull.dwFeatures       = 0;

	g_tsNull.cMaxBits         = 24;
}

static
void DestroyNullTypes () {}

CTypeSet g_tsNull (InitNullTypes, DestroyNullTypes);

/////////////////////////////////////////////////

struct category_info_t {
	const char * strDisplayName;
	COLORREF col;
};

static
category_info_t g_TypeCategories [] = {
	{"Roads",          RGB (240, 240, 240)},
	{"Boundaries",     c_colWhite},
	{"Elevation",      RGB (255, 255, 230)},
	{"Cities",         RGB (230, 230, 230)},
	{"Labels",         c_colWhite},
	{"Food",           RGB (255, 255, 230)},
	{"Lodging",        c_colWhite},
	{"Attractions",    c_colWhite},
	{"Entertainment",  c_colWhite},
	{"Shopping",       c_colWhite},
	{"Services",       RGB (240, 240, 255)},
	{"Emerg./Gov.",    RGB (255, 230, 230)},
	{"Transport",      RGB (230, 230, 230)},
	{"Religion",       RGB (255, 255, 230)},
	{"Education",      c_colWhite},
	{"Sports",         RGB (255, 240, 240)},
	{"Recreation",     RGB (230, 255, 240)},
	{"Water",          RGB (230, 230, 255)},
	{"Land",           RGB (230, 255, 230)},
	{"Man made",       RGB (230, 230, 230)},
	{"Building",       RGB (254, 225, 209)},
	{"Auto",           RGB (230, 230, 230)},
	{"Aviation",       c_colWhite},
	{"Marine",         RGB (230, 230, 255)},
	{"Marine navaids", RGB (240, 240, 255)},
	{"Marine danger",  RGB (230, 230, 255)},
	{"Others",         c_colWhite},
	{"Unknown",        RGB (255, 230, 230)},
	{"Custom",         c_colWhite},
};

const char * GetTypeCategoryDisplayName (type_category_t _tc) {
	if (_tc >= sizeof (g_TypeCategories)/sizeof (g_TypeCategories [0]))
		return "";
	return g_TypeCategories [_tc].strDisplayName;
}

COLORREF GetTypeCategoryColor (type_category_t _tc) {
	if (_tc >= sizeof (g_TypeCategories)/sizeof (g_TypeCategories [0]))
		return c_colWhite;
	return g_TypeCategories [_tc].col;
}
