//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// Unit tests.
//

# include "StdAfx.h"
# include "UnitTests.h"
# include "PtAlgo.h"

# include "Globals.h"
# include "PtFinder.h"

static
void _TestPtFinder () {
	CPointFinder<size_t> pf, pf1;

	::srand (0);

	const size_t N = 100;

	points_t points;
	points.reserve (N);
	size_t c;
	for (c = 0; c < N; ++ c) {
		const float x = float (::rand ())/RAND_MAX;
		const float y = float (::rand ())/RAND_MAX;
		point_t pt (x, y);
		points.push_back (pt);
		pf.Add (pt, c);
	}

	pf.Index ();

	const float c_fE = 1e-5f;

	for (c = 0; c < N; ++ c) {
		const point_t & pt = points [c];
		const size_t * const pc = pf.Find (pt, c_fE);
		assert (pc);
		if (pc)
			assert (* pc == c);
	}

	pf1.Add (points [0], 10);
	pf1.Index ();
	const size_t * const pc = pf1.Find (points [0], c_fE);
	assert (pc);
	if (pc)
		assert (* pc == 10);
}

/////////////////////////////////////////////////////////

# include "PrintUnits.h"

static
void _TestPrintUnits () {
	char strY [16];
	g_PositionFormat = pfSeconds;
	PrintLat (strY, 61 + 59.9995f/60);
	assert (::strncmp (strY, "N62", 3) == 0);
	g_PositionFormat = pfMinutes;
	PrintLat (strY, 61 + 59.9995f/60);
	assert (::strncmp (strY, "N62", 3) == 0);
}

/////////////////////////////////////////////////////////

static
void _TestRemoveRepeatingPoints () {
	points_t points;
	points.push_back (point_t (0, 0));
	points.push_back (point_t (0, 0));
	points.push_back (point_t (0, 0));
	points.push_back (point_t (2, 2));
	points.push_back (point_t (1, 1));
	points.push_back (point_t (1, 1));
	RemoveRepeatingPoints (points);
	assert (points.size () == 3);
	assert (points [0].x == 0);
	assert (points [1].x == 2);
	assert (points [2].x == 1);
}

///////////////////////////////////

void UnitTests () {
# ifdef _DEBUG
	_TestPtFinder ();
	_TestPrintUnits ();
	_TestRemoveRepeatingPoints ();

	////////////////////

	void Test_TM2LL ();
	Test_TM2LL ();

	void Test_Mercator ();
	Test_Mercator ();

	void Test_NZMG ();
	Test_NZMG ();

	void Test_Datums ();
	Test_Datums ();

	////////////////////

	void Test_fast_floor ();
	Test_fast_floor ();

	void Test_Distance ();
	Test_Distance ();

	void Test_Course ();
	Test_Course ();

	void Test_ParseArc ();
	Test_ParseArc ();
# endif // _DEBUG
}
