//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2002-2009
//
// VTU (Military Topographic Service of Russia) types conversion utility.
//

# include "StdAfx.h"
# include "VTUTypes.h"
# include "Globals.h"

# pragma const_seg (".text")

static
const vtu_type_t g_VTUTypes [] = {
	{11101,     0x2800, VNPL, 0x80, VTU_NAME (" ")},

	{11100000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{11200000,  0xF305, VNPL, VNPG, VTU_NAME (" ")},
	{11300000,  0xF305, VNPL, VNPG, VTU_NAME ("  ")},
	{11400000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{12000000,  0x6300, VNPL, VNPG, VTU_NAME (" ")},
	{12110000,  0x6300, VNPL, VNPG, VTU_NAME ("  ")},
	{12120000,  0x6300, VNPL, VNPG, VTU_NAME ("  ")},
	{12200000,  0x6300, VNPL, VNPG, VTU_NAME ("   ")},
	{12300000,  0x6300, VNPL, VNPG, VTU_NAME ("   ..")},
	{13110000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{13111000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{13120000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{13121000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{13210000,  VNPT,   VNPL, VNPG, VTU_NAME (". .")},
	{13220000,  VNPT,   VNPL, VNPG, VTU_NAME (". .")},
	{13410000,  VNPT,   VNPL, VNPG, VTU_NAME (". ...")},
	{13420000,  VNPT,   VNPL, VNPG, VTU_NAME (". ...")},

	{21100000,  VNPT,   0x22, VNPG, VTU_NAME ("  ")},
	{21200000,  VNPT,   0x21, VNPG, VTU_NAME (" ")},
	{21300000,  VNPT,   0x21, VNPG, VTU_NAME (" ")},
	{21400000,  VNPT,   0x20, VNPG, VTU_NAME (" ")},
	{21410000,  VNPT,   0x20, VNPG, VTU_NAME ("  ..")},
	{21500000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},

	{22110000,  VNPT,   VNPL, 0x4d, VTU_NAME ("")},
	{22111000,  VNPT,   VNPL, 0x4d, VTU_NAME (" ")},
	{22120000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22121000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22121100,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{22130000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22140000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22150000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22151000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22155000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22160000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{22170000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},

	{22211000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{22212000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{22213000,  VNPT,   VNPL, VNPG, VTU_NAME (" , ")},
	{22214000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22221000,  VNPT,   0x26, VNPG, VTU_NAME (" ")},
	{22222000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{22222100,  VNPT,   VNPL, VNPG, VTU_NAME ("  , .")},
	{22230000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22231000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22232000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22241000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{22242000,  VNPT,   VNPL, VNPG, VTU_NAME ("Regions of karst spread")},
	{22250000,  0xf308, VNPL, VNPG, VTU_NAME ("")},
	{22260000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{22263000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},

	{22310000,  0x6614, VNPL, VNPG, VTU_NAME ("-")},
	{22320000,  VNPT,   VNPL, VNPG, VTU_NAME ("  . ")},
	{22330000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},

	{22410000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22413000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{22421000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{22422000,  0x6608, VNPL, VNPG, VTU_NAME ("  ")},
	{22431000,  0x660e, VNPL, VNPG, VTU_NAME (" ")},
	{22432000,  0x660e, VNPL, VNPG, VTU_NAME (" ")},

	{22511000,  VNPT,   VNPL, 0x4d, VTU_NAME ("")},
	{22512000,  VNPT,   VNPL, 0x4d, VTU_NAME (" ")},
	{22512100,  VNPT,   VNPL, 0x4d, VTU_NAME ("   ")},
	{22520000,  0xf309, VNPL, VNPG, VTU_NAME ("  ")},
	{22521000,  0xf309, VNPL, VNPG, VTU_NAME ("")},
	{22522000,  0xf309, VNPL, VNPG, VTU_NAME ("")},

	{22611000,  0xf30a, VNPL, VNPG, VTU_NAME (" ")},
	{22612000,  0xf30a, VNPL, VNPG, VTU_NAME ("  ")},
	{22613000,  0xf30a, VNPL, VNPG, VTU_NAME (" ")},
	{22620000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{22623000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22630000,  0x6615, VNPL, VNPG, VTU_NAME ("")},
	{22633000,  0x6615, VNPL, VNPG, VTU_NAME (" ")},
	{22640000,  0xf30d, VNPL, VNPG, VTU_NAME ("   ()")},
	{22650000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{22660000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{22670000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},

	{23100000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{23110000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ,  ")},

	{31110000,  VNPT,   VNPL, 0x28, VTU_NAME ("  ")},
	{31120000,  VNPT,   VNPL, 0x41, VTU_NAME ("")},
	{31130000,  VNPT,   VNPL, 0x41, VTU_NAME ("")},
	{31131000,  VNPT,   VNPL, 0x41, VTU_NAME ("")},
	{31131200,  VNPT,   VNPL, 0x41, VTU_NAME (" ")},
	{31131300,  VNPT,   VNPL, 0x41, VTU_NAME (" ")},
	{31132000,  VNPT,   VNPL, 0x41, VTU_NAME ("")},
	{31133000,  VNPT,   VNPL, 0x41, VTU_NAME ("")},
	{31134000,  VNPT,   VNPL, 0x4c, VTU_NAME (" , .   ")},
	{31136000,  VNPT,   VNPL, VNPG, VTU_NAME (" . .")},
	{31140000,  VNPT,   VNPL, 0x4c, VTU_NAME (" ,  .")},

	{31211000,  VNPT,   VNPL, 0x53, VTU_NAME ("")},
	{31212000,  VNPT,   VNPL, 0x53, VTU_NAME ("")},
	{31220000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{31230000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{31241000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{31242000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{31250000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{31260000,  VNPT,   VNPL, VNPG, VTU_NAME ("Water-plant")},
	{31261000,  VNPT,   VNPL, 0x8d, VTU_NAME ("")},

	{31310000,  VNPT,   0x23, VNPG, VTU_NAME ("")},
	{31320000,  0x6200, VNPL, VNPG, VTU_NAME (" ")},
	{31331000,  0x1c0a, VNPL, VNPG, VTU_NAME ("")},
	{31332000,  0x1c09, VNPL, VNPG, VTU_NAME (" ")},
	{31333000,  0x1c09, VNPL, VNPG, VTU_NAME (" (.)")},
	{31334000,  VNPT,   VNPL, 0x53, VTU_NAME (" (.)")},
	{31335100,  0x6508, VNPL, VNPG, VTU_NAME ("")},
	{31335200,  0x4100, VNPL, VNPG, VTU_NAME ("")},

	{31410000,  VNPT,   0x1f, 0x49, VTU_NAME (", ")},
	{31420000,  VNPT,   0x18, 0x49, VTU_NAME ("Streamlets")},
	{31431000,  VNPT,   0x1f, 0x49, VTU_NAME ("")},
	{31431110,  VNPT,   0x1f, 0x49, VTU_NAME (" ")},
	{31431111,  VNPT,   0x1f, 0x49, VTU_NAME ("  ")},
	{31431120,  VNPT,   0x1f, 0x49, VTU_NAME (" ")},
	{31431121,  VNPT,   0x1f, 0x49, VTU_NAME ("  ")},
	{31431200,  VNPT,   0x1f, 0x49, VTU_NAME (" ")},
	{31431310,  VNPT,   0x1f, 0x49, VTU_NAME ("  ")},
	{31431320,  VNPT,   0x1f, 0x49, VTU_NAME ("  ")},
	{31431400,  VNPT,   0x1f, 0x49, VTU_NAME (" ")},
	{31431410,  VNPT,   0x1f, 0x49, VTU_NAME ("  ")},
	{31431510,  VNPT,   0x1f, 0x49, VTU_NAME (" ")},
	{31431520,  VNPT,   0x1f, 0x49, VTU_NAME ("   ")},
	{31432000,  VNPT,   0x18, 0x49, VTU_NAME ("")},
	{31432100,  VNPT,   0x18, 0x49, VTU_NAME ("  ..")},
	{31432200,  VNPT,   0x18, 0x4c, VTU_NAME (" ")},
	{31432210,  VNPT,   0x18, 0x4c, VTU_NAME ("  ")},

	{31510000,  0x6300, VNPL, VNPG, VTU_NAME (" ")},
	{31521000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{31522000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{31531000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{31532000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{31540000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ..")},
	{31541000,  VNPT,   VNPL, VNPG, VTU_NAME ("....")},

	{31611000,  0x6511, VNPL, VNPG, VTU_NAME (" () .")},
	{31612000,  0x6511, VNPL, VNPG, VTU_NAME (" () .")},
	{31620000,  0x6509, VNPL, VNPG, VTU_NAME ("")},
	{31630000,  0x6414, VNPL, VNPG, VTU_NAME ("")},
	{31631000,  0x6414, VNPL, VNPG, VTU_NAME (" ")},
	{31632000,  0x6414, VNPL, VNPG, VTU_NAME ("   ")},
	{31633000,  0x6414, VNPL, VNPG, VTU_NAME ("    ")},
	{31634000,  0x6414, VNPL, VNPG, VTU_NAME ("")},
	{31635000,  0x6414, VNPL, VNPG, VTU_NAME (" ")},
	{31636000,  0x6414, VNPL, VNPG, VTU_NAME (" ")},
	{31640000,  0x6414, VNPL, VNPG, VTU_NAME ("Artesian chinks")},
	{31650000,  0xf302, VNPL, VNPG, VTU_NAME ("")},
	{31662000,  0x5000, VNPL, VNPG, VTU_NAME (" ")},

	{31700000,  VNPT,   0x15, VNPG, VTU_NAME (" ")},
	{32110000,  0x5500, VNPL, 0x13, VTU_NAME ("")},
	{32110100,  VNPT,   VNPL, VNPG, VTU_NAME (" , ")},
	{32115100,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{32120000,  VNPT,   VNPL, 0x13, VTU_NAME ("")},
	{32121000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{32121100,  VNPT,   VNPL, VNPG, VTU_NAME ("  c ")},
	{32122000,  VNPT,   VNPL, VNPG, VTU_NAME ("Lock shambers")},
	{32130000,  VNPT,   0x26, VNPG, VTU_NAME ("Dikes and artificial ramparts")},
	{32133000,  VNPT,   VNPL, VNPG, VTU_NAME (" ()")},
	{32140000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{32150000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{32151100,  VNPT,   VNPL, VNPG, VTU_NAME ("  . . (.>1)")},
	{32152000,  VNPT,   VNPL, VNPG, VTU_NAME ("  . . .")},
	{32160000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},

	{32210000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{32220000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{32230000,  0x4300, VNPL, 0x09, VTU_NAME ("  ")},
	{32240000,  0x2f09, VNPL, 0x09, VTU_NAME ("   ")},
	{32250000,  0x2f09, VNPL, 0x09, VTU_NAME ("Wharfs without berthing facilities")},
	{32251000,  0x2f09, VNPL, 0x09, VTU_NAME ("   ")},
	{32260000,  VNPT,   VNPL, VNPG, VTU_NAME (" ()  ")},
	{32283000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},

	{32310000,  VNPT,   0x28, VNPG, VTU_NAME ("")},
	{32320000,  VNPT,   0x28, VNPG, VTU_NAME ("")},
	{32330000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{32331000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{32332000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{32340000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{32345000,  VNPT,   VNPL, VNPG, VTU_NAME ("  , ")},
	{32350000,  VNPT,   0x28, VNPG, VTU_NAME ("   ")},
	{32360000,  VNPT,   VNPL, VNPG, VTU_NAME ("    ")},

	{32410000,  0x1600, VNPL, VNPG, VTU_NAME ("")},
	{32420000,  0x1600, VNPL, VNPG, VTU_NAME ("")},
	{32430000,  0x1600, VNPL, VNPG, VTU_NAME ("   ")},
	{32431000,  0x1600, VNPL, VNPG, VTU_NAME (" ")},
	{32440000,  0x1600, VNPL, VNPG, VTU_NAME (" ")},
	{32450000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{32460000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{32461000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{32462000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{32470000,  VNPT,   VNPL, VNPG, VTU_NAME ("Tidepoles")},

	{33111000,  VNPT,   0x1b, VNPG, VTU_NAME ("  ")},
	{33112000,  VNPT,   0x1a, VNPG, VTU_NAME ("  ")},
	{33113000,  VNPT,   0x1a, VNPG, VTU_NAME (" ")},
	{33113200,  VNPT,   0x1a, VNPG, VTU_NAME (" . ")},
	{33120000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{33130000,  0xf307, VNPL, VNPG, VTU_NAME ("")},
	{33200000,  VNPT,   0x1a, VNPG, VTU_NAME ("Sea routes")},
	{34000000,  0x650c, VNPL, 0x80, VTU_NAME ("")},

	{41100000,  0x0900, VNPL, 0x02, VTU_NAME ("")},
	{41200000,  0x0b00, VNPL, 0x02, VTU_NAME ("  ")},
	{42100000,  0x0c00, VNPL, 0x03, VTU_NAME ("  ")},
	{42200000,  0x1100, VNPL, 0x03, VTU_NAME (" , ")},
	{42300000,  0x0d00, VNPL, 0x03, VTU_NAME ("  , ")},
	{43100000,  0x0d00, VNPL, 0x03, VTU_NAME ("  ")},
	{43200000,  0x0d00, VNPL, 0x03, VTU_NAME (",  .  ")},
	{43300000,  0x6415, VNPL, 0x03, VTU_NAME (". . ")},
	{43400000,  0x0d00, VNPL, 0x03, VTU_NAME ("   ")},

	{44100000,  0x6402, VNPL, 0x13, VTU_NAME ("  ")},
	{44200000,  0x6402, VNPL, 0x13, VTU_NAME ("  ")},
	{44210000,  0x6416, VNPL, 0x13, VTU_NAME (" ")},
	{45100000,  VNPT,   VNPL, 0x02, VTU_NAME ("")},
	{45111000,  VNPT,   VNPL, 0x01, VTU_NAME ("   ")},
	{45140000,  VNPT,   VNPL, 0x02, VTU_NAME ("   . ")},
	{45160000,  VNPT,   VNPL, 0x02, VTU_NAME (" ")},
	{45200000,  VNPT,   0x06, 0x6b, VTU_NAME ("")},
	{45210000,  VNPT,   VNPL, 0x6b, VTU_NAME (" , ")},
	{45300000,  VNPT,   VNPL, 0x6a, VTU_NAME ("")},
	{45400000,  VNPT,   VNPL, 0x02, VTU_NAME ("  ")},
	{45510000,  VNPT,   VNPL, 0x0c, VTU_NAME ("   ")},
	{45520000,  VNPT,   VNPL, 0x0c, VTU_NAME ("   ")},
	{46100000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ()")},
	{46510000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{46520000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ()")},
	{47110000,  VNPT,   VNPL, 0x13, VTU_NAME ("")},
	{47120000,  VNPT,   VNPL, 0x13, VTU_NAME (", ")},

	{51111000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{51111100,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{51112000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{51113000,  0xf30b, VNPL, VNPG, VTU_NAME ("")},
	{51121100,  0x640c, VNPL, VNPG, VTU_NAME ("")},
	{51121200,  VNPT,   VNPL, VNPG, VTU_NAME ("Adits")},
	{51121300,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{51122000,  0x640c, VNPL, VNPG, VTU_NAME (", , ")},
	{51123000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")}, 
	{51123100,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{51123200,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{51123210,  0x6300, VNPL, VNPG, VTU_NAME (" , ")},
	{51124000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{511241000, VNPT,   VNPL, VNPG, VTU_NAME ("Wells")},
	{511242000, VNPT,   VNPL, VNPG, VTU_NAME ("Dug holes")},
	{51124310,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{51124320,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{51124330,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{511250000, 0x640d, VNPL, VNPG, VTU_NAME ("   ")},

	{51131000,  VNPT,   VNPL, 0x0c, VTU_NAME (".  ")}, 
	{511310000, VNPT,   VNPL, 0x0c, VTU_NAME (".  ")},
	{51132000,  VNPT,   VNPL, 0x0c, VTU_NAME (".  ")},
	{511320000, VNPT,   VNPL, 0x0c, VTU_NAME (".  ")},

	{51133000,  0x2900, VNPL, VNPG, VTU_NAME (" ")},
	{51133100,  VNPT,   VNPL, 0x06, VTU_NAME ("")},
	{51133200,  0x2f0b, VNPL, 0x05, VTU_NAME ("")},

	{51140000,  VNPT,   VNPL, 0x1c, VTU_NAME ("")},
	{51141100,  VNPT,   VNPL, 0x1c, VTU_NAME (" ")},
	{51141200,  VNPT,   VNPL, 0x1c, VTU_NAME (" ")},
	{51141300,  VNPT,   VNPL, 0x1c, VTU_NAME ("")},
	{51141400,  VNPT,   VNPL, 0x1c, VTU_NAME (" ")},
	{51141500,  VNPT,   VNPL, 0x1c, VTU_NAME (" ")},
	{51142100,  VNPT,   VNPL, 0x1c, VTU_NAME ("")},
	{51142200,  VNPT,   VNPL, 0x1c, VTU_NAME (" .")},
	{51142300,  VNPT,   VNPL, 0x1c, VTU_NAME (" ")},

	{51151000,  VNPT,   VNPL, 0x6f, VTU_NAME (" ")},
	{51152000,  VNPT,   VNPL, 0x6f, VTU_NAME (" ")},
	{51153100,  VNPT,   VNPL, 0x6f, VTU_NAME ("    ")},
	{51153200,  VNPT,   VNPL, 0x6f, VTU_NAME ("    ")},
	{51160000,  VNPT,   VNPL, 0x6f, VTU_NAME ("")},
	{51170000,  VNPT,   VNPL, 0x6f, VTU_NAME ("  ")},

	{51210000,  VNPT,   VNPL, 0x6f, VTU_NAME ("")},
	{51211000,  VNPT,   VNPL, 0x0c, VTU_NAME ("  ")},
	{51211100,  VNPT,   VNPL, 0x0c, VTU_NAME (" ")},
	{51212000,  VNPT,   VNPL, 0x0c, VTU_NAME ("  ")},
	{51220000,  0x2f01, VNPL, 0x6b, VTU_NAME (" ")},
	{51230000,  VNPT,   VNPL, VNPG, VTU_NAME (" , ")},

	{51310000,  VNPT,   0x28, VNPG, VTU_NAME ("")},
	{51311000,  VNPT,   0x28, VNPG, VTU_NAME ("")},
	{51312000,  VNPT,   0x28, VNPG, VTU_NAME ("")},
	{51313000,  VNPT,   0x28, VNPG, VTU_NAME ("")},
	{51320000,  VNPT,   0x29, VNPG, VTU_NAME ("  ., .,  / ")},
	{51322200,  VNPT,   0x29, VNPG, VTU_NAME ("  ")},
	{51322300,  VNPT,   0x29, VNPG, VTU_NAME ("  ")},
	{51330000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{51340000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{513500000, VNPT,   VNPL, VNPG, VTU_NAME ("Siphons on pipelines")},

	{51410000,  0x6411, VNPL, VNPG, VTU_NAME (".   ")},
	{51411000,  0x6411, VNPL, VNPG, VTU_NAME (" ")},
	{51420000,  0x6411, VNPL, VNPG, VTU_NAME ("   ")},
	{51431000,  0x6411, VNPL, VNPG, VTU_NAME ("  ")},
	{51432000,  0x640d, VNPL, VNPG, VTU_NAME (" ")},
	{51432100,  0x640d, VNPL, VNPG, VTU_NAME (" ")},
	{51432200,  0x640d, VNPL, VNPG, VTU_NAME (" ")},
	{51440000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{51450000,  VNPT,   VNPL, 0x6f, VTU_NAME ("  ")},
	{51461000,  VNPT,   VNPL, 0x6f, VTU_NAME (" ")},
	{51462000,  VNPT,   VNPL, 0x6f, VTU_NAME (" ")},
	{51463000,  VNPT,   VNPL, 0x6f, VTU_NAME ("")},
	{51470000,  VNPT,   VNPL, VNPG, VTU_NAME ("  , ")},
	{51480000,  VNPT,   VNPL, 0x41, VTU_NAME ("")},
	{514900000, VNPT,   VNPL, VNPG, VTU_NAME ("Salt-pan (cool-tower)")},

	{51500000,  VNPT,   VNPL, 0x6f, VTU_NAME (" .  / ")},
	{51710000,  VNPT,   VNPL, 0x0c, VTU_NAME ("")},
	{51731000,  VNPT,   VNPL, 0x0c, VTU_NAME (" ")},
	{52100000,  VNPT,   VNPL, 0x0c, VTU_NAME (". ")},
	{52300000,  VNPT,   VNPL, 0x6f, VTU_NAME ("  ")},
	{52400000,  VNPT,   VNPL, 0x0c, VTU_NAME ("")},
	{52510000,  VNPT,   VNPL, 0x6f, VTU_NAME (",   ")},
	{52530000,  VNPT,   VNPL, 0x6f, VTU_NAME ("")},
	{52540000,  VNPT,   VNPL, 0x13, VTU_NAME ("")},

	{53110000,  0x2f04, VNPL, VNPG, VTU_NAME (", ")},
	{531200000, VNPT,   VNPL, VNPG, VTU_NAME ("Airfields")},
	{53130000,  VNPT,   0x27, 0x0e, VTU_NAME ("- ")},
	{53140000,  VNPT,   0x27, 0x0e, VTU_NAME ("   ")},
	{53151000,  VNPT,   0x27, 0x0e, VTU_NAME (" ")},
	{53152000,  VNPT,   0x27, 0x0e, VTU_NAME ("   ")},
	{53160000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{53170000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{53180000,  VNPT,   VNPL, VNPG, VTU_NAME ("    ")},

	{53210000,  0x640b, VNPL, 0x04, VTU_NAME ("   ")},
	{53211000,  0x640b, VNPL, 0x04, VTU_NAME (" ")},
	{53212000,  0x640b, VNPL, 0x04, VTU_NAME (" ")},
	{53220000,  VNPT,   VNPL, 0x04, VTU_NAME (", , ")},
	{53221000,  VNPT,   VNPL, 0x04, VTU_NAME ("")},
	{53230000,  0xf00a, VNPL, VNPG, VTU_NAME (" , ")},
	{53240000,  0xf00a, VNPL, VNPG, VTU_NAME ("   ")},
	{53310000,  0xf304, VNPL, VNPG, VTU_NAME ("")},
	{53320000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{53330000,  0xf304, VNPL, VNPG, VTU_NAME (" ")},
	{53340000,  0xf304, VNPL, VNPG, VTU_NAME (" (,,)")},
	{53350000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{53360000,  0x2f12, VNPL, VNPG, VTU_NAME (" ,  ")},

	{53410000,  0x6404, VNPL, 0x6e, VTU_NAME ("Monasteries")},
	{53411000,  0xf101, VNPL, 0x6e, VTU_NAME (" ")},
	{53411100,  0xf101, VNPL, 0x6e, VTU_NAME ("  ")},
	{53411200,  0xf101, VNPL, 0x6e, VTU_NAME ("  ")},
	{53420000,  0xf101, VNPL, 0x6e, VTU_NAME (", , ")},
	{53430000,  0xf104, VNPL, 0x6e, VTU_NAME ("")},
	{53440000,  0xf106, VNPL, 0x6e, VTU_NAME ("Buddhist temples, pagodas")},
	{53450000,  0x6404, VNPL, 0x6e, VTU_NAME ("")},
	{53460000,  VNPT,   VNPL, VNPG, VTU_NAME ("  , ")},
	{53511000,  0x6403, VNPL, 0x1a, VTU_NAME ("  ")},
	{53512000,  0x6403, VNPL, 0x1a, VTU_NAME ("  ")},
	{53516000,  0x6403, VNPL, 0x1a, VTU_NAME ("")},
	{53520000,  0xf301, VNPL, VNPG, VTU_NAME (" ")},
	{53522000,  0xf301, VNPL, VNPG, VTU_NAME (" ")},
	{53530000,  0xf301, VNPL, VNPG, VTU_NAME (", , ")},
	{53531000,  0x6403, VNPL, 0x1a, VTU_NAME (" ")},
	{53532000,  0x2c0b, VNPL, VNPG, VTU_NAME ("    . .")},
	{53540000,  0x6403, VNPL, 0x1a, VTU_NAME (", , ")},
	{53550000,  VNPT,   VNPL, 0x1a, VTU_NAME ("")},

	{53600000,  VNPT,   VNPL, VNPG, VTU_NAME ("- ")},
	{53610000,  VNPT,   VNPL, 0x6e, VTU_NAME (" - ")},
	{53622100,  0xf701, VNPL, 0x6e, VTU_NAME (" ")},
	{53622200,  0xf701, VNPL, 0x08, VTU_NAME (" ")},

	{53700000,  0x2d0a, VNPL, 0x19, VTU_NAME (" ")},
	{53710000,  0x2d0a, VNPL, 0x19, VTU_NAME (" ")},
	{53711000,  0x2d0a, VNPL, 0x19, VTU_NAME ("")},
	{53712000,  0x2d0a, VNPL, 0x19, VTU_NAME (" ")},
	{53713000,  0x2d06, VNPL, 0x19, VTU_NAME ("")},
	{53720000,  0x2d0a, VNPL, 0x6e, VTU_NAME (" ")},

	{53810000,  0x2c02, VNPL, 0x6e, VTU_NAME (",  .")},
	{53820000,  0x2d01, VNPL, 0x6e, VTU_NAME ("")},
	{53830000,  0x2d03, VNPL, 0x6e, VTU_NAME ("")},
	{53840000,  0x2c09, VNPL, 0x6e, VTU_NAME (" , ")},
	{53850000,  VNPT,   VNPL, 0x6e, VTU_NAME (" ")},
	{53870000,  0x2d01, VNPL, 0x13, VTU_NAME ("")},
	{53880000,  0x2d0a, VNPL, 0x6e, VTU_NAME ("")},
	{53890000,  0x2c06, VNPL, 0x17, VTU_NAME ("  (  )")},
	{53891000,  0x2c06, VNPL, 0x17, VTU_NAME ("    ")},
	{53892000,  VNPT,   VNPL, 0x6e, VTU_NAME ("    ")},

	{54100000,  VNPT,   VNPL, VNPG, VTU_NAME ("     .")},

	{61111000,  VNPT,   0x14, VNPG, VTU_NAME (".. ()")},
	{61112000,  VNPT,   0x14, VNPG, VTU_NAME (".. ()")},
	{61113000,  VNPT,   0x40, VNPG, VTU_NAME (".. ")},
	{61114000,  VNPT,   VNPL, VNPG, VTU_NAME (" . ..")},
	{61121000,  VNPT,   0x14, VNPG, VTU_NAME ("Railway station lines")},
	{61121100,  VNPT,   0x14, VNPG, VTU_NAME ("  ")},
	{61121200,  VNPT,   0x14, VNPG, VTU_NAME ("  ")},
	{61122100,  VNPT,   0x14, VNPG, VTU_NAME ("")},
	{61122200,  VNPT,   0x14, VNPG, VTU_NAME (" ")},
	{61123000,  VNPT,   0x14, VNPG, VTU_NAME (" ")},
	{61124000,  VNPT,   0x3f, VNPG, VTU_NAME (" ")},

	{61210000,  VNPT,   0x01, 0x69, VTU_NAME ("")},
	{61220000,  VNPT,   0x02, 0x68, VTU_NAME (" ")},
	{61230000,  VNPT,   0x03, 0x6b, VTU_NAME ("")},
	{61240000,  VNPT,   0x03, 0x6b, VTU_NAME ("Road with a covering (highway)")},
	{61310000,  VNPT,   0x42, 0x6b, VTU_NAME ("  ")},
	{61320000,  VNPT,   0x0a, 0x6b, VTU_NAME ("  ")},
	{61330000,  VNPT,   0x0a, 0x6b, VTU_NAME ("   ")},
	{61410000,  VNPT,   0x16, 0x6b, VTU_NAME ("    ")},
	{61420000,  VNPT,   0x16, 0x6b, VTU_NAME ("   ")},
	{61500000,  VNPT,   0x0a, 0x6b, VTU_NAME ("   ")},
	{61600000,  VNPT,   0x0a, 0x6b, VTU_NAME ("")},
	{61610000,  VNPT,   0x0a, 0x6b, VTU_NAME ("")},
	{61700000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{61810000,  VNPT,   0x41, VNPG, VTU_NAME (" ")},
	{61820000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},

	{61910000,  VNPT,   VNPL, VNPG, VTU_NAME ("    ")},
	{61920000,  VNPT,   VNPL, VNPG, VTU_NAME ("     ")},
	{61940000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{61950000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{61960000,  0x6406, VNPL, VNPG, VTU_NAME ("")},
	{61970000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{61980000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},

	{62110000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{62120000,  0xf007, VNPL, 0x6e, VTU_NAME ("")},
	{62131000,  0xf006, VNPL, 0x13, VTU_NAME (" /")},
	{62132000,  0xf005, VNPL, 0x6e, VTU_NAME (" ")},
	{62133000,  0xf006, VNPL, 0x13, VTU_NAME ("   ")},
	{62134000,  0xf006, VNPL, 0x13, VTU_NAME ("")},
	{62135000,  VNPT,   VNPL, VNPG, VTU_NAME ("Cargo handling areas")},
	{62136000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{62141000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{62142000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{62142200,  VNPT,   VNPL, VNPG, VTU_NAME ("  ..")},
	{62142300,  VNPT,   VNPL, 0x13, VTU_NAME ("  .")},
	{62143000,  0xf201, VNPL, VNPG, VTU_NAME ("  ")},
	{62144000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{62145000,  VNPT,   VNPL, 0x13, VTU_NAME ("Posts at watched crossings")},
	{62145100,  VNPT,   VNPL, 0x13, VTU_NAME ("  /")},
	{62210000,  VNPT,   VNPL, 0x13, VTU_NAME ("˸  ")},
	{62221000,  0x5a00, VNPL, VNPG, VTU_NAME (" ")},
	{62230000,  VNPT,   VNPL, VNPG, VTU_NAME ("   .")},

	{62300000,  VNPT,   VNPL, VNPG, VTU_NAME (", ")},
	{62310000,  0x6401, VNPL, VNPG, VTU_NAME ("")},
	{62311000,  0x6401, VNPL, VNPG, VTU_NAME ("   . ")},
	{62312000,  0x6401, VNPL, VNPG, VTU_NAME ("   . ")},
	{62313000,  0x6401, VNPL, VNPG, VTU_NAME (" ")},
	{62314000,  0x6401, VNPL, VNPG, VTU_NAME ("   ")},
	{62315000,  0x6401, VNPL, VNPG, VTU_NAME (" ")},
	{62317000,  0xf30c, VNPL, VNPG, VTU_NAME (" ")},
	{62317100,  0xf30c, VNPL, VNPG, VTU_NAME ("  ")},
	{62317200,  0xf30c, VNPL, VNPG, VTU_NAME ("  ")},
	{62317300,  0xf30c, VNPL, VNPG, VTU_NAME ("   ")},
	{62318000,  0xf30c, VNPL, VNPG, VTU_NAME (", ")},
	{62321000,  VNPT,   VNPL, VNPG, VTU_NAME ("Joint span overbridges")},
	{62322000,  VNPT,   VNPL, VNPG, VTU_NAME ("Disconnected span overbridges")},
	{62324000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{62329000,  VNPT,   VNPL, VNPG, VTU_NAME (" , ")},
	{62330000,  0x6413, 0x43, VNPG, VTU_NAME ("Tunnels")},
	{62331000,  0x6413, 0x43, VNPG, VTU_NAME ("")},
	{62340000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{62350000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{62360000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{62370000,  VNPT,   0x28, VNPG, VTU_NAME ("Pipe")},
	{62371000,  VNPT,   0x28, VNPG, VTU_NAME (" ")},
	{62380000,  VNPT,   VNPL, VNPG, VTU_NAME ("Galleries")},
	{62385200,  VNPT,   VNPL, VNPG, VTU_NAME ("  ()")},
	{62386000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ()  /")},
	{62391000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{62392000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{62393000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},

	{63100000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{63200000,  0x6400, VNPL, VNPG, VTU_NAME ("Distance post sign")},
	{63300000,  0x2800, VNPL, VNPG, VTU_NAME (" ")},
	{63310000,  VNPT,   VNPL, VNPG, VTU_NAME ("    ")},

	{71100000,  0xf406, VNPL, 0x50, VTU_NAME (" ")},
	{71111110,  0xf406, VNPL, 0x50, VTU_NAME ("   ()")},
	{71111113,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{71111120,  0x660a, VNPL, 0x82, VTU_NAME ("  ")},
	{71111210,  0xf404, VNPL, 0x83, VTU_NAME ("   ()")},
	{71111220,  0x660a, VNPL, 0x83, VTU_NAME ("  ")},
	{71112100,  0x660a, VNPL, 0x8f, VTU_NAME ("")},
	{71112200,  0x660a, VNPL, 0x84, VTU_NAME ("   ")},
	{71112300,  0x660a, VNPL, 0x85, VTU_NAME (" ")},
	{71113000,  0x660a, VNPL, 0x82, VTU_NAME ("Young wood")},
	{71113100,  0x660a, VNPL, 0x50, VTU_NAME ("  ")},
	{71114000,  0x660a, VNPL, 0x90, VTU_NAME ("")},
	{71121000,  0x660a, VNPL, 0x50, VTU_NAME ("Arboreta, plantations of young trees")},
	{71121200,  0x660a, VNPL, 0x82, VTU_NAME (" ")},
	{71121300,  0x660a, VNPL, 0x82, VTU_NAME (" , . ")},
	{71122000,  0x660a, VNPL, 0x82, VTU_NAME ("  . ")},
	{71123000,  0x660a, VNPL, 0x4e, VTU_NAME ("   ")},
	{71123100,  0x660a, VNPL, 0x4e, VTU_NAME ("- ")},
	{71131000,  0x660a, VNPL, 0x50, VTU_NAME (",  ")},
	{71132000,  0xf404, VNPL, VNPG, VTU_NAME ("Separate trees")},
	{71132100,  0xf404, VNPL, 0x50, VTU_NAME (",   . ")},
	{71132210,  0xf404, VNPL, 0x50, VTU_NAME (" , . ()")},
	{71132220,  0xf404, VNPL, 0x50, VTU_NAME (" , . (.)")},

	{71211100,  VNPT,   VNPL, 0x8e, VTU_NAME (" ")},
	{71211200,  0xf405, VNPL, 0x4f, VTU_NAME (" ")},
	{71212000,  VNPT,   VNPL, 0x92, VTU_NAME ("")},
	{71213000,  VNPT,   VNPL, 0x94, VTU_NAME (" ")},
	{71214000,  VNPT,   VNPL, 0x91, VTU_NAME ("")},
	{71221000,  VNPT,   VNPL, 0x4f, VTU_NAME (" . . ")},
	{71222000,  VNPT,   VNPL, 0x86, VTU_NAME ("")},
	{71223000,  VNPT,   VNPL, 0x4e, VTU_NAME (" ")},
	{71224000,  0xf405, VNPL, VNPG, VTU_NAME ("  ")},
	{71224300,  0xf405, VNPL, VNPG, VTU_NAME ("  ")},

	{71311000,  0x6400, VNPL, 0x8c, VTU_NAME ("   ")},
	{71312000,  VNPT,   VNPL, 0x95, VTU_NAME (" ")},
	{71313000,  VNPT,   VNPL, 0x88, VTU_NAME (" ")},
	{71314000,  VNPT,   VNPL, 0x88, VTU_NAME (" ")},
	{71315000,  VNPT,   VNPL, 0x95, VTU_NAME ("  ")},
	{71321000,  VNPT,   VNPL, 0x98, VTU_NAME ("   ")},
	{71322100,  VNPT,   VNPL, 0x87, VTU_NAME ("  ()")},
	{71322200,  VNPT,   VNPL, 0x87, VTU_NAME ("  ()")},
	{71323000,  VNPT,   VNPL, 0x13, VTU_NAME (", , ")},
	{71324000,  VNPT,   VNPL, VNPG, VTU_NAME ("Arable lands")},
	{71324100,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{71324170,  VNPT,   VNPL, 0x4e, VTU_NAME ("")},
	{71325000,  VNPT,   VNPL, 0x98, VTU_NAME ("  ")},
	{71410000,  VNPT,   VNPL, 0x88, VTU_NAME ("Forbs")},
	{71420000,  VNPT,   VNPL, 0x88, VTU_NAME ("")},
	{71500000,  VNPT,   VNPL, 0x96, VTU_NAME ("   ")},
	{71510000,  VNPT,   VNPL, 0x96, VTU_NAME ("")},
	{71520000,  VNPT,   VNPL, 0x97, VTU_NAME ("")},
	{71610000,  VNPT,   0x48, VNPG, VTU_NAME ("")},
	{71620000,  VNPT,   VNPL, 0x50, VTU_NAME (" ")},
	{71630000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},

	{72110000,  VNPT,   VNPL, 0x8a, VTU_NAME (" ")},
	{72120000,  VNPT,   VNPL, 0x8a, VTU_NAME ("Bouldery placers and broken stone grounds")},
	{72130000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{72210000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{72221000,  VNPT,   VNPL, VNPG, VTU_NAME ("Polygonic surfaces")},
	{72222000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{72223000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{72230000,  VNPT,   VNPL, 0x8a, VTU_NAME ("Shingle and gravel grounds")},
	{72240000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{72251000,  VNPT,   VNPL, 0x89, VTU_NAME ("Barchan sands")},
	{72251200,  VNPT,   VNPL, 0x89, VTU_NAME ("   ")},
	{72252000,  VNPT,   VNPL, 0x89, VTU_NAME (" ")},
	{72253000,  VNPT,   VNPL, 0x89, VTU_NAME ("Cumolose sands")},
	{72254000,  VNPT,   VNPL, 0x89, VTU_NAME ("Crater and sell sands")},
	{72255000,  VNPT,   VNPL, 0x89, VTU_NAME ("Plane sands")},

	{72310000,  VNPT,   VNPL, 0x51, VTU_NAME ("")},
	{72321000,  VNPT,   VNPL, 0x8b, VTU_NAME ("")},
	{72330000,  VNPT,   VNPL, 0x88, VTU_NAME (" ()")},
	{72331100,  VNPT,   VNPL, 0x51, VTU_NAME ("   ")},
	{72331300,  VNPT,   VNPL, 0x51, VTU_NAME ("  . . ")},
	{72332000,  VNPT,   VNPL, 0x8c, VTU_NAME ("    ")},
	{72333000,  VNPT,   VNPL, 0x51, VTU_NAME ("  ")},
	{72340000,  0x6513, VNPL, 0x51, VTU_NAME ("")},

	{73000000,  VNPT,   VNPL, VNPG, VTU_NAME (" . , ")},

	{79310000,  VNPT,   VNPL, VNPG, VTU_NAME ("   . ")},
	{79361200,  VNPT,   VNPL, VNPG, VTU_NAME ("  -   ")},

	{79671100,  0x6413, VNPL, VNPG, VTU_NAME ("    ")},
	{79671200,  0x6413, VNPL, VNPG, VTU_NAME ("    ")},
	{79674000,  VNPT,   VNPL, 0x93, VTU_NAME (" .  ")},

	{79714000,  VNPT,   VNPL, 0x93, VTU_NAME ("  ")},
	{79714000,  VNPT,   VNPL, 0x83, VTU_NAME ("   ")},
	{79715000,  VNPT,   VNPL, 0x83, VTU_NAME ("   ")},
	{79716000,  VNPT,   VNPL, 0x8a, VTU_NAME (" ")},
	{79717000,  VNPT,   VNPL, 0x84, VTU_NAME ("    ")},
	{79718000,  VNPT,   VNPL, 0x85, VTU_NAME (" ")},
	{79719000,  VNPT,   VNPL, 0x50, VTU_NAME ("  ")},
	{79719200,  0xf402, VNPL, 0x50, VTU_NAME ("  ")},
	{79719300,  0xf401, VNPL, 0x50, VTU_NAME ("  ")},
	{79719510,  0xf403, VNPL, 0x50, VTU_NAME ("  , /.")},
	{79719520,  0xf403, VNPL, 0x50, VTU_NAME ("  , /.")},
	{79731000,  VNPT,   VNPL, 0x8e, VTU_NAME ("  ")},
	{79732000,  VNPT,   VNPL, 0x4f, VTU_NAME ("  ()")},
	{79741000,  VNPT,   VNPL, 0x8c, VTU_NAME ("    -")},
	{79742000,  VNPT,   VNPL, 0x95, VTU_NAME ("  ")},
	{79743000,  VNPT,   VNPL, 0x88, VTU_NAME ("  ")},
	{79744000,  VNPT,   VNPL, 0x88, VTU_NAME ("  ")},
	{79745000,  VNPT,   VNPL, 0x95, VTU_NAME ("  ")},
	{79746000,  VNPT,   VNPL, 0x87, VTU_NAME ("  ")},
	{79747000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{79748000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{79749000,  VNPT,   VNPL, 0x96, VTU_NAME ("  . .")},
	{79749100,  VNPT,   VNPL, 0x96, VTU_NAME ("  .")},
	{79749200,  VNPT,   VNPL, 0x97, VTU_NAME (" . .")},
	{79751000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{79751100,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{79753000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{79754000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{79755000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{79756000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{79758000,  0x6513, VNPL, 0x51, VTU_NAME (" , .")},

	{79820000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},

	{81110000,  VNPT,   0x45, VNPG, VTU_NAME ("- ")},
	{81111000,  VNPT,   VNPL, VNPG, VTU_NAME ("Region")},
	{81121000,  VNPT,   VNPL, VNPG, VTU_NAME ("   ")},
	{81122100,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{81122200,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{81130000,  VNPT,   VNPL, VNPG, VTU_NAME ("")},
	{81140000,  VNPT,   0x45, VNPG, VTU_NAME ("  ")},
	{81140100,  VNPT,   0x45, VNPG, VTU_NAME (" ")},
	{81150000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},
	{81200000,  VNPT,   0x45, VNPG, VTU_NAME (" ")},

	{82100000,  VNPT,   VNPL, VNPG, VTU_NAME ("  ")},
	{82200000,  VNPT,   0x46, VNPG, VTU_NAME ("., ., . , . / .")},
	{82210000,  VNPT,   0x46, VNPG, VTU_NAME (" .   /")},
	{82220000,  VNPT,   0x46, VNPG, VTU_NAME (" ")},
	{82300000,  VNPT,   0x46, VNPG, VTU_NAME (" ")},
	{82320000,  VNPT,   0x46, VNPG, VTU_NAME (" ")},

	{83000000,  VNPT,   VNPL, VNPG, VTU_NAME ("Polar circles")},
	{84000000,  VNPT,   VNPL, VNPG, VTU_NAME ("Tropics")},
	{85100000,  VNPT,   VNPL, VNPG, VTU_NAME ("Isogones")},
	{85200000,  VNPT,   VNPL, VNPG, VTU_NAME ("Points of magnetic variation anomals")},
	{85300000,  VNPT,   VNPL, VNPG, VTU_NAME ("Areas of magnetic variation anomals")},

	{86000000,  VNPT,   VNPL, VNPG, VTU_NAME ("Compactly disposed objects")},

	{87000000,  VNPT,   VNPL, VNPG, VTU_NAME (",   ...")},

//	{88000000,  VNPT,   VNPL, VNPG, VTU_NAME (" ")},

    {90100000,  VNPT,   VNPL, 0x80, VTU_NAME ("Enclosed map")},
	{91000000,  VNPT,   VNPL, 0x80, VTU_NAME (" ")},
	{91010000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91012000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91020000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91022000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91030000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91032000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91042000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91050000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91060000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91070000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91100000,  VNPT,   VNPL, 0x80, VTU_NAME ("Line of short circuit")},
	{91130000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91140000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91150000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91170000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91190000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91200000,  0x1f00, 0x3e, 0x80, VTU_NAME (". .")},
	{91300000,  VNPT,   VNPL, 0x80, VTU_NAME ("Cartographic line")},
	{91400000,  VNPT,   VNPL, 0x80, VTU_NAME ("Cartographic area")},
	{91500000,  0x2800, 0x3e, 0x80, VTU_NAME ("Text")},
	{91600000,  VNPT,   VNPL, 0x80, VTU_NAME ("Colour area")},
	{91700000,  VNPT,   VNPL, 0x80, VTU_NAME ("Text in a marker")},
	{92020000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92022000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92130000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92150000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92170000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92171000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92172000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92180000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92181000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92182000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{92183000,  0x2800, 0x3e, 0x80, VTU_NAME (". , .")},
	{1000000004, 0x2800, 0x3e, 0x80, VTU_NAME (" ")},

	/////////////////////////////////////////////////////////////////////
	//
	// Non-standard ones (from 10_50otkr.RSC)
	//
	{92170001,  0x2800, 0x3e, 0x80, VTU_NAME (" ")},

	{93022000,  0x2800, 0x3e, 0x80, VTU_NAME (". ")},
	{93020000,  0x2800, 0x3e, 0x80, VTU_NAME (". ")},
	{93170000,  0x2800, 0x3e, 0x80, VTU_NAME (". ")},
	{93172000,  0x2800, 0x3e, 0x80, VTU_NAME (". ")},
};

# pragma const_seg ()

typedef std::map<DWORD, const vtu_type_t *> id2idx_t;
static id2idx_t g_id2idx;

const vtu_type_t * GetVTUType (DWORD _dwCode) {
	if (g_id2idx.empty ()) {
		for (size_t c = 0; c < ARRAY_SIZE (g_VTUTypes); ++ c) {
			const vtu_type_t & t = g_VTUTypes [c];
			g_id2idx.insert (id2idx_t::value_type (t.dwCode, & t));
		}
	}
	const id2idx_t::const_iterator i = g_id2idx.find (_dwCode);
	return i == g_id2idx.end () ? NULL : i->second;
}
