//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2009
//
// View plug-in's.
//

# include "StdAfx.h"
# include "Resource.h"
# include "ViewPlugins.h"
# include "GPSMapEditItf.h"
# include "COMRoot.h"
# include "COMViewPluginHost.h"
# include "Position.h"
# include "Toolbar.h"
# include "Menu.h"
# include "Log.h"
# include "PtAlgo.h"
# include "Reg.h"
# include "EHFilter.h"
# include "Settings.h"
# include "Wrappers.h"
# include "Globals.h"

struct vplugin_t {
	CComPtr<IViewPlugin> pPlugin;

	int nShowOrder;
	bool BeforeMap () const {return nShowOrder < 0;}
	bool operator < (const vplugin_t & _other) const {return nShowOrder < _other.nShowOrder;}

//	UINT uiCmd0, uiCmd1;
};

typedef std::list<vplugin_t> vplugins_t;
static vplugins_t g_PluginsBeforeMap;
static vplugins_t g_PluginsAfterMap;

struct vp_cmd_t {
	CComPtr<IViewPlugin> pPlugin;
	int nSwitchIdx;
	string_t strProgId;
};
typedef std::vector<vp_cmd_t> vp_cmds_t;
static vp_cmds_t g_vp_cmds;

static 
bool _StartPlugin (const char * _strProgId, UINT & _uiCmd) {
	USES_CONVERSION;

	// Create plug-in.
	CComPtr<IViewPlugin> pPlugin;
	const HRESULT hr = pPlugin.CoCreateInstance (A2OLE (_strProgId));
	if (! pPlugin || FAILED (hr)) {
		ReportWarning ("Failed to load view plug-in '%s'. Error code: 0x%08x.", _strProgId, hr);
		return false;
	}

	long lItfVer = 0;
	pPlugin->GetInterfaceVersion (& lItfVer);
	if (lItfVer != ViewPluginInterfaceVer) {
		ReportWarning ("The view plug-in '%s' implements incompatible interface.", _strProgId);
		return false;
	}

	long lSwitches = 0;
	pPlugin->GetSwitches (& lSwitches);
	if (lSwitches < 0)
		lSwitches = 0;

	long lShowOrder = 0;
	pPlugin->GetShowOrder (& lShowOrder);

	// Add separator to toolbar.
	if (lSwitches != 0) {
		TBBUTTON tbb;
		::memset (& tbb, 0, sizeof (tbb));
		tbb.fsStyle = TBSTYLE_SEP;
		tbb.fsState = TBSTATE_ENABLED;
		::SendMessage (g_hToolbarWnd, TB_ADDBUTTONS, 1, reinterpret_cast<LPARAM> (& tbb));
	}

	// Save g_vp_cmds.
	vp_cmds_t vp_cmds_saved = g_vp_cmds;

	CClientDC dcScreen (NULL);

	for (int nSwitch = 0; nSwitch < lSwitches; ++ nSwitch) {
		// Add menu item.
		CComBSTR bstrName;
		pPlugin->GetSwitchName (nSwitch, & bstrName);

		::InsertMenu (g_hmenuView, ID_GPS_TRACKING, MF_BYCOMMAND, _uiCmd + nSwitch, OLE2A (bstrName.m_str));

		// Add check-button to the toolbar.
		CMemDC dcMem (dcScreen, 16, 16);
		RECT rect = {0, 0, 16, 16};
		::FillRect (dcMem, & rect, ::GetSysColorBrush (COLOR_BTNFACE));
		pPlugin->DrawSwitchIcon (nSwitch, 16, 16, (long) HDC (dcMem));

		dcMem.UnselectBmp ();
		const TBADDBITMAP tbab = {NULL, (UINT) dcMem.DetachBitmap ()};
		const int nBitmap = ::SendMessage (g_hToolbarWnd, TB_ADDBITMAP, 1, reinterpret_cast<LPARAM> (& tbab));

		TBBUTTON tbb;
		::memset (& tbb, 0, sizeof (tbb));
		tbb.iBitmap   = nBitmap;
		tbb.fsStyle   = TBSTYLE_CHECK;
		tbb.fsState   = TBSTATE_ENABLED;
		tbb.idCommand = _uiCmd + nSwitch;
		::SendMessage (g_hToolbarWnd, TB_ADDBUTTONS, 1, reinterpret_cast<LPARAM> (& tbb));

		// Associate command ID.
		vp_cmd_t vpc;
		vpc.pPlugin    = pPlugin;
		vpc.nSwitchIdx = nSwitch;
		vpc.strProgId  = _strProgId;
		assert (_uiCmd - ID_PLUGINS_SWITCHES_0 + nSwitch == g_vp_cmds.size ());
		g_vp_cmds.push_back (vpc);

		// Set check state.
		char strSetting [128];
		::sprintf (strSetting, "VP_%s_%d", vpc.strProgId.c_str (), vpc.nSwitchIdx);
		const bool bChecked = GetSetting (strSetting, false);
		if (bChecked) {
			::SendMessage (g_hToolbarWnd, TB_CHECKBUTTON, _uiCmd + nSwitch, MAKELONG (bChecked, 0));
			::CheckMenuItem (g_hmenuView, _uiCmd + nSwitch, MF_BYCOMMAND | (bChecked ? MF_CHECKED : MF_UNCHECKED));
		}
	}

	// Initialize the plug-in.
	CComPtr<IApplication> pApplication;
	GetApplication (& pApplication);
	VARIANT_BOOL vbResult = VARIANT_FALSE;
	const HRESULT hrInit = pPlugin->Init (pApplication, & g_ComViewPluginHost, (long) g_hWnd, & vbResult);
	if (FAILED (hrInit) || vbResult == VARIANT_FALSE) {
		ReportWarning ("The view plug-in '%s' failed to load (error code 0x%08X).", _strProgId, hrInit);

		for (int nSwitch = 0; nSwitch < lSwitches; ++ nSwitch) {
			// Remove toolbar buttons.
			const size_t cBtns = ::SendMessage (g_hToolbarWnd, TB_BUTTONCOUNT, 0, 0);
			if (cBtns != 0)
				::SendMessage (g_hToolbarWnd, TB_DELETEBUTTON, cBtns - 1, 0);

			// Remove menu items.
			::DeleteMenu (g_hmenuView, _uiCmd + nSwitch, MF_BYCOMMAND);
		}
		// Remove toolbar separator.
		const size_t cBtns = ::SendMessage (g_hToolbarWnd, TB_BUTTONCOUNT, 0, 0);
		if (cBtns != 0)
			::SendMessage (g_hToolbarWnd, TB_DELETEBUTTON, cBtns - 1, 0);

		// Restore g_vp_cmds.
		vp_cmds_saved.swap (g_vp_cmds);

		return false;
	}

	// Add menu separator.
	if (lSwitches != 0)
		::InsertMenu (g_hmenuView, ID_GPS_TRACKING, MF_SEPARATOR | MF_BYCOMMAND, -1, NULL);

	// Add plug-in to the lists.
	vplugin_t vp;
	vp.pPlugin    = pPlugin;
	vp.nShowOrder = lShowOrder;
//	vp.uiCmd0     = _uiCmd;
//	vp.uiCmd1     = _uiCmd + lSwitches - 1;
	if (vp.BeforeMap ())
		g_PluginsBeforeMap.push_back (vp);
	else
		g_PluginsAfterMap.push_back (vp);

	_uiCmd += lSwitches;

	return true;
}

static
bool _StartPlugin_Safe (const char * _strProgId, UINT & _uiCmd) {
	__try {
		return _StartPlugin (_strProgId, _uiCmd);
	} __except (EHFilter (_exception_info ())) {
		ReportWarning ("Failed to load plug-in '%s' due to exception.", _strProgId);
		return false;
	}
}

void InitViewPlugins () {
	const DWORD dwStart = ::GetTickCount ();

	CRegistryKey hkeyGlobal (HKEY_LOCAL_MACHINE, c_strSettingsKey, KEY_READ);
	if (! hkeyGlobal.IsValid ())
		return;
	CRegistryKey hkeyViewPlugins (hkeyGlobal, "ViewPlugins", KEY_READ);
	if (! hkeyViewPlugins.IsValid ())
		return;

	UINT uiCmd = ID_PLUGINS_SWITCHES_0;

	// Enumerate ProgId's of plug-in's.
	DWORD dwIndex = 0;
	while (true) {
		if (uiCmd >= ID_PLUGINS_SWITCHES_0 + 100)
			break;

		char strValueName [64] = "";
		DWORD dwValueName = ARRAY_SIZE (strValueName);
		if (::RegEnumValue (hkeyViewPlugins, dwIndex, strValueName, & dwValueName, NULL, NULL, NULL, NULL) == ERROR_NO_MORE_ITEMS)
			break;
		++ dwIndex;

		char strSettingDisabled [128];
		::sprintf (strSettingDisabled, "VP_%s_disabled", strValueName);
		const bool bDisabled = GetSetting (strSettingDisabled, false);
		if (bDisabled)
			continue;

		const bool bOK = _StartPlugin_Safe (strValueName, uiCmd);
		if (! bOK)
			continue;
	}

	// Sort by show order.
	g_PluginsBeforeMap.sort ();
	g_PluginsAfterMap.sort ();

	ReportText ("* Plug-in's initialized in %d ms", ::GetTickCount () - dwStart);
}

void DestroyViewPlugins () {
	vplugins_t::const_iterator i;
	for (i = g_PluginsBeforeMap.begin (); i != g_PluginsBeforeMap.end (); ++ i) {
		const vplugin_t & vp = * i;
		vp.pPlugin->Destroy ();
	}
	g_PluginsBeforeMap.clear ();

	for (i = g_PluginsAfterMap.begin (); i != g_PluginsAfterMap.end (); ++ i) {
		const vplugin_t & vp = * i;
		vp.pPlugin->Destroy ();
	}
	g_PluginsAfterMap.clear ();

	g_vp_cmds.clear ();
}

bool ViewPlugins_HasToDrawBefore () {return ! g_PluginsBeforeMap.empty ();}
bool ViewPlugins_HasToDrawAfter  () {return ! g_PluginsAfterMap.empty ();}

////////////////////////////////////////////////////////////////////////
static
void _Render_Safe (IViewPlugin * _pPlugin, HDC _hDC, const rect_t & _rectVisible) {
	__try {
		_pPlugin->Render (
			reinterpret_cast<long> (_hDC),
			g_lWidth, g_lHeight,
			_rectVisible.y0, _rectVisible.x0, _rectVisible.y1, _rectVisible.x1
		);
	} __except (EHFilter (_exception_info ())) {}
}

void ViewPlugins_Draw (HDC _hDC, const rect_t & _rectVisible, bool _bBeforeMap) {
	const vplugins_t & vps = _bBeforeMap ? g_PluginsBeforeMap : g_PluginsAfterMap;
	for (vplugins_t::const_iterator i = vps.begin (); i != vps.end (); ++ i) {
		const vplugin_t & vp = * i;
		_Render_Safe (vp.pPlugin, _hDC, _rectVisible);
	}
}

void ViewPlugins_OnSwitch (UINT _uiCmd, bool _bMenu) {
	assert (_uiCmd - ID_PLUGINS_SWITCHES_0 < g_vp_cmds.size ());
	if (_uiCmd - ID_PLUGINS_SWITCHES_0 < g_vp_cmds.size ()) {
		const vp_cmd_t & vpc = g_vp_cmds [_uiCmd - ID_PLUGINS_SWITCHES_0];

		bool bChecked = ::SendMessage (g_hToolbarWnd, TB_ISBUTTONCHECKED, _uiCmd, 0) != 0;
		if (_bMenu)
			bChecked = ! bChecked;

		// Save check state.
		char strSetting [128];
		::sprintf (strSetting, "VP_%s_%d", vpc.strProgId.c_str (), vpc.nSwitchIdx);
		SaveSetting (strSetting, bChecked);

		// Update menu and toolbar.
		::SendMessage (g_hToolbarWnd, TB_CHECKBUTTON, _uiCmd, MAKELONG (bChecked, 0));
		::CheckMenuItem (g_hmenuView, _uiCmd, MF_BYCOMMAND | (bChecked ? MF_CHECKED : MF_UNCHECKED));

		// Notify plug-in.
		__try {
			vpc.pPlugin->OnSwitch (vpc.nSwitchIdx, bChecked ? VARIANT_TRUE : VARIANT_FALSE);
		} __except (EHFilter (_exception_info ())) {}
	}
}

////////////////////////////////////////////////////////////////////////

static
size_t _FindVpCmd (IViewPlugin * _pPlugin, long _lSwitchIdx) {
	const size_t cCmds = g_vp_cmds.size ();
	for (size_t c = 0; c < cCmds; ++ c) {
		const vp_cmd_t & vpc = g_vp_cmds [c];
		if (_pPlugin == vpc.pPlugin && vpc.nSwitchIdx == _lSwitchIdx)
			return c;
	}
	return -1;
}

bool ViewPlugins_GetSwitch (IViewPlugin * _pPlugin, long _lSwitchIdx) {
	const size_t cCmd = _FindVpCmd (_pPlugin, _lSwitchIdx);
	if (cCmd == -1)
		return false;
	return ::SendMessage (g_hToolbarWnd, TB_ISBUTTONCHECKED, ID_PLUGINS_SWITCHES_0 + cCmd, 0) != 0;
}

void ViewPlugins_SetSwitch (IViewPlugin * _pPlugin, long _lSwitchIdx, bool _bCheck) {
	const size_t cCmd = _FindVpCmd (_pPlugin, _lSwitchIdx);
	if (cCmd == -1)
		return;

	// Save check state.
	const vp_cmd_t & vpc = g_vp_cmds [cCmd];
	char strSetting [128];
	::sprintf (strSetting, "VP_%s_%d", vpc.strProgId.c_str (), vpc.nSwitchIdx);
	SaveSetting (strSetting, _bCheck);

	::SendMessage (g_hToolbarWnd, TB_CHECKBUTTON, ID_PLUGINS_SWITCHES_0 + cCmd, MAKELONG (_bCheck, 0));
	::CheckMenuItem (g_hmenuView, ID_PLUGINS_SWITCHES_0 + cCmd, MF_BYCOMMAND | (_bCheck ? MF_CHECKED : MF_UNCHECKED));
}

void ViewPlugins_EnableSwitch (IViewPlugin * _pPlugin, long _lSwitchIdx, bool _bEnable) {
	const size_t cCmd = _FindVpCmd (_pPlugin, _lSwitchIdx);
	if (cCmd == -1)
		return;

	::SendMessage (g_hToolbarWnd, TB_ENABLEBUTTON, ID_PLUGINS_SWITCHES_0 + cCmd, MAKELONG (_bEnable, 0));
	::EnableMenuItem (g_hmenuView, ID_PLUGINS_SWITCHES_0 + cCmd, MF_BYCOMMAND | (_bEnable ? MF_ENABLED : MF_GRAYED));
}

////////////////////////////////////////////////////////////////////////

static
void _OnLeftDown_Safe (IViewPlugin * _pPlugin, int _x, int _y, VARIANT_BOOL * _pvbProcessed) {
	__try {
		_pPlugin->OnLeftDown (_x, _y, _pvbProcessed);
	} __except (EHFilter (_exception_info ())) {}
}

bool ViewPlugins_OnLeftDown (int _x, int _y, bool _bBeforeMap) {
	const vplugins_t & vps = ! _bBeforeMap ? g_PluginsBeforeMap : g_PluginsAfterMap;
	for (vplugins_t::const_reverse_iterator i = vps.rbegin (); i != vps.rend (); ++ i) {
		const vplugin_t & vp = * i;

		VARIANT_BOOL vbProcessed = VARIANT_FALSE;
		_OnLeftDown_Safe (vp.pPlugin, _x, _y, & vbProcessed);
		if (vbProcessed != VARIANT_FALSE)
			return true;
	}
	return false;
}

static
void _OnLeftUp_Safe (IViewPlugin * _pPlugin, int _x, int _y, VARIANT_BOOL * _pvbProcessed) {
	__try {
		_pPlugin->OnLeftUp (_x, _y, _pvbProcessed);
	} __except (EHFilter (_exception_info ())) {}
}

bool ViewPlugins_OnLeftUp (int _x, int _y, bool _bBeforeMap) {
	const vplugins_t & vps = ! _bBeforeMap ? g_PluginsBeforeMap : g_PluginsAfterMap;
	for (vplugins_t::const_reverse_iterator i = vps.rbegin (); i != vps.rend (); ++ i) {
		const vplugin_t & vp = * i;

		VARIANT_BOOL vbProcessed = VARIANT_FALSE;
		_OnLeftUp_Safe (vp.pPlugin, _x, _y, & vbProcessed);
		if (vbProcessed != VARIANT_FALSE)
			return true;
	}
	return false;
}

static
void _OnLeftDblClk_Safe (IViewPlugin * _pPlugin, int _x, int _y, VARIANT_BOOL * _pvbProcessed) {
	__try {
		_pPlugin->OnLeftDblClk (_x, _y, _pvbProcessed);
	} __except (EHFilter (_exception_info ())) {}
}

bool ViewPlugins_OnLeftDblClk (int _x, int _y, bool _bBeforeMap) {
	const vplugins_t & vps = ! _bBeforeMap ? g_PluginsBeforeMap : g_PluginsAfterMap;
	for (vplugins_t::const_reverse_iterator i = vps.rbegin (); i != vps.rend (); ++ i) {
		const vplugin_t & vp = * i;

		VARIANT_BOOL vbProcessed = VARIANT_FALSE;
		_OnLeftDblClk_Safe (vp.pPlugin, _x, _y, & vbProcessed);
		if (vbProcessed != VARIANT_FALSE)
			return true;
	}
	return false;
}

static
void _OnRightDown_Safe (IViewPlugin * _pPlugin, int _x, int _y, VARIANT_BOOL * _pvbProcessed) {
	__try {
		_pPlugin->OnRightDown (_x, _y, _pvbProcessed);
	} __except (EHFilter (_exception_info ())) {}
}

bool ViewPlugins_OnRightDown (int _x, int _y, bool _bBeforeMap) {
	const vplugins_t & vps = ! _bBeforeMap ? g_PluginsBeforeMap : g_PluginsAfterMap;
	for (vplugins_t::const_reverse_iterator i = vps.rbegin (); i != vps.rend (); ++ i) {
		const vplugin_t & vp = * i;

		VARIANT_BOOL vbProcessed = VARIANT_FALSE;
		_OnRightDown_Safe (vp.pPlugin, _x, _y, & vbProcessed);
		if (vbProcessed != VARIANT_FALSE)
			return true;
	}
	return false;
}

static
void _OnRightUp_Safe (IViewPlugin * _pPlugin, int _x, int _y, VARIANT_BOOL * _pvbProcessed) {
	__try {
		_pPlugin->OnRightUp (_x, _y, _pvbProcessed);
	} __except (EHFilter (_exception_info ())) {}
}

bool ViewPlugins_OnRightUp (int _x, int _y, bool _bBeforeMap) {
	const vplugins_t & vps = ! _bBeforeMap ? g_PluginsBeforeMap : g_PluginsAfterMap;
	for (vplugins_t::const_reverse_iterator i = vps.rbegin (); i != vps.rend (); ++ i) {
		const vplugin_t & vp = * i;

		VARIANT_BOOL vbProcessed = VARIANT_FALSE;
		_OnRightUp_Safe (vp.pPlugin, _x, _y, & vbProcessed);
		if (vbProcessed != VARIANT_FALSE)
			return true;
	}
	return false;
}

static
void _OnMove_Safe (IViewPlugin * _pPlugin, int _x, int _y) {
	__try {
		_pPlugin->OnMove (_x, _y);
	} __except (EHFilter (_exception_info ())) {}
}

void ViewPlugins_OnMove (int _x, int _y) {
	vplugins_t::const_iterator i;
	for (i = g_PluginsBeforeMap.begin (); i != g_PluginsBeforeMap.end (); ++ i) {
		const vplugin_t & vp = * i;
		_OnMove_Safe (vp.pPlugin, _x, _y);
	}
	for (i = g_PluginsAfterMap.begin (); i != g_PluginsAfterMap.end (); ++ i) {
		const vplugin_t & vp = * i;
		_OnMove_Safe (vp.pPlugin, _x, _y);
	}
}

static
void _OnHover_Safe (IViewPlugin * _pPlugin, int _x, int _y, CComBSTR & _bstrHint) {
	__try {
		_pPlugin->OnHover (_x, _y, & _bstrHint);
	} __except (EHFilter (_exception_info ())) {}
}

void ViewPlugins_OnHover (int _x, int _y, bool _bBeforeMap, string_t & _strHint) {
	const vplugins_t & vps = ! _bBeforeMap ? g_PluginsBeforeMap : g_PluginsAfterMap;
	for (vplugins_t::const_reverse_iterator i = vps.rbegin (); i != vps.rend (); ++ i) {
		const vplugin_t & vp = * i;

		CComBSTR bstrHint;
		_OnHover_Safe (vp.pPlugin, _x, _y, bstrHint);
		if (bstrHint.Length () != 0) {
			USES_CONVERSION;
			_strHint = OLE2A (bstrHint.m_str);
			return;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////////

# include "Dialog.h"
# include "ListView.h"

class CCustomizeViewPlugins : public CDialog {
  public:
	CCustomizeViewPlugins () : CDialog (IDD_VIEW_PLUGINS) {}

	// Override.
	virtual bool OnInit (HWND _hwndFocus);
	virtual void OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl);
	virtual bool OnNotify (int _idCtrl, NMHDR * _pInfo);

  private:
	HWND m_hList;
	void _UpdateList ();

	std::vector<string_t> m_vecProgIds;
};

void CCustomizeViewPlugins::_UpdateList () {
	ListView_DeleteAllItems (m_hList);

	CRegistryKey hkeyGlobal (HKEY_LOCAL_MACHINE, c_strSettingsKey, KEY_READ);
	if (! hkeyGlobal.IsValid ())
		return;
	CRegistryKey hkeyViewPlugins (hkeyGlobal, "ViewPlugins", KEY_READ);
	if (! hkeyViewPlugins.IsValid ())
		return;

	size_t cItem = 0;

	// Enumerate ProgId's of plug-in's.
	DWORD dwIndex = 0;
	while (true) {
		char strValueName [64] = "";
		DWORD dwValueName = ARRAY_SIZE (strValueName);
		if (::RegEnumValue (hkeyViewPlugins, dwIndex, strValueName, & dwValueName, NULL, NULL, NULL, NULL) == ERROR_NO_MORE_ITEMS)
			break;
		++ dwIndex;

		m_vecProgIds.push_back (strValueName);

		// Get plug-in's display name.
		char strSettingDesc [128];
		::sprintf (strSettingDesc, "VP_%s_desc", strValueName);
		char strDescription [128] = "";
		GetSetting (strSettingDesc, strDescription, ARRAY_SIZE (strDescription));

		InsertItem (m_hList, cItem, cItem, "%s", strDescription [0] == '\0' ? strValueName : strDescription);

		char strSettingDisabled [128];
		::sprintf (strSettingDisabled, "VP_%s_disabled", strValueName);
		const bool bDisabled = GetSetting (strSettingDisabled, false);
		ListView_SetCheckState (m_hList, cItem, bDisabled ? FALSE : TRUE);

		++ cItem;
	}

	AutoSizeColumns (m_hList);
	EnsureSelectionVisible (m_hList);
}

bool CCustomizeViewPlugins::OnInit (HWND _hwndFocus) {
	m_hList = GetDlgItem (IDC_LIST);
	ListView_SetExtendedListViewStyleEx (m_hList, LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES, LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES);
	AddColumn (m_hList, 0, "Description");
	_UpdateList ();

	return true;
}

void CCustomizeViewPlugins::OnCommand (WORD _wCmd, WORD _wNotifyCode, HWND _hwndCtl) {
	switch (_wCmd) {
		case IDCANCEL:
			EndDialog (0);
			break;

		case IDOK: {
			const size_t cItems = ListView_GetItemCount (m_hList);
			for (size_t c = 0; c < cItems; ++ c) {
				const bool bChecked = ListView_GetCheckState (m_hList, c) != FALSE;

				assert (c < m_vecProgIds.size ());
				char strSettingDisabled [128];
				::sprintf (strSettingDisabled, "VP_%s_disabled", m_vecProgIds [c].c_str ());

				SaveSetting (strSettingDisabled, ! bChecked);
			}
			EndDialog (0);
			break;
		}
	}
}

bool CCustomizeViewPlugins::OnNotify (int _idCtrl, NMHDR * _pInfo) {
	return false;
}

void ViewPlugins_Customize (HWND _hWnd) {
	CCustomizeViewPlugins cvp;
	cvp.ShowModal (_hWnd);
}

/////////////////////////////////////////////////////////////////////////////////

bool ViewPlugins_CanCustomize () {
	CRegistryKey hkeyGlobal (HKEY_LOCAL_MACHINE, c_strSettingsKey, KEY_READ);
	if (! hkeyGlobal.IsValid ())
		return false;
	CRegistryKey hkeyViewPlugins (hkeyGlobal, "ViewPlugins", KEY_READ);
	if (! hkeyViewPlugins.IsValid ())
		return false;

	char strValueName [64] = "";
	DWORD dwValueName = ARRAY_SIZE (strValueName);
	if (::RegEnumValue (hkeyViewPlugins, 0, strValueName, & dwValueName, NULL, NULL, NULL, NULL) == ERROR_NO_MORE_ITEMS)
		return false;

	return true;
}