//
// GPSMapEdit
// (c) Konstantin Galichsky (kg@geopainting.com), 2008
//
// Variety of system wrappers.
//

# include "StdAfx.h"
# include "Wrappers.h"

CMemDC::CMemDC () :
	m_hdcMem (NULL),
	m_hbmp (NULL),
	m_hbmpPrev (NULL)
{}

CMemDC::CMemDC (HDC _hdc, int _w, int _h) :
	m_hdcMem (::CreateCompatibleDC (_hdc)),
	m_hbmp (::CreateCompatibleBitmap (_hdc, _w, _h)),
	m_hbmpPrev (NULL)
{
	SelectBmp ();
}

CMemDC::~CMemDC () {
	_Destroy ();
}

void CMemDC::_Destroy () {
	UnselectBmp ();

	if (m_hdcMem)
		::DeleteDC (m_hdcMem);
	if (m_hbmp)
		::DeleteObject (m_hbmp);
}

void CMemDC::Resize (HDC _hdc, int _w, int _h) {
	_Destroy ();

	m_hdcMem = ::CreateCompatibleDC (_hdc);
	m_hbmp = ::CreateCompatibleBitmap (_hdc, _w, _h);
	SelectBmp ();
}

void CMemDC::SelectBmp () {
	if (m_hdcMem && m_hbmp)
		m_hbmpPrev = (HBITMAP) ::SelectObject (m_hdcMem, m_hbmp);
}

void CMemDC::UnselectBmp () {
	if (m_hdcMem && m_hbmpPrev) {
		::SelectObject (m_hdcMem, m_hbmpPrev);
		m_hbmpPrev = NULL;
	}
}

HBITMAP CMemDC::DetachBitmap () {
	UnselectBmp ();
	const HBITMAP hbmp = m_hbmp;
	m_hbmp = NULL;
	return hbmp;
}

void CMemDC::Swap (CMemDC & _other) {
	std::swap (m_hdcMem,   _other.m_hdcMem);
	std::swap (m_hbmp,     _other.m_hbmp);
	std::swap (m_hbmpPrev, _other.m_hbmpPrev);
}
